/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.actions;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.windows.BrowserDialog;
import net.sf.dc.console.windows.itemforms.ItemForm;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.objects.ValidationException;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.fileimporters.FileImporter;
import net.sf.dc.fileimporters.ParseException;
import net.sf.dc.util.FileFilter;

public class AttachFileInfoAction extends DcAction {
    
    private ItemForm form;
    
    public AttachFileInfoAction(ItemForm form) {
        super();
        this.form = form;
    }

    public AttachFileInfoAction() {
        super();
    }
    
    public void actionPerformed(ActionEvent e) {
        DcModule module = getModule().getChild() != null ? getModule().getChild() : getModule();

        DcObject dco = form != null ? form.getObject() :
                                      module.getSearchView().getCurrent().getSelectedObject();
        
        String filename = dco.getFilename();
        
        FileImporter importer = dco.getModule().getImporter();
        if (importer == null)
            return;
        
        if (filename == null || filename.length() == 0 || !new File(filename).exists()) {
            String[] extensions = importer.getSupportedExtensions();
            FileFilter filter = extensions.length > 0 ? new FileFilter(extensions) : null;
            BrowserDialog dlg = new BrowserDialog(DcResources.getText("lblSelectFile"), filter);
            File file = dlg.showOpenFileDialog(form != null ? (JFrame) form : (JFrame) DataCrow.mainFrame, null);
            filename = file != null ? file.toString() : null;
        }
        
        if (filename != null && filename.length() > 0) {
            try {
                // overwrite empty information
                DcObject dcoNew = importer.parse(filename);
                
                if (dcoNew.getModule().getIndex() != dco.getModule().getIndex()) {
                    Collection children = dcoNew.getChildren(); 
                    for (Iterator iter = children.iterator(); iter.hasNext(); ) 
                        dcoNew = (DcObject) iter.next();
                }
                
                dco.merge(dcoNew);
                
                // overwrite parsed technical information, images and the file information
                for (Iterator iter = dcoNew.getFields().iterator(); iter.hasNext(); ) {
                    DcField field = (DcField) iter.next();
                    if (field.getValueType() == DcRepository.datatypes._PICTURE) {
                        Picture picture = (Picture) dcoNew.getValue(field.getIndex());
                        if (picture != null)
                            dco.setValue(field.getIndex(), picture.getValue(Picture._D_IMAGE));
                    
                    } else if (field.isTechnicalInfo() && field.getIndex() != DcObject._ID) {
                        dco.setValue(field.getIndex(), dcoNew.getValue(field.getIndex()));
                    } else if (field.getFieldType() == ComponentFactory._FILEFIELD ||
                               field.getFieldType() == ComponentFactory._FILELAUNCHFIELD) {
                        dco.setValue(field.getIndex(), filename);
                    }
                }
                
                if (form != null) {
                    form.setData(dco, true);
                } else {
                    if (dco.isChanged()) {
                        try {
                            dco.saveUpdate(true);
                        } catch (ValidationException ve) {
                            new MessageBox(ve.getMessage(), MessageBox._WARNING);
                        }
                    }
                }
                
            } catch (ParseException pe) {
                new MessageBox(pe.getMessage(), MessageBox._ERROR);
            }
        }
    }
    
    public KeyStroke getKeyStroke() {
        return KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK);        
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoImport;
    }
    
    public String getLabelShort() {
        return DcResources.getText("lblReadFileInfo");
    }

    public String getLabel() {
        return DcResources.getText("lblReadFileInfo");
    }
}