/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.actions;

import java.awt.event.ActionEvent;

import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.resources.DcResources;

public class ChangeViewAction extends DcAction {

    private final int viewIndex;

    public ChangeViewAction(int index) {
        super();
        viewIndex = index;
    }
    
    public void actionPerformed(ActionEvent e) {
         if (getModule().getCurrentSearchView().getIndex() != viewIndex) {
             if (getModule().getSearchView() != null)
                 getModule().getSearchView().setView(viewIndex);
             
             if (getModule().getInsertView() != null)
                 getModule().getInsertView().setView(viewIndex);
         }
    }
    
    public KeyStroke getKeyStroke() {
        if (viewIndex == MasterView._TABLE_VIEW)
            return KeyStroke.getKeyStroke("F2");
        else
            return KeyStroke.getKeyStroke("F3");
    }
    
    public ImageIcon getIcon() {
        if (viewIndex == MasterView._TABLE_VIEW)
            return IconLibrary._icoTableView;
        else
            return IconLibrary._icoCardView;
    }

    public String getLabel() {
        if (viewIndex == MasterView._TABLE_VIEW)
            return DcResources.getText("lblTableView");
        else
            return DcResources.getText("lblCardView");
    } 
}
