/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.actions;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;

public abstract class DcAction implements Action {

    protected static final int _SEARCHTAB = 0;
    protected static final int _INSERTTAB = 1;
    protected static final int _NOTETAB = 2;

    private int module = -1;

    public DcAction() {
    }

    public abstract ImageIcon getIcon();
    public abstract String getLabel();
    
    public String getLabelShort() {
        return getLabel();
    }
    
    public DcAction(int module) {
        this.module = module;
    }

    public int getCurrentTab() {
    	return DataCrow.mainFrame.getSelectedTab();
    }

    public DcModule getModule() {
        return module == -1 ? DcModules.getCurrent() : DcModules.get(module);
    }

    public void changeModule(int moduleIndex) {
        int activeModule = getModule().getIndex();
        if (moduleIndex != activeModule) {
            DataCrow.mainFrame.changeModule(moduleIndex, true);
        }
    }

    public abstract void actionPerformed(ActionEvent ae);

    public KeyStroke getKeyStroke() {
        return null;
    }
    
    public boolean isEnabled() {
        return true;
    }

    public void setEnabled(boolean arg0) {}

    public void addPropertyChangeListener(PropertyChangeListener arg0) {
    }

    public void removePropertyChangeListener(PropertyChangeListener arg0) {
    }

    public Object getValue(String arg0) {
        return null;
    }

    public void putValue(String arg0, Object arg1) {
    }
}
