/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.actions;

import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;

import net.sf.dc.console.windows.itemforms.ItemForm;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.resources.DcResources;

public class EditAsNewAction extends DcAction {

    private DcObject dco;
    
    public EditAsNewAction(DcObject dco) {
        this.dco = dco;
        
    }
    
    public void actionPerformed(ActionEvent ae) {
        DcModule module = dco.getModule();
        DcObject dcoCopy = module.getDcObject();
        dcoCopy.copy(dco);
        
        // mark images as new
        int[] fields = dco.getModule().getFieldIndices();
        for (int i = 0; i < fields.length; i++) {
            Object value = dcoCopy.getValue(fields[i]);
            if (value instanceof Picture) 
                ((Picture) value).isNew(true);
        }
        
        Collection children = dco.getChildren();
        if (children != null) {
            for (Iterator iter = children.iterator(); iter.hasNext(); ) {
                DcObject child = (DcObject) iter.next();
                DcObject childCopy = child.getModule().getDcObject();
                childCopy.copy(child);
                dcoCopy.addChild(childCopy);
            }
        }

        ItemForm form = new ItemForm(false, false, dcoCopy, false);
        form.setVisible(true);
        
        dco = null;
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoAdd;
    }

    public String getLabel() {
        return DcResources.getText("lblEditItemAsNew");
    }
}
