/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.actions;

import java.awt.event.ActionEvent;

import javax.swing.ImageIcon;

import net.sf.dc.console.wizards.Wizard;
import net.sf.dc.console.wizards.module.AlterModuleWizard;
import net.sf.dc.console.wizards.module.CreateModuleWizard;
import net.sf.dc.console.wizards.module.RelateModuleWizard;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.resources.DcResources;

public class StartModuleWizard extends DcAction {

    public static final int _CREATE_MODULE = 0;
    public static final int _ALTER_MODULE = 1;
    public static final int _RELATE_MODULE = 2;
    
    private final int type;
    
    public StartModuleWizard(int type) {
        super();
        this.type = type;
    }

    public void actionPerformed(ActionEvent e) {
        Wizard wizard;
        if (type == _CREATE_MODULE)
            wizard = new CreateModuleWizard();
        else if (type == _ALTER_MODULE)
            wizard = new AlterModuleWizard();
        else 
            wizard = new RelateModuleWizard();
         
        wizard.setVisible(true);
    }
    
    public ImageIcon getIcon() {
        return IconLibrary._icoWizard;
    }

    public String getLabel() {
        switch (type) {
        case _CREATE_MODULE:
            return DcResources.getText("lblModuleCreateWizard");
        case _ALTER_MODULE:
            return DcResources.getText("lblModuleAlterWizard");
        case _RELATE_MODULE:
            return DcResources.getText("lblModuleRelateWizard");
        }
        return "";
    }
}
