/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Insets;
import java.awt.ItemSelectable;
import java.awt.event.ActionListener;
import java.awt.event.ComponentListener;
import java.awt.event.ContainerListener;
import java.awt.event.FocusListener;
import java.awt.event.HierarchyBoundsListener;
import java.awt.event.HierarchyListener;
import java.awt.event.InputMethodListener;
import java.awt.event.ItemListener;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelListener;
import java.beans.PropertyChangeListener;
import java.beans.VetoableChangeListener;
import java.util.EventListener;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JTextField;
import javax.swing.JToolTip;
import javax.swing.LookAndFeel;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.AncestorListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.JTextComponent;
import javax.swing.tree.DefaultMutableTreeNode;

import net.sf.dc.actions.DcAction;
import net.sf.dc.console.components.DcButton;
import net.sf.dc.console.components.DcCheckBox;
import net.sf.dc.console.components.DcComboBox;
import net.sf.dc.console.components.DcDateField;
import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcFileLauncherField;
import net.sf.dc.console.components.DcFontSelector;
import net.sf.dc.console.components.DcIconSelectField;
import net.sf.dc.console.components.DcImageLabel;
import net.sf.dc.console.components.DcLabel;
import net.sf.dc.console.components.DcList;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.components.DcLookAndFeelSelector;
import net.sf.dc.console.components.DcMenu;
import net.sf.dc.console.components.DcMenuBar;
import net.sf.dc.console.components.DcMenuItem;
import net.sf.dc.console.components.DcModuleSelector;
import net.sf.dc.console.components.DcMultiLineToolTip;
import net.sf.dc.console.components.DcNumberField;
import net.sf.dc.console.components.DcPasswordField;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.console.components.DcProgramDefinitionsField;
import net.sf.dc.console.components.DcRadioButton;
import net.sf.dc.console.components.DcRatingComboBox;
import net.sf.dc.console.components.DcReferencesField;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.DcTabbedPane;
import net.sf.dc.console.components.DcTextPane;
import net.sf.dc.console.components.DcTimeField;
import net.sf.dc.console.components.DcTitledBorder;
import net.sf.dc.console.components.DcTree;
import net.sf.dc.console.components.DcUrlField;
import net.sf.dc.console.components.IComponent;
import net.sf.dc.console.components.renderers.AvailabilityComboBoxRenderer;
import net.sf.dc.console.components.renderers.ComboBoxRenderer;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcPropertyModule;
import net.sf.dc.core.objects.DcLookAndFeel;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;

import com.l2fprod.gui.plaf.skin.Skin;
import com.l2fprod.gui.plaf.skin.SkinLookAndFeel;

public final class ComponentFactory {

    public static final int _YESNOCOMBO = 1;
    public static final int _LONGTEXTFIELD = 2;
    public static final int _CHECKBOX = 3;
    public static final int _NUMBERFIELD = 4;
    public static final int _SHORTTEXTFIELD = 5;
    public static final int _URLFIELD = 6;
    public static final int _PICTUREFIELD = 7;
    public static final int _FONTSELECTOR = 8;
    public static final int _TIMEFIELD = 9;
    public static final int _THEMEFIELD = 10;
    public static final int _RATINGCOMBOBOX = 11;
    public static final int _FILEFIELD = 12;
    public static final int _PASSWORDFIELD = 13;
    public static final int _LOOKANDFEELSELECTOR = 14;
    public static final int _MODULESELECTOR = 15;
    public static final int _FILELAUNCHFIELD = 16;
    public static final int _COLORSELECTOR = 17;
    public static final int _DATEFIELD = 18;
    public static final int _PROGRAMDEFINITIONFIELD = 19;
    public static final int _REFERENCEFIELD = 20;
    public static final int _REFERENCESFIELD = 21;
    public static final int _AVAILABILITYCOMBO = 22;
    public static final int _SIMPLEPICTUREFIELD = 23;
    
    private static final Font fontUnreadable = new Font("Dialog", Font.PLAIN, 1);
    private static final Dimension iconButtonSize = new Dimension(25, 23);
    private static final Color colorDisabled = new Color(240,240,240);
    private static final Color colorRequired = new Color(120, 0, 0);

    private static LookAndFeel defaultLaf;

    public static void setLookAndFeel() {
        try {
            
            if (defaultLaf == null)
                defaultLaf = UIManager.getLookAndFeel();
            
            DcLookAndFeel laf = DcSettings.getLookAndFeel(DcRepository.settings.stLookAndFeel);
            if (laf.getType() == DcLookAndFeel._SKINLF) {
                Skin theSkinToUse = SkinLookAndFeel.loadThemePack(DataCrow.baseDir + "/themes/" +  laf.getFileName());
                SkinLookAndFeel.setSkin(theSkinToUse);
                UIManager.setLookAndFeel(new SkinLookAndFeel());
            } else if (laf.getType() == DcLookAndFeel._SYSTEMLF) {
                UIManager.setLookAndFeel(laf.getClassName());
            } else {
                UIManager.setLookAndFeel(defaultLaf);
            }
            
            UIManager.getLookAndFeelDefaults().put("SplitPane.border", null);
            UIManager.put("TabbedPane.lightHighlight", UIManager.get("TabbedPane.background") );
            UIManager.put("TabbedPane.darkShadow", UIManager.get("TabbedPane.background") );
            UIManager.put("TabbedPane.shadow", UIManager.get("TabbedPane.background") );
            
            UIManager.put("Tree.leafIcon", IconLibrary._icoTreeLeaf);
            UIManager.put("Tree.openIcon", IconLibrary._icoTreeOpen);
            UIManager.put("Tree.closedIcon", IconLibrary._icoTreeClosed);
            
        } catch (Exception e) {
            DcLog.add(e);
        }        
    }
    
    private static final void removeListeners(JComponent c) {
        removeListeners(MouseListener.class, c);
        removeListeners(ActionListener.class, c);
        removeListeners(ItemListener.class, c);
        removeListeners(AncestorListener.class, c);
        removeListeners(ComponentListener.class, c);
        removeListeners(ContainerListener.class, c);
        removeListeners(HierarchyBoundsListener.class, c);
        removeListeners(HierarchyListener.class, c);
        removeListeners(InputMethodListener.class, c);
        removeListeners(MouseMotionListener.class, c);
        removeListeners(MouseWheelListener.class, c);
        removeListeners(PropertyChangeListener.class, c);
        removeListeners(VetoableChangeListener.class, c);
    }
    
    private static final void removeListeners(Class clazz, JComponent c) {
         EventListener[] listeners = c.getListeners(clazz);
         for (int i = 0; i < listeners.length; i++) {
            
             if (listeners[i] instanceof java.awt.event.MouseListener) 
                 c.removeMouseListener((MouseListener) listeners[i]);
             if (listeners[i] instanceof java.awt.event.ActionListener) {
                 if (c instanceof JComboBox)
                     ((JComboBox)c).removeActionListener((ActionListener) listeners[i]);    
                 if (c instanceof AbstractButton)
                     ((AbstractButton)c).removeActionListener((ActionListener) listeners[i]);
             }
             
             if (listeners[i] instanceof java.awt.event.ItemListener)
                 ((ItemSelectable) c).removeItemListener((ItemListener) listeners[i]);
             

             if (listeners[i] instanceof AncestorListener)
                 c.removeAncestorListener((AncestorListener) listeners[i]);

             if (listeners[i] instanceof ComponentListener)
                 c.removeComponentListener((ComponentListener) listeners[i]);

             if (listeners[i] instanceof ContainerListener)
                 c.removeContainerListener((ContainerListener) listeners[i]);

             if (listeners[i] instanceof FocusListener)
                 c.removeFocusListener((FocusListener) listeners[i]);

             if (listeners[i] instanceof HierarchyBoundsListener)
                 c.removeHierarchyBoundsListener((HierarchyBoundsListener) listeners[i]);

             if (listeners[i] instanceof HierarchyListener)
                 c.removeHierarchyListener((HierarchyListener) listeners[i]);

             if (listeners[i] instanceof InputMethodListener)
                 c.removeInputMethodListener((InputMethodListener) listeners[i]);

             if (listeners[i] instanceof KeyListener)
                 c.removeKeyListener((KeyListener) listeners[i]);

             if (listeners[i] instanceof MouseMotionListener)
                 c.removeMouseMotionListener((MouseMotionListener) listeners[i]);

             if (listeners[i] instanceof MouseWheelListener)
                 c.removeMouseWheelListener((MouseWheelListener) listeners[i]);

             if (listeners[i] instanceof PropertyChangeListener)
                 c.removePropertyChangeListener((PropertyChangeListener) listeners[i]);

             if (listeners[i] instanceof VetoableChangeListener)
                 c.removeVetoableChangeListener((VetoableChangeListener) listeners[i]);
         }
    }
    
    public static final void clean(Component c) {
        if (c == null || !(c instanceof JComponent))
            return; 
        
        if (c instanceof DcList) 
            ((net.sf.dc.console.components.lists.DcList) c).clear();

        if (c instanceof DcTable) 
            ((DcTable) c).clearTable();
        
        if (c instanceof JComboBox)
            ((JComboBox) c).removeAllItems();
        
        if (c instanceof DcImageLabel)
            ((DcImageLabel) c).flush();

        if (c instanceof IComponent)
            ((IComponent) c).clear();

        removeListeners((JComponent) c);
        
        for (int i = 0; i < ((JComponent) c).getComponentCount(); i++)
            clean(((JComponent) c).getComponent(i));
        
        ((JComponent) c).removeAll();
    }

    public static final JComponent getComponent(int majormodule, 
                                                int minormodule, 
                                                int index, 
                                                String label, 
                                                int maxTextLength) {
        switch (index) {
            case _YESNOCOMBO:
                return getYesNoCombo();
            case _LONGTEXTFIELD:
                return getLongTextField(maxTextLength);
            case _CHECKBOX:
                return getCheckBox(label);
            case _NUMBERFIELD:
                return getNumberField();
            case _URLFIELD:
                return getURLField(maxTextLength);
            case _SIMPLEPICTUREFIELD:
                return getPictureField(true, false, false, label);
            case _PICTUREFIELD:
                return getPictureField(true, true, false, label);
            case _FONTSELECTOR:
                return getFontSelector();
            case _TIMEFIELD:
                return getTimeField();
            case _AVAILABILITYCOMBO:
                return getAvailabilityCombo();
            case _RATINGCOMBOBOX:
                return getRatingComboBox();
            case _FILEFIELD:
                return getFileField(false, false);
            case _PASSWORDFIELD:
                return getPasswordField();
            case _REFERENCEFIELD:
                return getObjectCombo(minormodule);
            case _PROGRAMDEFINITIONFIELD:
                return getProgramDefinitionField();
            case _LOOKANDFEELSELECTOR:
                return getLookAndFeelSelector();
            case _MODULESELECTOR:
                return getModuleSelector();
            case _FILELAUNCHFIELD:
                return getFileLaunchField();
            case _REFERENCESFIELD:
                return getReferencesField(DcModules.getMappingModIdx(majormodule, minormodule));
            case _DATEFIELD:
                return getDateField();                
            default:
                return getShortTextField(maxTextLength);
        }
    }

	public static final JComponent setUneditable(JComponent component) {
		if (component instanceof JComboBox)
			 ((JComboBox)component).setEnabled(false);
		else if (component instanceof JTextField)
			((JTextField) component).setEditable(false);

        component.setBackground(ComponentFactory.getDisabledColor());
		return component;
	}

	public static final DcProgramDefinitionsField getProgramDefinitionField() {
		return new DcProgramDefinitionsField();
	}

    public static final DcFileLauncherField getFileLaunchField() {
        return new DcFileLauncherField();
    }
    
    public static final DcModuleSelector getModuleSelector() {
        return new DcModuleSelector();
    }

    public static final DcReferencesField getReferencesField(int module) {
        return new DcReferencesField(module);
    }
    
    public static final DcPasswordField getPasswordField() {
    	return new DcPasswordField();
    }

    public static final DcFontSelector getFontSelector() {
        return new DcFontSelector();
    }

    public static final DcLookAndFeelSelector getLookAndFeelSelector() {
        return new DcLookAndFeelSelector();
    }    
    
    public static final DcTimeField getTimeField() {
    	return new DcTimeField();
    }
    
    public static final DcComboBox getMediaCombo(int module) {
        DcModule m = DcModules.get(module);
        int modIx = !(m instanceof DcPropertyModule) ? m.getPropertyModule(DcModules._STORAGEMEDIA).getIndex() : m.getIndex();

        DcComboBox combo = getComboBox();
        DataManager.registerUiComponent(combo, modIx);
        return combo;
    }    
    
    public static final JComboBox getStateCombo(int module) {
        DcModule m = DcModules.get(module);
        int modIx = !(m instanceof DcPropertyModule) ? m.getPropertyModule(DcModules._STATE).getIndex() : m.getIndex();

        DcComboBox combo = getComboBox();
        DataManager.registerUiComponent(combo, modIx);
        return combo;
    }    

    public static final DcComboBox getContactPersonCombo() {
        DcComboBox comboBox = getComboBox();
        DataManager.registerUiComponent(comboBox, DcModules._CONTACTPERSON);
        return comboBox;
    }

    public static final DcComboBox getObjectCombo(int module) {
        DcComboBox comboBox = getComboBox();
        DataManager.registerUiComponent(comboBox, module);
        return comboBox;
    }
    
    public static final DcComboBox getAvailabilityCombo() {
        DcComboBox comboBox = new DcComboBox();
        comboBox.setFont(getStandardFont());
        comboBox.setBorder(getBorder());
        comboBox.setRenderer(new AvailabilityComboBoxRenderer());

        comboBox.addItem(null);
        comboBox.addItem(Boolean.TRUE);
        comboBox.addItem(Boolean.FALSE);

        return comboBox;
    }
    
    public static final DcRadioButton getRadioButton(String label, ImageIcon icon, String command) {
        DcRadioButton radioButton = new DcRadioButton(label, icon, false);
        radioButton.setSelectedIcon(icon);
        radioButton.setActionCommand(command);
        return radioButton;
    }
    
    public static final DcRadioButton getRadioButton(String label, ImageIcon icon) {
        DcRadioButton radioButton = new DcRadioButton(label, icon, false);
        return radioButton;
    }

    public static final DcFileField getFileField(boolean save, boolean dirsOnly) {
        DcFileField ff = new DcFileField(null);
        ff.setModus(save, dirsOnly);
        return ff;
    }

    public static final DcFileField getFileField(boolean save, boolean dirsOnly, FileFilter filter) {
        DcFileField ff = new DcFileField(filter);
        ff.setModus(save, dirsOnly);
        return ff;
    }

    public static final DcDateField getDateField() {
        DcDateField dateField = new DcDateField();
        dateField.setFont(getStandardFont());
        dateField.setPreferredSize(new Dimension(dateField.getWidth(), 23));
        return dateField;
    }

    public static final DcPictureField getPictureField(boolean scaled,
                                                       boolean allowActions,
                                                       boolean thumbnail,
                                                       String name) {
        DcPictureField pictureField = new DcPictureField(scaled, allowActions, thumbnail, name) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        pictureField.setBorder(getBorder());
        return pictureField;
    }

    public static final DcPictureField getPictureField(boolean allowZoom) {
        DcPictureField pictureField = new DcPictureField(allowZoom) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        pictureField.setBorder(getBorder());
        return pictureField;
    }

    public static final DcUrlField getURLField(int maxLength) {
        DcUrlField urlField = new DcUrlField(maxLength);
        urlField.setFont(getStandardFont());
        return urlField;
    }

    public static final DcTree getTree(DefaultMutableTreeNode model) {
        DcTree tree = new DcTree(model);
        tree.setFont(getStandardFont());
        return tree;
    }

    public static final DcNumberField getNumberField() {
        DcNumberField numberField = new DcNumberField();
        numberField.setPreferredSize(new Dimension(numberField.getWidth(), 21));
        numberField.setFont(getStandardFont());
        numberField.setBorder(getBorder());
        return numberField;
    }

    public static final DcRatingComboBox getRatingComboBox() {
        return new DcRatingComboBox();
    }

    public static final DcComboBox getMP3GenreComboBox() {
        DcComboBox genreComboBox = getComboBox();
        genreComboBox.addItem("");
        genreComboBox.setSelectedIndex(0);
        genreComboBox.setFont(getStandardFont());

        for (int i = 0; i < DcRepository.collections.colMusicGenres.length; i++) {
            genreComboBox.addItem(DcRepository.collections.colMusicGenres[i]);
        }

        return genreComboBox;
    }

    public static final DcComboBox getComboBox(Object[] items) {
        DcComboBox comboBox = new DcComboBox(items);
        comboBox.setFont(getStandardFont());
        comboBox.setBorder(getBorder());
        comboBox.setRenderer(new ComboBoxRenderer());
        return comboBox;
    }    

    public static final DcImageLabel getImageLabel() {
        return new DcImageLabel();
    }

    public static final DcIconSelectField getIconSelectField(ImageIcon icon) {
        return new DcIconSelectField(icon);
    }
    
    public static final DcImageLabel getImageLabel(ImageIcon icon) {
        return new DcImageLabel(icon);
    }
    
    public static final DcComboBox getComboBox() {
        DcComboBox comboBox = new DcComboBox();
        comboBox.setFont(getStandardFont());
        comboBox.setBorder(getBorder());
        comboBox.setRenderer(new ComboBoxRenderer());
        return comboBox;
    }

    public static final DcComboBox getComboBox(DefaultComboBoxModel model) {
        DcComboBox comboBox = new DcComboBox(model);
        comboBox.setFont(getStandardFont());
        comboBox.setBorder(getBorder());
        comboBox.setRenderer(new ComboBoxRenderer());
        return comboBox;
    }
    
    public static final DcCheckBox getCheckBox(String labelText) {
        DcCheckBox checkBox = new DcCheckBox(labelText);
        checkBox.setFont(getSystemFont());
        return checkBox;
    }

    public static final DcMenuItem getMenuItem(String text) {
        DcMenuItem menuItem = new DcMenuItem(text);
        menuItem.setFont(getSystemFont());
        menuItem.setLayout(new FlowLayout(FlowLayout.LEFT));
        return menuItem;
    }
    
    public static final DcMenuItem getMenuItem(AbstractAction action) {
        DcMenuItem menuItem = new DcMenuItem(action);
        menuItem.setFont(getSystemFont());
        menuItem.setLayout(new FlowLayout(FlowLayout.LEFT));
        return menuItem;
    }    
    
    public static final DcMenuItem getMenuItem(ImageIcon icon, String text) {
        DcMenuItem menuItem = getMenuItem(text);
        menuItem.setIcon(icon);
        return menuItem;
    }

    public static final DcMenuItem getMenuItem(DcAction action) {
        DcMenuItem menuItem = getMenuItem(action.getLabel());
        menuItem.setIcon(action.getIcon());
        menuItem.addActionListener(action);
        menuItem.setAccelerator(action.getKeyStroke());
        return menuItem;
    }

    public static final DcMenu getMenu(String text) {
        DcMenu menu = new DcMenu(text);
        return menu;
    }

    public static final DcMenu getMenu(ImageIcon icon, String text) {
        DcMenu menu = new DcMenu(text);
        menu.setFont(getSystemFont());
        menu.setIcon(icon);
        return menu;
    }

    public static final DcButton getTableHeader(String title) {
        DcButton button = getButton(title);
        button.setFont(ComponentFactory.getStandardFont());
        button.setPreferredSize(new Dimension(button.getWidth(), 20));
        button.setBorder(new EtchedBorder(5));
        return button;
    }

    public static final DcButton getIconButton(ImageIcon icon) {
        DcButton button = getButton("");
        button.setIcon(icon);
        button.setFont(getSystemFont());
        
        button.setMaximumSize(iconButtonSize);
        button.setMinimumSize(iconButtonSize);
        button.setPreferredSize(iconButtonSize);
        
        return button;
    }
    
    public static final DcButton getButton(ImageIcon icon) {
        DcButton button = getButton("");
        button.setIcon(icon);
        button.setFont(getSystemFont());
        return button;
    }

    public static final DcButton getButton(String buttonText) {
        DcButton button = new DcButton() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        
        ToolTipManager.sharedInstance().registerComponent(button);
        button.setName("bt" + buttonText);
        button.setText(buttonText);
        
        int height = 25;
        
        button.setPreferredSize(new Dimension(110, height));
        button.setMaximumSize(new Dimension(110, height));
        button.setMinimumSize(new Dimension(110, height));
        button.setFont(getSystemFont());
        return button;
    }

    public static final DcLongTextField getTextArea() {
        DcLongTextField textArea = new DcLongTextField(Integer.MAX_VALUE);
        textArea.setWrapStyleWord(true);
        textArea.setLineWrap(true);
        textArea.setEditable(true);
        textArea.setMargin(new Insets(5,5,5,5));
        textArea.setFont(getStandardFont());
        return textArea;
    }

    public static final DcTextPane getTextPane() {
        DcTextPane textpane = new DcTextPane();
        textpane.setFont(getStandardFont());
        return textpane;
    }

    public static final DcLongTextField getLongTextField(int maxTextLength) {
        DcLongTextField longText = new DcLongTextField(maxTextLength);
        longText.setWrapStyleWord(true);
        longText.setLineWrap(true);
        longText.setEditable(true);
        longText.setMargin(new Insets(5,5,5,5));
        longText.setFont(getStandardFont());
        return longText;
    }

    public static final DcLabel getLabel(ImageIcon icon) {
        DcLabel label = new DcLabel(icon);
        return label;
    }

    public static final JLabel getLabel(String labelText, ImageIcon icon) {
        JLabel label = new DcLabel(labelText);
        label.setIcon(icon);
        return label;
    }

    public static final  JLabel getLabel(String labelText, int length) {
        JLabel label = getLabel(labelText);
        label.setPreferredSize(new Dimension(length, 23));
        label.setText(labelText);
        return label;
    }

    public static final DcLabel getLabel(String labelText) {
        DcLabel label = new DcLabel() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };

        label.setText(labelText);
        label.setRequestFocusEnabled(false);
        label.setFont(getSystemFont());
        label.setToolTipText(labelText);

        return label;
    }

    public static final DcShortTextField getShortTextField(int maxTextLength) {
        DcShortTextField textField = new DcShortTextField(maxTextLength);
        textField.setFont(getStandardFont());
        textField.setPreferredSize(new Dimension(50, 21));
        textField.setBorder(getBorder());
        return textField;
    }

    public static final DcShortTextField getTextFieldDisabled() {
        DcShortTextField textField = new DcShortTextField(4000);
        textField.setPreferredSize(new Dimension(50, 21));
        textField.setBorder(getBorder());
        textField.setEnabled(false);
        textField.setEditable(false);
        textField.setFont(ComponentFactory.getStandardFont());
        textField.setForeground(ComponentFactory.getDisabledColor());

        return textField;
    }
    
    public static final DcNumberField getIdFieldDisabled() {
        DcNumberField textField = new DcNumberField();
        textField.setPreferredSize(new Dimension(50, 21));
        textField.setBorder(getBorder());
        textField.setEnabled(false);
        textField.setEditable(false);
        textField.setFont(ComponentFactory.getStandardFont());
        textField.setForeground(ComponentFactory.getDisabledColor());
        return textField;
    }    

    public static final DcTabbedPane getTabbedPane() {
        DcTabbedPane tabbedPane = new DcTabbedPane();
        tabbedPane.setFont(getSystemFont());
        return tabbedPane;
    }

    public static final DcTable getDCTable(boolean readonly, boolean caching) {
        DcTable table = new DcTable(readonly, caching) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        return table;
    }

    public static final DcTable getDCTable(DcModule module, boolean readonly, boolean caching) {
    	DcTable table = new DcTable(module, readonly, caching) {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        return table;
    }

    public static final Icon getIcon(String iconLocation) {
        Icon icon = new ImageIcon(iconLocation);
        return icon;
    }

    public static final JMenuBar getMenuBar() {
        DcMenuBar menuBar = new DcMenuBar() {
            public JToolTip createToolTip() {
                return new DcMultiLineToolTip();
            }
        };
        menuBar.setFont(getStandardFont());
        return menuBar;
    }

    public static final DcComboBox getYesNoCombo() {
        DcComboBox comboBox = getComboBox();
        comboBox.addItem("");
        comboBox.addItem(DcResources.getText("lblYes"));
        comboBox.addItem(DcResources.getText("lblNo"));
        comboBox.setFont(getStandardFont());

        return comboBox;
    }

    public static final Border getBorder() {
        return BorderFactory.createLineBorder(Color.GRAY);
    }

    public static final TitledBorder getTitleBorder(String title) {
        TitledBorder border = new DcTitledBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY, 1), title);
        border.setTitleFont(getSystemFont());
        return border;
    }

    public static final Color getCurrentForegroundColor() {
        Color color = UIManager.getLookAndFeelDefaults().getColor("TextField.foreground");
        return color == null ? Color.BLACK : color;
    }
    
    public static final Font getStandardFont() {
        return DcSettings.getFont(DcRepository.settings.stSystemFontNormal);
    }

    public static final Font getSystemFont() {
        return DcSettings.getFont(DcRepository.settings.stSystemFontBold);
    }

    public static final Font getUnreadableFont() {
        return fontUnreadable;
    }

    public static final Color getDisabledColor() {
       return colorDisabled;
    }

    public static final Color getRequiredColor() {
        return colorRequired;
    }

    public static final Color getTableHeaderColor() {
        return new Color(220, 220, 220);
    }

    public static final void setValue(JComponent c, Object o) {
        if (c instanceof DcPictureField) {
            Object value = o instanceof Picture ? ((Picture) o).getValue(Picture._D_IMAGE) : o;
            ((DcPictureField) c).setValue(value);
        } else if (c instanceof IComponent) {
            ((IComponent) c).setValue(o); 
        } else if (c instanceof JComboBox) { 
            ((JComboBox) c).setSelectedItem(o);
        } else if (c instanceof JCheckBox) { 
            Boolean b = o instanceof String ? Boolean.valueOf((String) o) : ((Boolean) o);
            ((JCheckBox) c).setSelected(b.booleanValue());
        } else if (c instanceof JTextComponent) {
            String s = o instanceof String ? (String) o : o != null ? o.toString() : "";
            ((JTextComponent) c).setText(s);
        }
    }

    public static Object getValue(JComponent c) {
        
        if (c instanceof IComponent)
            return ((IComponent) c).getValue(); 
        
        if (c instanceof JComboBox) 
            return ((JComboBox) c).getSelectedItem();
        
        if (c instanceof JCheckBox) 
            return new Boolean(((JCheckBox) c).isSelected());

        if (c instanceof JTextComponent)
            return ((JTextComponent) c).getText();
        
        return null;
    }
}
