/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JTextField;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.windows.datepicker.DatePickerDialog;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.settings.DcSettings;

public class DcDateField extends JComponent implements IComponent, ActionListener {

    //private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    
    private static final SimpleDateFormat sdf = new SimpleDateFormat("EEEEE, d MMMMM yyyy");
    
    private JTextField text;
    private JButton button;

    public DcDateField() {

        text = ComponentFactory.getTextFieldDisabled();
        button = ComponentFactory.getIconButton(IconLibrary._icoCalendar);
        
        this.setLayout(Layout.getGBL());
        
        button.addActionListener(this);

        add( text,   Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                    ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                     new Insets(0, 0, 0, 0), 0, 0));
        add( button, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.EAST, GridBagConstraints.NONE,
                     new Insets(0, 0, 0, 0), 0, 0));
    }

    public void clear() {
        text = null;
        button = null;
    }
    
    public void setValue(Object value) {
        if (value instanceof Date) {
            setValue((Date) value);
        } else if (value instanceof String) {
            try {
                setValue(sdf.parse((String) value));
            } catch (Exception ignore) {}
        }
    }      
    
    public void setValue(Date date) {
        if (date == null)
            text.setText("");
        else 
            text.setText(sdf.format(date));
    }
    
    public Object getValue() {
        String date = text.getText();
        try {
            return sdf.parse(date);
        } catch (Exception e) {}
        
        return null;
    }
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getBoolean(DcRepository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        super.paintComponent( g2d );
    }

    public void actionPerformed(ActionEvent e) {
        
        Component top = null;
        Component parent = getParent();
        while (parent != null) {
            parent = parent.getParent();
            top = parent != null ? parent : top;
        }
        
        DatePickerDialog dp;
        if (top != null && top instanceof JFrame)
            dp = new DatePickerDialog((JFrame) top);
        else
            dp = new DatePickerDialog();
            
        dp.setDate((Date) getValue());
        dp.setVisible(true);
        
        setValue(dp.getDate() != null ? dp.getDate().getTime() : null);
    }
}