/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.ComponentInputMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.windows.help.HelpDialog;
import net.sf.dc.util.Utilities;

public class DcDialog extends JDialog {

    private String helpIndex = null;

    public DcDialog(JFrame parent) {
        super(parent);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setModal(true);
        addKeyListener(KeyStroke.getKeyStroke("F1"), new HelpAction(), "HelpAction");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), new CloseAction(), "CloseAction");
    }

    public DcDialog() {
        super();
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setModal(true);
        addKeyListener(KeyStroke.getKeyStroke("F1"), new HelpAction(), "HelpAction");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), new CloseAction(), "CloseAction");
    }
    
    public void close() {
        ComponentFactory.clean(this);
        dispose();
    }

    public void setHelpIndex(String helpIndex) {
    	this.helpIndex = helpIndex;
    }

    public void setVisible(boolean visible) {
        super.setVisible(visible);
    }

    public void addKeyListener(KeyStroke keyStroke, Action action, String name) {
        rootPane.getInputMap().put(keyStroke, name);
        rootPane.getActionMap().put(name, action);

        ActionMap parentActionMap = rootPane.getActionMap();
        ActionMap am = new ActionMap();
        am.setParent(parentActionMap);
        am.put(name, action);
        rootPane.setActionMap(am);

        InputMap parentInputMap = rootPane.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        ComponentInputMap im = new ComponentInputMap(getRootPane());
        
        try {
            im.setParent(parentInputMap);
        } catch (Exception e) {}            
        
        im.put(keyStroke, name);
        rootPane.setInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT, im);
    }

    public void setCenteredLocation() {
        setLocation(Utilities.getCenteredWindowLocation(getSize()));
    }

    protected void showHelp() {
        if (helpIndex != null) {
            HelpDialog.setHelpIndex(helpIndex);
        }
        new HelpDialog(this);
    }

    private class HelpAction extends AbstractAction {
        public void actionPerformed(ActionEvent arg0) {
            showHelp();
        }
    }
    
    private class CloseAction extends AbstractAction {
        public void actionPerformed(ActionEvent arg0) {
            close();
        }
    }
}
