/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;

public class DcFieldSelectorField extends JComponent implements IComponent {

    private Map componentMap = new HashMap();
    private final int module;

    public DcFieldSelectorField(int module) {
        this.module = module;
        buildComponent();
    }

    public Object getValue() {
        return getSelectedFields();
    }

    public void setValue(Object o) {
        if (o instanceof int[]) {
            setSelectedFields((int[]) o);
        }
    }
    
    public void clear() {
        componentMap.clear();
        componentMap = null;
    }    

    public void invertSelection() {
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext();) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);
            checkBox.setSelected(!checkBox.isSelected());
        }
    }
    
    public void selectAll() {
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext();) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);
            checkBox.setSelected(true);
        }
    }

    public void unselectAll() {
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext();) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);
            checkBox.setSelected(false);
        }
    }

    public Object getSelectedFieldIndices() {
        Collection fields = getSelectedFields();
        int[] indices = new int[fields.size()];
        int counter = 0;
        for (Iterator iter = fields.iterator(); iter.hasNext(); counter++) {
            indices[counter] = ((DcField) iter.next()).getIndex();
        }
        
        return indices;
    }    

    public Collection getSelectedFields() {
        Collection fields = new ArrayList();
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext();) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);

            if (checkBox.isSelected())
                fields.add(field);
        }

        return fields;
    }

    public void setSelectedFields(int[] fields) {
        for (Iterator iter = componentMap.keySet().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            JCheckBox checkBox = (JCheckBox) componentMap.get(field);

            if (fields.length == 0) {
                checkBox.setSelected(!field.isHidden());
            } else {
                for (int j = 0; j < fields.length; j++) {
                    if (field.getIndex() == fields[j])
                        checkBox.setSelected(true);
                }
            }
        }
    }

    private void buildComponent() {
        setLayout(Layout.getGBL());

        JPanel panel = new JPanel();
        panel.setLayout(Layout.getGBL());

        DcObject dco = DcModules.get(module).getDcObject();

        int x = 0;
        int y = 0;

        Collection fields = dco.getFields();
        for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();

            if (field.getIndex() != DcObject._ID) {
                JCheckBox checkBox = ComponentFactory.getCheckBox(field.getLabel());
                componentMap.put(field, checkBox);
    
                panel.add(checkBox, Layout.getGBC(x, y++, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets( 0, 10, 0, 0), 0, 0));
    
                if (y == 12) {
                    x++;
                    y = 0;
                }
            }
        }

        add(panel, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                   new Insets( 0, 0, 0, 0), 0, 0));

    }
}
