/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.ComponentInputMap;
import javax.swing.ImageIcon;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.windows.help.HelpDialog;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.util.SwingUtil;
import net.sf.dc.util.Utilities;

public class DcFrame extends JFrame {

    private String helpIndex = null;

    public DcFrame(String title, ImageIcon icon) {
        super(title);
        
        if (icon != null)
            setIconImage(icon.getImage());
        else
            setIconImage(IconLibrary._icoMain.getImage());
        
        SwingUtil.setRootFrame(this);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            }
        });

        setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        
        addKeyListener(KeyStroke.getKeyStroke("F1"), new HelpAction(), "HelpAction");
        addKeyListener(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), new CloseAction(), "CloseAction");
    }
    
    public void close() {
        ComponentFactory.clean(this);
        dispose();
    }

    protected void addKeyListener(KeyStroke keyStroke, Action action, String name) {
        rootPane.getInputMap().put(keyStroke, name);
        rootPane.getActionMap().put(name, action);

        JRootPane rootPane = getRootPane();
        ActionMap parentActionMap = rootPane.getActionMap();
        ActionMap am = new ActionMap();
        am.setParent(parentActionMap);
        am.put(name, action);
        rootPane.setActionMap(am);

        InputMap parentInputMap = rootPane.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        ComponentInputMap im = new ComponentInputMap(getRootPane());
        
        try {
            im.setParent(parentInputMap);
        } catch (Exception e) {}

        im.put(keyStroke, name);
        rootPane.setInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT, im);
    }

    public void setVisible(boolean visible) {
        super.setVisible(visible);
        if (visible) SwingUtil.setRootFrame(this);
    }

    public void setHelpIndex(String helpID) {
        helpIndex =  helpID;
    }

    protected void setCenteredLocation() {
        setLocation(Utilities.getCenteredWindowLocation(getSize()));
    }

    protected void showHelp() {
        if (helpIndex != null) 
            HelpDialog.setHelpIndex(helpIndex);
        new HelpDialog(this);
    }

    private class HelpAction extends AbstractAction {
        public void actionPerformed(ActionEvent arg0) {
            showHelp();
        }
    }
    
    private class CloseAction extends AbstractAction {
        public void actionPerformed(ActionEvent arg0) {
            close();
        }
    }  
}
