/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import javax.swing.ImageIcon;

import net.sf.dc.console.windows.BrowserDialog;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.util.PictureFileFilter;
import net.sf.dc.util.Utilities;

public class DcIconSelectField extends DcImageLabel {

    private final Dimension size;
    private boolean changed = false;
    
    public DcIconSelectField(Dimension size) {
        super();
        
        setOpaque(true);
        addMouseListener(new LoadImageAction());
        
        this.size = size;
    }
    
    public DcIconSelectField(ImageIcon icon) {
        image = Utilities.toBufferedImage(icon, false);
        repaint();
        
        setOpaque(true);
        addMouseListener(new LoadImageAction());
        
        this.size = new Dimension(icon.getIconWidth(), icon.getIconHeight());
    }
    
    public void setIcon(ImageIcon icon) {
        image = Utilities.toBufferedImage(icon, false);
        repaint();
        revalidate();
    }
    
    public Dimension getSize() {
        return size;
    }

    public Dimension getPreferredSize() {
        return size;
    }

    public Dimension getMinimumSize() {
        return size;
    }

    public Dimension getMaximumSize() {
        return size;
    }
    
    private void loadImage() {
        PictureFileFilter filter = new PictureFileFilter();
        BrowserDialog dlg = new BrowserDialog("", filter);
        File file = dlg.showOpenFileDialog(this, null);
        
        if (file == null)
            return;
        
        try {
            byte[] bytes = Utilities.readFile(file);
            Image image = Utilities.getScaledImage(bytes, size.width, size.height);
            setIcon(new ImageIcon(image));
            image = null;
            changed = true;
        } catch (Exception exp) {
            new MessageBox(exp.getMessage(), MessageBox._ERROR);
        }
    }
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        if (image != null) {
            int centerX = ((super.getWidth() - image.getWidth()) / 2);
            int centerY = ((super.getHeight() - image.getHeight()) / 2);
            g2d.drawImage(image, centerX, centerY, this);
        }
    }      
    
    private class LoadImageAction implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            loadImage();
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }

    public boolean isChanged() {
        return changed;
    }
}
