/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JComponent;

import net.sf.dc.util.Utilities;

public class DcImageLabel extends JComponent {
    
    protected BufferedImage image;

    public DcImageLabel() {
        setOpaque(false);
    }
    
    public void setIcon(ImageIcon icon) {
        setImage(icon);
    }
    
    public ImageIcon getIcon() {
        return new ImageIcon(image);
    }
    
    public DcImageLabel(ImageIcon icon) {
        this();
        setImage(icon);
    }
    
    private void setImage(ImageIcon icon) {
        image = Utilities.toBufferedImage(icon, true);
        repaint();
    }

    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        g2d.setColor(this.getBackground());
        g2d.fillRect(0, 0, getWidth(), getHeight());
        
        if (image != null) {
            int centerX = ((super.getWidth() - image.getWidth()) / 2);
            int centerY = ((super.getHeight() - image.getHeight()) / 2);
            g2d.drawImage(image, centerX, centerY, this);
        }
    }    

    public void flush() {
        if (image != null)
            image.flush();
        
        image = null;
        removeAll();
    }
}
