/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;

import net.sf.dc.core.DcRepository;
import net.sf.dc.settings.DcSettings;

public class DcList extends JList {

    public DcList(Collection data) {
        this();
        this.setListData(data.toArray());
    }
	
    public DcList() {
        setCellRenderer(new CustomCellRenderer());   
    }
    
    public Collection getData() {
    	Collection data = new ArrayList();
    	for (int i = 0; i < getModel().getSize(); i++) {
    		data.add(getModel().getElementAt(i));
    	}
    	return data;
    }
    
    public void setFont(Font font) {
        super.setFont(DcSettings.getFont(DcRepository.settings.stSystemFontBold));  
        for (Iterator iter = getData().iterator(); iter.hasNext(); ) {
            JPanel panel = (JPanel) iter.next();
            Component[] components = panel.getComponents();
            for (int j = 0; j < components.length; j++) {
                components[j].setFont(DcSettings.getFont(DcRepository.settings.stSystemFontBold));
            }
        }
    }     
    
    public class CustomCellRenderer implements ListCellRenderer {
        public Component getListCellRendererComponent(
                JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            
            JComponent component = (JComponent) value;
            Color selectionColor = DcSettings.getColor(DcRepository.settings.stRowSelectionColor);
            component.setBackground(isSelected ? selectionColor : Color.WHITE);
            return component;
        }
    }    
}
