/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import net.sf.dc.core.DcRepository;
import net.sf.dc.settings.DcSettings;

public class DcNumberField extends JTextField implements IComponent {

    public DcNumberField() {
        super();
    }

    protected Document createDefaultModel() {
        return new WholeNumberDocument();
    }

    public Object getValue() {
        Object o = null;
        try {
            o = new Long(getText());
        } catch(NumberFormatException ignore) {}
        return o;
    }

    public void clear() {} 
    
    public void setValue(int i) {
        setText(i + "");
    }

    /**
     * Sets the value. The value is first transformed to a String and
     * then parsed back to an int. If this is not successfull, 0 is set
     */
    public void setValue(Object o) {
        Long i = null;
        try {
            i = new Long(o.toString());
        } catch (Exception ignore) {}
        
        setText(i != null ? i.toString() : "");
    }

    /**
     * Document implementation. This document makes sure that only 
     * digits can be entered by the user. 
     */
    protected class WholeNumberDocument extends PlainDocument {

        protected WholeNumberDocument() {
        }

        public void insertString(int i, String s, AttributeSet attributeset)
        throws BadLocationException {

            char ac[] = s.toCharArray();
            char ac1[] = new char[ac.length];
            int j = 0;
            for(int k = 0; k < ac1.length; k++)
                if(Character.isDigit(ac[k]))
                    ac1[j++] = ac[k];
                else if (i == 0 && (s.equals("-") || ac[k] == '-'))
                    ac1[j++] = ac[k];

            super.insertString(i, new String(ac1, 0, j), attributeset);
        }
    } 
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getBoolean(DcRepository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        super.paintComponent( g2d );
    }    
}
