/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.windows.BrowserDialog;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.windows.messageboxes.QuestionBox;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.util.Base64;
import net.sf.dc.util.FileFilter;
import net.sf.dc.util.PictureFileFilter;
import net.sf.dc.util.Utilities;
import net.sf.ext.ij.ImageJ;
import net.sf.ext.ij.ImagePlus;
import net.sf.ext.ij.WindowManager;
import net.sf.ext.ij.gui.ImageWindow;

public class DcPictureField extends JComponent implements IComponent, ActionListener, WindowListener, MouseListener {

    byte[] bytes = null;
    private BufferedImage img = null;

    private boolean scaled = true;
    private boolean thumbnail = false;
    private Dimension size = null;
    private Insets insets = new Insets(0, 0, 0, 0);
    
    private ImageJ imageJ;
    private String name;
    
    public DcPictureField(boolean allowZoom) {
        this(true, allowZoom, false, "");
    }

    public DcPictureField(boolean scaled, boolean allowActions, boolean thumbnail, String name) {
        this.setLayout(Layout.getGBL());
        this.name = name;
        
        if (allowActions) {
            this.add(getMenu(),    Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                     GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(5, 5, 5, 5), 0, 0));
        }
        
        Panel panel = new Panel();
        
        if (allowActions)
            panel.addMouseListener(this);
        
        this.add(panel,  Layout.getGBC(0, 1, 1, 1, 80.0, 80.0,
                 GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets(5, 5, 5, 5), 0, 0));
        
        this.scaled = scaled;
        this.thumbnail = thumbnail;
    }
    
    private JMenuBar getMenu() {
        JMenuBar mb = ComponentFactory.getMenuBar();
        
        JMenu menuFile = ComponentFactory.getMenu(DcResources.getText("lblFile"));
        JMenu menuEdit = ComponentFactory.getMenu(DcResources.getText("lblEdit"));

        JMenuItem miSaveAs = ComponentFactory.getMenuItem(DcResources.getText("lblSaveAs"));
        JMenuItem miOpen = ComponentFactory.getMenuItem(DcResources.getText("lblOpen"));
        JMenuItem miEdit = ComponentFactory.getMenuItem(DcResources.getText("lblEdit"));
        JMenuItem miDelete = ComponentFactory.getMenuItem(DcResources.getText("lblDelete"));
        
        miEdit.setActionCommand("edit");
        miEdit.addActionListener(this);
        
        miOpen.setActionCommand("new");
        miOpen.addActionListener(this);

        miDelete.setActionCommand("delete");
        miDelete.addActionListener(this);
        
        miSaveAs.setActionCommand("Save as");
        miSaveAs.addActionListener(this);
        
        menuFile.add(miOpen);
        menuFile.add(miSaveAs);
        
        menuEdit.add(miEdit);
        menuEdit.add(miDelete);
        
        mb.add(menuFile);
        mb.add(menuEdit);
        
        return mb;
    }

    public void setValue(Object o) {
        clear();

        try {
            if (o == null) {
                bytes = null;
            } else {
                if (o instanceof URL) {
                    URL url = (URL) o;
                    bytes = Utilities.readFile(new File(url.getFile()));
                } else if (o instanceof String) {
                    String value = (String) o;
                    if (value.endsWith("jpg")) {
                        URL url;
                        if (!value.startsWith("file:"))
                        	url = new URL("file:" + value);
                        else
                            url = new URL(value);

                        bytes = Utilities.readFile(new File(url.getFile()));
                    } else {
                        String base64 = (String) o;
                        if (base64 != null && base64.length() > 0)
                            bytes = Base64.decode(base64.toCharArray());
                    }
                } else if (o instanceof byte[]) {
                    bytes = (byte[]) o;
                }
            }
        } catch (Exception exp) {
            DcLog.add(DcResources.getText("msgCouldNotLoadPicture"), DcLog._ERROR);
            DcLog.add(exp);
        }

        bytes = bytes == null || bytes.length == 0 ? null : bytes; 
        if (bytes != null)
            img = Utilities.toBufferedImage(new ImageIcon(bytes), true);

        revalidate();
        repaint();
    }

    public void clear() {
        bytes = null;
        size = null;
        img = null;
        size = null;
        insets = null;      
    } 
    
    public boolean isEmpty() {
    	return img == null;
    }

    public Object getValue() {
        return bytes;
    }

    private Image getImage() {
    	return img;
    }

    private boolean scalingAllowed(int width, int height) {
        return scaled && (height >= 50 && width >= 50);
    }

        
    private class Panel extends JComponent {

        public void paint(Graphics g) {
            insets = getInsets(insets);
            size = getSize(size);

      
            Image image = getImage();
            int w;
            int h;
            
            if (image == null) {
                image = IconLibrary._icoEmptyPictureLarge;
                scaled = true;
            }
        
            if (image != null && scalingAllowed(image.getWidth(null), image.getHeight(null))) {
                int imageWidth = image.getWidth(null);
                int imageHeight = image.getHeight(null);
        
                if (g instanceof Graphics2D)
                    ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
              
                w = 0;
                h = 0;
                if (thumbnail) {
                    w =  Math.min(size.width, image.getWidth(this) / 5);
                    h = Math.min(size.height, image.getHeight(this) / 5);
                } else {
                    w =  Math.min(size.width, image.getWidth(this));
                    h = Math.min(size.height, image.getHeight(this));
        
                    double scaledRatio = (double) w / (double) h;
                    double imageRatio = (double) imageWidth / (double) imageHeight;
                    if (scaledRatio < imageRatio) 
                        h = (int)(w / imageRatio);
                    else
                        w = (int)(h * imageRatio);
                }
            } else {
                w = image.getWidth(null);
                h = image.getHeight(null);
            }

            g.drawImage(image, getInset(size.width, w), getInset(size.height, h), w, h, null);
        }
    }
    
    private int getInset(int sizeComp, int sizePic) {
        int remainingSpace = sizeComp - sizePic;
        return remainingSpace >= 2 ? remainingSpace / 2 : 0;
    }

    public Dimension getMinimumSize() {
        int imgw = 32, imgh = 32;
        Image image = getImage();
        if (thumbnail && image != null) {
            imgw =  image.getWidth(this)/ 5;
            imgh = image.getHeight(this)/ 5;
        } else if (image != null) {
            imgw = image.getWidth(this);
            imgh = image.getHeight(this);
        }
        insets = getInsets(insets);
        return new Dimension(
            insets.left + Math.max(32, imgw / 10) + insets.right,
            insets.top + Math.max(32, imgh / 10) + insets.bottom);
    }

    public Dimension getPreferredSize() {
        int imgw = 32, imgh = 32;
        Image image = getImage();
        if (thumbnail && image != null) {
            imgw =  image.getWidth(this)/ 5;
            imgh = image.getHeight(this)/ 5;
        } else if (image != null) {
            imgw = image.getWidth(this);
            imgh = image.getHeight(this);
        }

        insets = getInsets(insets);
        return new Dimension(
            insets.left + imgw + insets.right,
            insets.top + imgh + insets.bottom);
    }

    public Dimension getMaximumSize() {
        return getPreferredSize();
    }

    private void openImageDialog() {
        try {
            BrowserDialog dialog = new BrowserDialog("Select a new Image", new PictureFileFilter());
            File file = dialog.showOpenFileDialog(this, null);
            if (file != null) {
                bytes = Utilities.readFile(file);
                setValue(bytes);
                
                boolean delete = DcSettings.getBoolean(DcRepository.settings.stDeleteImageFileAfterImport);
                if (delete) 
                    file.delete();
            }
        } catch (Exception exp) {
            DcLog.add(exp);
        }
    }
    
    private void saveToFile() {
        if (bytes != null) {
            BrowserDialog dlg = new BrowserDialog(DcResources.getText("lblSelectFile"), 
                                                  new FileFilter("jpg"));
            
            File file = dlg.showCreateFileDialog(this, null);

            try {
                if (file != null) {
                    String filename = file.toString();
                    filename += filename.toLowerCase().endsWith("jpg") || filename.toLowerCase().endsWith("jpeg") ?
                                "" : ".jpg";
                    
                    Utilities.writeToFile(bytes, filename);
                }
            } catch (Exception e) {
                new MessageBox(e.getMessage(), MessageBox._ERROR);
                DcLog.add(e);
            }
        }
    }
    
    private void openEditDialog() {
        imageJ = new ImageJ();
        imageJ.setVisible(true);
        ImagePlus imgPlus = new ImagePlus(name, img);
        imgPlus.setIgnoreFlush(true);
        ImageWindow imgWindow = new ImageWindow(imgPlus, false);
        WindowManager.addWindow(imgWindow);
        
        imgWindow.addWindowListener(this);
        imageJ.addWindowListener(this);
        imgWindow.setVisible(true);
    }
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        super.paintComponent(g2d);
    }

    public void actionPerformed(ActionEvent e) {
        String action = e.getActionCommand();
        if (action.equals("new")) {
            openImageDialog();
        } else if (action.equals("Save as")) {
            saveToFile();
        } else if (action.equals("edit")) {
            openEditDialog();
        } else if (action.equals("delete")) {
            setValue(null);
        }
    }
    
    public void windowClosing(WindowEvent e) {
        ImagePlus imgPlus = WindowManager.getImage(name);
        WindowManager.closeAllWindows();
        
        if (imgPlus == null)
            return;
        
        try {
            Image image = imgPlus.getImage();
            if (image != null && imgPlus.changes) {
                QuestionBox qb = new QuestionBox(DcResources.getText("msgKeepChanges"));
                if (qb.isAffirmative()) {
                    img = Utilities.toBufferedImage(new ImageIcon(image), true);
                    bytes = Utilities.getBytes(img);
                    revalidate();
                    repaint();
                }
            }
            
            imgPlus.setIgnoreFlush(false);
            imgPlus.flush();
        } catch (Exception exp) {
            DcLog.add(exp);
        }
        
        if (imageJ != null) {
            imageJ.dispose();
            imageJ = null;
        }
    }

    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}

    public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
            if (img != null)
                openEditDialog();
            else 
                openImageDialog();
        }
    }

    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}     
}