/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.windows.DcReferencesDialog;
import net.sf.dc.console.windows.itemforms.DcMinimalisticItemForm;
import net.sf.dc.console.windows.itemforms.ItemForm;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.MappingModule;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcProperty;
import net.sf.dc.settings.DcSettings;

public class DcReferencesField extends JComponent implements IComponent {

    private DcLongTextField fld = ComponentFactory.getLongTextField(4000);

    private Collection dcos = new ArrayList();
    
    private final int mappingModIdx;
    
    public DcReferencesField(int mappingModIdx) {
        super();
        
        this.mappingModIdx = mappingModIdx;
        
        setBounds(0,0,0,0);
        setLayout(Layout.getGBL());
        
        JButton btn = ComponentFactory.getIconButton(IconLibrary._icoOpenNew);
        JButton btnOpen = ComponentFactory.getIconButton(IconLibrary._icoOpen);

        fld.setEditable(false);
        fld.setBackground(getBackground());
        JScrollPane scrollIn = new JScrollPane(fld);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollIn.setPreferredSize(new Dimension(350,50));
        
        add(scrollIn, Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));        
        add(btnOpen, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));        
        add(btn,     Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));        
        
        btn.addActionListener(new OpenDialogAction());
        btnOpen.addActionListener(new OpenItemsAction());
    }

    public Object getValue() {
        return dcos;
    }
    
    public void setValue(Collection c) {
        dcos = new ArrayList();
        
        if (c != null)
            dcos.addAll(c);
        
        setDescription();
    }

    public void setValue(Object o) {
        setValue((Collection) o);
    }
    
    public void clear() {
        fld = null;
        dcos.clear();
        dcos = null;
        removeAll();
    }
    
    private void setDescription() {
        fld.setText("");
        if (dcos == null)
            return;

        String desc = "";
        boolean first = true;
        for (Iterator iter = dcos.iterator(); iter.hasNext(); ) {
            DcMapping dco = (DcMapping) iter.next();
            if (!first) desc += ", ";
            first = false;
            desc += dco;
        }
        fld.setText(desc);
    }
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getBoolean(DcRepository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        super.paintComponent(g2d);
    }  
    
    private class OpenItemsAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            for (Iterator iter = dcos.iterator(); iter.hasNext(); ) {
                DcMapping mapping = (DcMapping) iter.next();
                DcObject reference = mapping.getReferencedObject();
                reference.markAsUnchanged();
                
                ItemForm itemForm;
                if (reference instanceof DcProperty)
                    itemForm = new DcMinimalisticItemForm(true, reference, null);
                else 
                    itemForm = new ItemForm(null, false, true, reference, false);
                    
                itemForm.setVisible(true);
            }
        }
    }
    
    private class OpenDialogAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            MappingModule mappingModule = (MappingModule) DcModules.get(mappingModIdx);
            DcReferencesDialog dlg = new DcReferencesDialog(dcos, mappingModule);
            
            dlg.setVisible(true);
            
            if (dlg.isSaved()) {
                dcos.clear();
                dcos = new ArrayList();
                dcos.addAll(dlg.getDcObjects());
                setDescription();
            }
            dlg.dispose();
        }
    }
}