/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Toolkit;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

import net.sf.dc.core.DcRepository;
import net.sf.dc.settings.DcSettings;

public class DcShortTextField extends JTextField implements IComponent {

    private int MAX_TEXT_LENGTH = 0;
    
    private static final Toolkit toolkit = Toolkit.getDefaultToolkit();
    private DcUndoListenerer undoListener;

    public DcShortTextField(int maxTextLength) {
        super();
        MAX_TEXT_LENGTH = maxTextLength;
        undoListener = new DcUndoListenerer(this);
    }    
    
    public DcUndoListenerer getUndoListener() {
        return undoListener;
    }
    
    protected class ShortStringDocument extends PlainDocument {
        public void insertString(int i, String s, AttributeSet attributeset)
            throws BadLocationException {

            if (i + 1 > MAX_TEXT_LENGTH) {
                toolkit.beep();
            } else {
                super.insertString(i, s, attributeset);
            }
        }
    }
    
    public void clear() {
        undoListener = null;
    }
    
    protected Document createDefaultModel() {
        return new ShortStringDocument();
    }

    public Object getValue() {
        return getText();
    }

    public void setValue(Object o) {
        setText(o.toString());
    }
    
    protected void paintComponent ( Graphics g ) {
        Graphics2D g2d = (Graphics2D)g;
        if (DcSettings.getBoolean(DcRepository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        super.paintComponent( g2d );
    } 
}
