/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JTabbedPane;
import javax.swing.UIDefaults;
import javax.swing.UIManager;

import net.sf.dc.core.DcRepository;
import net.sf.dc.settings.DcSettings;

public class DcTabbedPane extends JTabbedPane {
    
    private static final String  uiClassID = "JTabbedPaneUI";
    
    static {
        UIDefaults ui = UIManager.getDefaults();
        if(!ui.contains(uiClassID))
            ui.put(uiClassID, "net.sf.dc.console.components.DcTabbedPaneUI");
    }    

    public String getUIClassID() {
        return uiClassID;
    }    
    
    public DcTabbedPane() {
        super();
    }

    public String getTitleAt(int index) {
        try {
            return super.getTitleAt(index);
        } catch (Exception ne) {
            return "";
        }
    }    
    
    public boolean isEnabledAt(int index) {
        return true;
    }
    
    public Color getBackgroundAt(int index) {
        try {
            return super.getBackgroundAt(index);
        } catch (Exception ne) {
            return null;
        }
    }
    
    protected void paintComponent(Graphics g) {
        try {
            Graphics2D g2d = (Graphics2D) g;
            
            if (DcSettings.getBoolean(DcRepository.settings.stFontAntiAliasing)) {
               g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
               g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
               g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
            }
    
            super.paintComponent( g2d );
        } catch (Exception e) {}
    }      
}
