/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JList;

import net.sf.dc.console.components.lists.elements.DcFieldListElement;
import net.sf.dc.console.components.renderers.DcListRenderer;
import net.sf.dc.core.objects.DcField;

public class DcFieldList extends DcList {
    
    public DcFieldList() {
        super(new DcListModel());
        
        setCellRenderer(new DcListRenderer(true));
        setLayoutOrientation(JList.VERTICAL_WRAP);
    }    

    public Collection getFields() {
        Collection fields = new ArrayList();
        for (Iterator iter = getElements().iterator(); iter.hasNext(); ) {
            DcFieldListElement element = (DcFieldListElement) iter.next();
            fields.add(element.getField());
        }
        return fields;
    }
    
    public DcField getSelected() {
        DcFieldListElement element = (DcFieldListElement) getSelectedValue();
        return element.getField();
    }
    
    public void add(DcField field) {
        getDcModel().addElement(new DcFieldListElement(field));
        try {
            ensureIndexIsVisible(getModel().getSize());
        } catch (Exception exp) {}
    }
    
    public void remove(DcField field) {
        for (Iterator iter = getElements().iterator(); iter.hasNext(); ) {
            DcFieldListElement element = (DcFieldListElement) iter.next();
            if (element.getField().equals(field))
                getDcModel().removeElement(element);                
        }
    }    
}
