/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JToolTip;
import javax.swing.ListCellRenderer;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcList;
import net.sf.dc.console.components.DcMultiLineToolTip;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcPropertyModule;
import net.sf.dc.core.modules.MappingModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.settings.DcSettings;

public class DcModuleList extends DcList {

    private int currentIndex = -1;
    private final JPanel panelHeader = new JPanel(Layout.getGBL());
    
    private boolean listenForChanges = true;
    private Map elements = new HashMap();
    
	public DcModuleList() {
        super();
        addModules();
        addListSelectionListener(new moduleSelectAction());
        setCellRenderer(new ModuleCellRenderer());
	}

    public void setBackground(Color color) {
        super.setBackground(Color.WHITE);
    }
    
    public void setFont(Font font) {
        Collection elements = getData();
        for (Iterator iter = elements.iterator(); iter.hasNext(); ) {
            Object value = iter.next();
            if (value instanceof JPanel)
                ((JPanel) value).setFont(font);
        }
    }    
    
	private void addModules() {
        for (Iterator iter =  DcModules.getAllModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            if (     module.isTopModule() && module.isEnabled() && 
                    !module.hasDependingModules() ||
                     module.getIndex() == DcModules._CONTACTPERSON) {
                
                Collection c = new ArrayList();
                c.add(new ModulePanel(module, ModulePanel._ICON32));
                for (Iterator iterFields = module.getFields().iterator(); iterFields.hasNext(); ) {
                    DcField field = (DcField) iterFields.next();
                    
                    DcModule referencedMod = DcModules.getReferencedModule(field);
                    
                    if (    referencedMod.getIndex() != module.getIndex() && 
                          !(referencedMod instanceof DcPropertyModule) &&
                            referencedMod.getIndex() != DcModules._CONTACTPERSON) {
                        
                        c.add(new ModulePanel(referencedMod, ModulePanel._ICON16));
                    }
                }
                elements.put(new Integer(module.getIndex()), c);
            }
        }
        
        setModules(DcSettings.getInt(DcRepository.settings.stModule));
	}
    
    private void setModules(int current) {
        listenForChanges = false;
        
        int module = current; 
        if (!elements.containsKey(new Integer(current))) {
            for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
                DcModule m = (DcModule) iter.next();
                if (!(m instanceof MappingModule) && m.hasReferenceTo(module) && m.isTopModule())
                    module = m.getIndex();
            }
        }
        
        Vector v = new Vector();
        for (Iterator iter = elements.values().iterator(); iter.hasNext(); ) {
            ArrayList e = (ArrayList) iter.next();
            ModulePanel panel = (ModulePanel) e.get(0);
            if (panel.getModule() == module)
                v.addAll(e);
            else
                v.add(panel);
        }
        setListData(v);
        
        for (Iterator iter = v.iterator(); iter.hasNext(); ) {
            ModulePanel panel = (ModulePanel) iter.next();
            if (panel.getModule() == current)
                setSelectedValue(panel, true);                
        }
        
        listenForChanges = true;
    }
    
    public void setSelectedModule(int module) {
        if ((DcModules.get(module).isTopModule() && !DcModules.get(module).hasDependingModules())) {
            add(panelHeader);
            setModules(module);
        }
    }
    
    private void setActiveModule() {
        listenForChanges = false;
        Object item = getSelectedValue();
        if (item instanceof ModulePanel) {
            int module = ((ModulePanel) item).getModule();
            if (module != currentIndex) {
                currentIndex = module;
                setSelectedModule(currentIndex);
                if (DataCrow.mainFrame != null) 
                    DataCrow.mainFrame.changeModule(module, true);
            }
        }
        listenForChanges = true;
    }
    
    public JToolTip createToolTip() {
        return new DcMultiLineToolTip();
    }    
    
    private class ModulePanel extends JPanel {
        private final int module;
        
        public static final int _ICON16 = 0;
        public static final int _ICON32 = 1;
        
        public ModulePanel(DcModule module, int icon) {
            super(new FlowLayout(FlowLayout.LEFT));
            this.module = module.getIndex();

            if (icon == _ICON16) {
                JLabel label = ComponentFactory.getLabel("");
                label.setPreferredSize(new Dimension(12, 12));
                label.setMinimumSize(new Dimension(12, 12));
                label.setMaximumSize(new Dimension(12, 12));
            
                add(label);
                add(ComponentFactory.getLabel(module.getIcon16()));
            } else {
                add(ComponentFactory.getLabel(module.getIcon32()));
            }

            add(ComponentFactory.getLabel(module.getLabel()));
        }
        
        public int getModule() {
            return module;
        }
        
        public JToolTip createToolTip() {
            return new DcMultiLineToolTip();
        }        
        
        public String getToolTipText() {
            return DcModules.get(module).getDescription();
        }
        
        public void setFont(Font font) {
            Component[] components = getComponents();
            for (int i = 0; i < components.length; i++) {
                components[i].setFont(font);
                components[i].setForeground(ComponentFactory.getCurrentForegroundColor());
            }
        }          
    }
    
    private class moduleSelectAction implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent arg0) {
            if (listenForChanges)
                setActiveModule();
        }
    }
    
    private class ModuleCellRenderer implements ListCellRenderer {
        public Component getListCellRendererComponent(
                JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            
            JComponent component = (JComponent) value;
            
            if (value instanceof ModulePanel) {
                int module = ((ModulePanel) value).getModule();
                component.setToolTipText(DcModules.get(module).getDescription());
            }
            
            Color selectionColor = DcSettings.getColor(DcRepository.settings.stRowSelectionColor);
            component.setBackground(isSelected ? selectionColor : Color.WHITE);
            return component;
        }
    }    
    
}