/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JList;

import net.sf.dc.console.components.lists.elements.DcAudioCDListHwElement;
import net.sf.dc.console.components.lists.elements.DcAudioTrackListElement;
import net.sf.dc.console.components.lists.elements.DcBookListHwElement;
import net.sf.dc.console.components.lists.elements.DcCardObjectListElement;
import net.sf.dc.console.components.lists.elements.DcMovieListHwElement;
import net.sf.dc.console.components.lists.elements.DcMusicAlbumListHwElement;
import net.sf.dc.console.components.lists.elements.DcMusicTrackListElement;
import net.sf.dc.console.components.lists.elements.DcObjectListElement;
import net.sf.dc.console.components.lists.elements.DcPropertyListElement;
import net.sf.dc.console.components.lists.elements.DcShortObjectListElement;
import net.sf.dc.console.components.lists.elements.DcSoftwareListHwElement;
import net.sf.dc.console.components.lists.elements.DcTemplateListElement;
import net.sf.dc.console.components.renderers.DcListRenderer;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.IChildModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcProperty;
import net.sf.dc.core.objects.DcTemplate;
import net.sf.dc.core.objects.helpers.AudioTrack;
import net.sf.dc.core.objects.helpers.MusicTrack;
import net.sf.dc.core.resources.DcResources;

public class DcObjectList extends DcList {

    private int style = _CARDS;
    
    public static final int _ELABORATE = 0;
    public static final int _CARDS = 1;
    public static final int _LISTING = 2;
    
    private DcListRenderer renderer = new DcListRenderer();
    
    public DcObjectList(int style, boolean wrap, boolean evenOddColors) {
        super(new DcListModel());

        this.style = style;
        setCellRenderer(renderer);
        renderer.setEventOddColors(evenOddColors);
        
        if (wrap)
            setLayoutOrientation(JList.HORIZONTAL_WRAP);
        else 
            setLayoutOrientation(JList.VERTICAL_WRAP);
    }    
    
    public void setSelected(DcObject dco, int field) {
        for (int row = 0 ; row < getDcModel().getSize(); row++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(row);
            if (element.getDcObject() != null) {
                if (dco.getDisplayString(field).equals(element.getDcObject().getDisplayString(field))) {
                    setSelectedIndex(row);
                    break;
                }
            }
        }
    }
    
    public void setSelected(Collection dcos) {
        int[] indices = new int[dcos.size()];
        
        int counter = 0;
        for (Iterator iter = dcos.iterator(); iter.hasNext(); counter++) {
            Object o = iter.next();
            for (int row = 0 ; row < getDcModel().getSize(); row++) {
                DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(row);
                if (element.getDcObject() != null) {
                    if (element.getDcObject().equals(o)) {
                        indices[counter] = row;
                        break;
                    }
                }
            }
        }
        setSelectedIndices(indices);
    }
    
    public int[] getChangedIndices() {
        Collection rows = new ArrayList();
        for (int i = 0 ; i < getDcModel().getSize(); i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            if (element.getDcObject() != null && element.getDcObject().isChanged()) 
                rows.add(new Integer(i));
        }
        
        int[] indices = new int[rows.size()];
        int counter = 0;
        for (Iterator iter = rows.iterator(); iter.hasNext(); counter++) {
            indices[counter] = ((Integer) iter.next()).intValue();
        }
        return indices;        
    }
    
    public void markAsUnchanged() {
        int[] rows = getChangedIndices();
        for (int i = 0; i < rows.length; i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            element.getDcObject().markAsUnchanged(); 
        }
    }    
    
    public DcObject getObjectForID(String ID) {
        DcObjectListElement element = getElementForID(ID);
        return element != null ? element.getDcObject() : null;
    }    

    public DcObjectListElement getElementForID(String ID) {
        for (int i = 0 ; i < getDcModel().getSize(); i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            if (element.getDcObject() != null && element.getDcObject().getID().equals(ID)) 
                return element;
        }
        return null;
    }      
    
    public Collection getObjects() {
        Collection objects = new ArrayList();
        for (int i = 0 ; i < getDcModel().getSize(); i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            if (element.getDcObject() != null) 
                objects.add(element.getDcObject());
        }
        return objects;
    }
    
    public Collection getSelectedObjects() {
        int[] indices = getSelectedIndices();
        Collection objects = new ArrayList();

        if (indices == null || indices.length == 0) {
            new MessageBox(DcResources.getText("msgNoRowsSelected"), MessageBox._WARNING);
        } else {
            for (int i = 0; i < indices.length; i++) {
                DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(indices[i]);
                DcObject dco = element.getDcObject();
                dco.setRow(indices[i]);
                objects.add(dco);
            }
        }
        return objects;
    }
    
    public void remove(String[] ids) {
        for (int i = 0; i < ids.length; i++)
            getDcModel().removeElement(getElementForObjectID(ids[i]));
    }

    public void updateElement(int index, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(index);
        element.update(dco, overwrite, allowDeletes, mark);
        
        try {
            setSelectedValue(getDcModel().getElementAt(index), true);
        } catch (Exception ignore) {}
    }
    
    public void updateElement(String objectID, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        DcObjectListElement element = getElementForObjectID(objectID);
        if (element != null)
            element.update(dco, overwrite, allowDeletes, mark);
        
        try {
            setSelectedValue(getElementForObjectID(objectID), true);
        } catch (Exception ignore) {}
    }
    
    private DcObjectListElement getElementForObjectID(String objectID) {
        for (int i = 0 ; i < getDcModel().getSize(); i++) {
            DcObjectListElement element = (DcObjectListElement) getDcModel().getElementAt(i);
            if (objectID.equals(element.getDcObject().getID()))
                return element;
        }
        return null;
    }    
    
    public DcObject getSelectedObject() {
        DcObjectListElement element = (DcObjectListElement) getSelectedValue();
        
        DcObject dco = null;
        if (element != null) 
            dco = element.getDcObject(); 
        
        return dco;
    }
    
    public void add(DcObject dco) {
        if (dco.getID() == null || dco.getID().equals("")) 
            dco.setIDs();        
        
        dco.markAsUnchanged();
        getDcModel().addElement(getDisplayElement(dco));
        
        try {
            ensureIndexIsVisible(getModel().getSize());
        } catch (Exception exp) {}
    }
    
    public void add(DcObject[] objects) {
        listenersEnabled(false);
        
        for (int i = 0; i < objects.length; i++) {
            DcObject dco = objects[i];
            if (dco.getID() == null || dco.getID().equals("")) 
                dco.setIDs(); 
            
            add(dco);
        }
        
        listenersEnabled(true);
        
        getDcModel().fireIntervalAdded(getDcModel(), 0, objects.length - 1);
    }    
    
    public void add(Collection objects) {
        listenersEnabled(false);
        
        for (Iterator iter = objects.iterator(); iter.hasNext(); ) {
            DcObject dco = (DcObject) iter.next();
            
            if (dco.getID() == null || dco.getID().equals("")) 
                dco.setIDs(); 
            
            add(dco);
        }
        
        listenersEnabled(true);
        
        getDcModel().fireIntervalAdded(getDcModel(), 0, objects.size() - 1);
    }
    
    public DcObjectListElement getDisplayElement(DcObject dco) {
        DcObjectListElement element = null;
        DcModule module = dco.getModule();
        
        if (style == _ELABORATE) {
            if (module.getIndex() == DcModules._AUDIOCD) 
                element = new DcAudioCDListHwElement(dco);
            else if (module.getIndex() == DcModules._MUSICALBUM) 
                element = new DcMusicAlbumListHwElement(dco);
            else if (module.getIndex() == DcModules._SOFTWARE)
                element = new DcSoftwareListHwElement(dco);
            else if (module.getIndex() == DcModules._MOVIE)
                element = new DcMovieListHwElement(dco);
            else if (module.getIndex() == DcModules._BOOK)
                element = new DcBookListHwElement(dco);
            else 
                element = new DcCardObjectListElement(dco);
        } else if (style == _CARDS) {
            if (dco instanceof AudioTrack)
                element = new DcAudioTrackListElement(dco);
            else if (dco instanceof DcTemplate)
                element = new DcTemplateListElement(dco);
            else if (dco instanceof MusicTrack)
                element = new DcMusicTrackListElement(dco);
            else if (dco instanceof DcProperty)
                element = new DcPropertyListElement(dco);
            else if (module instanceof IChildModule)
                element = new DcShortObjectListElement(dco);
            else 
                element = new DcCardObjectListElement(dco);       
        } else if (style == _LISTING) {
            if (dco instanceof DcProperty)
                element = new DcPropertyListElement(dco);
            else if (dco instanceof MusicTrack)
                element = new DcMusicTrackListElement(dco);
            else if (dco instanceof AudioTrack)
                element = new DcAudioTrackListElement(dco);
            else
                element = new DcShortObjectListElement(dco);
        }

        return element;
    }   
}
