/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists.elements;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JLabel;
import javax.swing.JTextArea;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.MusicAlbum;
import net.sf.dc.core.objects.helpers.MusicTrack;

public class DcMusicAlbumListHwElement extends DcObjectListHwElement {

    private JTextArea tracksField;
    
    public DcMusicAlbumListHwElement(DcObject dco) {
        super(dco);
    }
    
    public Collection getPictures() {
        Collection pictures = new ArrayList();
        pictures.add(dco.getValue(MusicAlbum._J_PICTUREFRONT));
        pictures.add(dco.getValue(MusicAlbum._K_PICTUREBACK));
        return pictures;
    }

    public void setBackground(Color color) {
        super.setBackground(color);
        if (tracksField != null)
            tracksField.setBackground(new Color(color.getRed() - 20, color.getGreen() - 20, color.getBlue() - 20));
    }    
    
    public void build() {
        setLayout(Layout.getGBL());
        
        tracksField = getTracksField();
        JLabel titleLabel = getLabel(MusicAlbum._A_TITLE, true, label1Length);
        if (DcModules.getCurrent().getIndex() == DcModules._MEDIA)
            titleLabel.setIcon(dco.getModule().getIcon16());

        addComponent(titleLabel, 0, 0);
        addComponent(getLabel(MusicAlbum._A_TITLE, false, field1Length), 1, 0);
        addComponent(getLabel(MusicAlbum._G_GENRES, true, label2Length), 2, 0);
        addComponent(getLabel(MusicAlbum._G_GENRES, false, field2Length), 3, 0);
        
        addComponent(getLabel(MusicAlbum._F_ARTISTS, true, label1Length), 0, 1);
        addComponent(getLabel(MusicAlbum._F_ARTISTS, false, field1Length), 1, 1);
        addComponent(getLabel(DcMediaObject._E_RATING, true, label2Length), 2, 1);
        addComponent(getRatingValueLabel(), 3, 1);        
        
        add(tracksField, Layout.getGBC( 0, 2, 4, 1, 10.0, 10.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(0, 5, 5, 0), 0, 0));
        add(getPicturePanel(getPictures()), Layout.getGBC( 4, 0, 1, 3, 10.0, 10.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(1, 1, 1, 1), 0, 0));        
    }   
    
    protected JTextArea getTracksField() {
        MusicTrack track = new MusicTrack();
        String description = ""; 
        int counter = 0;
        for (Iterator iter = dco.getChildren().iterator(); iter.hasNext(); ) {
            track = (MusicTrack) iter.next();
            
            if (counter > 0) description += " / ";
            
            String title = (String) track.getValue(MusicTrack._A_TITLE);
            title = title == null ? "" : title; 
            
            Long i = (Long) track.getValue(MusicTrack._J_PLAYLENGTH);
            String length = i == null || i.equals(new Long(0)) ? "" : " " + track.getDisplayString(MusicTrack._J_PLAYLENGTH);
            
            if (track.getValue(MusicTrack._F_TRACKNUMBER) != null)
                description += track.getValue(MusicTrack._F_TRACKNUMBER) + " - ";
            
            description += title;
            description += length;
            counter++;
        }

        JTextArea textField = ComponentFactory.getTextArea();
        textField.setPreferredSize(new Dimension(600, 120));
        textField.setText(description);
        return textField;        
    }
}
