/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists.elements;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Image;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcImageLabel;
import net.sf.dc.console.components.DcLabel;
import net.sf.dc.console.components.DcTextPane;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.util.Utilities;

public abstract class DcObjectListElement extends DcListElement {
    
    private Color bg = new Color(255, 255, 230);
    private Dimension dimTxt = new Dimension(145, 45);
    private Dimension dimPicLbl = new Dimension(146, 140);
    
    protected int fieldHeight = 21;
    protected DcObject dco;
    
    protected DcObjectListElement() {}
    
    public DcObjectListElement(DcObject dco) {
        this.dco = dco;
        dco.markAsUnchanged();
        setBackground(Color.WHITE);
        build();
    }

    public DcObject getDcObject() {
        return dco;
    }
    
    public abstract Collection getPictures();
    
    public void update(DcObject o, boolean overwrite, boolean allowDeletes, boolean mark) {
        int[] indices = dco.getFieldIndices();
        
        for (int i = 0; i < indices.length; i++) {
            try {
                if (o.isChanged(indices[i])) {
                    Object newValue = o.getValue(indices[i]);
                    Object existingValue = dco.getValue(indices[i]);

                    if (newValue instanceof Picture)
                        newValue = ((Picture) newValue).isDeleted() ? null : newValue;
                    if (overwrite && ((newValue != null && !newValue.equals("")) || allowDeletes)) {
                        dco.setValue(indices[i], newValue);
                    } else if (existingValue == null || existingValue.equals("")) {
                        dco.setValue(indices[i], newValue);
                    }
                }
            } catch (Exception ignore) {}
        }        

        dco.removeChildren();
        dco.loadChildren();
        
        if (!mark)
            dco.markAsUnchanged();
        
        update();
        revalidate();
    }    

    protected void build() {
        addPicture(getPictures());
        
        DcTextPane title = ComponentFactory.getTextPane();
        JScrollPane scroller = new JScrollPane(title);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        title.setText(dco.getName());
        title.setBackground(bg);
        
        title.setPreferredSize(dimTxt);
        title.setMinimumSize(dimTxt);
        title.setMaximumSize(dimTxt);
        add(title);
    }
    
    private void addPicture(Collection pictures) {
        DcImageLabel lbl = null;
        for (Iterator iter = pictures.iterator(); iter.hasNext(); ) {
            Picture p = (Picture) iter.next();
            if (p != null) {
                p.loadImage();
                ImageIcon scaledImage = p.getScaledPicture();
                byte[] bytes = (byte[]) p.getValue(Picture._D_IMAGE);
                DcImageLabel label = new DcImageLabel();
                if (scaledImage != null) { 
                    label = ComponentFactory.getImageLabel(scaledImage);
                } else if (bytes != null && bytes.length > 10) {
                    Image img = Utilities.getScaledImage(bytes);
                    label = ComponentFactory.getImageLabel(new ImageIcon(img));
                }                
                
                lbl = label;
                break;
            }
        }
        
        if (lbl == null) lbl = ComponentFactory.getImageLabel(new ImageIcon(IconLibrary._icoEmptyPictureSmall));
        
        lbl.setPreferredSize(dimPicLbl);
        lbl.setMinimumSize(dimPicLbl);
        lbl.setMaximumSize(dimPicLbl);
        add(lbl);
    }
    
    public JPanel getPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
        return panel;
    }
    
    protected DcLabel getLabel(int field, boolean label, int width) {
        DcLabel lbl = new DcLabel();
        if (label) {
            Font font = DcSettings.getFont(DcRepository.settings.stSystemFontBold);
            lbl.setFont(new Font(font.getName(), Font.BOLD, font.getSize()));
            lbl.setText(dco.getLabel(field));
        } else {
            lbl.setFont(DcSettings.getFont(DcRepository.settings.stSystemFontNormal));
            lbl.setText(dco.getDisplayString(field));
        }
        
        lbl.setPreferredSize(new Dimension(width, fieldHeight));
        
        return lbl;
    }       
    
    public void setBackground(Color color) {
        super.setBackground(color);
        for (int i = 0; i < getComponents().length; i++) {
            getComponents()[i].setBackground(color);
        }
    }
    
    public void clear() {
        super.clear();
        dco = null;
        bg = null;
        dimTxt = null;
        dimPicLbl = null;
    }
}
