/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.lists.elements;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcImageLabel;
import net.sf.dc.console.components.DcLabel;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.util.Utilities;

public abstract class DcObjectListHwElement extends DcObjectListElement {
    
    protected int label1Length = 80;
    protected int label2Length = 100;
    protected int field1Length = 300;
    protected int field2Length = 120;
    
    public DcObjectListHwElement(DcObject dco) {
        super();
        
        this.dco = dco;
        setLayout(new FlowLayout(FlowLayout.LEFT));
        setBorder(ComponentFactory.getBorder());
        build();
    }

    protected Color lighter(Color color) {
        int red = color.getRed() > 10 ? color.getRed() - 10 : color.getRed();
        int green = color.getGreen() > 10 ? color.getGreen() - 10 : color.getGreen();
        int blue = color.getBlue() > 10 ? color.getBlue() - 10 : color.getBlue();
        return new Color(red, green, blue);
    }
    
    public JPanel getPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
        return panel;
    }
    
    public abstract void build();
    
    protected void addComponent(JComponent component, int x, int y) {
        Insets insets = y == 0 && x == 0 ? 
                            new Insets(5, 5, 0, 0) : y == 0 ?  
                                    new Insets(5, 0, 0, 0) : x == 0 ? 
                                            new Insets(0, 5, 0, 0) :
                                                new Insets(0, 0, 0, 0);
        add(component, Layout.getGBC( x, y, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, insets, 0, 0));          
    }
    
    protected DcLabel getRatingValueLabel() {
        DcLabel labelRating = ComponentFactory.getLabel(Utilities.getRatingIcon(dco.getValue(DcMediaObject._E_RATING)));
        labelRating.setPreferredSize(new Dimension(55, fieldHeight));
        return labelRating;
    }
    
    protected DcLabel getLabel(String text, boolean label, int width) {
        DcLabel lbl = new DcLabel();
        
        if (label) {
            Font font = DcSettings.getFont(DcRepository.settings.stSystemFontBold);
            lbl.setFont(new Font(font.getName(), Font.BOLD, font.getSize()));
        } else {
            lbl.setFont(DcSettings.getFont(DcRepository.settings.stSystemFontNormal));
        }
        
        lbl.setText(text);
        lbl.setPreferredSize(new Dimension(width, fieldHeight));
        return lbl;
    }
    
    protected DcLabel getLabel(int field, boolean label, int width) {
        DcLabel lbl = new DcLabel();
        if (label) {
            Font font = DcSettings.getFont(DcRepository.settings.stSystemFontBold);
            lbl.setFont(new Font(font.getName(), Font.BOLD, font.getSize()));
            lbl.setText(dco.getLabel(field));
        } else {
            lbl.setFont(DcSettings.getFont(DcRepository.settings.stSystemFontNormal));
            lbl.setText(dco.getDisplayString(field));
        }
        
        lbl.setPreferredSize(new Dimension(width, fieldHeight));
        
        return lbl;
    }    
    
    protected JPanel getPicturePanel(Collection pictures) {
        JPanel panel = new JPanel();
        panel.setPreferredSize(new Dimension(200,150));
        panel.setLayout(Layout.getGBL());
        
        for (Iterator iter = pictures.iterator(); iter.hasNext(); ) {
            Picture picture = (Picture) iter.next();
            
            if (picture != null && !picture.isDeleted()) {
                ImageIcon scaledImage = picture.getScaledPicture();
                byte[] bytes = (byte[]) picture.getValue(Picture._D_IMAGE);
                
                DcImageLabel label = new DcImageLabel();
                if (bytes != null && bytes.length > 10) {
                    Image img = Utilities.getScaledImage(bytes);
                    label = ComponentFactory.getImageLabel(new ImageIcon(img));                    
                } else if (scaledImage != null) { 
                    label = ComponentFactory.getImageLabel(scaledImage);
                } else {
                    // nothing available, try to load the image instead
                    picture.loadImage();
                    bytes = (byte[]) picture.getValue(Picture._D_IMAGE);
                    if (bytes != null && bytes.length > 10) {
                        Image img = Utilities.getScaledImage(bytes);
                        label = ComponentFactory.getImageLabel(new ImageIcon(img));
                    }
                }
                
                panel.add(label,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                          new Insets(0, 0, 0, 0), 0, 0));
                break;
            }
        }
        
        return panel;
    }

    protected String getShortDescription(String description) {
        int max = 500;
        String s = description;
        
        if (description == null) {
            s = "";
        } else if (description.length() > max) {
            s = s.substring(0, max);
            int index = description.indexOf(" ", max);
            if (index > -1) {
                String remainder = description.substring(max, index);
                s += remainder + ".....";
            }
        }
        
        s = s.replaceAll("\r", "");
        s = s.replaceAll("\n", "");
        
        return s;
    }
    
    public void setBackground(Color color) {
        super.setBackground(color);
        for (int i = 0; i < getComponents().length; i++) {
            getComponents()[i].setBackground(color);
        }
    }    

    public void setFont(Font font) {
        Font fontNormal = DcSettings.getFont(DcRepository.settings.stSystemFontNormal);
        Font fontSystem = DcSettings.getFont(DcRepository.settings.stSystemFontNormal);        
        
        super.setFont(fontNormal);
        for (int i = 0; i < getComponents().length; i++) {
            Component component = getComponents()[i];
            if (component instanceof JPanel) {
                
                for (int j = 0; j < ((JPanel) component).getComponents().length; j++) {
                    Component subComponent = ((JPanel) component).getComponents()[j];
                    if (subComponent instanceof JLabel) 
                        subComponent.setFont(fontSystem);
                    else
                        subComponent.setFont(fontNormal);
                }
            }
        }
    }     
}
