/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComponent;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.views.table.TableInsertView;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;

public class ChildInsertTablePanel extends TableInsertView {

    private ArrayList children = new ArrayList();

    public void add(Collection objects) {
        children.addAll(objects);
    }
    
    public ChildInsertTablePanel(int module) {
        super(DcModules.get(module),
              DcResources.getText("lblNewItem", DcModules.get(module).getObjectName()),
              IconLibrary._icoItemsNew);
    }

    public void setParentID(String sID) {
        cacheChildren();
        if (sID != null && !sID.equals(parentID)) {
            parentID = sID;
            loadChildren();
        }
    }

    public void clear() {
        clearTable(true);
        children.clear();
        parentID = null;
    }

    public void open() {
    }

    public void removeRows(String[] sIDs) {
        super.removeRows(sIDs);
        
        DcObject dco = null;
        for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            DcObject o = (DcObject) iter.next();
            for (int i = 0; i < sIDs.length; i++) {
                if (o.getID().equals(sIDs[i])) {
                    dco = o;
                }
            }
        }

        children.remove(dco);
    }

    public void clearTable(boolean cache) {
        table.clearTable();
        if (cache) {
            children.clear();
        }
    }

    public void cacheChildren() {
        for (int i = 0; i < table.getRowCount(); i++) {
            DcObject dco = getObjectForSaving(i);
            
            if (dco != null) { 
                dco.setValue(dco.getParentReferenceFieldIndex(), getParentID());
    
                boolean exists = false;
                DcObject[] c = new DcObject[children.size()];
                c = (DcObject[]) children.toArray(c);
                for (int j = 0; j < c.length; j++) {
                    DcObject dcoCached = c[j];
                    if (dcoCached.getID().equals(dco.getID())) {
                        dcoCached.copy(dco);
                        exists = true;
                        break;
                    }
                }
    
                if (!exists)
                    children.add(dco);
            }
        }
    }

    public void removeChildren(String parentID) {
        Collection children = getChildren(parentID);
        children.removeAll(children);
        clearTable(false);
    }

    public void loadChildren() {
        clearTable(false);
        Collection children = getChildren(parentID);
        for (Iterator iter = children.iterator(); iter.hasNext(); ) {
            DcObject child = ((DcObject) iter.next());
            child.setValue(child.getParentReferenceFieldIndex(), parentID);
        }
        
        super.add(children);
    }

    public Collection getChildren(String parentID) {
        ArrayList al = new ArrayList();
        for (int i = 0; i < children.size(); i++) {
            DcObject dco = (DcObject) children.get(i);
            if (dco.getValue(dco.getParentReferenceFieldIndex()).equals(parentID) ) {
                al.add(dco);
            }
        }
        return al;
    }

    public JComponent[] getExtraComponents() {
        JButton buttonAddChild = ComponentFactory.getButton(DcResources.getText("lblAddChild", getModule().getObjectName()));
        buttonAddChild.addActionListener(new AddChildAcionListener());
        buttonAddChild.setMnemonic('T');
        return new JComponent[] {buttonAddChild};
    }

    private class AddChildAcionListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            if (getParentID() != null && !getParentID().equals("")) {
                DcObject dco = getModule().getDcObject();
                dco.setValue(dco.getParentReferenceFieldIndex(), getParentID());
                add(dco);
                children.add(dco);
            } else {
                new MessageBox(DcResources.getText("msgAddSelectParent"), MessageBox._ERROR);
            }
        }
    }
}