/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.core.wf.requests.RefreshChildView;

public class ChildSearchTablePanel extends TableSearchView {

    private int parentModIdx;
    
    public ChildSearchTablePanel(int module, int parentModIdx) {
        super(DcModules.get(module), DcModules.get(module).getObjectName());
        
        this.parentModIdx = parentModIdx;
        
        table.getSelectionModel().removeListSelectionListener(rowSelectionListener);
        table.removeKeyListener(changeRowByKey);
        treePanel = null;
    }
    
    public JComponent[] getExtraComponents() {
        JButton buttonAddChild = ComponentFactory.getButton(DcResources.getText("lblAddChild", getModule().getObjectName()));
        buttonAddChild.addActionListener(new AddChildAcionListener());
        buttonAddChild.setMnemonic('T');
        return new JComponent[] {buttonAddChild};
    }

    /**
     * Indicates whether there are unsaved changes
     */
    public boolean changesSaved() {
        return table.noChanges();
    }    
    
    protected void addAfterDeleteRequest(DcObject dco) {
        dco.addRequest(new RefreshChildView(null, parentModIdx, null, null, getParentID(), false));
    }    
    
    /**
     * Clears the table and its cache
     */
    public void clear() {
        table.clearTable();
        table.clearCache();
    }
    
    public void loadChildValues() {}
    
    private class AddChildAcionListener implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            if (getParentID() != null && !getParentID().equals("")) {
                DcObject dco = getModule().getDcObject();
                dco.setIDs();
                dco.setValue(dco.getParentReferenceFieldIndex(), getParentID());
                open(dco, false, true);
            } else {
                new MessageBox(DcResources.getText("msgAddSelectParent"), MessageBox._ERROR);
            }
        }
    }  
}
