/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.windows.FieldSelectorDialog;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.Settings;

public class OnlineServiceSettingsPanel extends JPanel {
    
    private JButton buttonSettings = ComponentFactory.getButton(IconLibrary._icoSettings);
    private JCheckBox checkOverwrite;
    private JCheckBox checkAutoCreateSubItems;
    private JCheckBox checkAutoAdd;
    private JCheckBox checkOnlineSearchSubItems;
    
    private JFrame parent;
    
    private int module;
    private boolean updateMode = false;

    public OnlineServiceSettingsPanel(int module, boolean updateMode) {
        this(null, updateMode, module);
    }

    public OnlineServiceSettingsPanel(JFrame parent, boolean updateMode, int module) {
        this.module = module;
        this.parent = parent;
        this.updateMode = updateMode;
        
        build(updateMode);
        
        Settings settings = DcModules.get(module).getSettings();
        setOverwrite(settings.getBoolean(DcRepository.modulesettings.stOnlineSearchOverwrite));
        setAutoCreateSubItems(settings.getBoolean(DcRepository.modulesettings.stAutoCreateSubItems));
        setAutoAdd(settings.getBoolean(DcRepository.modulesettings.stAutoAddPerfectMatch));
        setOnlineSearchSubItems(settings.getBoolean(DcRepository.modulesettings.stOnlineSearchSubItems));
        
        if (updateMode)
            buttonSettings.setEnabled(isOverwriteAllowed());
    }

    public void save() {
        Settings settings = DcModules.get(module).getSettings();
        settings.set(DcRepository.modulesettings.stOnlineSearchOverwrite, new Boolean(checkOverwrite.isSelected()));
        settings.set(DcRepository.modulesettings.stAutoCreateSubItems, new Boolean(checkAutoCreateSubItems.isSelected()));
        settings.set(DcRepository.modulesettings.stAutoAddPerfectMatch, new Boolean(checkAutoAdd.isSelected()));
        settings.set(DcRepository.modulesettings.stOnlineSearchSubItems, new Boolean(checkOnlineSearchSubItems.isSelected()));
    }

    private void setAutoCreateSubItems(boolean b) {
        checkAutoCreateSubItems.setSelected(b);
    }
    
    private void setOverwrite(boolean b) {
        checkOverwrite.setSelected(b);
    }
    
    public boolean isOverwriteAllowed() {
        return checkOverwrite.isSelected();
    }
    
    public boolean isOnlineSearchSubItems() {
        return checkOnlineSearchSubItems.isSelected();
    }
    
    private void setOnlineSearchSubItems(boolean b) {
        checkOnlineSearchSubItems.setSelected(b);
    }
    
    private void setAutoAdd(boolean b) {
        checkAutoAdd.setSelected(b);
    }
    
    public boolean isAutoAddAllowed() {
        return checkAutoAdd.isSelected();
    }    
    
    public void clear() {
        buttonSettings = null;
        checkOverwrite = null;
        checkOnlineSearchSubItems = null;
        checkAutoCreateSubItems = null;
        checkAutoAdd = null;
        parent = null;
        removeAll();
    }
    
    private void build(boolean updateMode) {
        JPanel panel = new JPanel();
        panel.setLayout(new FlowLayout(FlowLayout.LEFT));

        setLayout(Layout.getGBL());

        checkAutoCreateSubItems = ComponentFactory.getCheckBox(DcResources.getText("lblAutoCreateMissingItems"));
        checkAutoCreateSubItems.setToolTipText(DcResources.getText("tpAutoCreateMissingItems"));

        checkAutoAdd = ComponentFactory.getCheckBox(DcResources.getText("lblAutoAddPerfectMatch"));
        checkAutoAdd.setToolTipText(DcResources.getText("tpAutoAddPerfectMatch"));
        
        checkOverwrite = ComponentFactory.getCheckBox(DcResources.getText("lblOverwriteExistingValues"));
        checkOverwrite.setToolTipText(DcResources.getText("tpOverwriteExistingValues"));
        
        if (updateMode)
            checkOverwrite.addActionListener(new OverwriteListener());
        
        checkOnlineSearchSubItems = ComponentFactory.getCheckBox(DcResources.getText("lblOnlineSearchSubItems"));
        checkOnlineSearchSubItems.setToolTipText(DcResources.getText("tpOnlineSearchSubItems"));
        
        buttonSettings.setMnemonic('S');
        buttonSettings.setMaximumSize(new Dimension(25, 22));
        buttonSettings.setMinimumSize(new Dimension(25, 22));
        buttonSettings.setPreferredSize(new Dimension(25, 22));
        buttonSettings.addActionListener(new OpenSettingsDialogAction());
        
        if (updateMode)
            panel.add(checkOverwrite);
        else
            panel.add(ComponentFactory.getLabel(DcResources.getText("lblRetrieveValues")));
        
        panel.add(buttonSettings);
        
        add(panel, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
        add(checkOnlineSearchSubItems, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
            new Insets(0, 5, 0, 0), 0, 0));
        add(checkAutoAdd, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
            new Insets(0, 5, 0, 0), 0, 0));            
        add(checkAutoCreateSubItems, Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
           ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
            new Insets(0, 5, 0, 0), 0, 0));

        setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblSettings")));        
    }
    
    private void openOverwriteSettingsDlg() {
        String settingsKey = updateMode ?
                             DcRepository.modulesettings.stOnlineSearchFieldOverwriteSettings :
                             DcRepository.modulesettings.stOnlineSearchRetrievedFields;
        
        FieldSelectorDialog dlg = new FieldSelectorDialog(parent, module, settingsKey);
        dlg.setVisible(true);
    }    
    
    public class OpenSettingsDialogAction implements ActionListener{
        public void actionPerformed(ActionEvent e) {
            openOverwriteSettingsDlg();
        }
    }
    
    private class OverwriteListener implements ActionListener {
        public void actionPerformed(ActionEvent ae) {
            if (updateMode)
                buttonSettings.setEnabled(checkOverwrite.isSelected());
            else
                buttonSettings.setEnabled(true);
        }
    }       
}
