/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.panels;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.lists.DcFieldList;
import net.sf.dc.console.components.lists.elements.DcFieldListElement;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcField;

public class SortingPanel extends JPanel {
    
    private Vector elements;
    private DcFieldList listRight;
    private DcFieldList listLeft;
    
    private DcModule module;
    
    public SortingPanel(DcModule module) {
        this.module = module;

        build();

        for (Iterator iter = module.getFields().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            if (!field.isUiOnly() && field.isEnabled()) 
                listLeft.add(field);
        }

        elements = new Vector();
        elements.addAll(listLeft.getElements());

        applyOrderSettings(module.getSettings().getStringArray(DcRepository.modulesettings.stSearchOrder));
    }

    public DcField[] getOrder() {
        DcField[] order = new DcField[listRight.getDcModel().size()];
        return (DcField[]) listRight.getFields().toArray(order);
    }
    
    public void save() {
        if (module.getSetting(DcRepository.modulesettings.stSearchOrder) != null) {
            
            Collection fields = listRight.getFields();
            String[] order = new String[fields.size()];
            int counter = 0;
            for (Iterator iter = fields.iterator(); iter.hasNext(); counter++) 
                order[counter] = ((DcField) iter.next()).getDatabaseFieldName();
            
            module.setSetting(DcRepository.modulesettings.stSearchOrder, order);
        }
    }    

    public void applyOrderSettings(DcField[] ordering) {
        reset();
        for (int i = 0; i < ordering.length; i++) {
            listLeft.remove(ordering[i]);
            listRight.add(ordering[i]);
        }
    }    
    
    public void applyOrderSettings(String[] ordering) {
        reset();
        for (int i = 0; i < ordering.length; i++) {
            listLeft.remove(module.getField(ordering[i]));
            listRight.add(module.getField(ordering[i]));
        }
    }
    
    private void reset() {
        //listLeft.setListData(elements);
        listRight.setListData(new Vector());
    }
    
    public void clear() {
        if (listLeft != null)
            listLeft.clear();
        
        listLeft = null;
        
        if (listRight != null)
            listRight.clear();
        
        listRight = null;
        
        if (elements != null)
            elements.clear();
        
        elements = null;
        module = null;
    }    
    
    private void build() {
        setLayout(Layout.getGBL());
        
        JTextField txtFilter = ComponentFactory.getShortTextField(255);
        txtFilter.setPreferredSize(new Dimension(50, 23));
        txtFilter.setMinimumSize(new Dimension(50, 23));        
        txtFilter.addKeyListener(new FilterInputListener());

        listLeft = new DcFieldList();
        listLeft.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        listLeft.addMouseListener(new ListMouseListener(ListMouseListener._RIGHT));

        listRight = new DcFieldList();
        listRight.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        listRight.addMouseListener(new ListMouseListener(ListMouseListener._LEFT));
        
        JScrollPane scrollerLeft = new JScrollPane(listLeft);
        scrollerLeft.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollerLeft.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        JScrollPane scrollerRight = new JScrollPane(listRight);
        scrollerRight.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollerRight.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        add(txtFilter,     Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 0), 0, 0));
        add(scrollerLeft,  Layout.getGBC( 0, 1, 1, 1, 20.0, 20.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));
        add(scrollerRight, Layout.getGBC( 1, 1, 1, 1, 20.0, 20.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));
    }
    
    private class ListMouseListener implements MouseListener {
        
        public static final int _LEFT = 0;
        public static final int _RIGHT = 1;
        
        int direction;
        
        public ListMouseListener(int direction) {
            this.direction = direction;
        }
        
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                if (direction == _LEFT) {
                    DcField field = listRight.getSelected();
                    listLeft.add(field);
                    listRight.remove();
                    listRight.clearSelection();
                } else {
                    DcField field =  listLeft.getSelected();
                    listRight.add(field);
                    listLeft.remove();
                    listLeft.clearSelection();
                }
            }  
        }
        
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    } 
    
    private class FilterInputListener implements KeyListener {

        public void keyPressed(KeyEvent e) {}
        public void keyTyped(KeyEvent e) {}

        public void keyReleased(KeyEvent e) {
            DcShortTextField txtFilter = (DcShortTextField) e.getSource();
            String filter = txtFilter.getText();
            
            if (filter.trim().length() == 0) {
                listLeft.setListData(elements);
            } else {
                Vector newElements = new Vector();
                for (Iterator iter = elements.iterator(); iter.hasNext(); ) {
                    DcFieldListElement element = (DcFieldListElement) iter.next();
                    String displayValue = element.getField().getLabel();
                    if (displayValue.toLowerCase().startsWith(filter.toLowerCase()))
                        newElements.add(element);
                }
                listLeft.setListData(newElements);
            }
        }
    }    
}
