/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.renderers;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.ToolTipManager;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableCellRenderer;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.DcRepository;
import net.sf.dc.settings.DcSettings;

public class DcTableCellRenderer extends DefaultTableCellRenderer {

    private static boolean disabled = false;
    private static final Border focusBorder = ComponentFactory.getBorder();

    protected boolean allowTooltips() {
    	return true;
    }
    
    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, 
                                                   boolean hasFocus, int row, int column) {

        JComponent c = (JComponent) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        
        if (DcSettings.getBoolean(DcRepository.settings.stShowTableTooltip) && allowTooltips()) {
        	if (disabled) {
        		ToolTipManager.sharedInstance().registerComponent(table);
                ToolTipManager.sharedInstance().registerComponent(table.getTableHeader());
                disabled = false;
        	} 
            
            ToolTipManager.sharedInstance().setDismissDelay(2000);
            setToolTipText(getTipText());

        } else {
            if (!disabled) { 
                disabled = true;
            	ToolTipManager.sharedInstance().unregisterComponent(table);
            	ToolTipManager.sharedInstance().unregisterComponent(table.getTableHeader());
            }
        }
        
        Color colorOddRow = DcSettings.getColor(DcRepository.settings.stOddRowColor);
        Color colorEvenRow = DcSettings.getColor(DcRepository.settings.stEvenRowColor);
        Color colorRowSelection = DcSettings.getColor(DcRepository.settings.stRowSelectionColor);
        Font font = DcSettings.getFont(DcRepository.settings.stSystemFontNormal);
        setFont(font);
        
        setForeground(ComponentFactory.getCurrentForegroundColor());

        if (!isSelected) {
            if ((row % 2) == 0) {
        		setBackground(colorEvenRow);
            } else {
                setBackground(colorOddRow);
            }
        } else {
        	if ((row % 2) == 0) {
        		if (hasFocus) {
        			setBackground(colorEvenRow);
        		} else {
        			setBackground(colorRowSelection);
        		}
        	} else {
        		if (hasFocus) {
        			setBackground(colorOddRow);
        		} else {
        			setBackground(colorRowSelection);
        		}
        	}
        }
        
    	if (hasFocus) {
    		setBorder(focusBorder);
    	} else {
    		setBorder(noFocusBorder);
    	}
        
        return c;
    }
    
    protected String getTipText() {
        return super.getText();
    }
}