/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.event.MouseEvent;

import javax.swing.MenuSelectionManager;
import javax.swing.SwingUtilities;
import javax.swing.text.JTextComponent;

import net.sf.dc.core.log.DcLog;


public class DcEditPopupEventQueue extends EventQueue {

    private DcEditPopupMenu popupmenu;

    /**
     * Creates a new instance of DcPopupShortEventQueue
     */
    public DcEditPopupEventQueue() {
        super();
    }

    protected void dispatchEvent(AWTEvent event) {
        try {
            try {
                super.dispatchEvent(event);
            } catch (Exception e) {
                DcLog.add(e);
            }
    
            if (!(event instanceof MouseEvent)) {
                return;
            }
    
            MouseEvent me = (MouseEvent) event;
            if (!me.isPopupTrigger()) {
                if (me.getID() == MouseEvent.MOUSE_RELEASED && popupmenu != null
                        && popupmenu.isShowing())
                    popupmenu.setVisible(false);
    
                return;
            }
    
            Component comp = SwingUtilities.getDeepestComponentAt((Component) me
                    .getSource(), me.getX(), me.getY());
    
            if (!(comp instanceof JTextComponent))
                return;
    
            if (MenuSelectionManager.defaultManager().getSelectedPath().length > 0)
                return;
    
            popupmenu = new DcEditPopupMenu((JTextComponent) comp);
            popupmenu.validate();
            popupmenu.show((Component) me.getSource(), me.getX(), me.getY());
        } catch (Exception exp) {
            DcLog.add(exp);
        }
    }
}
