/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.Font;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JMenu;

import net.sf.dc.actions.AboutAction;
import net.sf.dc.actions.ChangeLookAndFeelAction;
import net.sf.dc.actions.ChangeViewAction;
import net.sf.dc.actions.CreateNewAction;
import net.sf.dc.actions.CreateReportAction;
import net.sf.dc.actions.DeleteAction;
import net.sf.dc.actions.ExpertUserAction;
import net.sf.dc.actions.HelpAction;
import net.sf.dc.actions.LogAction;
import net.sf.dc.actions.MigrationAction;
import net.sf.dc.actions.NewItemWizardAction;
import net.sf.dc.actions.OnlineSearchAction;
import net.sf.dc.actions.OpenAction;
import net.sf.dc.actions.OpenAutoIncrementSettingsDialog;
import net.sf.dc.actions.OpenBackupDialogAction;
import net.sf.dc.actions.OpenFieldsSettingsDialog;
import net.sf.dc.actions.OpenModuleAction;
import net.sf.dc.actions.OpenQuickViewSettingsDialog;
import net.sf.dc.actions.OpenResourceEditorAction;
import net.sf.dc.actions.OpenSynchronizerDialog;
import net.sf.dc.actions.OpenTitleRewriterSettingsDialog;
import net.sf.dc.actions.QuitApplicationAction;
import net.sf.dc.actions.SaveAllAction;
import net.sf.dc.actions.SearchAction;
import net.sf.dc.actions.SearchAgainAction;
import net.sf.dc.actions.SettingsAction;
import net.sf.dc.actions.ShowTipOfTheDayAction;
import net.sf.dc.actions.StartModuleWizard;
import net.sf.dc.actions.StartProgramAction;
import net.sf.dc.actions.ToggleGroupingPaneAction;
import net.sf.dc.actions.ToggleModuleListAction;
import net.sf.dc.actions.ToggleQuickFilterBar;
import net.sf.dc.actions.ToggleQuickViewAction;
import net.sf.dc.actions.ToggleToolbarLabelsAction;
import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcMenu;
import net.sf.dc.console.components.DcMenuItem;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcTemplate;
import net.sf.dc.core.objects.template.Templates;
import net.sf.dc.core.resources.DcResources;

public class DcMenuBar extends net.sf.dc.console.components.DcMenuBar {

	public static final int _SEARCHPANEL = 0;
    public static final int _INSERTPANEL = 1;
    public static final int _NOTEPANEL = 2;
    
    private final DcModule module;

    public DcMenuBar(DcModule module) {
        this.module = module;
    	build();
    }
    
    public void setPanel(int index) {
        switch (index) {
            case _SEARCHPANEL:
                break;
            case _INSERTPANEL:
                break;
            case _NOTEPANEL:
                break;
        }
    }
    
    public void setFont(Font font) {
        for (int i = 0; i < getMenuCount(); i++) {
            JMenu menu = getMenu(i);
            menu.setFont(ComponentFactory.getSystemFont());
            for (int j = 0; j < menu.getMenuComponents().length; j++) {
                menu.getMenuComponents()[j].setFont(ComponentFactory.getStandardFont());
            }
        }
    }
    
    private void build() {
        String dcoName = module.getObjectName();

        DcMenu menuTools = new DcModuleToolsMenu(module);
        DcMenu menuModules = ComponentFactory.getMenu(DcResources.getText("lblModules"));
        DcMenu menuSettings = ComponentFactory.getMenu(DcResources.getText("lblSettings"));
        DcMenu menuFile = ComponentFactory.getMenu(DcResources.getText("lblFile"));
        DcMenu menuFilter = ComponentFactory.getMenu(DcResources.getText("lblFilter"));
        DcMenu menuInformation = ComponentFactory.getMenu(DcResources.getText("lblHelp"));
        
        // view menu
        DcMenu menuView = ComponentFactory.getMenu(DcResources.getText("lblView"));
        menuView.add(ComponentFactory.getMenuItem(new ChangeViewAction(MasterView._TABLE_VIEW)));
        menuView.add(ComponentFactory.getMenuItem(new ChangeViewAction(MasterView._LIST_VIEW)));
        menuView.addSeparator();
        menuView.add(ComponentFactory.getMenuItem(new ChangeLookAndFeelAction()));
        menuView.addSeparator();
        menuView.add(ComponentFactory.getMenuItem(new ToggleQuickFilterBar())); 
        menuView.add(ComponentFactory.getMenuItem(new ToggleModuleListAction()));
        menuView.add(ComponentFactory.getMenuItem(new ToggleQuickViewAction()));
        menuView.add(ComponentFactory.getMenuItem(new ToggleGroupingPaneAction()));
        menuView.add(ComponentFactory.getMenuItem(new ToggleToolbarLabelsAction()));
        
        // modules menu
        DcMenu subMenuModule = ComponentFactory.getMenu(DcResources.getText("lblActiveModule"));
        for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            if (module.isTopModule() && !module.hasDependingModules() || module.getIndex() == DcModules._CONTACTPERSON) {
                DcMenuItem menuItem = ComponentFactory.getMenuItem(new OpenModuleAction(module.getIndex()));
                subMenuModule.add(menuItem);
                if (module.getKeyStroke() != null)
                    menuItem.setAccelerator(module.getKeyStroke());
            }
        }
        
        menuModules.add(ComponentFactory.getMenuItem(new StartModuleWizard(StartModuleWizard._CREATE_MODULE)));
        menuModules.add(ComponentFactory.getMenuItem(new StartModuleWizard(StartModuleWizard._ALTER_MODULE)));
        menuModules.add(ComponentFactory.getMenuItem(new StartModuleWizard(StartModuleWizard._RELATE_MODULE)));
        menuModules.addSeparator();
        menuModules.add(subMenuModule);
        
        // information menu
        menuInformation.add(ComponentFactory.getMenuItem(new HelpAction()));
        menuInformation.add(ComponentFactory.getMenuItem(new ShowTipOfTheDayAction()));
        menuInformation.addSeparator();
        menuInformation.add(ComponentFactory.getMenuItem(new AboutAction()));
        
        // search menu
        menuFilter.add(ComponentFactory.getMenuItem(new SearchAction()));
        menuFilter.add(ComponentFactory.getMenuItem(new SearchAgainAction()));
        
        // tools menu
        if (module.hasOnlineServices()) {
            menuTools.addSeparator();
            menuTools.add(ComponentFactory.getMenuItem(new OnlineSearchAction()));
            if (module.getSynchronizer() == null)
                menuTools.addSeparator();
        }
        
        if (module.getSynchronizer() != null) {
            menuTools.add(ComponentFactory.getMenuItem(new OpenSynchronizerDialog()));
            menuTools.addSeparator();
        }        
        
        // file menu
        DcMenu menuCreateNew = ComponentFactory.getMenu(IconLibrary._icoAdd, DcResources.getText("lblNewItem", dcoName));
        boolean templatesPresent = false;
        if (module.getTemplateModule() != null) {
            Collection templates = Templates.getTemplates(module.getTemplateModule().getIndex());
            for (Iterator iter = templates.iterator(); iter.hasNext(); ) {
                templatesPresent = true;
                DcTemplate template = (DcTemplate) iter.next();
                menuCreateNew.add(ComponentFactory.getMenuItem(new CreateNewAction(template)));
            }
        }        
        
        menuFile.add(ComponentFactory.getMenuItem(new NewItemWizardAction()));
        menuFile.addSeparator();
        menuFile.add(ComponentFactory.getMenuItem(new StartProgramAction()));
        menuFile.addSeparator();

        if (templatesPresent)
            menuFile.add(menuCreateNew);
        
        menuFile.add(ComponentFactory.getMenuItem(new CreateNewAction()));
        menuFile.add(ComponentFactory.getMenuItem(new OpenAction()));
        menuFile.add(ComponentFactory.getMenuItem(new SaveAllAction()));
        menuFile.add(ComponentFactory.getMenuItem(new DeleteAction()));
        menuFile.addSeparator();
        menuFile.add(ComponentFactory.getMenuItem(new LogAction()));
        menuFile.add(ComponentFactory.getMenuItem(new QuitApplicationAction()));

        // settings menu
        menuSettings.add(ComponentFactory.getMenuItem(new SettingsAction()));
        menuSettings.addSeparator();
        menuSettings.add(ComponentFactory.getMenuItem(new OpenFieldsSettingsDialog()));
        menuSettings.add(ComponentFactory.getMenuItem(new OpenQuickViewSettingsDialog()));
        menuSettings.addSeparator();
        menuSettings.add(ComponentFactory.getMenuItem(new OpenResourceEditorAction()));

        // tools menu
    	menuTools.add(ComponentFactory.getMenuItem(new CreateReportAction()));
        menuTools.add(ComponentFactory.getMenuItem(new MigrationAction()));
        
        if (module.getIndex() != DcModules._MEDIA) {
            menuTools.addSeparator();
            menuTools.add(ComponentFactory.getMenuItem(new OpenAutoIncrementSettingsDialog()));
            menuTools.add(ComponentFactory.getMenuItem(new OpenTitleRewriterSettingsDialog()));
        }
        
        menuTools.addSeparator();
        menuTools.add(ComponentFactory.getMenuItem(new OpenBackupDialogAction()));
        menuTools.addSeparator();        
        menuTools.add(ComponentFactory.getMenuItem(new ExpertUserAction()));
        
        this.add(menuFile);
        this.add(menuModules);
        this.add(menuView);
        this.add(menuFilter);
        this.add(menuSettings);
        this.add(menuTools);
        this.add(menuInformation);
    }
}
