/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.util.Iterator;

import net.sf.dc.actions.ImportAction;
import net.sf.dc.actions.ManageAction;
import net.sf.dc.actions.ManageTemplatesAction;
import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcMenu;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcPropertyModule;
import net.sf.dc.core.resources.DcResources;

public class DcModuleToolsMenu extends DcMenu {
    
    public DcModuleToolsMenu(DcModule module) {
        super(DcResources.getText("lblTools"));
        
        DcModule child = module.getChild();
        
        for (Iterator iter = DcModules.getPropertyModules(module).iterator(); iter.hasNext(); ) {
            DcPropertyModule sm = (DcPropertyModule) iter.next();
            int sourceIdx = sm.isServingMultipleModules() ? sm.getIndex() : sm.getIndex() - module.getIndex();
            if (child != null && child.hasReferenceTo(sourceIdx) && !sm.isServingMultipleModules()) {
                add(sm, module.getObjectName() + " " + sm.getObjectNamePlural());
                add((DcPropertyModule) DcModules.get(sourceIdx + child.getIndex()), 
                    child.getObjectName() + " " + DcModules.get(sourceIdx + child.getIndex()).getObjectNamePlural());
            } else {
                add(sm, sm.getObjectNamePlural());
            }
        }

        // add references to property modules for the child
        if (child != null) {
            for (Iterator iter = DcModules.getPropertyModules(child).iterator(); iter.hasNext(); ) {
                DcPropertyModule sm = (DcPropertyModule) iter.next();
                int sourceIdx = sm.isServingMultipleModules() ? sm.getIndex() : sm.getIndex() - child.getIndex();
                if (!module.hasReferenceTo(sourceIdx) && !sm.isServingMultipleModules())
                    add(sm, child.getObjectName() + " " + sm.getObjectNamePlural());
            }
        }
        
        if (module.isTopModule() &&  module.getTemplateModule() != null) {
            
            if (module.getChild() != null && module.getChild().getTemplateModule() != null) {
                add(ComponentFactory.getMenuItem(new ManageTemplatesAction(module.getIndex(), 
                    module.getObjectName() + " " +  module.getTemplateModule().getObjectNamePlural())));
                add(ComponentFactory.getMenuItem(new ManageTemplatesAction(module.getChild().getIndex(), 
                    module.getChild().getObjectName() + " " +  module.getTemplateModule().getObjectNamePlural())));
            } else {
                add(ComponentFactory.getMenuItem(new ManageTemplatesAction(module.getIndex(), 
                        module.getTemplateModule().getObjectNamePlural())));
            }
            
            if (module.getImporterClass() != null)
                addSeparator();
        }
        
        if (module.getImporterClass() != null)
            add(ComponentFactory.getMenuItem(new ImportAction()));
    }
    
    private void add(DcPropertyModule sm, String title) {
        if (sm != null)
            add(ComponentFactory.getMenuItem(new ManageAction(sm, title))); 
    }
}
