/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.util.Iterator;

import javax.swing.JSeparator;

import net.sf.dc.actions.AddRowAction;
import net.sf.dc.actions.AttachFileInfoAction;
import net.sf.dc.actions.CreateReportAction;
import net.sf.dc.actions.DeleteAction;
import net.sf.dc.actions.EditAsNewAction;
import net.sf.dc.actions.ManageAction;
import net.sf.dc.actions.OpenAction;
import net.sf.dc.actions.OpenFieldsSettingsDialog;
import net.sf.dc.actions.OpenLoanFormAction;
import net.sf.dc.actions.OpenSortingDialog;
import net.sf.dc.actions.RemoveRowAction;
import net.sf.dc.actions.SaveSelectedAction;
import net.sf.dc.actions.StartProgramAction;
import net.sf.dc.actions.UpdateAllAction;
import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcPropertyModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.fileimporters.FileImporter;
import net.sf.dc.settings.definitions.DcFieldDefinition;

public class DcPopupMenu extends net.sf.dc.console.components.DcPopupMenu {

    public static final int _INSERT = 0;
    public static final int _SEARCH = 1;
    
    public DcPopupMenu(DcObject dco, int modus) {
        DcModule module = dco.getModule();

        if (modus == _INSERT && !module.isChildModule()) {
            if (module.getIndex() != DcModules._MEDIA)
                add(ComponentFactory.getMenuItem(new SaveSelectedAction()));
            
            add(ComponentFactory.getMenuItem(new OpenAction()));
            add(ComponentFactory.getMenuItem(new EditAsNewAction(dco)));
        }
        
        if (modus == DcPopupMenu._SEARCH) { 
            add(ComponentFactory.getMenuItem(new DeleteAction(module.getIndex())));
            add(ComponentFactory.getMenuItem(new EditAsNewAction(dco)));
        } else {
            add(ComponentFactory.getMenuItem(new RemoveRowAction(dco.getModule().getIndex())));
            add(ComponentFactory.getMenuItem(new AddRowAction(dco.getModule().getIndex())));
        }   
        
        add(new JSeparator(JSeparator.HORIZONTAL));
        add(ComponentFactory.getMenuItem(new OpenSortingDialog()));
        
        if (modus == DcPopupMenu._SEARCH && module.canBeLended()) {
            add(new JSeparator(JSeparator.HORIZONTAL));
            add(ComponentFactory.getMenuItem(new OpenLoanFormAction()));
        }
        
        boolean sepAdded = false;
        for (Iterator iter = module.getFieldDefinitions().iterator(); iter.hasNext(); ) {
            
            DcFieldDefinition definition = (DcFieldDefinition) iter.next();
            DcField field = module.getField(definition.getIndex());
            
            DcPropertyModule pm = DcModules.getPropertyModule(field);
            if (pm != null) {
                if (!sepAdded)
                    add(new JSeparator(JSeparator.HORIZONTAL));

                sepAdded = true;
                add(ComponentFactory.getMenuItem(new ManageAction(pm, pm.getObjectNamePlural())));
            }
        }
        
        if (module.getCurrentSearchView().getIndex() == MasterView._TABLE_VIEW) {
            add(new JSeparator(JSeparator.HORIZONTAL));
            add(ComponentFactory.getMenuItem(new OpenFieldsSettingsDialog()));
        }
        
        if (modus == DcPopupMenu._SEARCH) {
            FileImporter importer = module.getImporter();
            if (importer != null && importer.allowReparsing() && module.getDcObject().getFileField() != null) { 
                add(new JSeparator(JSeparator.HORIZONTAL));                
                add(ComponentFactory.getMenuItem(new AttachFileInfoAction()));
            }
        }
        
        add(new JSeparator(JSeparator.HORIZONTAL));
        add(ComponentFactory.getMenuItem(new UpdateAllAction(module.getIndex())));
        add(ComponentFactory.getMenuItem(new StartProgramAction()));
        
        add(new JSeparator(JSeparator.HORIZONTAL));
        add(ComponentFactory.getMenuItem(new CreateReportAction()));
    }
}
