/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JToolBar;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcComboBox;
import net.sf.dc.console.components.DcEditableComboBox;
import net.sf.dc.console.components.renderers.ComboBoxRenderer;
import net.sf.dc.console.menu.DcToolBar.ToolBarButton;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataFilterEntry;
import net.sf.dc.core.data.DataFilters;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.data.Operator;
import net.sf.dc.core.data.Operators;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class DcQuickFilterToolBar extends JToolBar implements ActionListener {

    private final DcEditableComboBox comboCriteria = new DcEditableComboBox();
    private final DcComboBox comboFields = ComponentFactory.getComboBox(new DefaultComboBoxModel());
    private final DcComboBox comboFilters = ComponentFactory.getComboBox();
    
    private final SearchOnEnterListener searchOnEnterListener = new SearchOnEnterListener();
    private final SearchOnDoubleClickListener searchOnDblClickListener = new SearchOnDoubleClickListener();
    private final FilterSelectionListener filterSelListener = new FilterSelectionListener();
    private final FieldSelectionListener fieldSelListener = new FieldSelectionListener();
    
    private final DcModule module;
    
    public DcQuickFilterToolBar(DcModule module) {
        this.module = module;
        build();
        
        // set default value (if applicable)
        int fieldIdx = module.getSettings().getInt(DcRepository.modulesettings.stQuickFilterDefaultField);
        comboFields.setSelectedItem(module.getField(fieldIdx));
    }
    
    public void setFont(Font font) {
        Component c;
        int i = 0;
        while ((c = getComponentAtIndex(i++)) != null) {
            if (c instanceof ToolBarButton) {
                ToolBarButton button = (ToolBarButton) c;
                button.setFont(ComponentFactory.getSystemFont());
            }
        }
    }    
    
    private void search() {
        DataFilter df = getDataFilter();
        DataManager.bindData(module.getSearchView(), module.getIndex(), df);
        DataFilters.setCurrent(module.getIndex(), df);
    }
    
    private void search(DataFilter df) {
        DataManager.bindData(module.getSearchView(), module.getIndex(), df);
    }
    
    private Object getValue() {
        DcField field = (DcField) comboFields.getSelectedItem();
        
        Object value = comboCriteria.getSelectedItem();
        if (field.getFieldType() == ComponentFactory._RATINGCOMBOBOX)
            value = value != null && value.equals(new Long(-1)) ? null : value;
        
        value = value != null && value instanceof String ? ((String) value).trim() : value;
        
        return value;
    }
    
    private DataFilter getDataFilter() {
        DcField field = (DcField) comboFields.getSelectedItem();
        Object value = getValue();
        
        if (value != null && !value.equals("")) {
            Operator operator = Operators.getOperator(Operators._CONTAINS);
            
            if (value.equals(DcResources.getText("lblEmptySearchValue"))) {
                operator = Operators.getOperator(Operators._IS_EMPTY);
                value = null;
            } else if (field.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION) {
                Collection c = new ArrayList();
                c.add(value);
                value = c;
            } else if (field.getValueType() == DcRepository.datatypes._LONG) {
                operator = Operators.getOperator(Operators._EQUAL_TO);
            }
            
            DataFilter df = DataFilters.getDefaultDataFilter(module.getIndex());
            DataFilterEntry dfe = new DataFilterEntry(DataFilterEntry._AND, 
                                                      module.getIndex(), 
                                                      field.getIndex(), 
                                                      operator,
                                                      value);
            df.addEntry(dfe);
            return df;
        } else {
            return null;
        }
    }

    private void setSearchField(DcField field) {
        JComponent c = ComponentFactory.getComponent(field.getModule(), field.getReferenceIdx(), field.getFieldType(), field.getLabel(), 255);
        comboCriteria.removeAllItems();
        
        if (field.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION) {
            DcObject[] objects = DataManager.get(field.getReferenceIdx(), null);
            comboCriteria.addItem(" ");
            for (int i = 0; objects != null && i < objects.length; i++)
                comboCriteria.addItem(objects[i]);
            
            comboCriteria.setRenderer(new ComboBoxRenderer());
            comboCriteria.setEditable(false);
            
        } else if (c instanceof JComboBox) {
            JComboBox combo = (JComboBox) c;
            for (int i = 0; i < combo.getItemCount(); i++)
                comboCriteria.addItem(combo.getItemAt(i));
            
            comboCriteria.setRenderer(combo.getRenderer());
            comboCriteria.setEditable(false);
        } else {
        	comboCriteria.addItem(" ");
            comboCriteria.setEditable(true);
        }

        comboCriteria.addItem(DcResources.getText("lblEmptySearchValue"));

        revalidate();
    }
    
    private void build() {
        List fields = new ArrayList();
        for (Iterator iter = module.getFields().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            if ( field.isEnabled() && field.isSearchable() && 
               (!field.isUiOnly() || field.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION))
                fields.add(field);
        }
        
        Collections.sort(fields, new Comparator() {
            public int compare(Object o1, Object o2) {
                return ((DcField) o1).getLabel().compareTo(((DcField) o2).getLabel());
            }
        });   
        
        for (Iterator iter = fields.iterator(); iter.hasNext(); )
            comboFields.addItem(iter.next());
        
        Collection filters = DataFilters.get(module.getIndex());
        comboFilters.addItem(" ");
        for (Iterator iter = filters.iterator(); iter.hasNext(); ) {
            comboFilters.addItem(iter.next());
        }
        
        comboCriteria.getEditor().getEditorComponent().addKeyListener(searchOnEnterListener);
        comboCriteria.getEditor().getEditorComponent().addMouseListener(searchOnDblClickListener);
        comboCriteria.setRenderer(new DefaultListCellRenderer());
        
        comboFilters.addActionListener(filterSelListener);
        comboFields.addActionListener(fieldSelListener);
        
        comboFilters.setPreferredSize(new Dimension(200, 21));
        comboFields.setPreferredSize(new Dimension(200, 21));
        comboCriteria.setPreferredSize(new Dimension(200, 21));
        comboFilters.setMaximumSize(new Dimension(200, 21));
        comboFields.setMaximumSize(new Dimension(200, 21));
        comboCriteria.setMaximumSize(new Dimension(200, 21));
        
        add(ComponentFactory.getLabel(DcResources.getText("lblQuickFilter") + " "));
        add(comboFields);
        add(comboCriteria);
        
        JButton button1 = ComponentFactory.getIconButton(IconLibrary._icoAccept);
        button1.addActionListener(this);

        JButton button2 = ComponentFactory.getIconButton(IconLibrary._icoAccept);
        button2.setActionCommand("searchOnSelectedFilter");
        button2.addActionListener(this);

        add(button1);
        
        if (filters.size() > 0) {
            addSeparator();
            
            add(ComponentFactory.getLabel(DcResources.getText("lblFilters") + " "));
            add(comboFilters);
            add(button2);
        }
    }    
    
    public void actionPerformed(ActionEvent ae) {
        if (ae.getActionCommand().equals("searchOnSelectedFilter"))
            search(comboFilters.getSelectedIndex() > 0 ? (DataFilter) comboFilters.getSelectedItem() : null);    
        else
            search();
    }
    
    private class SearchOnEnterListener implements KeyListener {

        public void keyReleased(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                search();
            }
        }

        public void keyPressed(KeyEvent e) {}
        public void keyTyped(KeyEvent e) {}
    }

    private class SearchOnDoubleClickListener implements MouseListener {

        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                search();
            }            
        }
    }
    
    private class FieldSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            comboCriteria.removeAllItems();
            
            DcComboBox comboFields = (DcComboBox) e.getSource();
            DcField field = (DcField) comboFields.getSelectedItem();
            setSearchField(field);
            module.getSettings().set(DcRepository.modulesettings.stQuickFilterDefaultField, new Integer(field.getIndex()));
        }
    }
    
    private class FilterSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DataFilter df = comboFilters.getSelectedIndex() > 0 ? (DataFilter) comboFilters.getSelectedItem() : null;
            search(df);
        }
    } 
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getBoolean(DcRepository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        
        super.paintComponent( g2d );
    }
}
