/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.menu;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;

import javax.swing.JMenu;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.components.DcMenu;
import net.sf.dc.console.components.DcMenuItem;
import net.sf.dc.console.components.panels.TreePanel;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class DcTreePanelMenuBar extends net.sf.dc.console.components.DcMenuBar {
    
    private final int modIdx;
    private final TreePanel treePanel;

    public DcTreePanelMenuBar(int modIdx, TreePanel treePanel) {
        this.modIdx = modIdx;
        this.treePanel = treePanel;
        
        DcMenu menuEdit = ComponentFactory.getMenu(DcResources.getText("lblEdit"));
        DcMenuItem menuExpandAll = ComponentFactory.getMenuItem(DcResources.getText("lblExpandAll"));
        DcMenuItem menuCollapseAll = ComponentFactory.getMenuItem(DcResources.getText("lblCollapseAll"));
        DcMenuItem menuClear = ComponentFactory.getMenuItem(DcResources.getText("lblClear"));
        menuCollapseAll.addActionListener(new CollapseAllAction());
        
        menuExpandAll.addActionListener(new ExpandAllAction());
        menuClear.addActionListener(new ClearAction());
        
        menuEdit.add(menuCollapseAll);
        menuEdit.add(menuExpandAll);
        menuEdit.addSeparator();
        menuEdit.add(menuClear);
        
        DcModule module = DcModules.get(modIdx);
        DcMenu menuFields = ComponentFactory.getMenu(DcResources.getText("lblGroupBy"));

        for (Iterator iter = module.getFields().iterator(); iter.hasNext(); ){
            DcField fld = (DcField) iter.next();
            
            if (isValidGroupingField(fld)) {
                String label = fld.getLabel();
                DcMenu subMenu = ComponentFactory.getMenu(label);

                DcMenuItem mi = ComponentFactory.getMenuItem(label);
                mi.addActionListener(new FieldSelectionListener(new int[] {fld.getIndex()}));
                
                menuFields.add(mi);
                menuFields.add(subMenu);
                
                for (Iterator iterSub = module.getFields().iterator(); iterSub.hasNext(); ){
                    DcField subFld = (DcField) iterSub.next();
                    if (subFld.getIndex() != fld.getIndex() && isValidGroupingField(subFld)) {
                        DcMenuItem subItem = ComponentFactory.getMenuItem(subFld.getLabel());
                        subItem.addActionListener(new FieldSelectionListener(new int[] {fld.getIndex(), subFld.getIndex()}));
                        subMenu.add(subItem);
                    }
                }
            }
        }
        
        setPreferredSize(new Dimension(100, 22));
        setMaximumSize(new Dimension(100, 22));
        setMinimumSize(new Dimension(50, 22));
        
        add(menuFields);
        add(menuEdit);
    }
    
    private boolean isValidGroupingField(DcField fld) {
        return (fld.isEnabled() && !fld.isTechnicalInfo()) &&
               (fld.getValueType() != DcRepository.datatypes._BLOB && 
                fld.getValueType() != DcRepository.datatypes._BYTEARRAY &&
                fld.getValueType() != DcRepository.datatypes._COLOR &&
                fld.getValueType() != DcRepository.datatypes._DIMENSION &&
                fld.getValueType() != DcRepository.datatypes._FONT &&
                fld.getValueType() != DcRepository.datatypes._ICON &&
                fld.getValueType() != DcRepository.datatypes._PICTURE && 
                fld.getFieldType() != ComponentFactory._FILEFIELD &&
                fld.getFieldType() != ComponentFactory._FILELAUNCHFIELD &&
                fld.getFieldType() != ComponentFactory._LONGTEXTFIELD &&
                fld.getFieldType() != ComponentFactory._URLFIELD);
    }
    
    public void setFont(Font font) {
        for (int i = 0; i < getMenuCount(); i++) {
            JMenu menu = getMenu(i);
            menu.setFont(ComponentFactory.getSystemFont());
            for (int j = 0; j < menu.getMenuComponents().length; j++) {
                menu.getMenuComponents()[j].setFont(ComponentFactory.getStandardFont());
            }
        }
    }    
    
    private class CollapseAllAction implements ActionListener {
        
        public void actionPerformed(ActionEvent arg0) {
            treePanel.collapseAll();
        }
    }    

    private class ClearAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            treePanel.reset();
        }
    } 
    
    private class ExpandAllAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            treePanel.expandAll();
        }
    }    
    
    private class FieldSelectionListener implements ActionListener {

        private final int[] fields;
        
        public FieldSelectionListener(int[] fields) {
            this.fields = fields;
        }

        public void actionPerformed(ActionEvent arg0) {
            DcModules.get(modIdx).getCurrentSearchView().applyGrouping(fields);
        }
    }
    
    protected void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;
        
        if (DcSettings.getBoolean(DcRepository.settings.stFontAntiAliasing)) {
           g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
           g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
        }
        
        super.paintComponent( g2d );
    }    
}

