/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JPanel;

import net.sf.dc.console.Layout;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.settings.DcSettings;

public class MasterView {

    public static final int _TABLE_VIEW = 0;
    public static final int _LIST_VIEW = 1;
    
    protected final Map views = new HashMap();
    
    public JPanel getViewPanel() {
        JPanel panel = new JPanel();

        View view = getCurrent();
        
        panel.setLayout(Layout.getGBL());
        panel.add( view, Layout.getGBC( 0, 0, 2, 1, 2.0, 2.0
                  ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                   new Insets(5, 5, 5,  5), 0, 0));
        
        return panel;
    }
    
    public void setView(int index) {
        DataCrow.mainFrame.applyView(index);
    }
    
    public View get(int index) {
        return (View) views.get(new Integer(index));
    }
    
    public View getCurrent() {
        View current = get(DcSettings.getInt(DcRepository.settings.stSelectedView));
        current = current == null ? get(MasterView._TABLE_VIEW) : current;        
        return current;
    }      
    
    public void delete() {
        for (Iterator iter = views.values().iterator(); iter.hasNext(); ) {
            View view = (View) iter.next();
            view.delete();
        }          
    }
    
    public void addView(int index, View view) {
        views.put(new Integer(index), view);        
    }

    public void setStatus(String message) {
        getCurrent().setStatus(message);
    }    
    
    public void applySettings() {
        for (Iterator iter = views.values().iterator(); iter.hasNext(); ) {
            View view = (View) iter.next();
            view.applyDefinitions();
            view.applySettings();
        }        
    }
    
    public void sort() {
        Collection c = views.values();
        Object[] o = c.toArray();
        
        for (int i = 0; i < o.length; i++) {
            View view = (View) o[i];
            view.sort();
        }
    }

    public void removeRows(String[] ids) {
        Collection c = views.values();
        Object[] o = c.toArray();
        
        for (int i = 0; i < o.length; i++) {
            View view = (View) o[i];
            view.removeRows(ids);
        }
    }

    public void updateRow(String ID, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark) {
        for (Iterator iter = views.values().iterator(); iter.hasNext(); ) {
    		View view = (View) iter.next();
    		view.updateRow(ID, dco, overwrite, allowDeletes, mark);
        }
    }    
    
    public void reload(String ID) {
        for (Iterator iter = views.values().iterator(); iter.hasNext(); ) {
            View view = (View) iter.next();
            view.reload(ID);
        }
    }
    
    public void removeFromCache(String ID) {
        for (Iterator iter = views.values().iterator(); iter.hasNext(); ) {
            View view = (View) iter.next();
            view.removeFromCache(ID);
        }
    }    
    
    public void clear() {
        for (Iterator iter = views.values().iterator(); iter.hasNext(); ) {
            View view = (View) iter.next();
            view.clear();
        }
    }
    
    public void add(DcObject dco) {
        for (Iterator iter = views.values().iterator(); iter.hasNext(); ) {
            View view = (View) iter.next();
            view.add(dco);
        }
    }
    
    public void add(DcObject[] objects) {
        for (Iterator iter = views.values().iterator(); iter.hasNext(); ) {
            View view = (View) iter.next();
            view.add(objects);
        }
    }    
    
    public void add(Collection objects) {
        for (Iterator iter = views.values().iterator(); iter.hasNext(); ) {
            View view = (View) iter.next();
            view.add(objects);
        }
    }
    
    public Iterator getViews() {
        return views.values().iterator();
    }
}
