/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.KeyStroke;

import net.sf.dc.console.components.DcPanel;
import net.sf.dc.console.components.DcViewDivider;
import net.sf.dc.console.components.panels.TreePanel;
import net.sf.dc.console.windows.UpdateAllDialog;
import net.sf.dc.console.windows.itemforms.ItemForm;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.util.DataTask;

public abstract class View extends DcPanel {

    protected TreePanel treePanel;
    
    protected DataTask task;
    private final int index; 
    private final DcModule module;
    
    protected DcViewDivider vdQuickPane;
    protected DcViewDivider vdGroupingPane;
    
    private boolean overwriteVisibility = false;
    
    public View(DcModule module, String title, ImageIcon icon, int index) {
        super(title, icon);  
        this.index = index;
        this.module = module;
        
        setFocusable(true);

        getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), "DELETE");
        getActionMap().put("DELETE", new AbstractAction() {
            public void actionPerformed(ActionEvent ignored) {
                delete();
            }
        });        
    }    
    
    protected boolean isTaskRunning() {
        boolean isTaskRunning = task != null && task.isRunning();
        if (isTaskRunning)
            new MessageBox(DcResources.getText("msgJobRunning"), MessageBox._WARNING); 
        
        return isTaskRunning; 
    }
    
    public void setOverwriteVisibility(boolean b) {
        overwriteVisibility = b;
    }
    
    public boolean isVisibilityOverwritten() {
        return overwriteVisibility;
    }
    
    public void applyViewDividerLocation() {
        if (vdQuickPane != null)
            vdQuickPane.setDividerLocation(DcSettings.getInt(DcRepository.settings.stQuickViewDividerLocation));
        
        if (vdGroupingPane != null)
            vdGroupingPane.setDividerLocation(DcSettings.getInt(DcRepository.settings.stTreeDividerLocation));
    }
    
    public DataTask getCurrentTask() {
        return task;
    }
    
    public void markAsUnchanged() {}
    
    public int getIndex() {
        return index;
    }
    
    public void sort() {
        int rowCount = getRows();
        ArrayList objects = new ArrayList();
        for (int i = 0; i < rowCount; i++)
            objects.add(getObject(i));
        
        DataFilter df = new DataFilter(module.getIndex());
        df.setOrder(module.getSettings().getStringArray(DcRepository.modulesettings.stSearchOrder));
        df.sort(objects);

        add(objects, false);
    }
    
    public void open(DcObject dco, boolean update, boolean applyTemplate) {
        dco.setRow(-1);
        ItemForm itemForm = new ItemForm(    null,
                                             false,
                                             update,
                                             dco,
                                             applyTemplate);
        itemForm.setVisible(true);
    }
    
    public void openUpdateAllDialog() {
        UpdateAllDialog dialog = new UpdateAllDialog(this);
        dialog.setVisible(true);
    }     
    

    public DcModule getModule() {
        return module;
    }     
    
    public abstract void applyGrouping(int[] fields);
    public abstract void undoGrouping();
    
    public abstract JComponent[] getExtraComponents();
    
    public abstract DcObject getObject(int row);
    
    public abstract Collection getSelectedObjects();
    public abstract DcObject getSelectedObject();
    
    public abstract DcObject getDcObject(String id);
    
    public abstract int[] getSelectedRows();
    public abstract int getRows();
    
    public abstract void applySettings();
    
    public abstract boolean isChangesSaved();
    
    public abstract void reload(String ID);
    
    public abstract void add(DcObject dco);
    public abstract void add(DcObject dco, boolean select);
    
    public abstract void add(Collection objects, boolean updateTree);
    public abstract void add(DcObject[] objects, boolean updateTree);

    public abstract void add(Collection objects);
    public abstract void add(DcObject[] objects);

    public abstract void clear();
    public abstract void clear(boolean safe);
    public abstract void open();
    public abstract void createReport();
    public abstract void delete();
    
    public abstract void removeRows(String[] ids);
    
    public abstract void save();
    public abstract void saveSelected();
    public abstract void updateRow(String ID, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark);
    public abstract void updateRow(int row, DcObject dco, boolean overwrite, boolean allowDeletes, boolean mark);
    
    public abstract void showQuickView(boolean b);
    public abstract void removeFromCache(String id);    
    
    public abstract void applyDefinitions();
    

    
}
