/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.views.list;

import java.awt.Cursor;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.DcViewDivider;
import net.sf.dc.console.components.lists.elements.DcObjectListElement;
import net.sf.dc.console.components.panels.TreePanel;
import net.sf.dc.console.menu.DcPopupMenu;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.windows.messageboxes.QuestionBox;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.core.wf.requests.IRequest;
import net.sf.dc.core.wf.requests.StatusUpdateRequest;
import net.sf.dc.core.wf.requests.UpdateStatusProgressBarRequest;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.util.DataTask;

public class ListSearchView extends ListBaseView {

    protected RowSelectionListener rowSelectionListener = new RowSelectionListener();
    protected ChangeRowByKey changeRowByKey = new ChangeRowByKey();
    
    public ListSearchView(DcModule module, String title, ImageIcon icon) {
        super(module, title, icon);  
        setHelpIndex("dc.items.views");
        buildPanel();
    }    
    
    public void open() {
        DcObject dco = getSelectedObject();
        
        if (dco != null) {
            dco.markAsUnchanged();
            open(dco, true, true);
        }
    }
    
    public void reload(String ID) {
        DcObjectListElement element = list.getElementForID(ID);
        if (element != null)
            element.update();
        
        if (quickView != null)
            quickView.setImages();
        
        repaint();
    }    
    
    public void delete() {
        if (!isTaskRunning()) {
            Collection objects = list.getSelectedObjects();
            if (objects.size() > 0) {
                task = new DeletingThread(objects);
                task.start();
            } else {
                new MessageBox(DcResources.getText("msgSelectItemToDel"), MessageBox._WARNING);    
            }
        }
    }     
    
    public boolean isChangesSaved() {
        boolean isChangesSaved = true;
        boolean saved = list.getChangedIndices() == null || list.getChangedIndices().length == 0;
        if (!saved) {
            QuestionBox qb = new QuestionBox(DcResources.getText("msgNotSaved"));
            if (qb.isAffirmative()) {
                save();
                isChangesSaved = false;
            } else {
                list.markAsUnchanged();
            }
        }
        return isChangesSaved;
    }    
    
    public void applyGrouping(int[] fields) {
        treePanel.groupBy(fields);
    }
    
    public void undoGrouping() {
        
    }
    
    protected void setSelectedIndex() {
        try {
            list.setSelectedIndex(0);
            setDataForSelectedRow();
        } catch (Exception exp) {}        
    } 
    
    /**
     * Saves the selected values
     */
    public void saveSelected() {
        if (!isTaskRunning()) {
            task = new SavingThread(list.getSelectedObjects(), true);
            task.start();
        }
    }    
    
    public void markAsUnchanged() {
        list.markAsUnchanged();
    }
    
    public void save() {
        if (!isTaskRunning()) {
            int[] rows = list.getChangedIndices();
            if (rows.length > 0) {
                Collection objects = new ArrayList();
                for (int i = 0; i < rows.length; i++)
                    objects.add(getObject(rows[i]));
                
                task = new SavingThread(objects, true);
                task.startRunning();
                task.start();   
            } else {
                new MessageBox(DcResources.getText("msgNoChangesToSave"), MessageBox._WARNING);    
            }
        }
    }    
    
    public void setDataForSelectedRow() {
        if (quickView != null && quickView.isVisible() && updateQuickView) {
            int index = list.getSelectedIndex() > -1 ? list.getSelectedIndex() : 0;
            quickView.setObject(getObjectForIndex(index));
        }
    }     
    
    public void showQuickView(boolean b) {
        quickView.setVisible(b);
    }    
    
    protected DcPopupMenu getPopupMenu() {
        return new DcPopupMenu(getSelectedObject(), DcPopupMenu._SEARCH);
    }
    
    private void buildPanel() {

        //**********************************************************
        //Search result panel
        //**********************************************************
        panelResult.setLayout(Layout.getGBL());
        scrollResults = new JScrollPane(list);
        scrollResults.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        treePanel = new TreePanel(getModule().getIndex(), this);
       
        panelResult.add(scrollResults,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                       ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                        new Insets(5, 5, 5, 5), 0, 0));
        
        vdGroupingPane = new DcViewDivider(treePanel, panelResult, DcRepository.settings.stTreeDividerLocation);
        
        list.addMouseListener(mouseListener);
        list.getSelectionModel().addListSelectionListener(rowSelectionListener);
        list.addKeyListener(changeRowByKey);
        
        quickView = getModule().getQuickView();
        vdQuickPane = new DcViewDivider(vdGroupingPane, quickView, DcRepository.settings.stQuickViewDividerLocation);
        if (quickView != null)
            quickView.setVisible(DcSettings.getBoolean(DcRepository.settings.stShowQuickView));
        
        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());
        
        panelActions = getActionPanel(false, null);
        add(    vdQuickPane,               Layout.getGBC( 0, 1, 3, 1, 100.0, 100.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0)); 
        add(    panelActions,     Layout.getGBC( 0, 2, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));
        add(    panelStatus,      Layout.getGBC( 0, 3, 3, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 5), 0, 0));

        ToolTipManager.sharedInstance().registerComponent(list);
    }  
    
    protected class DeletingThread extends DataTask {

        public DeletingThread(Collection objects) {
            super(objects);
        }
        
        public void run() {
            try {
                updateProgressBar(0);
                initProgressBar(objects.size());   
                list.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                
                setStatus(DcResources.getText("msgDeletingXItems", "" + objects.size()));
                
                startRunning();
                denyActions();
                
                QuestionBox qb = new QuestionBox(DcResources.getText("msgDeleteQuestion")); 
                if (!qb.isAffirmative()) {
                    stopRunning();
                } else {
                    int counter = 1;
                    for (Iterator iter = objects.iterator(); iter.hasNext() && keepOnRunning(); counter++) {
                        DcObject dco = (DcObject) iter.next();
                        
                        dco.markAsUnchanged();
                        dco.addRequest(new UpdateStatusProgressBarRequest(
                                dco.getModule().getIndex(), MainFrame._SEARCHTAB, counter));                        
                        
                        if (counter == objects.size()) {
                            dco.setSilence(false);
                            dco.setEndOfBatch(true);
                            IRequest request = 
                                new StatusUpdateRequest(dco.getModule().getIndex(), MainFrame._SEARCHTAB, "msgDeleteSuccessfull");
                            request.setExecuteOnFail(true);
                            dco.addRequest(request);
                        } else {
                            dco.setSilence(true);
                            dco.setEndOfBatch(false);
                        }
                        
                        dco.delete(); 
                        
                        try {
                            sleep(300);
                        } catch (Exception ignore) {} // doesn't want to sleep..
                    }
                }
            } finally {
                list.setSelectedIndex(0);
                list.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                stopRunning();
                allowActions();
            }
        }
    }
    
    protected class RowSelectionListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            if (e.getValueIsAdjusting()) {
                setDataForSelectedRow();
            }
        }
    }    
    
    protected class ChangeRowByKey implements KeyListener {
        private int rowFrom = -1;
        private int rowTo = -1;

        public void keyPressed(KeyEvent e) {
           if (e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN) {
               rowFrom = list.getSelectedIndex();
           }
        }

        public void keyReleased(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_DOWN ||
                e.getKeyCode() == KeyEvent.VK_LEFT || e.getKeyCode() == KeyEvent.VK_RIGHT) {
                
                rowTo = list.getSelectedIndex();
                if (rowFrom != rowTo) {
                    if (quickView != null && quickView.isVisible() && updateQuickView)
                        quickView.setObject(getObjectForIndex(rowTo));
                }
            }
        }

        public void keyTyped(KeyEvent arg0) {}
    }
}
