/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.net.URL;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcPictureField;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.resources.DcResources;

public class AboutDialog extends DcDialog {

    public AboutDialog(JFrame parent) {
        super(parent);

        buildDialog();

        setTitle("");
        Image img =IconLibrary._icoMain.getImage();
        img.flush();

        setCenteredLocation();
    }

    private void buildDialog() {
        JTabbedPane tabbedPane = ComponentFactory.getTabbedPane();

        //**********************************************************
        //About panel
        //**********************************************************
        JPanel panelAbout = new JPanel();
        panelAbout.setLayout(Layout.getGBL());

        DcPictureField about = ComponentFactory.getPictureField(false, false, false, "");
        try {
            about.setValue(new URL("file://" + DataCrow.baseDir + "icons_system/aboutinformation.jpg"));
            panelAbout.add(about, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
            new Insets(5, 5, 5, 5), 0, 0));
        } catch (Exception exp) {
            DcLog.add(exp);
        }

        //**********************************************************
        //Credits
        //**********************************************************
        JPanel panelCredits = new JPanel();
        panelCredits.setLayout(Layout.getGBL());

        DcPictureField credits = ComponentFactory.getPictureField(false, false, false, "");
        try {
            credits.setValue(new URL("file://" + DataCrow.baseDir + "icons_system/aboutcredits.jpg"));
            panelCredits.add(credits, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
            GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
            new Insets(5, 5, 5, 5), 0, 0));
        } catch (Exception exp) {
            DcLog.add(exp);
        }

        //**********************************************************
        //Main panel
        //**********************************************************
        tabbedPane.addTab(DcRepository.system.version, IconLibrary._icoAbout, panelAbout);
        tabbedPane.addTab(DcResources.getText("lblCredits"), IconLibrary._icoHelp, panelCredits);

        getContentPane().setLayout(Layout.getGBL());
        getContentPane().add(tabbedPane,   Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                                           GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                           new Insets(5, 5, 5, 5), 0, 0));

        setTitle(DcResources.getText("msgQuestion"));
        pack();
        setSize(new Dimension(450,490));
        setResizable(false);
    }
}
