/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.lists.DcObjectList;
import net.sf.dc.console.components.lists.elements.DcObjectListElement;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.MappingModule;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class DcReferencesDialog extends DcDialog {
    
    private Collection selected;
    private Vector elements;
    private DcObjectList listRight;
    private DcObjectList listLeft;
    private MappingModule mappingModule;
    
    private boolean saved = false;
    
    public DcReferencesDialog(Collection currentItems, MappingModule mappingModule) {
        selected = new ArrayList();
        this.mappingModule = mappingModule;
        
        Collection current = currentItems == null ? new ArrayList() : currentItems;
        selected.addAll(current);
        
        setTitle(DcModules.get(mappingModule.getReferencedModIdx()).getObjectNamePlural());
        buildDialog();
        
        DcObject[] dcos = DataManager.get(mappingModule.getReferencedModIdx(), null);
        listLeft.add(dcos);
        
        int counter = 0;
        String[] ids = new String[current.size()];
        Collection selected = new ArrayList();
        for (Iterator iter = current.iterator(); iter.hasNext();) {
            DcMapping mapping = (DcMapping) iter.next();
            DcObject reference = mapping.getReferencedObject();
            
            if (reference != null) {
                selected.add(reference);
                ids[ counter++] = reference.getID();
            }
        }
        listRight.add(selected);
        listLeft.remove(ids);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });

        // store the elements for filtering purposes
        elements = new Vector();
        elements.addAll(listLeft.getElements());
        
        pack();
        
        setSize(DcSettings.getDimension(DcRepository.settings.stReferencesDialogSize));
        setCenteredLocation();
        setModal(true);
    }
    
    public Collection getDcObjects() {
        return selected;
    }
    
    public boolean isSaved() {
        return saved;
    }
    
    private void save() {
        selected.clear();
        
        Collection c = listRight.getObjects();
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            DcObject dco = (DcObject) iter.next();
            DcObject mapping = mappingModule.getDcObject();
            mapping.setValue(DcMapping._B_REFERENCED_ID, dco.getID());
            selected.add(mapping);
        }
        
        saved = true;
        close();
    }
    
    public void close() {
        DcSettings.set(DcRepository.settings.stReferencesDialogSize, getSize());
        setVisible(false);
    }
    
    public void dispose() {
        if (listLeft != null)
            listLeft.clear();
        
        listLeft = null;
        
        if (listRight != null)
            listRight.clear();
        
        listRight = null;
        
        if (elements != null)
            elements.clear();
        
        elements = null;
        
        mappingModule = null;

        if (selected != null)
            selected.clear();
        
        selected = null;
    }    
    
    private void buildDialog() {
        getContentPane().setLayout(Layout.getGBL());
        
        JTextField txtFilter = ComponentFactory.getShortTextField(255);
        txtFilter.setPreferredSize(new Dimension(50, 23));
        txtFilter.setMinimumSize(new Dimension(50, 23));        
        txtFilter.addKeyListener(new FilterInputListener());

        listLeft = new DcObjectList(DcObjectList._LISTING, false, true);
        listLeft.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        listLeft.addMouseListener(new ListMouseListener(ListMouseListener._RIGHT));

        listRight = new DcObjectList(DcObjectList._LISTING, false, true);
        listRight.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        listRight.addMouseListener(new ListMouseListener(ListMouseListener._LEFT));
        
        JScrollPane scrollerLeft = new JScrollPane(listLeft);
        scrollerLeft.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollerLeft.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        JScrollPane scrollerRight = new JScrollPane(listRight);
        scrollerRight.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollerRight.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
        JButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));
        
        JPanel panelActions = new JPanel();
        panelActions.add(buttonSave);
        panelActions.add(buttonClose);
        
        buttonSave.addActionListener(new SaveAction());
        buttonClose.addActionListener(new CloseAction());

        getContentPane().add(txtFilter,     Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 0), 0, 0));
        getContentPane().add(ComponentFactory.getLabel(DcResources.getText("lblAvailable")),  
                Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));
        getContentPane().add(ComponentFactory.getLabel(DcResources.getText("lblSelected")),  
                Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));
        getContentPane().add(scrollerLeft,  Layout.getGBC( 0, 2, 1, 1, 40.0, 40.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));
        getContentPane().add(scrollerRight, Layout.getGBC( 1, 2, 1, 1, 40.0, 40.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));
        getContentPane().add(panelActions,  Layout.getGBC( 0, 3, 2, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets( 5, 0, 5, 0), 0, 0));
    }
    
    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            save();
        }
    }     

    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            close();
        }
    }     

    private class ListMouseListener implements MouseListener {
        
        public static final int _LEFT = 0;
        public static final int _RIGHT = 1;
        
        int direction;
        
        public ListMouseListener(int direction) {
            this.direction = direction;
        }
        
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                if (direction == _LEFT) {
                    DcObject dco = listRight.getSelectedObject();
                    listLeft.add(dco);
                    listRight.remove();
                    listRight.clearSelection();
                } else {
                    DcObject dco = listLeft.getSelectedObject();
                    listRight.add(dco);
                    listLeft.remove();
                    listLeft.clearSelection();
                }
            }  
        }
        
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    } 
    
    private Vector getUnselectedItems() {
        Vector unselected = new Vector();
        for (Iterator iter1 = elements.iterator(); iter1.hasNext(); ) {
            DcObjectListElement element1 = (DcObjectListElement) iter1.next();
            String displayValue1 = element1.getDcObject().toString();
            
            boolean exists = false;
            for (Iterator iter2 = listRight.getElements().iterator(); iter2.hasNext(); ) {
                DcObjectListElement element2 = (DcObjectListElement) iter2.next();
                String displayValue2 = element2.getDcObject().toString();
                
                if (displayValue1.equals(displayValue2))
                    exists = true;
            }
            
            if (!exists)
                unselected.add(element1);
        }

        return unselected;
    }
    
    private class FilterInputListener implements KeyListener {

        public void keyPressed(KeyEvent e) {}
        public void keyTyped(KeyEvent e) {}

        public void keyReleased(KeyEvent e) {
            DcShortTextField txtFilter = (DcShortTextField) e.getSource();
            String filter = txtFilter.getText();
            
            if (filter.trim().length() == 0) {
                listLeft.setListData(getUnselectedItems());
            } else {
                Vector unselected = getUnselectedItems();
                Vector newElements = new Vector();
                for (Iterator iter1 = unselected.iterator(); iter1.hasNext(); ) {
                    DcObjectListElement element = (DcObjectListElement) iter1.next();
                    String displayValue = element.getDcObject().toString();
                    if (displayValue.toLowerCase().startsWith(filter.toLowerCase()))
                        newElements.add(element);
                }
                listLeft.setListData(newElements);                
            }
        }
    }
}
