/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.KeyStroke;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcFieldSelectorField;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.resources.DcResources;

public class FieldSelectorDialog extends DcDialog { 
    
    private final int modIdx;
    private DcFieldSelectorField fs;
    private String settingsKey;
    
    public FieldSelectorDialog(JFrame parent, int modIdx, String settingsKey) {
        
        super(parent);
        
        this.modIdx = modIdx;
        this.settingsKey = settingsKey;
        
        setTitle(DcResources.getText("lblOnlineUpdateFieldSettings"));
        
        try {
            setModal(true);
            
            addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    close();
                }
            });
            
            build();
            
            fs.setSelectedFields((int[]) getModule().getSetting(settingsKey));
        } catch (Exception e) {
            DcLog.add(e);
        }
        
        setSize(getModule().getSettings().getDimension(DcRepository.modulesettings.stOnlineSearchFieldSettingsDialogSize));
        setCenteredLocation();
    }
    
    private DcModule getModule() {
        return DcModules.get(modIdx);
    }
    
    public void close() {
        getModule().getSettings().set(DcRepository.modulesettings.stOnlineSearchFieldSettingsDialogSize, getSize());
        fs = null;
        super.close();
    }

    private void apply() {
        getModule().setSetting(settingsKey, fs.getSelectedFieldIndices());
        close();
    }    
    
    private void build() {
        getContentPane().setLayout(Layout.getGBL());
        
        fs = new DcFieldSelectorField(modIdx);
        
        getContentPane().add(fs, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 5, 5, 5), 0, 0));        
        
        JPanel panelActions = new JPanel();
        panelActions.setLayout(new FlowLayout(FlowLayout.RIGHT));

        JButton buttonApply = ComponentFactory.getButton(DcResources.getText("lblApply"));
        JButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));
        
        buttonApply.addActionListener(new ApplyAction());
        buttonClose.addActionListener(new CloseAction());
        
        panelActions.add(buttonApply);  
        panelActions.add(buttonClose);
        
        getContentPane().add(panelActions, Layout.getGBC(0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                             new Insets(0, 0, 5, 5), 0, 0));
        
        JMenu editMenu = createMenu();
        JMenuBar mb = ComponentFactory.getMenuBar();
        mb.add(editMenu);
        setJMenuBar(mb);
        
        pack();
    }
    
    private JMenu createMenu() {
        JMenu menu = ComponentFactory.getMenu("Edit");
        
        JMenuItem menuSelectAll = ComponentFactory.getMenuItem(DcResources.getText("lblSelectAll"));
        JMenuItem menuUnselectAll = ComponentFactory.getMenuItem(DcResources.getText("lblUnselectAll"));
        JMenuItem menuInvertSelection = ComponentFactory.getMenuItem(DcResources.getText("lblInvertSelection"));

        menuSelectAll.addActionListener(new SelectAllAction());
        menuUnselectAll.addActionListener(new UnselectAllAction());
        menuInvertSelection.addActionListener(new InvertSelectionAction());
        
        menuSelectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
        menuUnselectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, InputEvent.CTRL_MASK));
        menuInvertSelection.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));
        
        menu.add(menuSelectAll);
        menu.add(menuUnselectAll);
        menu.addSeparator();
        menu.add(menuInvertSelection);
        
        return menu;
    }    

    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            close();
        }
    }    
    
    private class ApplyAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            apply();
        }
    }
    
    private class SelectAllAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            fs.selectAll();
        }
    }

    private class InvertSelectionAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            fs.invertSelection();
        }
    }
    
    private class UnselectAllAction implements ActionListener {
        public void actionPerformed(ActionEvent arg0) {
            fs.unselectAll();
        }
    }
}

