/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.DcImageLabel;
import net.sf.dc.console.components.DcTextArea;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.resources.DcResources;

public class LogForm extends DcFrame {

    private JTextArea logger;
    private JPanel panelLogging = new JPanel();
    private JPanel panelPicture = new JPanel();
    private JPanel panelProductInfo = new JPanel();

    private JScrollPane scroller;
    
    private JLabel labelVersion = ComponentFactory.getLabel(DcRepository.system.version);
    private JLabel labelLastUpd = ComponentFactory.getLabel(DcRepository.system.releasedate);

    public LogForm() {
        super(DcResources.getText("lblLog"), IconLibrary._icoEventLog);
        buildPanel();
    }
    
    public void logToFront(boolean show) {
        this.setVisible(show);

        if (show) {
            this.toFront();
        }
    }

    public void setFont(Font font) {
        if (logger != null) {
            labelVersion.setFont(ComponentFactory.getSystemFont());
            labelLastUpd.setFont(ComponentFactory.getSystemFont());
            logger.setFont(ComponentFactory.getStandardFont());
        }
    }
    
    public void writeToScreen(String message) {
        logger.insert(message, 0);
    }    
    
    public void setVisible(boolean b) {
        SwingUtilities.updateComponentTreeUI(this.getContentPane());
        super.setVisible(b);
    }

    private void buildPanel() {
        //**********************************************************
        //Logo panel
        //**********************************************************
        panelPicture.setLayout(Layout.getGBL());
        DcImageLabel logo = ComponentFactory.getImageLabel(new ImageIcon(DataCrow.baseDir + "icons_system/logo.jpg"));
        panelPicture.setBorder(ComponentFactory.getBorder());

        panelPicture.setPreferredSize(new Dimension(575,200));
        panelPicture.setMinimumSize(new Dimension(575,200));
        panelPicture.add(logo,  Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                               ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                                new Insets(0, 0, 0, 0), 0, 0));

        //**********************************************************
        //Logging panel
        //**********************************************************
        panelLogging.setLayout(Layout.getGBL());

        logger = new DcTextArea();
        logger.setEditable(true);
        
        scroller = new JScrollPane(logger);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        panelLogging.add(scroller,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                    new Insets( 0, 5, 0, 5), 0, 0));

        //**********************************************************
        //Product information panel
        //**********************************************************
        panelProductInfo.setLayout(Layout.getGBL());

        panelProductInfo.add(labelVersion,     Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets( 0, 5, 0, 0), 0, 0));
        panelProductInfo.add(labelLastUpd,     Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                                              ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                               new Insets( 0, 5, 0, 0), 0, 0));


        //**********************************************************
        //Main panel
        //**********************************************************
        getContentPane().setLayout(Layout.getGBL());
        
        // Build the panel
        this.getContentPane().add(   panelPicture,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                    new Insets( 5, 5, 0, 5), 0, 0));
        this.getContentPane().add(   panelLogging,  Layout.getGBC( 0, 1, 1, 1, 20.0, 20.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                                    new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(panelProductInfo, Layout.getGBC( 0, 2, 1, 1, 0.0, 0.0
                                                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                                    new Insets( 5, 0, 5, 0), 0, 0));
        pack();

        setSize(590, 600);
        setResizable(false);
        
        setCenteredLocation();
    }
}
