/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcFileField;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.console.components.tables.DcTableModel;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.util.Base64;
import net.sf.dc.util.CSVReader;
import net.sf.dc.util.Utilities;

public class MigrationDialog extends DcDialog {

    private DcTable tableMappings;

    private DcFileField ffSource;
    private JButton buttonApply = ComponentFactory.getButton(DcResources.getText("lblApply"));
    private JButton buttonRun = ComponentFactory.getButton(DcResources.getText("lblRun"));

    private DcShortTextField textSep = ComponentFactory.getShortTextField(5);

    private JTextArea textLog = ComponentFactory.getTextArea();
    private JProgressBar progressBar = new JProgressBar();

    private String sep = "|";

    public MigrationDialog() {
        super(DataCrow.mainFrame);

        setTitle(DcResources.getText("lblFileImport"));
        setHelpIndex("dc.tools.migration");

        ffSource = ComponentFactory.getFileField(false, false);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });

        buildDialog();
    }

    private void addMessage(String message) {
        textLog.insert(message + '\n', 0);
        textLog.setCaretPosition(0);
    }

    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }

    public void updateProgressBar() {
        progressBar.setValue(progressBar.getValue() + 1);
    }

    public void close() {
        DcModules.getCurrent().getSettings().set(DcRepository.modulesettings.stFileImportDialogSize, getSize());
        
        sep = null;
        tableMappings = null;
        ffSource = null;
        buttonApply = null;
        buttonRun = null;
        textSep = null;
        textLog = null;
        progressBar = null;
        
        super.close();
    }

    private void buildDialog() {
        getContentPane().setLayout(Layout.getGBL());

        //**********************************************************
        //Create Import Panel
        //**********************************************************
        JPanel panelImport = new JPanel();
        panelImport.setLayout(Layout.getGBL());

        JLabel labelSep = ComponentFactory.getLabel(DcResources.getText("lblValueSeperator"));
        textSep.setText(",");

        buttonApply.addActionListener(new ApplyFileAction());
        panelImport.add(ffSource,       Layout.getGBC( 0, 0, 4, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelImport.add(labelSep,        Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelImport.add(textSep,        Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelImport.add(buttonApply,    Layout.getGBC( 2, 1, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 5, 5), 0, 0));

        panelImport.setBorder(
                ComponentFactory.getTitleBorder(DcResources.getText("lblSourceSelection")));

        //**********************************************************
        //Create Import Definition Panel
        //**********************************************************
        tableMappings = ComponentFactory.getDCTable(false, false);

        DcTableModel model = (DcTableModel) tableMappings.getModel();
        model.setColumnCount(2);

        TableColumn columnName = tableMappings.getColumnModel().getColumn(0);
        JTextField text = ComponentFactory.getTextFieldDisabled();
        columnName.setCellEditor(new DefaultCellEditor(text));
        columnName.setHeaderValue(DcResources.getText("lblSourceName"));

        TableColumn columnField = tableMappings.getColumnModel().getColumn(1);
        Collection fields = DcModules.getCurrent().getFields();
        JComboBox comboFields = ComponentFactory.getComboBox();
        columnField.setHeaderValue(DcResources.getText("lblTargetName"));

        for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
            DcField  field = (DcField) iter.next();
            if ((!field.isUiOnly() || field.getValueType() == DcRepository.datatypes._PICTURE) && 
                field.getIndex() != DcObject._ID)
                comboFields.addItem(field);
        }
        columnField.setCellEditor(new DefaultCellEditor(comboFields));

        tableMappings.applyHeaders();

        buttonRun.addActionListener(new ImportAction());

        JScrollPane tableScroller = new JScrollPane(tableMappings);
        tableScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        JPanel panelImportDef = new JPanel();
        panelImportDef.setLayout(Layout.getGBL());

        panelImportDef.add(tableScroller, Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 5, 5, 5), 0, 0));
        panelImportDef.add(buttonRun,   Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 5, 5), 0, 0));

        panelImportDef.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblImportDefinitions")));

        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel();
        panelLog.setLayout(Layout.getGBL());

        JScrollPane scroller = new JScrollPane(textLog);
        textLog.setEditable(false);
        textLog.setEnabled(false);

        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));

        panelLog.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblLog")));

        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));


        //**********************************************************
        //Main panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());

        JButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));

        buttonApply.setMnemonic('P');
        buttonClose.setMnemonic('C');
        buttonRun.setMnemonic('R');
        
        buttonClose.addActionListener(new CloseAction());

        this.getContentPane().add(panelImport,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelImportDef,  Layout.getGBC( 0, 1, 1, 1, 20.0, 20.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelLog,     Layout.getGBC( 0, 2, 1, 1, 10.0, 10.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panelProgress,Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(buttonClose,  Layout.getGBC( 0, 4, 1, 1, 0.0, 0.0
                                 ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                                  new Insets( 5, 5, 5, 5), 0, 0));
        pack();

        Dimension size = DcModules.getCurrent().getSettings().getDimension(DcRepository.modulesettings.stFileImportDialogSize);
        setSize(size);
        setCenteredLocation();
    }

    private boolean isCorrectSource(File file) {
        boolean correct = file != null && file.exists();
        if (file == null)

            new MessageBox(DcResources.getText("msgNoFileSelected"), MessageBox._WARNING);
        else if (!file.exists())
            new MessageBox(DcResources.getText("msgFileCannotBeUsed"), MessageBox._WARNING);
        return correct;
    }

    private void applySource() throws IOException {
        File file = ffSource.getFile();

        sep = textSep.getText().trim();

        if (sep.length() == 0) {
            sep = ",";
            textSep.setText(sep);
        }

        tableMappings.clearTable();
        
        if (isCorrectSource(file)) {
            InputStreamReader reader = new InputStreamReader(new FileInputStream(file), "UTF-8");
            CSVReader csvReader = new CSVReader(reader, getSepCharacter(sep), this);
            String[] headers = csvReader.readNext();
            
            Collection fields = DcModules.getCurrent().getFields();
            for (int i = 0; i < headers.length; i++) {
                String columnValue = headers[i];

                DcField field = null;
                for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
                    DcField fld = (DcField) iter.next();
                    if (fld.getLabel().toUpperCase().equals(columnValue.toUpperCase()))
                        field = fld;
                }

                tableMappings.addRow(
                        new Object[] {DcResources.getText("lblField") +
                                      " " + i + " (" + columnValue + ")", field});
            }
            initProgressBar(0);
        }
    }

    private String getSepCharacter(String sep) {
        String character = sep;
        if (sep.toUpperCase().equals("TAB"))
            character = "\t";
        return character;
    }

    private Long convertToLong(String s, DcField field) {
        try {
            if (s != null && s.length() > 0) {
                return new Long(s);
            }
        } catch (Exception exp) {
            addMessage(DcResources.getText("msgCouldNotConvertToInt", new String[] {s, field.getLabel()}));
        }
        return null;
    }

    private void importFile() {
        buttonRun.setEnabled(false);
        buttonApply.setEnabled(false);
        DataCrow.mainFrame.setSelectedTab(net.sf.dc.console.MainFrame._INSERTTAB);

        try {
            LinkedList fields = new LinkedList();
            for (int i = 0; i < tableMappings.getRowCount(); i++) {
                DcField field = (DcField) tableMappings.getValueAt(i, 1, true);

                if (field == null){
                    new MessageBox(DcResources.getText("msgNoDestination", (String) tableMappings.getValueAt(i, 0)), MessageBox._WARNING);
                    return;
                } else if (fields.contains(field)) {
                    new MessageBox(DcResources.getText("msgDestinationInUse", (String) tableMappings.getValueAt(i, 0)), MessageBox._WARNING);
                    return;
                } else {
                    fields.add(field);
                }
            }

            File file = ffSource.getFile();

            try {
                if (isCorrectSource(file)) {
                    InputStreamReader reader = new InputStreamReader(new FileInputStream(file), "UTF-8");
                    CSVReader csvReader = new CSVReader(reader, getSepCharacter(sep), this);
                    List lines = csvReader.readAll();                        
                    
                    initProgressBar(lines.size() - 1);
                    
                    DcModule module = DcModules.getCurrent();
                    int counter = 0;
                    for (Iterator iter = lines.iterator(); iter.hasNext(); counter++) {
                        String[] values = (String[]) iter.next();

                        if (counter != 0) {
                            DcObject dco = module.getDcObject();
    
                            for (int i = 0; i < values.length; i++) {
                                String s = values[i];
                                DcField field = (DcField) fields.get(i);
                                if (field.getValueType() == DcRepository.datatypes._LONG) {
                                    dco.setValue(field.getIndex(), convertToLong(s, field));
                                } else if (field.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION ||
                                         field.getValueType() == DcRepository.datatypes._DCOBJECTREFERENCE) {
                                    DataManager.createReference(dco, field.getIndex(), s);
                                } else if (field.getValueType() == DcRepository.datatypes._PICTURE) {
                                    
                                    try {
                                        byte[] image = Base64.decode(s.toCharArray());
                                        dco.setValue(field.getIndex(), image);
                                    } catch (Exception e) {
                                        if (s.trim().length() > 0) {
                                            byte[] image = Utilities.readFile(new File(s));
                                            dco.setValue(field.getIndex(), image);
                                        }
                                    }
                                } else {
                                    dco.setValue(field.getIndex(),s);
                                }
                            }
                            module.getCurrentInsertView().add(dco);
                            updateProgressBar();
                        }
                    }
                    reader.close();
                }
            } catch (Exception exp) {
                DcLog.add(exp);

            }
        } finally {
            buttonRun.setEnabled(true);
            buttonApply.setEnabled(true);
        }
    }
    
    private class ImportThread extends Thread {
        public void run() {
            importFile();
        }
    }

    private class ApplyFileAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            try {
                applySource();
            } catch (IOException ioExp) {
                new MessageBox(DcResources.getText("msgFileCannotBeUsed") + ": " + ioExp.getMessage(), MessageBox._ERROR);
                addMessage("Error while reading file : " + ioExp.getMessage());
                DcLog.add(ioExp);
            }
        }
    }

    private class ImportAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (tableMappings.getRowCount() == 0) {
                new MessageBox(DcResources.getText("msgNoFieldsFound"), MessageBox._WARNING);
            } else {
                ImportThread thread = new ImportThread();
                thread.start();
            }
        }
    }

    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
}
