/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.renderers.CheckBoxTableCellRenderer;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.Settings;
import net.sf.dc.settings.definitions.Definitions;
import net.sf.dc.settings.definitions.QuickViewFieldDefinition;
import net.sf.dc.settings.definitions.QuickViewFieldDefinitions;

public class QuickViewSettingsDialog extends DcDialog {

    private DefinitionPanel panelDefinitionsParent = null;
    private DefinitionPanel panelDefinitionsChild = null;

    public QuickViewSettingsDialog() {
        super(DataCrow.mainFrame);

        setHelpIndex("dc.Settings.QuickViewSettings");

        buildDialog();

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });

        setTitle(DcResources.getText("lblQuickViewSettings"));
        setModal(true);
    }

    public void close() {
        Settings settings = DcModules.getCurrent().getSettings();
        settings.set(DcRepository.modulesettings.stQuickViewSettingsDialogSize, getSize());
        panelDefinitionsParent = null;
        panelDefinitionsChild = null;
        super.close();
    }

    private void save() {
        panelDefinitionsParent.save();
        if (panelDefinitionsChild != null)
            panelDefinitionsChild.save();
    }

    private void buildDialog() {
        getContentPane().setLayout(Layout.getGBL());

        /***********************************************************************
         * ACTIONS PANEL
         **********************************************************************/
        JPanel panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());

        JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
        JButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));

        buttonSave.setMnemonic('S');
        buttonClose.setMnemonic('C');
        
        buttonSave.addActionListener(new SaveAction());
        buttonClose.addActionListener(new CloseAction());

        panelActions.add(buttonSave, Layout.getGBC(0, 0, 1, 4, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 5), 0, 0));
        panelActions.add(buttonClose, Layout.getGBC(1, 0, 1, 4, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));

        /***********************************************************************
         * MAIN PANEL
         **********************************************************************/

        DcModule module = DcModules.get(DcSettings.getInt(DcRepository.settings.stModule));
        panelDefinitionsParent = new DefinitionPanel(module);
        panelDefinitionsParent.setBorder(ComponentFactory.getTitleBorder(module.getObjectName()));

        getContentPane().add(panelDefinitionsParent, Layout.getGBC(0, 0, 1, 1, 10.0, 10.0,
                             GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));

        if (module.getChild() != null) {
            panelDefinitionsChild = new DefinitionPanel(module.getChild());
            getContentPane().add(panelDefinitionsChild, Layout.getGBC(0, 1, 1, 1, 10.0, 10.0,
                                 GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                 new Insets(5, 5, 5, 5), 0, 0));
            panelDefinitionsChild.setBorder(ComponentFactory.getTitleBorder(module.getChild().getObjectName()));
        }

        getContentPane().add(panelActions, Layout.getGBC(0, 2, 1, 1, 1.0, 1.0,
                             GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));

        pack();

        Settings settings = DcModules.getCurrent().getSettings();
        Dimension dim = settings.getDimension(DcRepository.modulesettings.stQuickViewSettingsDialogSize);
        setSize(dim);

        setCenteredLocation();
    }

    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }

    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            save();
        }
    }

    protected class DefinitionPanel extends JPanel {

        private final DcTable table;
        private DcModule module;

        public DefinitionPanel(DcModule module) {
            this.module = module;
            table = ComponentFactory.getDCTable(false, false);
            buildPanel();
        }

        private void buildPanel() {
            setLayout(Layout.getGBL());

            table.setColumnCount(4);
            TableColumn columnHidden = table.getColumnModel().getColumn(3);
            table.removeColumn(columnHidden);

            TableColumn columnField = table.getColumnModel().getColumn(0);
            JTextField textField = ComponentFactory.getTextFieldDisabled();
            columnField.setCellEditor(new DefaultCellEditor(textField));
            columnField.setHeaderValue(DcResources.getText("lblLabel"));

            TableColumn columnEnabled = table.getColumnModel().getColumn(1);
            JCheckBox checkEnabled = new JCheckBox();
            columnEnabled.setCellEditor(new DefaultCellEditor(checkEnabled));
            columnEnabled.setCellRenderer(new CheckBoxTableCellRenderer());
            columnEnabled.setHeaderValue(DcResources.getText("lblEnabled"));
            
            JComboBox comboDirection = ComponentFactory.getComboBox(
                    new String[] {DcResources.getText("lblHorizontal"), DcResources.getText("lblVertical")});

            TableColumn columnDirection = table.getColumnModel().getColumn(2);
            columnDirection.setCellEditor(new DefaultCellEditor(comboDirection));
            columnDirection.setHeaderValue(DcResources.getText("lblDirection"));
            
            applyDefinitions();

            JScrollPane scroller = new JScrollPane(table);
            scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
            scroller.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);

            JLabel labelNav = ComponentFactory.getLabel("");
            JButton buttonTop = ComponentFactory.getButton(IconLibrary._icoArrowTop);
            JButton buttonUp = ComponentFactory.getButton(IconLibrary._icoArrowUp);
            JButton buttonDown = ComponentFactory.getButton(IconLibrary._icoArrowDown);
            JButton buttonBottom = ComponentFactory.getButton(IconLibrary._icoArrowBottom);

            buttonTop.addActionListener(new RowToTopAction());
            buttonUp.addActionListener(new RowUpAction());
            buttonDown.addActionListener(new RowDownAction());
            buttonBottom.addActionListener(new RowToBottomAction());

            buttonBottom.setPreferredSize(new Dimension(25, 22));
            buttonBottom.setMaximumSize(new Dimension(25, 22));
            buttonBottom.setMinimumSize(new Dimension(25, 22));

            buttonTop.setPreferredSize(new Dimension(25, 22));
            buttonTop.setMaximumSize(new Dimension(25, 22));
            buttonTop.setMinimumSize(new Dimension(25, 22));
            
            buttonUp.setPreferredSize(new Dimension(25, 22));
            buttonUp.setMaximumSize(new Dimension(25, 22));
            buttonUp.setMinimumSize(new Dimension(25, 22));
            
            buttonDown.setPreferredSize(new Dimension(25, 22));
            buttonDown.setMaximumSize(new Dimension(25, 22));
            buttonDown.setMinimumSize(new Dimension(25, 22));
            
            add(scroller,  Layout.getGBC(0, 0, 1, 12, 50.0, 50.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                    new Insets(0, 0, 0, 0), 0, 0));
            add(labelNav,  Layout.getGBC(1, 0, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(0, 5, 5, 5), 0, 0));
            add(buttonTop, Layout.getGBC(1, 1, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(0, 5, 0, 5), 0, 0));
            add(buttonUp,  Layout.getGBC(1, 2, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(0, 5, 0, 5), 0, 0));
            add(buttonDown,Layout.getGBC(1, 10, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(5, 5, 0, 5), 0, 0));
            add(buttonBottom,Layout.getGBC(1, 11, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(5, 5, 0, 5), 0, 0));

            table.applyHeaders();
        }

        private void save() {
            module.setSetting(DcRepository.modulesettings.stQuickViewFieldDefinitions, getDefinitions());

            if (module.getSearchView() != null)
                module.getSearchView().applySettings();

            if (module.getInsertView() != null)
                module.getInsertView().applySettings();
        }

        public void applyDefinitions() {
            Definitions definitions = 
                module.getSettings().getDefinitions(DcRepository.modulesettings.stQuickViewFieldDefinitions);
            table.clearTable();
            for (Iterator iter = definitions.iterator(); iter.hasNext();) {
                QuickViewFieldDefinition definition = (QuickViewFieldDefinition) iter.next();
                table.addRow(definition.getDisplayValues(module));
            }            
        }

        public Definitions getDefinitions() {
            table.cancelEdit();
            
            QuickViewFieldDefinitions qvDefs = new QuickViewFieldDefinitions();
            for (int row = 0; row < table.getRowCount(); row++) {
                boolean enabled = ((Boolean) table.getValueAt(row, 1, true)).booleanValue();
                String direction = (String) table.getValueAt(row, 2, true);
                DcField field = (DcField) table.getValueAt(row, 3, true);
                qvDefs.addDefinition(new QuickViewFieldDefinition(field.getIndex(), enabled, direction));
            }
            return qvDefs;
        }
        
        private class RowToTopAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                table.moveRowToTop();
            }
        }        
        
        private class RowToBottomAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                table.moveRowToBottom();
            }
        }          

        private class RowUpAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                table.moveRowUp();
            }
        }

        private class RowDownAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                table.moveRowDown();
            }
        }
    }
}
