/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows;

import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Properties;
import java.util.Random;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcButton;
import net.sf.dc.console.components.DcCheckBox;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcTextPane;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.util.Utilities;

public class TipOfTheDayDialog extends DcDialog {

    private DcTextPane tipPane = ComponentFactory.getTextPane();
    private DcCheckBox checkShowTips = ComponentFactory.getCheckBox(DcResources.getText("lblShowTipsOnStartup"));
    
    private HTMLEditorKit kit = new HTMLEditorKit();
    private HTMLDocument document = new HTMLDocument();

    private LinkedList tips = new LinkedList();
    
    private int currentTip = 0;
    
    public TipOfTheDayDialog() {
        super(DataCrow.mainFrame);
        setTitle(DcResources.getText("lblTipOfTheDay"));
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        
        build();
        setSize(400, 300);
        setCenteredLocation();
    }
    
    private void showNextTip() {
        currentTip = currentTip < tips.size() -1 ? currentTip + 1 : 0; 
        showNextTip(currentTip);
    }    

    private void showNextTip(int idx) {
        try {
            String tip = (String) tips.get(idx);
            Font font = DcSettings.getFont(DcRepository.settings.stSystemFontNormal);
            String hexColor = Utilities.getHexColor(ComponentFactory.getCurrentForegroundColor());
            
            String fontstyle = "";
            if (font.isItalic())
                fontstyle = ";font-style:italic";
            else if (font.isBold())
                fontstyle = ";font-weigth:bolder";
            
            String style = "style=" + '"' + 
                           ";font-family:" + font.getFamily() + 
                           ";font-size:" + font.getSize() + 
                           fontstyle + 
                           ";color:" + hexColor + ";" + '"';            
            
            String html = "<html><body " + style + ">" + tip + "</body></html>";
            tipPane.setText("");
            kit.insertHTML(document, 0, html, 0, 0, null);
        } catch (Exception exp) {
            DcLog.add(exp);
        }
    }    

    
    private void build() {
        tipPane.setEditorKit(kit);
        tipPane.setDocument(document);
        tipPane.setEditable(false);
        tipPane.setBounds(1,1,1,1);
        
        getContentPane().setLayout(Layout.getGBL());
        
        DcButton buttonNext = ComponentFactory.getButton(DcResources.getText("lblNext"));
        DcButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));
        
        buttonNext.addActionListener(new ShowNextTipAction());
        buttonClose.addActionListener(new CloseAction());
        checkShowTips.addActionListener(new ToggleShowTipsAction());
        checkShowTips.setSelected(DcSettings.getBoolean(DcRepository.settings.stShowTipsOnStartup));
        
        JScrollPane scroller = new JScrollPane(tipPane);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        JPanel panelActions = new JPanel();
        panelActions.setLayout(new FlowLayout(FlowLayout.RIGHT));
        panelActions.add(buttonNext);
        panelActions.add(buttonClose);
        
        buttonNext.setMnemonic('N');
        buttonClose.setMnemonic('C');
        
        getContentPane().add(scroller,      Layout.getGBC( 0, 0, 4, 1, 10.0, 10.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 5, 5, 5, 5), 0, 0));
        getContentPane().add(checkShowTips, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0));
        getContentPane().add(panelActions,  Layout.getGBC( 3, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 0), 0, 0));
        
        setResizable(false);
        pack();

        initializeTips();
        currentTip = new Random().nextInt(tips.size());
        showNextTip(currentTip);
    }
    
    
    private void initializeTips() {
        Properties properties = DcResources.getResources();
        for (Iterator iter = properties.keySet().iterator(); iter.hasNext(); ) {
            String key = (String) iter.next();
            if (key.startsWith("tip")) 
                tips.add(properties.get(key));
        }
    }
    
    public void close() {
        kit = null;
        document = null;
        tipPane = null;
        checkShowTips = null;
        
        tips.clear();
        tips = null;
        super.close();
    }    
    
    private class ToggleShowTipsAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DcSettings.set(DcRepository.settings.stShowTipsOnStartup, new Boolean(checkShowTips.isSelected()));
        }
    }      

    private class ShowNextTipAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showNextTip();
        }
    }    
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }    
}
