/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.enhancers;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.ResultSet;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.db.DatabaseManager;
import net.sf.dc.core.db.Query;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.enhancers.IValueEnhancer;
import net.sf.dc.enhancers.TitleRewriter;
import net.sf.dc.enhancers.ValueEnhancers;

public class TitleRewriterDialog extends DcDialog {

    private boolean canceled = false;
    
    private JProgressBar progressBar = new JProgressBar();
    
    private JButton buttonRun = ComponentFactory.getButton(DcResources.getText("lblRun"));
    private JButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));
    private JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
    
    private JCheckBox checkEnabled = ComponentFactory.getCheckBox(DcResources.getText("lblEnabled"));
    private DcLongTextField txtWordList = ComponentFactory.getLongTextField(4000);
    
    private DcModule module = DcModules.getCurrent();

    public TitleRewriterDialog() {
        super(DataCrow.mainFrame);

        buildDialog();

        setHelpIndex("dc.tools.titlerewriter");
        setTitle(DcResources.getText("lblTitleRewriter"));

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        
        setModal(true);
    }

    private void cancel() {
        canceled = true;
    }
    
    public void close() {
        checkEnabled = null;
        txtWordList = null;
        progressBar = null;
        module = null;
        
        super.close();
    }

    private TitleRewriter getTitleRewriter() throws Exception {
        if (txtWordList.getText() == null || txtWordList.getText().trim().length() == 0)
            throw new Exception(DcResources.getText("msgNoMemberWordsDefined"));
        
        return new TitleRewriter(checkEnabled.isSelected(), txtWordList.getText());
    }
    
    private void save() {
        try {
            TitleRewriter titleRewriter = getTitleRewriter();
            module.removeEnhancers();
            DcField field = module.getField(titleRewriter.getField());
            if (field == null) {
                new MessageBox(DcResources.getText("msgCouldNotSaveTitleRewriter"), MessageBox._WARNING);
            } else {
                ValueEnhancers.registerEnhancer(field, titleRewriter);
                ValueEnhancers.save();
            }
        } catch (Exception exp) {
            new MessageBox(exp.getMessage(), MessageBox._WARNING);
        }
    }
    
    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }

    public void updateProgressBar() {
        int current = progressBar.getValue();
        progressBar.setValue(current + 1);
    }    

    private void buildDialog() {
        getContentPane().setLayout(Layout.getGBL());

        /***********************************************************************
         * Settings
         **********************************************************************/
        JPanel panelSettings = new JPanel(false);
        panelSettings.setLayout(Layout.getGBL());
        
        JLabel lblWords = ComponentFactory.getLabel(DcResources.getText("lblWords"));
        buttonSave.setMnemonic('S');
        buttonSave.addActionListener(new SaveAction());     
        
        JScrollPane textScroller = new JScrollPane(txtWordList);
        textScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        textScroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        panelSettings.add(checkEnabled,     Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        panelSettings.add(lblWords,         Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(10, 5, 0, 5), 0, 0));
        panelSettings.add(textScroller,     Layout.getGBC(0, 2, 1, 1, 5.0, 5.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(0, 5, 5, 5), 0, 0));
        panelSettings.add(buttonSave,       Layout.getGBC(0, 3, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 5), 0, 0));
        
        /***********************************************************************
         * Rewrite Titles
         **********************************************************************/
        JPanel panelRewrite = new JPanel(false);
        panelRewrite.setLayout(Layout.getGBL());
        
        DcLongTextField explanation = ComponentFactory.getLongTextField(2000);
        explanation.setText(DcResources.getText("msgTitleRewriterExplanation"));
        ComponentFactory.setUneditable(explanation);
        
        panelRewrite.add(explanation, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHEAST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));

        buttonRun.addActionListener(new RewriteTitlesAction());
        buttonRun.setMnemonic('R');
        
        JButton buttonCancel = ComponentFactory.getButton(DcResources.getText("lblCancel"));
        buttonCancel.addActionListener(new CancelAction());
        buttonCancel.setMnemonic('C');
        
        JPanel panel = new JPanel();
        panel.add(buttonRun);
        panel.add(buttonCancel);
        
        panelRewrite.add(panel,   Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        
        panelRewrite.add(progressBar, Layout.getGBC(0, 2, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        
        /***********************************************************************
         * Main
         **********************************************************************/
        panelRewrite.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblRewriteAll")));
        panelSettings.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblSettings")));
        
        buttonClose.setMnemonic('C');
        buttonClose.addActionListener(new CloseAction());

        getContentPane().add(panelSettings, Layout.getGBC(0, 0, 1, 1, 10.0, 10.0,
                GridBagConstraints.SOUTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelRewrite,  Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.SOUTHWEST, GridBagConstraints.BOTH,
                new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(buttonClose,  Layout.getGBC(0, 2, 1, 1, 1.0, 1.0,
                GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                new Insets(5, 5, 5, 10), 0, 0));        

        Collection enhancers = ValueEnhancers.getEnhancers(module.getIndex(), ValueEnhancers._TITLEREWRITERS);
        if (enhancers != null && enhancers.size() > 0)
            setEnhancers(enhancers.toArray()[0]);
        else 
            txtWordList.setText("the,un,de,le,a,la,une,de,het,een,der,die,das");
        
        setResizable(false);
        pack();
        setSize(new Dimension(500,400));
        setCenteredLocation();
    }
    
    private void setEnhancers(Object enhancer) {
        if (enhancer instanceof TitleRewriter) {
            TitleRewriter rewriter = (TitleRewriter) enhancer;
            checkEnabled.setSelected(rewriter.isEnabled());
            txtWordList.setText(rewriter.getWordList());
        }
    }
    
    private void rewrite() {
        save();
        this.canceled = false;
        Rewriter rewriter = new Rewriter();
        rewriter.start();
    }

    private class Rewriter extends Thread {
        
        public Rewriter() {}
        
        public void run() {
            boolean active = false;
            for (Iterator iter = module.getFields().iterator(); iter.hasNext() && !canceled; ) {
                DcField field = (DcField) iter.next();
                
                IValueEnhancer[] enhancers = field.getValueEnhancers();
                for (int i = 0; i < enhancers.length && !canceled; i++) {
                    if (enhancers[i].isEnabled() && enhancers[i] instanceof TitleRewriter) {
                        active = true;
                        rewrite((TitleRewriter) enhancers[i], 
                                module.getField(((TitleRewriter) enhancers[i]).getField()));
                    }
                }
            }
            
            if (!active && !canceled)
                new MessageBox(DcResources.getText("msgNoTitleRewritersFound"), MessageBox._ERROR);
            
        }
        
        private void rewrite(TitleRewriter rewriter, DcField field) {
            save();
            
            buttonClose.setEnabled(false);
            buttonRun.setEnabled(false);
            buttonSave.setEnabled(false);            
            
            try {
                ResultSet rs = DatabaseManager.executeSQL(
                        "SELECT COUNT(ID) AS TOTAL FROM " + module.getTableName() +
                        " WHERE " + field.getDatabaseFieldName() + " IS NOT NULL AND " + 
                        field.getDatabaseFieldName() + " != ''", false);
                
                int total = 0;
                while (rs.next()) {
                    total = rs.getInt("TOTAL");
                }
                rs.close();                
                
                initProgressBar(total);
                rs = DatabaseManager.executeSQL(
                        "SELECT ID, " + field.getDatabaseFieldName() + " FROM " + module.getTableName() + 
                        " WHERE " + field.getDatabaseFieldName() + " IS NOT NULL AND " + 
                        field.getDatabaseFieldName() + " != ''", false);
                
                while (rs.next() &&  !canceled) {
                    String ID = rs.getString("ID");
                    String title = rs.getString(field.getDatabaseFieldName());
                    String newTitle = (String) rewriter.apply(field, title);
                    
                    if (!title.equals(newTitle)) {
                        DatabaseManager.executeQuery(
                                "UPDATE " + module.getTableName() + " SET " + field.getDatabaseFieldName() + 
                                " = '" + newTitle + "' WHERE ID = " + ID, Query._UPDATE, false);
                        
                        DcObject dco = DataManager.getObject(module.getIndex(), ID);
                        dco.setValue(field.getIndex(), newTitle);
                        dco.markAsUnchanged();
                        updateUI(module, field, ID, newTitle);
                    }
                    
                    updateProgressBar();
                }
                rs.close();
            } catch (Exception exp) {
                DcLog.add(exp);
            } finally {
                if (buttonRun != null) {
                    buttonClose.setEnabled(true);
                    buttonRun.setEnabled(true);
                    buttonSave.setEnabled(true);
                }
            }
        }
    }
    
    private void updateUI(DcModule module, DcField field, String ID, String title) {
        DcObject dco = module.getDcObject();
        dco.setValue(field.getIndex(), title);
        module.getSearchView().updateRow(ID, dco, true, false, false);     
    }    
    
    private class RewriteTitlesAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            rewrite();
        }
    }    

    private class CancelAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancel();
        }
    }    
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }

    private class SaveAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            save();
        }
    }
}

