/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.expertuser;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Iterator;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.renderers.NumberTableCellRenderer;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class SystemInfoPanel extends JPanel {
    
    public SystemInfoPanel() {
        build();
    }
    
    private void build() {
        setLayout(Layout.getGBL());

        JLabel databaseNameLabel = ComponentFactory.getLabel(DcResources.getText("lblDatabase"));
        JLabel databaseNameTextLabel = ComponentFactory.getLabel(DcSettings.getString(DcRepository.settings.stConnectionString));
        JLabel databaseDriverLabel = ComponentFactory.getLabel(DcResources.getText("lblDatabaseDriver"));
        JLabel databaseDriverTextLabel = ComponentFactory.getLabel(DcSettings.getString(DcRepository.settings.stDatabaseDriver));
        
        DcTable table = ComponentFactory.getDCTable(true, false);
        table.setEnabled(false);
        table.setColumnCount(2);
        TableColumn columnName = table.getColumnModel().getColumn(0);
        TableColumn columnCount = table.getColumnModel().getColumn(1);
        columnName.setHeaderValue(DcResources.getText("lblName"));
        columnCount.setHeaderValue(DcResources.getText("lblItems"));
        table.applyHeaders();
        
        columnCount.setCellRenderer(new NumberTableCellRenderer());
        
        int total = 0;
        for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            if (module.getIndex() != DcModules._MEDIA && module.isTopModule()) {
                int count = getRecordCount(module);
                total += count;
                table.addRow(new Object[] {module.getLabel(), new Integer(count)});
            }
        }

        table.addRow(new Object[] {DcResources.getText("lblTotal"), new Integer(total)});

        JScrollPane sp = new JScrollPane(table);
        sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        add(databaseNameLabel,       Layout.getGBC( 0, 0, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));
        add(databaseNameTextLabel,   Layout.getGBC( 1, 0, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));
        add(databaseDriverLabel,     Layout.getGBC( 0, 1, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));
        add(databaseDriverTextLabel, Layout.getGBC( 1, 1, 1, 1, 1.0, 0.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 5, 0, 0), 0, 0));        
        add(sp,                      Layout.getGBC( 0, 2, 2, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 10, 5, 5, 5), 0, 0));
    }
    
    private int getRecordCount(DcModule module) {
    	DcObject[] o = DataManager.get(module.getIndex(), null);
    	return o.length;
    }    
}
