/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.filtering;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcLongTextField;
import net.sf.dc.console.components.lists.DcFilterEntryList;
import net.sf.dc.console.components.lists.elements.DcFilterEntryListElement;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataFilterEntry;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.data.Operator;
import net.sf.dc.core.data.Operators;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;

public class DefineFilterEntryPanel extends JPanel {

    private JPanel panelInput;
    private JComponent c;
    private JButton buttonAdd;
    
    private JComboBox comboAndOr;
    private JComboBox comboFields;
    private JComboBox comboOperators;
    private JComboBox comboModules;
    
    private DcFilterEntryList list;
    
    private DcModule module;
    
    private ModuleSelectionListener moduleSelectionListener = new ModuleSelectionListener();
    private FieldSelectionListener fieldSelectionListener = new FieldSelectionListener();
    private OperatorSelectionListener operatorSelectionListener = new OperatorSelectionListener();
    
    public DefineFilterEntryPanel(DcModule module) {
        this.module = module;
        build();
        comboModules.setSelectedIndex(0);
    }
    
    public void applyEntry(DataFilterEntry entry) {
        DcModule module = DcModules.get(entry.getModule());
        DcField field = module.getField(entry.getField());
        Operator operator = entry.getOperator();
        
        comboModules.setSelectedItem(module);
        
        setFields(module);
        
        comboAndOr.setSelectedItem(entry.getAndOr());
        comboFields.setSelectedItem(field);
        comboOperators.setSelectedItem(operator);
        
        if (operator.needsValues()) {
        	Object value = entry.getValue();
        	if (	field.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION && 
        			value instanceof Collection) {
        		Collection c = (Collection) value;
        		value = c != null && c.size() == 1 ?value = c.toArray()[0] : null;
        	} 
        	
            ComponentFactory.setValue(c, value);
        }

        comboOperators.setEnabled(true);
        comboFields.setEnabled(true);
        buttonAdd.setEnabled(true);
    }
    
    public void addEntry() {
        DcField field = (DcField) comboFields.getSelectedItem();
        Operator operator = (Operator) comboOperators.getSelectedItem();
        
        Object value = ComponentFactory.getValue(c);
        
        if (field.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION) {
        	Collection c = new ArrayList();
        	c.add(value);
        	value = c;
        }
        
        if (operator.needsValues() && (value == null || value.equals(""))) {
            new MessageBox(DcResources.getText("msgEnterFilterValue"), MessageBox._INFORMATION);
            return;
        }
        
        DataFilterEntry entry = 
            new DataFilterEntry((String) comboAndOr.getSelectedItem(),
                                field.getModule(), field.getIndex(), operator, 
                                operator.needsValues() ? value : null);
        
        list.add(entry);
        setFields(module);

        if (comboFields.getItemCount() > 0) {
            comboFields.setSelectedIndex(0);
        } else {
            comboOperators.removeAllItems();
            buttonAdd.setEnabled(false);
        } 
        
        list.clearSelection();
        setAndOr();
    }
    
    public void clear() {
        list.clear();
        setFields(module);
        setAndOr();
    }
    
    private void setAndOr() {
        if (list.getModel().getSize() == 0) {
            comboAndOr.removeItem(DcResources.getText("lblOr"));
            comboAndOr.setEnabled(false);
        } else {
            comboAndOr.removeAllItems();
            comboAndOr.addItem(DcResources.getText("lblAnd"));
            comboAndOr.addItem(DcResources.getText("lblOr"));
            comboAndOr.setEnabled(true);
            comboAndOr.setSelectedIndex(0);
        }
    }
    
    private void setFields(DcModule module) {
        comboFields.removeActionListener(fieldSelectionListener);
        comboFields.removeAllItems();
        
        for (Iterator iter = module.getFields().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            if (field.isSearchable())
                comboFields.addItem(field);
        }
        
        comboFields.addActionListener(fieldSelectionListener);
        
        if (comboFields.getItemCount() > 0)
            comboFields.setSelectedIndex(0);
    }      
    
    public void applyField(DcField field) {
        if (field == null) return;
        if (c != null) panelInput.remove(c);
        
        Dimension size = c != null ? c.getSize() : null; 
        
        if (field.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION) {
            DcObject[] objects = DataManager.get(field.getReferenceIdx(), null);
            c = ComponentFactory.getComboBox();
            ((JComboBox) c).addItem(" ");
            for (int i = 0; objects != null && i < objects.length; i++)
            	((JComboBox) c).addItem(objects[i]);
            
            ((JComboBox) c).setEditable(false);
        } else if (field.getFieldType() == ComponentFactory._FILEFIELD || 
        		   field.getFieldType() == ComponentFactory._FILELAUNCHFIELD) {
        	c = ComponentFactory.getShortTextField(255);
        } else {
        	c = ComponentFactory.getComponent(field.getModule(), field.getReferenceIdx(), field.getFieldType(), field.getLabel(), field.getMaximumLength());	
        }
        
        c = c instanceof DcLongTextField ? ComponentFactory.getShortTextField(field.getMaximumLength()) : c;
        
        int height = comboAndOr != null && comboAndOr.getHeight() > 0 ? comboAndOr.getHeight() : 23;
        c.setMinimumSize(new Dimension(100, height));
        c.setPreferredSize(new Dimension(100, height));
        if (size != null && size.getWidth() > 0 && size.getHeight() > 0) {
            size.setSize(size.getWidth(), height);
            c.setPreferredSize(size);
        }
        
        panelInput.add(c, Layout.getGBC( 4, 1, 1, 1, 50.0, 50.0
                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                       new Insets( 0,0,0,0), 0, 0));

        revalidate();
    }
    
    public Collection getEntries() {
        return list.getEntries();
    }
    
    private void setOperators(DcField field) {
        comboOperators.removeActionListener(operatorSelectionListener);
        comboOperators.removeAllItems();
        
        if (field == null)
            return;
        
        Collection operators = Operators.get(field);
        
        for (Iterator iter = operators.iterator(); iter.hasNext(); )
            comboOperators.addItem(iter.next());
        
        comboOperators.addActionListener(operatorSelectionListener);
    }    
        
    
    private void build() {
        setLayout(Layout.getGBL());
        
        // create the comboboxes
        comboFields = ComponentFactory.getComboBox(new DefaultComboBoxModel());
        comboOperators = ComponentFactory.getComboBox(new DefaultComboBoxModel());
        comboModules = ComponentFactory.getComboBox(new DefaultComboBoxModel());
        comboAndOr = ComponentFactory.getComboBox(new DefaultComboBoxModel());
        
        comboAndOr.addItem(DcResources.getText("lblAnd"));
        comboAndOr.setEnabled(false);
        
        comboModules.addItem(module);
        if (module.getChild() != null)
            comboModules.addItem(module.getChild());

        comboFields.addActionListener(fieldSelectionListener);
        comboModules.addActionListener(moduleSelectionListener);
        comboOperators.addActionListener(operatorSelectionListener);
        
        // create the list
        list = new DcFilterEntryList();
        list.addMouseListener(new ListMouseListener());

        JScrollPane scroller = new JScrollPane(list);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);

        panelInput = new JPanel();
        panelInput.setLayout(Layout.getGBL());
        
        // create the buttons
        buttonAdd = ComponentFactory.getButton(DcResources.getText("lblAdd"));
        buttonAdd.addActionListener(new AddEntryAction());
        
        panelInput.add(ComponentFactory.getLabel(DcResources.getText("lblModule")),  
                 Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 5), 0, 0));
        panelInput.add(ComponentFactory.getLabel(DcResources.getText("lblField")),  
                 Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 5), 0, 0));
        panelInput.add(ComponentFactory.getLabel(DcResources.getText("lblOperator")),
                 Layout.getGBC( 3, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 5), 0, 0));
        panelInput.add(ComponentFactory.getLabel(DcResources.getText("lblValue")),
                 Layout.getGBC( 4, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 5), 0, 0));

        panelInput.add(comboAndOr,      Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 5), 0, 0));
        panelInput.add(comboModules,    Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 5), 0, 0));
        panelInput.add(comboFields,     Layout.getGBC( 2, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 5), 0, 0));
        panelInput.add(comboOperators,  Layout.getGBC( 3, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 5), 0, 0));
        
        JPanel panelAction = new JPanel();
        panelAction.add(buttonAdd);

        add(scroller,     Layout.getGBC( 0, 0, 2, 1, 40.0, 40.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 5, 5, 5, 5), 0, 0));
        add(panelInput,   Layout.getGBC( 0, 1, 1, 1, 40.0, 1.0
                ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 5, 5, 5, 0), 0, 0));
        add(panelAction,  Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));
        

    }
    
    private class AddEntryAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            addEntry();
        }
    }
    
    private class ModuleSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            setFields((DcModule) comboModules.getSelectedItem());
        }
    }    
    
    private class FieldSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            DcField field = (DcField) comboFields.getSelectedItem();
            setOperators(field);
            applyField(field);
        }
    } 

    private class OperatorSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            Operator operator = (Operator) comboOperators.getSelectedItem();
            if (c != null && operator != null) {
                c.setEnabled(operator.needsValues());
                Color color = operator.needsValues() ? Color.WHITE : ComponentFactory.getDisabledColor();
                c.setBackground(color);
            }
        }
    } 

    
    private class ListMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2 && list.getSelectedIndex() > -1) {
                DcFilterEntryListElement elem = (DcFilterEntryListElement) list.getSelectedValue();
                DataFilterEntry entry = elem.getEntry();
                list.remove();
                setAndOr();
                applyEntry(entry);
                e.consume();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }       
}
