/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.filtering;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.components.panels.SortingPanel;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.View;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.windows.messageboxes.QuestionBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataFilterEntry;
import net.sf.dc.core.data.DataFilters;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.resources.DcResources;

public class FilterDialog extends DcDialog {

    private final DcModule module;
    
    private final MasterView parent;
    
    private JTabbedPane filterTabs = ComponentFactory.getTabbedPane();
    
    private DefineFilterEntryPanel defineFilterPanel;
    private SortingPanel sortingPanel;
    private ManageFiltersPanel manageFiltersPanel;
    
    public FilterDialog(DcModule module, MasterView parent) {
        super(DataCrow.mainFrame);
        
        setTitle(DcResources.getText("lblFilter"));
        setHelpIndex("dc.filters");
        
        this.parent = parent;
        this.module = module;
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });        
        
        buildForm(); 
        
        setSize(module.getSettings().getDimension(DcRepository.modulesettings.stFilterDialogSize));
        setCenteredLocation();
        
        setModal(true);
    }

    public void filter() {
        FilterThread thread = new FilterThread();
        thread.start();
    }   
    
    public DataFilter getDataFilter() {
        DataFilter filter = new DataFilter(module.getIndex());
        filter.setEntries(defineFilterPanel.getEntries());
        filter.setOrder(sortingPanel.getOrder());
        return filter;
    }

    private void buildForm() {
        getContentPane().setLayout(Layout.getGBL());
        
        defineFilterPanel = new DefineFilterEntryPanel(module);
        sortingPanel = new SortingPanel(module);
        manageFiltersPanel = new ManageFiltersPanel(); 
        
        filterTabs.addTab(DcResources.getText("lblFilter"), IconLibrary._icoSearch, defineFilterPanel);
        filterTabs.addTab(DcResources.getText("lblSort"), IconLibrary._icoSort, sortingPanel);
        filterTabs.addTab(DcResources.getText("lblManage"), IconLibrary._icoFilter, manageFiltersPanel);
        
        getContentPane().add(filterTabs,       Layout.getGBC( 0, 0, 1, 1, 100.0, 100.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.BOTH,
                 new Insets( 5, 5, 5, 2), 0, 0));

        getContentPane().add(getActionPanel(), Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets( 5, 5, 5, 5), 0, 0));

        pack();     
    }
    
    private void applyFilter(DataFilter filter) {
        defineFilterPanel.clear();
        for (Iterator iter = filter.getEntries().iterator(); iter.hasNext(); ) {
            DataFilterEntry entry = (DataFilterEntry) iter.next();
            defineFilterPanel.applyEntry(entry);
            defineFilterPanel.addEntry();
        }
        sortingPanel.applyOrderSettings(filter.getOrder());
    }
    
    private class ManageFiltersPanel extends JPanel {
        
        private JComboBox comboFilters;
        private DcShortTextField textName;
        
        public ManageFiltersPanel() {
            setLayout(Layout.getGBL());
            
            // ********* save ********
            JPanel panelSave = new JPanel();
            panelSave.setLayout(Layout.getGBL());
            
            textName = ComponentFactory.getShortTextField(255);
            JButton buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
            
            buttonSave.addActionListener(new SaveFilterAction());

            JLabel labelName = ComponentFactory.getLabel(DcResources.getText("lblName"));
            Dimension labelSize = new Dimension(150, 23);
            labelName.setPreferredSize(labelSize);
            labelName.setMinimumSize(labelSize);
            labelName.setMinimumSize(labelSize);
            
            panelSave.add(labelName,    Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                      new Insets( 5, 5, 5, 5), 0, 0));
            panelSave.add(textName,     Layout.getGBC( 1, 0, 1, 1, 30.0, 30.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets( 5, 5, 5, 5), 0, 0));
            panelSave.add(buttonSave,   Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                      new Insets( 5, 5, 5, 5), 0, 0));
            panelSave.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblSaveNewFilter")));
            
            // ********* manage ********
            JPanel panelManage = new JPanel();
            panelManage.setLayout(Layout.getGBL());
            
            comboFilters = ComponentFactory.getComboBox();
            JButton buttonApply = ComponentFactory.getButton(DcResources.getText("lblApply"));
            JButton buttonEdit = ComponentFactory.getButton(DcResources.getText("lblEdit"));
            JButton buttonDelete = ComponentFactory.getButton(DcResources.getText("lblDelete"));
            
            Collection filters = DataFilters.get(module.getIndex());
            for (Iterator iter = filters.iterator(); iter.hasNext(); )
                comboFilters.addItem(iter.next());
            
            buttonApply.addActionListener(new ApplyFilterAction());
            buttonEdit.addActionListener(new EditAction());
            buttonDelete.addActionListener(new DeleteAction());

            JLabel labelFilters = ComponentFactory.getLabel(DcResources.getText("lblFilters"));
            labelFilters.setPreferredSize(labelSize);
            labelFilters.setMinimumSize(labelSize);
            labelFilters.setMinimumSize(labelSize);
            
            panelManage.add(labelFilters,   Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                      new Insets( 5, 5, 5, 5), 0, 0));
            panelManage.add(comboFilters,   Layout.getGBC( 1, 0, 1, 1, 30.0, 30.0
                     ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                      new Insets( 5, 5, 5, 5), 0, 0));
            panelManage.add(buttonApply,    Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                     new Insets( 5, 5, 5, 5), 0, 0));
            panelManage.add(buttonEdit,     Layout.getGBC( 2, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                     new Insets( 5, 5, 5, 5), 0, 0));
            panelManage.add(buttonDelete,   Layout.getGBC( 2, 2, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                     new Insets( 5, 5, 5, 5), 0, 0));
            
            panelManage.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblManageExistingFilters")));
            
            // ********* main ********
            add(panelSave,   Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets( 5, 5, 5, 5), 0, 0));
            add(panelManage, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets( 5, 5, 5, 5), 0, 0));
        }

        private void setFilters() {
            comboFilters.removeAllItems();
            comboFilters.getEditor().setItem(null);
            comboFilters.repaint();
            
            Collection filters = DataFilters.get(module.getIndex());
            
            if (filters.size() == 0)
                comboFilters.addItem("");
            
            for (Iterator iter = filters.iterator(); iter.hasNext(); )
                comboFilters.addItem(iter.next());            
        }
        
        private void save() {
            DataFilter df = getDataFilter();
            if (textName.getText() == null || textName.getText().trim().length() == 0) {
                new MessageBox(DcResources.getText("msgEnterFilterName"), MessageBox._INFORMATION);
            } else {
                df.setName(textName.getText());
                DataFilters.add(df);
                setFilters();
            }
        }

        private class DeleteAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                DataFilter filter = (DataFilter) comboFilters.getSelectedItem();
                
                if (filter != null)
                    DataFilters.delete(filter);
                
                setFilters();
            }
        } 
        
        private DataFilter getSelected() {
            return comboFilters.getSelectedItem() instanceof DataFilter ? 
                    (DataFilter) comboFilters.getSelectedItem() : null;
        }
        
        private class EditAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                DataFilter filter = getSelected();
                if (filter != null) {
                    applyFilter(filter);
                    filterTabs.setSelectedIndex(0);
                    textName.setText(filter.getName());
                }
            }
        }         
        
        private class ApplyFilterAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                DataFilter filter = getSelected();
                if (filter != null) {
                    applyFilter(filter);
                    textName.setText(filter.getName());
                    filter();
                }
            }
        } 
        
        private class SaveFilterAction implements ActionListener {
            public void actionPerformed(ActionEvent e) {
                save();
            }
        } 
    }    
    
    private JPanel getActionPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        
        JButton buttonApply = ComponentFactory.getButton(DcResources.getText("lblApply"));
        JButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));
        
        buttonApply.setToolTipText(DcResources.getText("tpSearch"));
        buttonApply.addActionListener(new SearchButtonAction());
        buttonClose.addActionListener(new CloseButtonAction());
        buttonClose.setMnemonic('C');
        buttonApply.setMnemonic('A');

        panel.add(buttonApply);
        panel.add(buttonClose);
        
        buttonApply.requestFocus();
        
        return panel; 
    }
    
    public void close() {
        module.getSettings().set(DcRepository.modulesettings.stFilterDialogSize, getSize());
        DataCrow.mainFrame.updateQuickFilterBar();
        setVisible(false);
    }
    
    private class FilterThread extends Thread {

        public void run() {
            View view = parent.getCurrent();
            boolean saved = view.isChangesSaved();
            
            if (!saved && view.getCurrentTask() != null) {
                QuestionBox qb = new QuestionBox(DcResources.getText("msgNotSaved"));
                if (qb.isAffirmative())
                    view.save();
                
                try {
                    view.getCurrentTask().join();
                } catch (Exception exp) {
                    DcLog.add(exp);
                }
            }
            
            view.markAsUnchanged();

            DataFilter df = getDataFilter(); 
            DataFilters.setCurrent(module.getIndex(), df);
            DataManager.bindData(parent, module.getIndex(), df);

            parent.setStatus(DcResources.getText("msgSearchHasBeenExecuted"));

            DataCrow.mainFrame.setSelectedTab(0);        
            close();
        }
    }    
    
    private class CloseButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    } 
    
    private class SearchButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            filter();
        }
    }
}