/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.itemforms;

import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.windows.messageboxes.QuestionBox;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.ValidationException;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.core.wf.requests.CloseWindowRequest;
import net.sf.dc.core.wf.requests.RefreshChildView;

public class ChildItemForm extends ItemForm {

    private ChildForm childForm;
    
    public ChildItemForm(boolean update, DcObject o, ChildForm childForm) {
        super(false, update, o, true);
        this.childForm = childForm;
    }
    
    protected void saveValues() {
        DcObject clone = getObjectForSaving();
        clone.addRequest(new CloseWindowRequest(this));
        
        if (!update)  {
            try {
                clone.addRequest(new RefreshChildView(childForm, childForm.getParentModuleIdx(), clone, clone.getID(), (String) clone.getValue(clone.getParentReferenceFieldIndex()), true));
                clone.setSilence(true);
                clone.saveNew(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
            }
        } else if (isChanged()) {
            try {
                clone.addRequest(new RefreshChildView(childForm, childForm.getParentModuleIdx(), clone, clone.getID(), (String) clone.getValue(clone.getParentReferenceFieldIndex()), false));
                clone.setSilence(true);
                clone.saveUpdate(true);
            } catch (ValidationException vExp) {
                new MessageBox(vExp.getMessage(), MessageBox._WARNING);
            }
        } else if (! isChanged()) {
            new MessageBox(DcResources.getText("msgNoChangesToSave"), MessageBox._WARNING);
        }
    }  
    
    public void close(boolean aftersave) { 
        super.close(aftersave);
        childForm = null;
    }
    
    /**
     * Deletes this item from the database
     */
    protected void deleteValues() {
        QuestionBox qb = new QuestionBox(DcResources.getText("msgDeleteQuestion"), this);
        
        if (qb.isAffirmative()) {
            String id = dco.getID();
            dco.clearValues();
            dco.setValue(DcObject._ID, id);
            dco.setSilence(true);

            dco.addRequest(new CloseWindowRequest(this));
            dco.addRequest(new RefreshChildView(childForm,  childForm.getParentModuleIdx(), null, id, (String) dco.getValue(dco.getParentReferenceFieldIndex()), false));
            dco.delete();
        }
    }
}
