/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.itemforms;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.DcPanel;
import net.sf.dc.console.components.lists.DcObjectList;
import net.sf.dc.console.menu.DcPropertyViewPopupMenu;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.windows.messageboxes.QuestionBox;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.core.wf.requests.IRequest;
import net.sf.dc.core.wf.requests.RefreshPropertyItemViewRequest;
import net.sf.dc.core.wf.requests.RequestCollection;
import net.sf.dc.core.wf.requests.StatusUpdateRequest;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.Settings;
import net.sf.dc.util.DataTask;
import net.sf.dc.util.SwingUtil;

public class DcMinimalisticItemView extends DcFrame {

    protected DataTask task;
    
    private final boolean readonly;
    
    protected ListMouseListener mouseListener = new ListMouseListener();
    
    private JScrollPane scroller;
    
    private JPanel panelActions = new JPanel();
    private JPanel statusPanel;
    private JButton buttonNew = ComponentFactory.getButton(DcResources.getText("lblNew"));
    private JButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));
    
    protected DcObjectList list;
    
    private int module;
    private DcPanel panel = new DcPanel("", null);
    
    public DcMinimalisticItemView(int module, boolean readonly) {
        super(DcModules.get(module).getObjectNamePlural(), IconLibrary._icoAnchor);
        
        this.list = new DcObjectList(DcObjectList._LISTING, false, true);
        this.readonly = readonly;
        
        SwingUtil.setRootFrame(this);
        this.module = module;
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });        
        
        buildPanel();

        pack();
        
        Settings settings = DcModules.get(module).getSettings();
        setSize(settings.getDimension(DcRepository.modulesettings.stSimpleItemViewSize));
        setCenteredLocation();
    }
    
    public void hideDialogActions(boolean b) {
        buttonClose.setVisible(!b);
        statusPanel.setVisible(!b);
    }
    
    public void close() {
        setVisible(false);
    }
    
    public void open() {
        DcObject dco = list.getSelectedObject();
        if (dco != null) {
            dco.markAsUnchanged();
            DcMinimalisticItemForm itemForm = new DcMinimalisticItemForm(true, dco, this);
            itemForm.setVisible(true);
        }
    }
    
    public void createNew() {
        DcMinimalisticItemForm itemForm = new DcMinimalisticItemForm(false, getModule().getDcObject(), this);
        itemForm.setVisible(true);
    }
    
    public RequestCollection getAfterDeleteRequests() {
        RequestCollection requests = new RequestCollection();
        requests.addRequest(new RefreshPropertyItemViewRequest(this));
        return requests;
    }    
    
    public void setObjects(Collection objects) {
        list.clear();
        list.add(objects);
    }
    
    public Collection getObjects() {
        return list.getObjects();
    }
    
    protected JPopupMenu getPopupMenu() {
        return new DcPropertyViewPopupMenu(this);
    }
    
    protected int getModuleIdx() {
        return module;
    }
    
    protected DcModule getModule() {
        return DcModules.get(module);
    }
    
    public void setVisible(boolean b) {
        super.setVisible(b);
        if (b)
            addObjects();
    }    
    
    public void addObjects() {
        list.clear();
        DcObject dco = getModule().getDcObject();
        DataFilter filter = new DataFilter(dco);
        filter.setOrder(new DcField[] {dco.getField(getModule().getDefaultSortFieldIdx())});
        list.add(DataManager.get(getModule().getIndex(), filter));
    }    
    
    public void setBackground(Color color) {
        super.setBackground(color);
        getContentPane().setBackground(color);
        Component[] components = getComponents();
        for (int i = 0; i < components.length; i++) {
            JComponent component = (JComponent) components[i];
            setBackground(component, color);
        }
    }

    private void setBackground(JComponent component, Color color) {
        if (component instanceof JLabel || component instanceof JPanel ||
            component instanceof JCheckBox || component instanceof JTabbedPane ||
            component instanceof JRootPane || component instanceof JLayeredPane) {

            component.setBackground(color);

            Component[] components = component.getComponents();
            for (int i = 0; i < components.length; i++) {
                JComponent subComponent = (JComponent) components[i];
                setBackground(subComponent, color);
            }
        }
    }

    public void setFont(Font font) {
        Font fontNormal = DcSettings.getFont(DcRepository.settings.stSystemFontNormal);
        Font fontSystem = DcSettings.getFont(DcRepository.settings.stSystemFontBold);

        super.setFont(fontNormal);
        if (panel != null) {
            panel.setFont(fontSystem);
            buttonClose.setFont(fontSystem);
            buttonNew.setFont(fontSystem);
            list.setFont(fontNormal);
        }
    }      
    
    public void denyActions() {
        list.removeMouseListener(mouseListener);    
    }

    public void allowActions() {
        list.addMouseListener(mouseListener);
    }    
    
    /**
     * Indicates whether there is a data task running at this moment
     */
    protected boolean isTaskRunning() {
        boolean isTaskRunning = task != null && task.isRunning();
        if (isTaskRunning)
            new MessageBox(DcResources.getText("msgJobRunning"), MessageBox._WARNING); 
        return isTaskRunning; 
    }     

    private void buildPanel() {
        getContentPane().setLayout(Layout.getGBL());
        
        scroller = new JScrollPane(list);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);        
        
        list.addMouseListener(mouseListener);
        
        //**********************************************************
        //Result panel
        //**********************************************************
        panel.setLayout(Layout.getGBL());
        panel.add(scroller,  Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                 new Insets(5, 5, 5, 5), 0, 0));
        
        //**********************************************************
        //Action panel
        //**********************************************************
        panelActions.setLayout(new FlowLayout(FlowLayout.LEFT));
        buttonNew.addActionListener(new NewAction());
        buttonClose.addActionListener(new CloseAction());
        
        buttonNew.setMnemonic('N');
        buttonClose.setMnemonic('C');
        
        panelActions.add(buttonNew);
        panelActions.add(buttonClose);
        
        //**********************************************************
        //Main panel
        //**********************************************************
        statusPanel = panel.getStatusPanel();
        
        getContentPane().add(panel,         Layout.getGBC( 0, 0, 1, 1, 10.0, 10.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets(0, 0, 0, 0), 0, 0));
        
        if (!readonly) {
            getContentPane().add(panelActions,  Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                     new Insets(0, 0, 0, 0), 0, 0));
            getContentPane().add(statusPanel,  Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                     new Insets(0, 0, 0, 0), 0, 0));
        }
        
    }
    
    
    public void initProgressBar(int maxValue) {
        panel.initProgressBar(maxValue);
    }
    
    public void updateProgressBar(int value) {
        panel.updateProgressBar(value);
    }
    
    public void setStatus(String text) {
        panel.setStatus(text);
    }    
    
    public void delete() {
        if (!isTaskRunning()) {
            Collection objects = list.getSelectedObjects();
            if (objects.size() > 0) {
                task = new DeletingThread(objects);
                task.start();
            } else {
                new MessageBox(DcResources.getText("msgSelectItemToDel"), MessageBox._WARNING);    
            }
        }
    }     
    
    protected class DeletingThread extends DataTask {

        public DeletingThread(Collection objects) {
            super(objects);
        }
        
        public void run() {
            try {
                updateProgressBar(0);
                initProgressBar(objects.size());   
                list.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                
                setStatus("Deleting " + objects.size() + " items from to the database");
                startRunning();
                denyActions();
                
                QuestionBox qb = new QuestionBox(DcResources.getText("msgDeleteQuestion")); 
                if (!qb.isAffirmative()) {
                    stopRunning();
                } else {
                    int counter = 1;
                    for (Iterator iter = objects.iterator(); iter.hasNext(); counter++) {
                        DcObject dco = (DcObject) iter.next();                  
                        updateProgressBar(counter);
                        
                        dco.markAsUnchanged();
                        dco.setSilence(true);
                        dco.delete(); 
                        
                        if (counter == objects.size()) {
                            RequestCollection requests = getAfterDeleteRequests();
                            for (int j = 0; j < requests.getRequests().length; j++) {
                                dco.addRequest(requests.getRequests()[j]);
                            }
                            
                            dco.setEndOfBatch(true);
                            IRequest request = 
                                new StatusUpdateRequest(dco.getModule().getIndex(), MainFrame._SEARCHTAB, "msgDeleteSuccessfull");
                            request.setExecuteOnFail(true);
                            dco.addRequest(request);
                        } else {
                            dco.setEndOfBatch(false);
                        }
                        
                        try {
                            sleep(300);
                        } catch (Exception ignore) {} // doesn't want to sleep..
                    }
                    
                    list.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                }
            } finally {
                list.setSelectedIndex(0);
                stopRunning();
                allowActions();
            }
        }
    }     
    
    /**
     * Listens on right clicks: shows the popup menu 
     * 
     * Listens on double clicks:
     * opens the item form for the row on which is double clicked
     */
    private class ListMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (!readonly) {
                if (SwingUtilities.isRightMouseButton(e)) {
                    if (list.getSelectedIndex() == -1) {
                        int index = list.locationToIndex(e.getPoint());
                        list.setSelectedIndex(index);
                    }
                    
                    if (list.getSelectedIndex() > -1) {
                        JPopupMenu menu = getPopupMenu();                
                        menu.setInvoker(list);
                        menu.show(list, e.getX(), e.getY());
                    }
                }
    
                if (e.getClickCount() == 2 && list.getSelectedIndex() > -1) 
                    open();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }

    private class NewAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            createNew();
        }
    }
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }    
}