/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.messageboxes;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.util.SwingUtil;

public class MessageBox extends DcDialog {

    private JTextArea textMessage;
    private JButton buttonOk;
    private JPanel panel = new JPanel();
    private JLabel labelIcon = ComponentFactory.getLabel("");

    public  static final int _ERROR = 1;
    public  static final int _WARNING = 2;
    public  static final int _INFORMATION = 3;

    public MessageBox(JFrame frame, String message, int type) {
        super(frame);
        init();
        setModal(true);
        display(message, type);
    }

    public MessageBox(String message, int type, boolean modal) {
        super(SwingUtil.getRootFrame());
        init();
        setModal(modal);
        display(message, type);
    }

    public MessageBox(String message, int type) {
        super(SwingUtil.getRootFrame());
        init();
        setModal(true);
        display(message, type);
    }

    public void close() {
        textMessage = null;
        buttonOk = null;
        panel = null;
        labelIcon = null;
        super.close();
    }

    private void setIcon(ImageIcon icon) {
        labelIcon.setIcon(icon);
    }

    private void display(String message, int type) {
        textMessage.setText(message);

        if (type == _ERROR) {
            setIcon(IconLibrary._icoError);
        } else if (type == _WARNING) {
            setIcon(IconLibrary._icoWarning);
        } else {
            setIcon(IconLibrary._icoAbout);
        }

        this.pack();
        this.toFront();

        try {
            Dimension size = DataCrow.mainFrame.getSize();
            Point location = DataCrow.mainFrame.getLocation();
            int x = ((location.x + size.width - getSize().width) / 2);
            int y = ((location.y + size.height  - getSize().height) / 2);
            setLocation(x, y);
        // ignore the errors as the messagebox should always be displayed!
        } catch (Exception ignore) {
            setLocation(150, 150);
        }

        buttonOk.requestFocus();
        this.setVisible(true);
    }

    private void init() {
        this.getContentPane().add(panel);

        this.setResizable(false);
        this.getContentPane().setLayout(Layout.getGBL());

        textMessage = ComponentFactory.getTextArea();
        textMessage.setEditable(false);
        textMessage.setBackground(panel.getBackground());
        JScrollPane scrollIn = new JScrollPane(textMessage);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollIn.setPreferredSize(new Dimension(350,50));
        scrollIn.setBorder(null);
        buttonOk = ComponentFactory.getButton(DcResources.getText("lblOK"));
        buttonOk.addActionListener(new MessageButtonAction());
        buttonOk.setMnemonic('O');

        panel.setLayout(Layout.getGBL());

        panel.add(labelIcon,  Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0));
        panel.add(scrollIn,   Layout.getGBC( 1, 0, 6, 3, 90.0, 90.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                              new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonOk, Layout.getGBC(0, 3, 2, 1, 0.0, 0.0
                              ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                               new Insets(5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panel);
    }

    private class MessageButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
}
