/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.messageboxes;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.util.SwingUtil;

public class QuestionBox extends JDialog {

    private JTextArea textMessage;
    private JButton buttonYes;
    private JButton buttonNo;
    private JPanel panel = new JPanel();

    private boolean affirmative = false;
    
    private final JLabel labelIcon = ComponentFactory.getLabel("");

    public QuestionBox(String message) {
        super(SwingUtil.getRootFrame());
        init(message);
    }

    public QuestionBox(String message, JFrame parent) {
        super(parent);
        init(message);
    }

    public void close() {
        this.dispose();
    }

    private void init(String message) {
        buildDialog();
        textMessage.setText(message);

        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

        this.pack();
        this.setModal(true);

        if (DataCrow.mainFrame != null) {
            Dimension size = DataCrow.mainFrame.getSize();
            Point location = DataCrow.mainFrame.getLocation();
            int x = ((location.x + size.width - getSize().width) / 2);
            int y = ((location.y + size.height  - getSize().height) / 2);
            setLocation(x, y);
        } else {
        	setLocation(150, 150);
        }

        buttonYes.requestFocus();
        this.setVisible(true);
    }

    private void buildDialog() {
        this.setResizable(false);
        this.getContentPane().setLayout(new GridLayout());

        // Input panel
        panel.setLayout(Layout.getGBL());

        textMessage = ComponentFactory.getTextArea();
        textMessage.setEditable(false);
        textMessage.setBackground(panel.getBackground());
        
        JScrollPane scrollIn = new JScrollPane(textMessage);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollIn.setPreferredSize(new Dimension(350,50));
        scrollIn.setBorder(null);

        buttonYes = ComponentFactory.getButton(DcResources.getText("lblYes"));
        buttonNo = ComponentFactory.getButton(DcResources.getText("lblNo"));

        buttonYes.addActionListener(new YesButtonAction());
        buttonNo.addActionListener(new NoButtonAction());
        
        buttonYes.setMnemonic('Y');
        buttonNo.setMnemonic('N');

        labelIcon.setIcon(IconLibrary._icoQuestion);

        panel.setLayout(Layout.getGBL());
        panel.add(labelIcon,  Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0));
        panel.add(scrollIn,   Layout.getGBC( 1, 0, 6, 3, 90.0, 90.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                              new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonYes, Layout.getGBC(  0, 3, 2, 1, 0.0, 0.0
                             ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonNo,  Layout.getGBC(  2, 3, 1, 1, 0.0, 0.0
                             ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0));

        this.pack();
        this.setModal(true);
        this.getContentPane().add(panel);
    }

    private class YesButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            affirmative = true;
            close();
        }
    }

    private class NoButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            affirmative = false;
            close();
        }
    }

    public boolean isAffirmative() {
        return affirmative;
    }
}