/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.onlinesearch;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.DcFrame;
import net.sf.dc.console.components.lists.DcObjectList;
import net.sf.dc.console.components.panels.OnlineServiceSettingsPanel;
import net.sf.dc.console.windows.help.HelpDialog;
import net.sf.dc.console.windows.itemforms.ItemForm;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearch;
import net.sf.dc.onlinesearch.OnlineSearchTask;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.settings.Settings;
import net.sf.dc.util.StringUtils;

public class OnlineSearchForm extends DcFrame implements IOnlineSearchClient, ActionListener {

    private int module;
    private String ID;

    protected OnlineSearchTask task;
    private boolean startSearchOnOpen = false;
    
    private ItemForm itemForm;
    private DcObjectList list;
    private ArrayList items = new ArrayList();

    private OnlineSearch onlineSearch;
    
    private OnlineServiceSettingsPanel panelSettings;
    private OnlineServicePanel panelService;
    private JTextArea textLog = ComponentFactory.getTextArea();
    private JProgressBar progressBar = new JProgressBar();

    private MouseListener mouseListener = new TableMouseListener();

    public OnlineSearchForm(OnlineSearch onlineSearch, DcObject dco, ItemForm itemForm, boolean advanced) {
        super(DcResources.getText("lblOnlineXSearch", DcModules.getCurrent().getObjectName()),
                                  IconLibrary._icoSearchOnline);

        startSearchOnOpen = dco != null;

        this.ID = dco != null ? dco.getID() : null;
        this.itemForm = itemForm;
        this.module = DcSettings.getInt(DcRepository.settings.stModule);
        this.onlineSearch = onlineSearch;
        
        buildDialog(advanced);

        HelpDialog.setHelpIndex("dc.onlinesearch");
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close(true);
            }
        });
        
        if (panelService.getQuery() == null || panelService.getQuery().trim().length() == 0)
            panelService.setQuery(dco != null ? dco.getName() : "");

        setSize(getModule().getSettings().getDimension(DcRepository.modulesettings.stOnlineSearchFormSize));
        setCenteredLocation();
        stopped();

        if (startSearchOnOpen && panelService.getQuery().length() > 0)
            start();
    }
    
    public DcModule getModule() {
        return DcModules.get(module);
    }

    public void addObject(DcObject dco) {
        if (task != null && !task.isCancelled()) {
            dco.applyTemplate();
                
            if (ID == null)
                removeValues(dco);
            
            dco.setValue(DcObject._ID, ID);
            list.add(dco);
            items.add(dco);
            checkPerfectMatch(dco);
        }
    }
    
    private void removeValues(DcObject dco) {
        int[] fields = getModule().getSettings().getIntArray(DcRepository.modulesettings.stOnlineSearchRetrievedFields);

        for (Iterator iter = dco.getFields().iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();

            boolean allowed = false;
            for (int i = 0; fields != null && i < fields.length; i++) {
                if (field.getIndex() == fields[i])
                    allowed = true;
            }
            
            if (!allowed)
                dco.setValueLowLevel(field.getIndex(), null);
        }
    }

    public DcObject getDcObject() {
        return ID != null ? DataManager.getObject(module, ID) : null;
    }    
    
    public DcObject getSelectedObject() {
        int row = list.getSelectedIndex();

        DcObject dco = null;
        
        if (row > -1) {
            dco = (DcObject) items.get(row);
            dco.setValue(DcObject._ID, ID);
        }

        return dco;
    }

    public Collection getSelectedObjects() {
        int row = list.getSelectedIndex();

        if (row < 0) {
            new MessageBox(DcResources.getText("msgSelectRowForTransfer"), MessageBox._WARNING);
            return null;
        }

        ArrayList result = new ArrayList();
        if (ID == null) {
            int[] rows = list.getSelectedIndices();
            for (int i = 0; i < rows.length; i++) {
                result.add(items.get(rows[i]));
            }
        } else {
            DcObject o = (DcObject) items.get(row);
        	o.setValue(DcObject._ID, ID);
            result.add(o);
        }
        return result;
    }

    public Collection getServers() {
        return onlineSearch.getServers();
    }    
    
    public int getResultCount() {
        return list.getModel().getSize();
    }

    private void open() {
        int selectedRow = list.getSelectedIndex();
        if (selectedRow == -1) {
            new MessageBox(DcResources.getText("msgSelectRowToOpen"), MessageBox._WARNING);
            return;
        }

        DcObject o = getSelectedObject();
        if (o != null) {
        	ItemForm itemForm = new ItemForm(true, false, o, true);
            itemForm.setVisible(true);
        }
    }
    
    private void checkPerfectMatch(DcObject dco) {
        if (!panelSettings.isAutoAddAllowed())
            return;
        
        DcSearchMode mode = panelService.getMode();
        if (mode != null && mode.singleIsPerfect()) {
            panelService.hasPerfectMatchOccured(true);
        } else {
            String string = panelService.getQuery().toLowerCase();
            String item = StringUtils.normalize(dco.toString()).toLowerCase();
            panelService.hasPerfectMatchOccured(string.equals(item));
        }
        
        if (panelService.hasPerfectMatchOccured()) {
            list.setSelected(dco, DcMediaObject._A_TITLE);
            if (ID != null) { 
                update(true);
                close(true);
            } else { 
                addNew(false); 
                stop();
                list.clear();
                toFront();
            }
        }            
    }
    
    protected void saveSettings() {
        getModule().setSetting(DcRepository.modulesettings.stOnlineSearchFormSize, getSize());
        panelService.save();
        panelSettings.save();
    }

    public void update(boolean closeForm) {
        DcObject o = getSelectedObject();

        if (o == null) return;
            
        if (itemForm.isVisible()) {
            DcModule mod = DcModules.get(module);
            DcObject dco = mod.getDcObject();

            Settings settings = getModule().getSettings();
            
            int[] fields = 
                settings.getBoolean(DcRepository.modulesettings.stOnlineSearchOverwrite) ?
                settings.getIntArray(DcRepository.modulesettings.stOnlineSearchFieldOverwriteSettings) :
                dco.getModule().getFieldIndices();

            for (int i = 0; i < fields.length; i++) {
                int field = fields[i];
                
                if (dco.isFilled(field)) {
                    if (settings.getBoolean(DcRepository.modulesettings.stOnlineSearchOverwrite) && 
                        (o.getValue(fields[i]) != null && !o.getValue(fields[i]).equals("") && !o.getValue(fields[i]).equals("-1"))) {
                        dco.setValue(field, o.getValue(fields[i]));
                    }
                } else {
                    dco.setValue(field, o.getValue(fields[i]));
                }
            }  
            
            if (o.getChildren() != null && o.getChildren().size() > 0)
                dco.setChildren(o.getChildren());
            
            itemForm.setData(dco, panelSettings.isOverwriteAllowed());
        }
        
        if (closeForm)
            close(true);
    }

    public void addNew(boolean closeForm) {
        Collection objects = getSelectedObjects();
        if (objects != null) {
            getModule().getCurrentInsertView().add(objects);
            DataCrow.mainFrame.setSelectedTab(MainFrame._INSERTTAB);
            DataCrow.mainFrame.toFront();
            if (closeForm)
                close(true);
        }
    }    

    public void setSelectionMode(int selectionMode) {
        list.setSelectionMode(selectionMode);        
    }
    
    private void clear() {
    	list.clear();
        updateProgressBar(0);
        items.clear();
        textLog.setText("");
    }

    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }

    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }
    
    public void stop() {
        if (task != null)
            task.cancelSearch();

        addMessage(DcResources.getText("msgStoppedSearch"));
        stopped();
    }
    
    public void start() {
        if (panelService.getQuery() == null || panelService.getQuery().trim().equals("")) {
            new MessageBox(this, DcResources.getText("msgEnterKeyword"), MessageBox._INFORMATION);
            return;
        }
        
        saveSettings();
        processing();

        task = panelService.getServer().getTask(this, panelService.getQuery(), panelService.getMode(), panelService.getRegion());;
        task.setPriority(Thread.MAX_PRIORITY);
        task.start();
    }     

    public void processed(int i) {
        updateProgressBar(i);
    }

    public void processing() {
        panelService.busy(true);
    }

    public void stopped() {
        if (panelService != null)
            panelService.busy(false);
    }    
    
    public int resultCount() {
        return list.getObjects().size();
    }

    public void processingTotal(int i) {
        initProgressBar(i);
    }    
    
    public void addError(Throwable t) {
        new MessageBox(t.getMessage(), MessageBox._ERROR);
        DcLog.add(t);
    }

    public void addError(String message) {
        new MessageBox(message, MessageBox._ERROR);
        addMessage(message);
    }

    public void addWarning(String warning) {
        if (!panelService.hasPerfectMatchOccured())
            new MessageBox(warning, MessageBox._WARNING);
    }    
    
    public void setFocus() {
        panelService.setFocus();
    }
    
    public void addDoubleClickListener(MouseListener ml) {
        list.removeMouseListener(mouseListener);
        mouseListener = ml;
        list.addMouseListener(ml);
    }
    
    private void buildDialog(boolean advanced) {
        getContentPane().setLayout(Layout.getGBL());
        
        getContentPane().add(getContentPanel(advanced), Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(0, 0, 0, 0), 0, 0));
        pack();
    }    
    
    public JPanel getContentPanel(boolean advanced) {
        setResizable(true);
        getContentPane().setLayout(Layout.getGBL());

        //**********************************************************
        //Settings
        //**********************************************************
        panelSettings = new OnlineServiceSettingsPanel(this, ID != null, module);


        //**********************************************************
        //Servers
        //**********************************************************
        panelService = new OnlineServicePanel(this, onlineSearch);
        
        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel();
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));

        
        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel();
        panelLog.setLayout(Layout.getGBL());

        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));

        panelLog.setBorder(ComponentFactory.getTitleBorder(DcResources.getText("lblLog")));

        //**********************************************************
        //Result panel
        //**********************************************************
        JPanel panelResults = new JPanel();
        panelResults.setLayout(Layout.getGBL());

        list = new DcObjectList(DcObjectList._ELABORATE, false, true);

        JScrollPane scrollTable = new JScrollPane(list);
        list.addMouseListener(mouseListener);
        scrollTable.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        panelResults.add(scrollTable,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                         new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Actions panel
        //**********************************************************
        JPanel panelActions = new JPanel();
        panelActions.setLayout(Layout.getGBL());

        JButton buttonDetails = ComponentFactory.getButton(DcResources.getText("lblDetails"));
        
        JButton buttonUpdate = ComponentFactory.getButton(DcResources.getText("lblUpdate"));
        JButton buttonAddNew = ComponentFactory.getButton(DcResources.getText("lblAddNew"));

        JButton buttonClear = ComponentFactory.getButton(DcResources.getText("lblClear"));
        JButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));

        buttonClose.setToolTipText(DcResources.getText("tpClose"));
        buttonUpdate.setToolTipText(DcResources.getText("tpUpdate"));
        buttonAddNew.setToolTipText(DcResources.getText("tpAddNew"));
        
        buttonDetails.setMnemonic('E');
        buttonClose.setMnemonic('C');        
        buttonUpdate.setMnemonic('U');
        buttonAddNew.setMnemonic('A');
        buttonClear.setMnemonic('L');

        buttonDetails.addActionListener(this);
        buttonDetails.setActionCommand("showdetails");
        buttonUpdate.addActionListener(this);
        buttonUpdate.setActionCommand("update");
        buttonClose.addActionListener(this);
        buttonClose.setActionCommand("close");
        buttonClear.addActionListener(this);
        buttonClear.setActionCommand("clear");
        buttonAddNew.addActionListener(this);
        buttonAddNew.setActionCommand("addnew");

        panelActions.add(buttonDetails,    Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets(5, 5, 5, 5), 0, 0));
        
        if (ID != null && advanced) {
            panelActions.add(buttonUpdate,    Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));
        } 

        if (advanced)
            panelActions.add(buttonAddNew,    Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));

        panelActions.add(buttonClear,        Layout.getGBC( 3, 0, 1, 1, 1.0, 1.0
                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                        new Insets(5, 5, 5, 5), 0, 0));

        if (advanced)
            panelActions.add(buttonClose,     Layout.getGBC( 4, 0, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Main Panel
        //**********************************************************
        JPanel panel = new JPanel();
        panel.setLayout(Layout.getGBL());

        panel.add(panelService, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets(5, 5, 5, 5), 0, 0));
        
        
        if (advanced)
            panel.add(panelSettings, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                             new Insets(5, 5, 5, 5), 0, 0));

        panel.add(panelResults,  Layout.getGBC( 0, 2, 1, 1, 50.0, 50.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        panel.add(panelActions,  Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));
        panel.add(panelProgress, Layout.getGBC( 0, 4, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));

        if (advanced)
            panel.add(panelLog,  Layout.getGBC( 0, 5, 1, 1, 10.0, 10.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));

        return panel;
    }    
    
    public void addMessage(String message) {
        if (textLog != null)
            textLog.insert(message + '\n', 0);
    }    
    
    public void setVisible(boolean b) {
        if (b)
            panelService.setFocus();
        super.setVisible(b);
    }    
    
    public void close(boolean saveSettings) {
        if (saveSettings)
            saveSettings();
        
        stop();

        ClearThread clrThread  = new ClearThread();
        clrThread.start();
        
        dispose();
    }     
    
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("stopsearch"))
            stop();
        else if (e.getActionCommand().equals("showdetails"))
            open();
        else if (e.getActionCommand().equals("update"))
            update(true);
        else if (e.getActionCommand().equals("close"))
            close(true);
        else if (e.getActionCommand().equals("clear"))        
            clear();
        else if (e.getActionCommand().equals("addnew"))        
            addNew(true);
        else if (e.getActionCommand().equals("search")) {        
            panelService.hasPerfectMatchOccured(false);
            start();        
        }
    }
    
    private class TableMouseListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                if (ID != null)
                    update(true);
                else 
                    addNew(true);
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }    
    
    private class ClearThread extends Thread {

        public void run() {
            
            while(task != null && task.isAlive()) {
                try {
                    sleep(100);
                } catch (Exception ignore) {}
            }
            
            list.removeMouseListener(mouseListener);
            list.clear();
            itemForm  = null;
            ID = null;
            items.clear();
            textLog = null;
            progressBar = null;
            panelService.clear();
            panelService = null;
            
            panelSettings.clear();
            panelSettings = null;
            mouseListener = null;
            task = null;
            close();
        }
    }    
}