/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.onlinesearch;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.OnlineSearch;
import net.sf.dc.util.StringUtils;

public class OnlineServicePanel extends JPanel implements ActionListener, KeyListener  {
    
    private JButton buttonSearch;
    private JTextField fldQuery;

    private JButton buttonStop = ComponentFactory.getButton(DcResources.getText("lblStop"));
    
    private JComboBox comboRegions = ComponentFactory.getComboBox(new DefaultComboBoxModel());
    private JComboBox comboModes = ComponentFactory.getComboBox(new DefaultComboBoxModel());
    private JComboBox comboServers = ComponentFactory.getComboBox(new DefaultComboBoxModel());
    
    private boolean listenForServerChanges = true;
    private boolean perfectMatchOccured = false;

    private OnlineSearchForm osf;
    private OnlineSearch onlineSearch;
    
    public OnlineServicePanel(OnlineSearchForm osf, OnlineSearch onlineSearch) {
        this.osf = osf;
        this.onlineSearch = onlineSearch;
        build(true);
        applySettings();
    }

    public void hasPerfectMatchOccured(boolean b) {
        perfectMatchOccured = b;
    }
    
    public boolean hasPerfectMatchOccured() {
        return perfectMatchOccured;
    }
    
    public String getQuery() {
        return StringUtils.normalize(fldQuery.getText().trim());
    }
    
    public void setQuery(String query) {
        fldQuery.setText(query);
    }    
    
    public DcSearchMode getMode() {
        DcServer server = getServer();
        DcSearchMode mode = null;
        if (server.getModes() != null && server.getModes().size() > 0) {
            mode = (DcSearchMode) comboModes.getSelectedItem();
        }
        return mode;
    }
    
    public DcServer getServer() {
        return (DcServer) comboServers.getSelectedItem();
    }

    public DcRegion getRegion() {
        DcServer server = getServer();
        DcRegion region = null;
        if (server.getRegions() != null && server.getRegions().size() > 0) {
            region = (DcRegion) comboRegions.getSelectedItem();
        }
        return region;
    }
    
    protected void busy(boolean b) {
        if (b) {
            buttonSearch.setEnabled(false);
            buttonStop.setEnabled(true);
            buttonStop.requestFocus();
        } else {
            buttonSearch.setEnabled(true);
            buttonStop.setEnabled(false);

            if (perfectMatchOccured) {
                fldQuery.setText("");
                fldQuery.requestFocusInWindow();
            } else {
                fldQuery.requestFocusInWindow();
            }
        }
    }

    protected void setFocus() {
        fldQuery.requestFocusInWindow();
    }    
    
    protected void save() {
        String server = getServer().getName();
        String mode = getMode() != null ? getMode().toString() : null;
        String region = getRegion() != null ? getRegion().getCode() : null;

        osf.getModule().setSetting(DcRepository.modulesettings.stOnlineSearchDefaultMode, mode);
        osf.getModule().setSetting(DcRepository.modulesettings.stOnlineSearchDefaultServer, server);
        osf.getModule().setSetting(DcRepository.modulesettings.stOnlineSearchDefaultRegion, region);        
    }
    
    protected void clear() {
        buttonSearch = null;
        fldQuery = null;
        buttonStop = null;
        comboRegions = null;
        comboModes = null;
        comboServers = null;
        osf = null;
        onlineSearch = null;
    }    
    
    private void applySettings() {
        DcObject dco = osf.getDcObject();
        if (dco != null) {
            DcServer server = onlineSearch.getServer(dco);
            if (server != null) {
                comboServers.setSelectedItem(server);
                
                DcRegion region = onlineSearch.getRegion(dco);
                if (region != null) comboRegions.setSelectedItem(region);
                
                DcSearchMode mode = onlineSearch.getMode(dco);
                if (mode != null) comboModes.setSelectedItem(mode);

                String query = onlineSearch.getQuery(dco);
                if (query != null && !query.toLowerCase().equals("null")) 
                    fldQuery.setText(query.trim());
                
                return;
            }
        }
        
        if (onlineSearch.getDefaultServer() != null)
            comboServers.setSelectedItem(onlineSearch.getDefaultServer());
        else
            comboServers.setSelectedIndex(0);
        
        if (onlineSearch.getDefaultRegion() != null)
            comboRegions.setSelectedItem(onlineSearch.getDefaultRegion());
            
        if (onlineSearch.getDefaultSearchMode() != null)
            comboModes.setSelectedItem(onlineSearch.getDefaultSearchMode());
    }
        
    private void rebuild() {
        listenForServerChanges = false;
        removeAll();
        build(false);
        revalidate();
        repaint();
        listenForServerChanges = true;
    }    
    
    private void build(boolean initialize) {
        setLayout(Layout.getGBL());
        
        if (initialize) {
            fldQuery = ComponentFactory.getShortTextField(255);
            fldQuery.addKeyListener(this);            
            fldQuery.setMinimumSize(new Dimension(100,19));
        
            buttonStop.addActionListener(osf);
            buttonStop.setActionCommand("stopsearch");
            buttonStop.setMnemonic('T');
            
            comboServers.setActionCommand("changeserver");
            Collection servers = osf.getServers();
            for (Iterator iter = servers.iterator(); iter.hasNext(); )
                comboServers.addItem(iter.next());

            comboServers.addActionListener(this);
        }
        
        DcServer server = getServer();
        JLabel labelServer = ComponentFactory.getLabel(DcResources.getText("lblInternetServer"));
        
        if (server.isKeywordSearchAllowed()) {
            int stretchX = server.hasModes() ? 1 : 2;
            JLabel labelSearch = ComponentFactory.getLabel(DcResources.getText("lblSearchFor"));
            add(labelSearch,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                             ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                              new Insets(5, 5, 5, 5), 0, 0));
            add(fldQuery,     Layout.getGBC( 1, 0, stretchX, 1, 10.0, 10.0
                             ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                              new Insets(5, 5, 5, 5), 0, 0));                
        }

        if (server.hasModes()) {
            comboModes.removeAllItems();
            for (Iterator iter = server.getModes().iterator(); iter.hasNext(); )
                comboModes.addItem(iter.next());

            add(comboModes,   Layout.getGBC( 2, 0, 1, 1, 10.0, 10.0
                             ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                              new Insets(5, 5, 5, 5), 0, 0));
        }
  
        comboRegions.removeAllItems();
        for (Iterator iter = server.getRegions().iterator(); iter.hasNext(); )
            comboRegions.addItem(iter.next());

        buttonSearch = ComponentFactory.getButton(DcResources.getText("lblFind"));
        buttonSearch.addActionListener(osf);
        buttonSearch.setActionCommand("search");
        buttonSearch.setMnemonic('F');

        add(labelServer,  Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));
        add(comboServers, Layout.getGBC( 1, 1, 1, 1, 10.0, 10.0
                         ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));
        add(comboRegions, Layout.getGBC( 2, 1, 1, 1, 10.0, 10.0
                         ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));
        add(buttonSearch, Layout.getGBC( 3, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));
        add(buttonStop,   Layout.getGBC(4, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0));      
    }
    
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("stopsearch"))
            osf.stop();
        else if (e.getActionCommand().equals("changeserver") && listenForServerChanges)
            rebuild();
    }

    public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
            perfectMatchOccured = false;
            osf.start();
            e.consume();
        }
    }

    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}
}
