/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.windows.settings;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcImageLabel;
import net.sf.dc.console.components.DcTree;
import net.sf.dc.console.windows.help.HelpDialog;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.core.settings.Settings;
import net.sf.dc.core.settings.SettingsFile;
import net.sf.dc.core.settings.SettingsGroup;

/**
 * Tree view with panels. Loads and saves settings
 *
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class SettingsView extends DcDialog {

    protected DcTree tree;
    
    private Settings settings;

    private InformationPanel panelInfo;
    private JPanel panelActions;
    private JPanel panelBogus = new JPanel();

    private JButton buttonSave;
    private JButton buttonClose;

    public SettingsView(String title, Settings settings) {
        super(DataCrow.mainFrame);

        setTitle(title);
        
        this.settings = settings;
        buildView();
        this.setTitle(title);
    }

    private void buildView() {
        setResizable(false);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });

        panelInfo = new InformationPanel();
        panelInfo.setMinimumSize(new Dimension(700, 380));
        panelInfo.setPreferredSize(new Dimension(700, 380));
        panelInfo.setMaximumSize(new Dimension(700, 380));

        getContentPane().setLayout(Layout.getGBL());
        tree = new DcTree(buildTreeModel());
        tree.setFont(ComponentFactory.getSystemFont());
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.addTreeSelectionListener(new NodeSelectedAction());
        tree.setBorder(new EtchedBorder());

        tree.setPreferredSize(new Dimension(300, 400));
        tree.setMinimumSize(new Dimension(300, 400));
        tree.setMaximumSize(new Dimension(300, 400));

        panelBogus.add(panelInfo,  Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                       new Insets(0, 0, 0, 0), 0, 0));

        getContentPane().add(tree,  Layout.getGBC( 0, 0, 1, 2, 2.0, 2.0
                            ,GridBagConstraints.NORTH, GridBagConstraints.NONE,
                             new Insets(5, 5, 0, 5), 0, 0));
        getContentPane().add(getActionPanel(),  Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                             new Insets(5, 5, 5, 5), 0, 0));
        getContentPane().add(panelBogus,  Layout.getGBC( 1, 1, 1, 1, 1.0, 1.0
                            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                             new Insets(5, 5, 5, 5), 0, 0));
        initializeSettings();
        this.pack();
        setCenteredLocation();
    }

    public void close() {
        SettingsFile.save(settings);
        
        settings = null;
        tree = null;
        panelInfo = null;
        panelActions = null;
        panelBogus = null;
        buttonSave = null;
        buttonClose = null;
        
        super.close();
    }

    /**
     * Creates the action panel (save buttons and such)
     */
    private JPanel getActionPanel() {
        panelActions = new JPanel();

        buttonSave = ComponentFactory.getButton(DcResources.getText("lblSave"));
        buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));

        panelActions.add(buttonSave,Layout.getGBC( 1, 0, 1, 1, 40.0, 40.0
                        ,GridBagConstraints.EAST, GridBagConstraints.BOTH,
                         new Insets(5, 5, 5, 5), 0, 0));
        panelActions.add(buttonClose,  Layout.getGBC( 1, 0, 1, 1, 80.0, 80.0
                        ,GridBagConstraints.WEST, GridBagConstraints.BOTH,
                         new Insets(5, 5, 5, 5), 0, 0));
        panelActions.setVisible(false);

        buttonSave.addActionListener(new SaveAllSettingsAction());
        buttonClose.addActionListener(new CloseAction());
        
        buttonSave.setMnemonic('S');
        buttonClose.setMnemonic('C');

        return panelActions;
    }

    public void setFont(Font font) {

        if (tree != null && buttonClose != null) {
            buttonClose.setFont(ComponentFactory.getSystemFont());
            buttonSave.setFont(ComponentFactory.getSystemFont());
            tree.setFont(ComponentFactory.getSystemFont());

            Vector panels = getAllPanels();
            for (Enumeration enumerator = panels.elements(); enumerator.hasMoreElements(); ) {
                JPanel panel = (JPanel) enumerator.nextElement();

                Component[] comps = panel.getComponents();
                for (int i = 0; i < comps.length; i++) {
                    if (comps[i] instanceof JLabel || comps[i] instanceof JCheckBox)
                        comps[i].setFont(ComponentFactory.getSystemFont());
                    else
                        comps[i].setFont(ComponentFactory.getStandardFont());
                }
            }
        }
    }

    public void setDisclaimer(ImageIcon icon) {
        panelInfo.setImage(icon);
    }

    /**
     * Information panel, holder of the disclaimer information
     */
    public class InformationPanel extends JPanel {

        public InformationPanel() {
            // Build the panel
            JLabel labelProduct = ComponentFactory.getLabel(DcRepository.system.version);
            JLabel urlField = ComponentFactory.getLabel("www.datacrow.net");
            JLabel labelEmail = ComponentFactory.getLabel("info@datacrow.net");

            this.setLayout(Layout.getGBL());

            this.add(labelProduct,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
            		                , GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
									  new Insets( 0, 30, 0, 5), 0, 0));
            this.add(urlField,        Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                                    , GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
									  new Insets( 0, 30, 0, 5), 0, 0));
            this.add(labelEmail,      Layout.getGBC( 0, 3, 1, 1, 1.0, 1.0
                                    , GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
									  new Insets( 0, 30, 0, 5), 0, 0));
        }

        public void setImage(ImageIcon icon) {
            DcImageLabel imageLabel = new DcImageLabel(icon);
            
            
            this.add(imageLabel,        Layout.getGBC( 0, 0, 1, 1, 20.0, 20.0
                    , GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                      new Insets( 5, 5, 5, 5), 0, 0));            
            
            pack();
            revalidate();

        }

        public String toString() {
            return DcResources.getText("lblSettings");
        }
    }

    /**
     * Puts the current values of the settings in the panel
     */
    private void initializeSettings() {
        Vector vPanels = getAllPanels();
        for (Enumeration enumerator = vPanels.elements(); enumerator.hasMoreElements(); ) {
            SettingsPanel panel = (SettingsPanel) enumerator.nextElement();
            panel.initializeSettings();
        }
    }

    /**
     * Builds the tree model by reading the settings groups.
     * Each settingsgroup can contain several childeren, which are
     * added as leafs of the parent in the tree.
     * The settings panels are instantiated here as well.
     * Each leaf contains a specific panel with the settings components.
     */
    private DefaultMutableTreeNode buildTreeModel() {
        DefaultMutableTreeNode topTreeNode = new DefaultMutableTreeNode(DcResources.getText("lblSettings"));
        topTreeNode.setUserObject(panelInfo);

        LinkedHashMap htGroups = settings.getSettingsGroups();
        Vector vChildGroups = new Vector();
        for (Iterator iter = htGroups.values().iterator(); iter.hasNext(); ) {
            SettingsGroup group = (SettingsGroup) iter.next();
            JPanel panel = new SettingsPanel(group);
            DefaultMutableTreeNode treeNode = new DefaultMutableTreeNode(panel);

            // Add the treenode to the top note
            topTreeNode.add(treeNode);
            // Add the childs to child vector
            vChildGroups.addAll(group.getChildren().values());

            panelBogus.setPreferredSize(new Dimension(700, 380));

            panel.setMinimumSize(new Dimension(700, 380));
            panel.setPreferredSize(new Dimension(700, 380));
            panel.setMaximumSize(new Dimension(700, 380));

            getContentPane().add(panel, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                 new Insets(5, 5, 5, 5), 0, 0));

            Hashtable htChildren = group.getChildren();
            for (Enumeration enumChilds = htChildren.elements(); enumChilds.hasMoreElements(); ) {
                SettingsGroup childGroup = (SettingsGroup) enumChilds.nextElement();
                JPanel childPanel = new SettingsPanel(childGroup);
                DefaultMutableTreeNode childTreeNode = new DefaultMutableTreeNode(childPanel);
                treeNode.add(childTreeNode);

                childPanel.setMinimumSize(new Dimension(700, 380));
                childPanel.setPreferredSize(new Dimension(700, 380));
                childPanel.setMaximumSize(new Dimension(700, 380));

                getContentPane().add(childPanel, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                     new Insets(5, 5, 5, 5), 0, 0));
            }
        }
        return topTreeNode;
    }

    /**
     * Saves all settings as they are defined in the panels
     */
    protected void saveAllSettings() {
        Vector vPanels = getAllPanels();
        for (Enumeration enumerator = vPanels.elements(); enumerator.hasMoreElements(); ) {
            SettingsPanel panel = (SettingsPanel) enumerator.nextElement();
            panel.saveSettings();
        }
        DataCrow.mainFrame.applySettings();
        toFront();
    }

    /**
     * Saves the settings of the current visible panel
     */
    protected void saveCurrentVisibleSettings() {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
        if (node.getUserObject() instanceof SettingsPanel) {
            SettingsPanel panel = (SettingsPanel) node.getUserObject();
            panel.saveSettings();
        }
        DataCrow.mainFrame.applySettings();
        toFront();
    }

    private Vector getAllPanels() {
        Vector vPanels = new Vector();

        DefaultMutableTreeNode oRoot = (DefaultMutableTreeNode) tree.getModel().getRoot();
        for (Enumeration enumerator = oRoot.children(); enumerator.hasMoreElements(); ) {
            DefaultMutableTreeNode oCurrent = (DefaultMutableTreeNode) enumerator.nextElement();
            SettingsPanel nodePanel = (SettingsPanel) oCurrent.getUserObject();

            vPanels.addElement(nodePanel);

            for (Enumeration enumChilds = oCurrent.children(); enumChilds.hasMoreElements(); ) {
                DefaultMutableTreeNode oChild = (DefaultMutableTreeNode) enumChilds.nextElement();
                SettingsPanel nodePanelChild = (SettingsPanel) oChild.getUserObject();
                vPanels.addElement(nodePanelChild);
            }
        }
        return vPanels;
    }

    protected void setVisible(JPanel panel) {
        Vector vPanels = getAllPanels();
        for (Enumeration enumerator = vPanels.elements(); enumerator.hasMoreElements(); ) {
            SettingsPanel nodePanel = (SettingsPanel) enumerator.nextElement();
            if (nodePanel.equals(panel)) {
                nodePanel.setVisible(true);
            } else {
                nodePanel.setVisible(false);
            }
        }
    }

    protected void setPanelsVisible(boolean visible) {
        Vector vPanels = getAllPanels();
        for (Enumeration enumerator = vPanels.elements(); enumerator.hasMoreElements(); ) {
            SettingsPanel nodePanel = (SettingsPanel) enumerator.nextElement();
            nodePanel.setVisible(visible);
        }
    }

    private class NodeSelectedAction implements TreeSelectionListener {
        public void valueChanged(TreeSelectionEvent e) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();

            if (node == null)
                return;

            Object nodeInfo = node.getUserObject();

            if (nodeInfo.equals(panelInfo)) {
                JPanel panel = (JPanel) nodeInfo;
                panel.setVisible(true);
                setPanelsVisible(false);
                panelActions.setVisible(false);
            } else {
                SettingsPanel panel = (SettingsPanel) nodeInfo;
                HelpDialog.setHelpIndex(panel.getHelpIndex());
                setVisible(panel);
                panelInfo.setVisible(false);
                panelActions.setVisible(true);
            }
        }
    }

    public class SaveCurrentVisibleSettingsAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveCurrentVisibleSettings();
        }
    }

    public class SaveAllSettingsAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveAllSettings();
        }
    }

    public class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
}
