/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards.item;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ListSelectionModel;

import net.sf.dc.console.Layout;
import net.sf.dc.console.windows.onlinesearch.OnlineSearchForm;
import net.sf.dc.console.wizards.WizardException;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;

public class InternetWizardPanel extends ItemWizardPanel {

    private OnlineSearchForm internetSearchForm = null;
    private ItemWizard wizard;

    public InternetWizardPanel(ItemWizard wizard, DcModule module) {
        build(module);
        this.wizard = wizard;
    }

    public Object apply() throws WizardException {
        DcObject result = internetSearchForm.getSelectedObject();
        if (result == null) 
            throw new WizardException(DcResources.getText("msgWizardSelectItem"));

        internetSearchForm.stop();
        return result;
    }

    public String getHelpText() {
        return DcResources.getText("msgInternetSearch");
    }

    public void setObject(DcObject dco) {}

    public void setFocus() {
        internetSearchForm.setFocus();
    }
    
    public void setVisible(boolean b) {
        if (b && internetSearchForm != null)  
            internetSearchForm.setFocus();
        
        super.setVisible(b);
    }     
    
    private void build(DcModule module) {
        if (module.hasOnlineServices())
            internetSearchForm = module.getOnlineSearch().getUI(null, null, false);

        if (internetSearchForm != null) {
            internetSearchForm.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            internetSearchForm.addDoubleClickListener(new DblClickSelectionListener());
            setLayout(Layout.getGBL());
            add(internetSearchForm.getContentPanel(false), Layout.getGBC(0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                            new Insets(5, 5, 5, 5), 0, 0));
        }
    }
    
    public class DblClickSelectionListener implements MouseListener {
        public void mouseReleased(MouseEvent e) {
            if (e.getClickCount() == 2) {
                internetSearchForm.stop();
                wizard.next();
            }
        }
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mousePressed(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
    }     
}