/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards.item;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;

import net.sf.dc.console.windows.ItemTypeDialog;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.wizards.Wizard;
import net.sf.dc.console.wizards.WizardException;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.ValidationException;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.core.wf.requests.CloseWindowRequest;
import net.sf.dc.settings.DcSettings;

public class ItemWizard extends Wizard {

    private DcObject dco;
    private DcModule module;
    private boolean closed = false;

    public ItemWizard() {
        super();
        setTitle(DcResources.getText("lblItemWizard"));
        setHelpIndex("dc.items.wizard");

        setSize(DcSettings.getDimension(DcRepository.settings.stItemWizardFormSize));
        setCenteredLocation();
    }
    
    protected void initialize() {
        module = DcModules.getCurrent();
        if (module.getIndex() == DcModules._MEDIA) {
            ItemTypeDialog dialog = new ItemTypeDialog();
            dialog.setVisible(true);

            if (dialog.getSelectedModule() < 0) {
                closed = true;
                close();
            } else {
                module = DcModules.get(dialog.getSelectedModule());
            }
        }
        dco = module.getDcObject();
    }
    
    protected List getWizardPanels() {
        List panels = new ArrayList();
        
        if (module.hasOnlineServices()) {
            JPanel panelInternetSearch = new InternetWizardPanel(this, module);
            panels.add(panelInternetSearch);
        }
        panels.add(new ItemDetailsWizardPanel(dco));
        
        return panels;
    }

    public void setVisible(boolean b) {
        super.setVisible(b && !closed);
        if (b && getCurrent() instanceof InternetWizardPanel)
            ((InternetWizardPanel) getCurrent()).setFocus();
    }

    protected void saveSettings() {
        DcSettings.set(DcRepository.settings.stItemWizardFormSize, getSize());
    }

    public void finish() throws WizardException {
        dco = (DcObject) getCurrent().apply();
        dco.addRequest(new CloseWindowRequest(this));
        
        try {
            dco.saveNew(true);
        } catch (ValidationException ve) {
            throw new WizardException(ve.getMessage());
        }
    }

    public void next() {
        try {
            dco = (DcObject) getCurrent().apply();

            current += 1;
            if (current <= getStepCount()) {
                for (int i = 0; i < getStepCount(); i++) {
                    ItemWizardPanel panel = (ItemWizardPanel) getWizardPanel(i);
                    panel.setObject(dco);
                    panel.setVisible(i == current);
                }
            } else {
                current -= 1;
            }

            applyPanel();
        } catch (WizardException wzexp) {
            if (wzexp.getMessage().length() > 1)
                new MessageBox(wzexp.getMessage(), MessageBox._WARNING);
        }
    }

    public void close() {
        dco = null;
        module = null;
        super.close();
    }

    protected String getWizardName() {
        return DcResources.getText("msgNewItemWizard",
                                   new String[] {dco.getModule().getObjectName(),
                                   String.valueOf(current + 1),
                                   String.valueOf(getStepCount())});
    }

    protected void restart() {
        try {
            finish();
            saveSettings();
            ItemWizard wizard = new ItemWizard();
            wizard.setVisible(true);
        } catch (WizardException exp) {
            new MessageBox(exp.getMessage(), MessageBox._WARNING);
        }
    }
}
