/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards.module;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.wizards.Wizard;
import net.sf.dc.console.wizards.WizardException;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.ModuleJar;
import net.sf.dc.core.modules.xml.XmlField;
import net.sf.dc.core.modules.xml.XmlModule;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.DcSettings;

public class AlterModuleWizard extends Wizard {
    
    public AlterModuleWizard() {
        super();
        
        setTitle(DcResources.getText("lblModuleWizard"));
        setHelpIndex("dc.modules.alter");

        setSize(DcSettings.getDimension(DcRepository.settings.stModuleWizardFormSize));
        setCenteredLocation();
    }
    
    protected void initialize() {
    }
    
    protected List getWizardPanels() {
        List panels = new ArrayList();
        panels.add(new PanelModules(this));
        panels.add(new PanelBasicInfo(this, true));
        panels.add(new PanelFields(this, true));
        return panels;
    }

    protected void saveSettings() {
        DcSettings.set(DcRepository.settings.stModuleWizardFormSize, getSize());
    }

    public void finish() throws WizardException {
        XmlModule module = (XmlModule) getCurrent().apply();

        try {
            ModuleJar mj = new ModuleJar(module);
            mj.save();
            
            Collection fields = module.getFields();
            for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
                XmlField field = (XmlField) iter.next();
                if (field.getModuleReference() != module.getIndex() && field.getModuleReference() != 0) {
                    DcModule m = DcModules.get(field.getModuleReference()) == null ? 
                                 DcModules.get(field.getModuleReference() + module.getIndex()) : 
                                 DcModules.get(field.getModuleReference());
                    
                    if (m != null && m.getXmlModule() != null)
                        new ModuleJar(m.getXmlModule()).save();
                }
            }
            
            close();
            
        } catch (Exception e) {
            throw new WizardException(DcResources.getText("msgCouldNotWriteModuleFile", e.getMessage()));
        }
    }

    public void next() {
        try {
            XmlModule module = (XmlModule) getCurrent().apply();

            if (module == null)
                return;
            
            current += 1;
            if (current <= getStepCount()) {
                for (int i = 0; i < getStepCount(); i++) {
                    ModuleWizardPanel panel = (ModuleWizardPanel) getWizardPanel(i);
                    panel.setModule(module);
                    panel.setVisible(i == current);
                }
            } else {
                current -= 1;
            }

            applyPanel();
        } catch (WizardException wzexp) {
            if (wzexp.getMessage().length() > 1)
                new MessageBox(wzexp.getMessage(), MessageBox._WARNING);
        }
    }

    public void close() {
        if (!isCancelled() && !isRestarted())
            new RestartDataCrowDialog(this);
        
        super.close();
    }

    protected String getWizardName() {
        return DcResources.getText("msgModuleWizard",
                                   new String[] {String.valueOf(current + 1), String.valueOf(getStepCount())});
    }

    protected void restart() {
        try {
            finish();
            saveSettings();
            CreateModuleWizard wizard = new CreateModuleWizard();
            wizard.setVisible(true);
        } catch (WizardException exp) {
            new MessageBox(exp.getMessage(), MessageBox._WARNING);
        }
    }
}
