/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards.module;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcCheckBox;
import net.sf.dc.console.components.DcDialog;
import net.sf.dc.console.components.DcNumberField;
import net.sf.dc.console.components.DcShortTextField;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.wizards.WizardException;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.xml.XmlField;
import net.sf.dc.core.modules.xml.XmlModule;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.util.StringUtils;

public class DefineFieldDialog extends DcDialog {
    
    private boolean canHaveReferences;
    
    private DcShortTextField textName;
    private DcCheckBox checkSearchable;
    private DcCheckBox checkTechinfo;
    private DcNumberField numberMaxLength;
    
    private JComboBox comboReference;
    private JComboBox comboFieldType;
    
    private XmlModule module;
    private XmlField field;
    
    public DefineFieldDialog(JFrame parent, XmlModule module, boolean canHaveReferences) {
        super(parent);
        this.canHaveReferences = canHaveReferences;
        this.setModal(true);
        this.module = module;
        this.setTitle(DcResources.getText("lblDefineField"));
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        
        build();
        setResizable(false);
        pack();
        setSize(new Dimension(400,300));
        setCenteredLocation();
    }

    public XmlField getField() {
        return field;
    }
    
    private void createField() {
        field =  new XmlField();
        
        String name = textName.getText();
        
        try {
            checkValue(name, DcResources.getText("lblName"));
            
            if (numberMaxLength.isEnabled())
                checkValue(name, DcResources.getText("lblMaxTextLength"));
            
            String column = StringUtils.normalize(name).replaceAll(" ", "");
            boolean exists = true;
            int counter = 1;
            while (exists) {
                exists = false;
                for (Iterator iter = module.getFields().iterator(); iter.hasNext(); ) {
                    XmlField field = (XmlField) iter.next();
                    if (column.toLowerCase().equals(field.getColumn().toLowerCase())) {
                        column += (counter++);
                        exists = true;
                    }
                }
            }
            
            field.setColumn(column);
            field.setName(name);
            field.setSearchable(checkSearchable.isSelected());
            field.setTechinfo(checkTechinfo.isSelected());
            
            field.setMaximumLength(numberMaxLength.getValue() == null ? 255 : 
            						((Long) numberMaxLength.getValue()).intValue());
            
            FieldType ft = (FieldType) comboFieldType.getSelectedItem();
            if (ft.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION ||
                ft.getValueType() == DcRepository.datatypes._DCOBJECTREFERENCE) {
                
                DcModule m = (DcModule) comboReference.getSelectedItem();
                field.setModuleReference(m.getIndex());
                
                if (m.getXmlModule() != null)
                    m.getXmlModule().setHasDependingModules(true);
            }

            field.setUiOnly(ft.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION ||
                            ft.getValueType() == DcRepository.datatypes._PICTURE);
            
            field.setValueType(ft.getValueType());
            field.setFieldType(ft.getIndex());
            field.setOverwritable(true);
            
            close();
            
        } catch (WizardException we) {
        	field = null;
            new MessageBox(we.getMessage(), MessageBox._WARNING);
        }
    }
    
    private void checkValue(String s, String desc) throws WizardException { 
        if (s == null || s.trim().length() == 0)
            throw new WizardException(DcResources.getText("msgXNotEntered", desc));
    }
    
    public void close() {
        textName = null;
        checkSearchable = null;
        checkTechinfo = null;
        numberMaxLength = null;
        comboReference = null;
        comboFieldType = null;
        //field = null;
        super.close();
    }
    
    public void clear() {
        field = null;
    }
    
    private void build() {
        JPanel panel = new JPanel();
        panel.setLayout(Layout.getGBL());
        
        textName = ComponentFactory.getShortTextField(255);
        checkSearchable = ComponentFactory.getCheckBox("");
        checkTechinfo = ComponentFactory.getCheckBox("");
        numberMaxLength = ComponentFactory.getNumberField();
        
        comboReference = ComponentFactory.getComboBox();
        comboFieldType = ComponentFactory.getComboBox();
        initializeTypes();
        
        panel.add(ComponentFactory.getLabel(DcResources.getText("lblName")),
                Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        panel.add(textName,
                Layout.getGBC(1, 0, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 0), 0, 0));
        panel.add(ComponentFactory.getLabel(DcResources.getText("lblSearchable")),
                Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        panel.add(checkSearchable,
                Layout.getGBC(1, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 0), 0, 0));
        panel.add(ComponentFactory.getLabel(DcResources.getText("lblHoldsTechnicalInfo")),
                Layout.getGBC(0, 2, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        panel.add(checkTechinfo,
                Layout.getGBC(1, 2, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 0), 0, 0));
        panel.add(ComponentFactory.getLabel(DcResources.getText("lblMaxTextLength")),
                Layout.getGBC(0, 3, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        panel.add(numberMaxLength,
                Layout.getGBC(1, 3, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 0), 0, 0));
        panel.add(ComponentFactory.getLabel(DcResources.getText("lblFieldType")),
                Layout.getGBC(0, 4, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        panel.add(comboFieldType,
                Layout.getGBC(1, 4, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 0), 0, 0));
        
        if (canHaveReferences) {
            panel.add(ComponentFactory.getLabel(DcResources.getText("lblReferencedModule")),
                    Layout.getGBC(0, 5, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets(0, 0, 0, 0), 0, 0));
            panel.add(comboReference,
                    Layout.getGBC(1, 5, 1, 1, 1.0, 1.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                    new Insets(0, 0, 0, 0), 0, 0));
            
            Collection modules = new ArrayList();
            modules.addAll(DcModules.getPropertyBaseModules());

            for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
                DcModule module = (DcModule) iter.next();
                if (module.isTopModule() && !module.hasDependingModules() &&
                    !module.isParentModule() && !module.isChildModule())
                    modules.add(module);
            }
            
            Collections.sort((List) modules, new ModuleComparator());
            for (Iterator iter = modules.iterator(); iter.hasNext(); )
                comboReference.addItem(iter.next());
        }

        comboFieldType.addActionListener(new FieldTypeSelectionListener());
        comboFieldType.setSelectedIndex(0);
        
        JPanel panelActions = new JPanel();
        panelActions.setLayout(new FlowLayout(FlowLayout.RIGHT));
        
        JButton buttonOk = ComponentFactory.getButton(DcResources.getText("lblOK"));
        JButton buttonClose = ComponentFactory.getButton(DcResources.getText("lblClose"));
        
        panelActions.add(buttonOk);
        panelActions.add(buttonClose);
        
        buttonClose.addActionListener(new CloseAction());
        buttonOk.addActionListener(new OkAction());
        
        
        getContentPane().setLayout(Layout.getGBL());
        add(panel, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
        add(panelActions, Layout.getGBC(0, 1, 1, 1, 1.0, 1.0,
                GridBagConstraints.SOUTHEAST, GridBagConstraints.HORIZONTAL,
                new Insets(5, 5, 5, 5), 0, 0));
    }
    
    private void initializeTypes() {
        List fieldTypes = new ArrayList();
        fieldTypes.add(new FieldType(ComponentFactory._CHECKBOX, 
                                     DcRepository.datatypes._BOOLEAN, 
                                     DcResources.getText("lblCheckbox")));
        fieldTypes.add(new FieldType(ComponentFactory._DATEFIELD, 
                                     DcRepository.datatypes._DATE, 
                                     DcResources.getText("lblDateField")));
        fieldTypes.add(new FieldType(ComponentFactory._FILEFIELD, 
                                     DcRepository.datatypes._STRING, 
                                     DcResources.getText("lblFileField")));
        fieldTypes.add(new FieldType(ComponentFactory._FILELAUNCHFIELD, 
                                     DcRepository.datatypes._STRING, 
                                     DcResources.getText("lblFileLauncherField")));
        fieldTypes.add(new FieldType(ComponentFactory._LONGTEXTFIELD, 
                                     DcRepository.datatypes._STRING, 
                                     DcResources.getText("lblLongTextField")));
        fieldTypes.add(new FieldType(ComponentFactory._NUMBERFIELD, 
                                     DcRepository.datatypes._LONG, 
                                     DcResources.getText("lblNumberField")));
        fieldTypes.add(new FieldType(ComponentFactory._PICTUREFIELD, 
                                     DcRepository.datatypes._PICTURE, 
                                     DcResources.getText("lblPictureField")));
        fieldTypes.add(new FieldType(ComponentFactory._RATINGCOMBOBOX, 
                                     DcRepository.datatypes._LONG, 
                                     DcResources.getText("lblRatingField")));
        fieldTypes.add(new FieldType(ComponentFactory._SHORTTEXTFIELD, 
                                     DcRepository.datatypes._STRING, 
                                     DcResources.getText("lblShortTextField")));
        fieldTypes.add(new FieldType(ComponentFactory._URLFIELD, 
                                     DcRepository.datatypes._STRING, 
                                     DcResources.getText("lblUrlField")));

        if (canHaveReferences) {
            fieldTypes.add(new FieldType(ComponentFactory._REFERENCESFIELD, 
                                         DcRepository.datatypes._DCOBJECTCOLLECTION, 
                                         DcResources.getText("lblMultiReferenceField")));
            fieldTypes.add(new FieldType(ComponentFactory._REFERENCEFIELD, 
                                         DcRepository.datatypes._DCOBJECTREFERENCE, 
                                         DcResources.getText("lblSingleReferenceField")));
        }
        
        Collections.sort(fieldTypes, new FieldTypeComparator());
        for (Iterator iter = fieldTypes.iterator(); iter.hasNext(); )
            comboFieldType.addItem(iter.next());
    }    
    
    private class FieldTypeSelectionListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            FieldType type = (FieldType) comboFieldType.getSelectedItem();
            
            if (type.getIndex() != ComponentFactory._SHORTTEXTFIELD &&
                type.getIndex() != ComponentFactory._NUMBERFIELD && 
                type.getIndex() != ComponentFactory._LONGTEXTFIELD) {
                numberMaxLength.setEnabled(false);
            } else {
                numberMaxLength.setEnabled(true);
            }
            
            if (type.getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION ||
                type.getValueType() == DcRepository.datatypes._DCOBJECTREFERENCE) {
                comboReference.setEnabled(true);
            } else {
                comboReference.setEnabled(false);
            }
        }
    }
    
    private class FieldType {
        private int index;
        private int valueType;
        private String name;
        
        public FieldType(int index, int valueType, String name) {
            this.index = index;
            this.name = name;
            this.valueType = valueType;
        }

        public String toString() {
            return getName();
        }

        public int getIndex() {
            return index;
        }

        public String getName() {
            return name;
        }

        public int getValueType() {
            return valueType;
        }
    }
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
    
    private class OkAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            createField();
        }
    }

    private class FieldTypeComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            FieldType ft1 = (FieldType) o1;
            FieldType ft2 = (FieldType) o2;
            return ft1.getName().compareTo(ft2.getName());
        }
    }
    
    private class ModuleComparator implements Comparator {
        public int compare(Object o1, Object o2) {
            DcModule m1 = (DcModule) o1;
            DcModule m2 = (DcModule) o2;
            return m1.getLabel().compareTo(m2.getLabel());
        }
    }
}