/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards.module;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.wizards.Wizard;
import net.sf.dc.console.wizards.WizardException;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcMediaParentModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcParentModule;
import net.sf.dc.core.modules.xml.XmlModule;
import net.sf.dc.core.resources.DcResources;

public class PanelChildModule extends ModuleWizardPanel {

    private int selectedModule = -1;
    
    private Map fields = new HashMap();
    
    public PanelChildModule(Wizard wizard) {
        super(wizard);
        build();
    }

    public Object apply() {
        XmlModule module = getModule();
        
        if (selectedModule == -1) {
            new MessageBox(DcResources.getText("msgSelectChildModuleFirst"), MessageBox._INFORMATION);
            return null;
        }
        
        module.setChildIndex(selectedModule);
        
        if (  !(module.getModuleClass().equals(DcMediaParentModule.class) || 
                module.getModuleClass().equals(DcParentModule.class))) {
            
            if (module.getModuleClass().equals(DcMediaModule.class))
                module.setModuleClass(DcMediaParentModule.class);
            else 
                module.setModuleClass(DcParentModule.class);
        }
        
        return module;
    }
    
    public void setModule(XmlModule module) {
        super.setModule(module);
        
        JRadioButton rb = (JRadioButton) fields.get(new Integer(module.getIndex()));
        rb.setVisible(false);
    }

    public String getHelpText() {
        return DcResources.getText("msgSelectChildModule");
    }
    
    private void build() {
        setLayout(Layout.getGBL());
        
        final ButtonGroup bg = new ButtonGroup();
        class ModuleSelectionListener implements ItemListener {
            public void itemStateChanged(ItemEvent ev) {
                String command = bg.getSelection().getActionCommand();
                selectedModule = Integer.parseInt(command);
                try {
                    getWizard().finish();
                } catch (WizardException wi) {
                    new MessageBox(wi.getMessage(), MessageBox._WARNING);
                }
            }
        } 

        int y = 0;
        int x = 0;
        
        for (Iterator iter = DcModules.getAllModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            
            if (      module.getIndex() != DcModules._MEDIA &&
                      module.isTopModule() && 
                     !module.isParentModule() && !module.isChildModule()) {

                JRadioButton rb = ComponentFactory.getRadioButton(
                        module.getName(), module.getIcon16(), "" + module.getIndex());

                fields.put(new Integer(module.getIndex()), rb);
                
                rb.addItemListener(new ModuleSelectionListener());
                bg.add(rb);
                add(rb, Layout.getGBC( x, y++, 1, 1, 1.0, 1.0
                   ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                    new Insets( 0, 5, 5, 5), 0, 0));
                
                if (y == 7) {
                    ++x;
                    y = 0;
                }
            }
        }
    }
}