/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.wizards.module;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JViewport;
import javax.swing.table.TableColumn;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.Layout;
import net.sf.dc.console.components.DcLabel;
import net.sf.dc.console.components.tables.DcTable;
import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.console.windows.messageboxes.QuestionBox;
import net.sf.dc.console.wizards.Wizard;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcPropertyModule;
import net.sf.dc.core.modules.xml.XmlField;
import net.sf.dc.core.modules.xml.XmlModule;
import net.sf.dc.core.resources.DcResources;

public class PanelFields extends ModuleWizardPanel {

    private JButton buttonUp = ComponentFactory.getButton(IconLibrary._icoArrowUp);
    private JButton buttonDown = ComponentFactory.getButton(IconLibrary._icoArrowDown);
    private JButton buttonAdd = ComponentFactory.getButton(DcResources.getText("lblAddField"));
    private JButton buttonRemove = ComponentFactory.getButton(DcResources.getText("lblRemoveField"));
    
    private boolean canHaveReferences;
    private final DcTable table;
    private final DcTable tableSysFields;
    
    private final boolean dbCheck;
    
    public PanelFields(Wizard wizard, boolean dbCheck) {
        super(wizard);
        
        this.dbCheck = dbCheck;
        table = ComponentFactory.getDCTable(true, false);
        tableSysFields = ComponentFactory.getDCTable(true, false);
        build();
    }
    
    public void setModule(XmlModule module) {
        super.setModule(module);
        canHaveReferences = !getModule().getModuleClass().equals(DcPropertyModule.class); 
        addDefaultFields();
        
        table.clearTable();
        for (Iterator iter = module.getFields().iterator(); iter.hasNext(); )
            table.addRow(new Object[] {iter.next()});
    }
    
    public Object apply() {
        XmlModule module = getModule();

        Collection fields = new ArrayList();
        for (int i = 0; i < table.getRowCount(); i++) {
            XmlField field = (XmlField) table.getValueAt(i, 0);
            fields.add(field);
        }
        
        module.setFields(fields);
        
        return module;
    }
    
    public String getHelpText() {
        return DcResources.getText("msgModuleFields");
    }
    
    private void createField() {
        DefineFieldDialog dlg = new DefineFieldDialog(getWizard(), getModule(), canHaveReferences);
        dlg.setVisible(true);
        XmlField field = dlg.getField();
        if (field != null)
            table.addRow(new Object[] {field});
    }
    
    private Collection getDefaultFields() {
        XmlModule xmlModule = getModule();

        DcModule module = null;
        if (xmlModule.getModuleClass() != null) { 
            if (xmlModule.getModuleClass().equals(DcMediaModule.class))
                module = new DcMediaModule(10, false, "", "", "", "", "", "", "");
            else if (xmlModule.getModuleClass().equals(DcPropertyModule.class))
                module = new DcPropertyModule(10, "", "", "", "", "");
        }
        
        return module != null ? module.getFields() : new ArrayList();
    }
    
    private void addDefaultFields() {
        tableSysFields.clearTable();
        Collection fields = getDefaultFields(); 
        for (Iterator iter = fields.iterator(); iter.hasNext(); )
            tableSysFields.addRow(new Object[] {iter.next()});
    }

    private void build() {
        // Actions
        JPanel panelActions = new JPanel();
        panelActions.setLayout(new FlowLayout(FlowLayout.LEFT));
        
        buttonAdd.addActionListener(new AddNewAction());
        buttonRemove.addActionListener(new DeleteAction());
        panelActions.add(buttonRemove);
        panelActions.add(buttonAdd);
        
        // Fields
        JPanel panelFields = new JPanel();
        panelFields.setLayout(Layout.getGBL());

        DcLabel labelPredefined = ComponentFactory.getLabel(DcResources.getText("lblPredefinedFields"));
        DcLabel labelNew = ComponentFactory.getLabel(DcResources.getText("lblNewFields"));
        
        tableSysFields.setColumnCount(1);
        TableColumn colSysField = tableSysFields.getColumnModel().getColumn(0);
        colSysField.setCellEditor(new DefaultCellEditor(ComponentFactory.getTextFieldDisabled()));
        colSysField.setHeaderValue(DcResources.getText("lblName"));

        JScrollPane scrollerSysFields = new JScrollPane(tableSysFields);
        scrollerSysFields.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollerSysFields.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
        
        table.setColumnCount(1);
        TableColumn colField = table.getColumnModel().getColumn(0);
        JTextField textField = ComponentFactory.getTextFieldDisabled();
        colField.setCellEditor(new DefaultCellEditor(textField));
        colField.setHeaderValue(DcResources.getText("lblName"));
        colField.setCellRenderer(new XmlFieldCellRenderer());

        JScrollPane scroller = new JScrollPane(table);
        scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
        
        JLabel labelNav = ComponentFactory.getLabel("");

        buttonUp.addActionListener(new RowUpAction());
        buttonDown.addActionListener(new RowDownAction());

        buttonUp.setPreferredSize(new Dimension(25, 22));
        buttonUp.setMaximumSize(new Dimension(25, 22));
        buttonUp.setMinimumSize(new Dimension(25, 22));
        
        buttonDown.setPreferredSize(new Dimension(25, 22));
        buttonDown.setMaximumSize(new Dimension(25, 22));
        buttonDown.setMinimumSize(new Dimension(25, 22));

        panelFields.add(labelPredefined, Layout.getGBC(0, 0, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        panelFields.add(scrollerSysFields, Layout.getGBC(0, 1, 1, 1, 50.0, 50.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(0, 0, 0, 0), 0, 0));
        panelFields.add(labelNew, Layout.getGBC(0, 2, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(10, 0, 0, 0), 0, 0));
        panelFields.add(scroller, Layout.getGBC(0, 3, 1, 4, 50.0, 50.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                new Insets(0, 0, 0, 0), 0, 0));
        panelFields.add(labelNav, Layout.getGBC(1, 3, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 5, 5, 5), 0, 0));
        panelFields.add(buttonUp, Layout.getGBC(1, 4, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(0, 5, 0, 5), 0, 0));
        panelFields.add(buttonDown, Layout.getGBC(1, 5, 1, 1, 1.0, 1.0,
                GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                new Insets(5, 5, 0, 5), 0, 0));

        table.applyHeaders();
        tableSysFields.applyHeaders();
        
        setLayout(Layout.getGBL());
        add(panelFields,    Layout.getGBC(0, 1, 1, 1, 50.0, 50.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets(5, 5, 5, 5), 0, 0)); 
        add(panelActions,   Layout.getGBC(0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
                 new Insets(5, 5, 5, 10), 0, 0)); 
    }
    
    private class AddNewAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            createField();
        }
    }

    private class DeleteAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            if (table.getSelectedRow() == -1)
                return;
            
            if (dbCheck) {
                XmlField field = (XmlField) table.getValueAt(table.getSelectedRow(), 0);
                
                if (field.isOverwritable()) {
                    QuestionBox qb = new QuestionBox(DcResources.getText("msgDeleteField"));
                    if (!qb.isAffirmative())
                        return;
                } else {
                    new MessageBox(DcResources.getText("msgFieldCannotBeRemoved"), MessageBox._INFORMATION);
                    return;
                }
            }
            
            if (table.getSelectedRow() != -1)
                table.removeRow(table.getSelectedRow());
        }
    }
    
    
    private class RowUpAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            table.moveRowUp();
        }
    }

    private class RowDownAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            table.moveRowDown();
        }
    }
}