/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.settings.DcSettings;

public class FreeResourcesTask {

    // only schedule garbage collection for the application once
    private static boolean done = false;
    private static GCStimulatorTask instance;
    
    public FreeResourcesTask() {
        int interval = DcSettings.getInt(DcRepository.settings.stGarbageCollectionIntervalMs);
        scheduleRegularGC(interval);
    }    

    private void scheduleRegularGC(long intervalMilliSecs) {
        if (!done){ 
            // only schedule 1 garbage collector per application
            GCStimulatorTask stimulator = getInstance () ;
            java.util.Timer scheduler = new java.util.Timer () ;
            scheduler.scheduleAtFixedRate (stimulator, 10, intervalMilliSecs);
            done = true ;
        }
    }
    
    private static void suggestGCNow() {
        DataManager.unloadPictures();
        ImageStore.clean();
        System.gc();
    }

    private class GCStimulatorTask extends java.util.TimerTask {
        public void run() {
            suggestGCNow();
        }
    }

    private synchronized GCStimulatorTask getInstance() {
        if (instance == null) {
            instance = new GCStimulatorTask();
        }
        return instance;
    }

}
