/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core;

import java.awt.Image;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.resources.DcResources;

public class ImageStore {

    private static final Collection references = new ArrayList();
    
    public synchronized static void addImage(Image image) {
        if (!exists(image) && image != null)
            references.add(new WeakReference(image));
    }
    
    private synchronized static boolean exists(Image image) {
        for (Iterator iter = references.iterator(); iter.hasNext(); ) {
            WeakReference reference = (WeakReference) iter.next();
            if (reference.get() != null && reference.get().equals(image))
                return true;
        }
        
        return false;
    }

    public synchronized static void clean() {
        Collection remove = new ArrayList();
        
        int collected = 0;
        for (Iterator iter = references.iterator(); iter.hasNext(); ) {
            WeakReference reference = (WeakReference) iter.next();
            Image image = (Image) reference.get();
            if (image == null) {
                remove.add(reference);
                collected++;
            } else {
                image.flush();
                reference.enqueue();
                image = null;
            }
        }
        
        DcLog.add(DcResources.getText("msgImagesInStore", new String[] {"" + references.size(), "" + collected}), DcLog._STANDARD);
        references.removeAll(remove);
        remove.clear();
    }
}
