/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.ModuleJar;
import net.sf.dc.core.modules.xml.XmlField;
import net.sf.dc.core.modules.xml.XmlModule;
import net.sf.dc.core.modules.xml.XmlObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.util.StringUtils;
import net.sf.dc.util.XMLParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Upgrades the actual module jar file. Fields can be added, removed or altered.
 * @author rj.vanderwaals
 */
public class ModuleUpgrade extends XmlObject {
    
    private File add;
    private File alter;
    private File remove;
    
    public byte[] upgrade(byte[] xml) throws ModuleUpgradeException {
        try {
            String s = new String(xml, "UTF-8");
            
            String moduleIdx = StringUtils.getValueBetween("<index>", "</index>", s);
            if (moduleIdx.equals("" + DcModules._MUSICALBUM)) {
                s = s.replaceAll("OnlineAudioCdSearchForm", "OnlineMusicAlbumSearchForm");
            }

            // tag name changes
            s = s.replaceAll("online-search-ui-class", "onlinesearch-class");
            s = s.replaceAll("import-ui-class", "importer-class");
            
            // object definitions have changed
            s = s.replaceAll("net.sf.dc.core.objects.audiocd", "net.sf.dc.core.objects.helpers");
            s = s.replaceAll("net.sf.dc.core.objects.book", "net.sf.dc.core.objects.helpers");
            s = s.replaceAll("net.sf.dc.core.objects.contactperson", "net.sf.dc.core.objects.helpers");
            s = s.replaceAll("net.sf.dc.core.objects.generic", "net.sf.dc.core.objects.helpers");
            s = s.replaceAll("net.sf.dc.core.objects.image", "net.sf.dc.core.objects.helpers");
            s = s.replaceAll("net.sf.dc.core.objects.movie", "net.sf.dc.core.objects.helpers");
            s = s.replaceAll("net.sf.dc.core.objects.musicalbum", "net.sf.dc.core.objects.helpers");
            s = s.replaceAll("net.sf.dc.core.objects.software", "net.sf.dc.core.objects.helpers");
            s = s.replaceAll("GenericObject", "Media");
            
            // class location changes
            s = s.replaceAll("net.sf.dc.processes.synchronizer", "net.sf.dc.synchronizers");
            s = s.replaceAll("net.sf.dc.console.onlinesearch", "net.sf.dc.console.windows.onlinesearch");
            
            s = s.replaceAll("net.sf.dc.console.windows.onlinesearch.OnlineSoftwareSearchForm", "net.sf.dc.onlinesearch.OnlineSoftwareSearch");
            s = s.replaceAll("net.sf.dc.console.windows.onlinesearch.OnlineAudioCdSearchForm", "net.sf.dc.onlinesearch.OnlineAudioCdSearch");
            s = s.replaceAll("net.sf.dc.console.windows.onlinesearch.OnlineBookSearchForm", "net.sf.dc.onlinesearch.OnlineBookSearch");
            s = s.replaceAll("net.sf.dc.console.windows.onlinesearch.OnlineMoviePersonSearchForm", "net.sf.dc.onlinesearch.OnlineMoviePersonSearch");
            s = s.replaceAll("net.sf.dc.console.windows.onlinesearch.OnlineMovieSearchForm", "net.sf.dc.onlinesearch.OnlineMovieSearch");
            s = s.replaceAll("net.sf.dc.console.windows.onlinesearch.OnlineMusicAlbumSearchForm", "net.sf.dc.onlinesearch.OnlineMusicAlbumSearch");

            s = s.replaceAll("DcPerson", "DcAssociate");
            
            if (moduleIdx.equals("" + DcModules._MUSICTRACK)) {
                s = s.replaceAll("<importer-class />", 
                                 "<importer-class>net.sf.dc.fileimporters.MusicAlbumImporter</importer-class>");
            }
            
            if (moduleIdx.equals("" + DcModules._MUSICARTIST)) {
                s = s.replaceAll("<onlinesearch-class></onlinesearch-class>", 
                                 "<onlinesearch-class>net.sf.dc.onlinesearch.OnlineArtistSearch</onlinesearch-class>");
                s = s.replaceAll("<synchronizer-class></synchronizer-class>", 
                                 "<synchronizer-class>net.sf.dc.synchronizers.AssociateSynchronizer</synchronizer-class>");
            }

            if (    moduleIdx.equals("" + DcModules._ACTOR) || 
                    moduleIdx.equals("" + DcModules._DEVELOPER) ||
                    moduleIdx.equals("" + DcModules._SOFTWAREPUBLISHER) || 
                    moduleIdx.equals("" + DcModules._DIRECTOR)) {
                s = s.replaceAll("<synchronizer-class></synchronizer-class>", 
                                 "<synchronizer-class>net.sf.dc.synchronizers.AssociateSynchronizer</synchronizer-class>");
            }
            
            if (moduleIdx.equals("" + DcModules._BOOK)) {
                s = s.replaceAll("<importer-class></importer-class>", "<importer-class>net.sf.dc.fileimporters.EbookImport</importer-class>");
            }
            
            if (moduleIdx.equals("" + DcModules._SOFTWAREPUBLISHER) || moduleIdx.equals("" + DcModules._DEVELOPER)) {
                s = s.replaceAll("<onlinesearch-class></onlinesearch-class>", 
                                 "<onlinesearch-class>net.sf.dc.onlinesearch.OnlineSoftwareCompanySearch</onlinesearch-class>");
            }
            
            return s.getBytes("UTF-8");

        } catch (Exception e) {
            throw new ModuleUpgradeException(e);
        }
    }
    
    public void upgrade() throws ModuleUpgradeException {
        add = new File(DataCrow.baseDir + "upgrade/add.xml");
        alter = new File(DataCrow.baseDir + "upgrade/alter.xml");
        remove = new File(DataCrow.baseDir + "upgrade/remove.xml");
        
        try {
            if (add.exists())
                add();
            
            if (alter.exists())
                alter();
            
            if (remove.exists())
                remove();
                
        } catch (Exception exp) {
            throw new ModuleUpgradeException(exp);
        }
    }

    private void save(XmlModule module) throws Exception {
        new ModuleJar(module).save();
    }
    
    private XmlField getField(int index, Collection fields) {
        for (Iterator iterCurrent = fields.iterator(); iterCurrent.hasNext(); ) {
            XmlField field = (XmlField) iterCurrent.next();
            if (field.getIndex() == index) return field;
        }
        return null;
    }
    
    private void add() throws Exception {
        Document document = read(add);
        
        Element element = document.getDocumentElement();
        NodeList nodes = element.getElementsByTagName("module");
        
        for (int i = 0; i < nodes.getLength(); i++) {
            Element module = (Element) nodes.item(i);
            String jarfile = XMLParser.getString(module, "module-jar");
            int index = XMLParser.getInt(module, "module-index");
            
            if (!new File(DataCrow.moduleDir + jarfile).exists()) continue;
            
            ModuleJar jar = new ModuleJar(DataCrow.moduleDir + jarfile);
            jar.load();
            
            // get the fields to add
            Collection upgradeFlds = getFields(module, index);
            XmlModule xmlModule = jar.getModule();
            
            for (Iterator iter = upgradeFlds.iterator(); iter.hasNext(); ) {
                XmlField field = (XmlField) iter.next();
                if (getField(field.getIndex(), xmlModule.getFields()) == null) {
                    xmlModule.getFields().add(field);
                    DcLog.add(DcResources.getText("msgUpgradedModuleXAdded", 
                                 new String[]{xmlModule.getName(), field.getName()}),
                                 DcLog._STANDARD);                    
                }
            }
            
            save(xmlModule);
        }
    }

    private void alter() throws Exception {
        Document document = read(alter);
        
        Element element = document.getDocumentElement();
        NodeList nodes = element.getElementsByTagName("module");
        
        for (int i = 0; i < nodes.getLength(); i++) {
            Element module = (Element) nodes.item(i);
            String jarfile = XMLParser.getString(module, "module-jar");
            int index = XMLParser.getInt(module, "module-index");
            
            if (!new File(DataCrow.moduleDir + jarfile).exists()) continue;
            
            ModuleJar jar = new ModuleJar(DataCrow.moduleDir + jarfile);
            jar.load();
            
            // get the fields to add
            Collection upgradeFlds = getFields(module, index);
            XmlModule xmlModule = jar.getModule();
            
            for (Iterator iter = upgradeFlds.iterator(); iter.hasNext(); ) {
                XmlField fieldNew = (XmlField) iter.next();
                XmlField fieldOrg = getField(fieldNew.getIndex(), xmlModule.getFields());
                
                if (fieldOrg == null) continue;
                
                fieldOrg.setColumn(fieldNew.getColumn());
                fieldOrg.setEnabled(fieldNew.isEnabled());
                fieldOrg.setFieldType(fieldNew.getFieldType());
                fieldOrg.setIndex(fieldNew.getIndex());
                fieldOrg.setMaximumLength(fieldNew.getMaximumLength());
                fieldOrg.setModuleReference(fieldNew.getModuleReference());
                fieldOrg.setName(fieldNew.getName());
                fieldOrg.setOverwritable(fieldNew.isOverwritable());
                fieldOrg.setReadonly(fieldNew.isReadonly());
                fieldOrg.setSearchable(fieldNew.isSearchable());
                fieldOrg.setTechinfo(fieldNew.isTechinfo());
                fieldOrg.setUiOnly(fieldNew.isUiOnly());
                fieldOrg.setValueType(fieldNew.getValueType());
                
                DcLog.add(DcResources.getText("msgUpgradedModuleXAltered", 
                        new String[]{xmlModule.getName(), fieldOrg.getName()}),
                        DcLog._STANDARD);
            }
            save(xmlModule);
        }        
    }

    private void remove() throws Exception {
        Document document = read(remove);
        
        Element element = document.getDocumentElement();
        NodeList nodes = element.getElementsByTagName("module");
        
        for (int i = 0; i < nodes.getLength(); i++) {
            Element module = (Element) nodes.item(i);
            String jarfile = XMLParser.getString(module, "module-jar");
            int index = XMLParser.getInt(module, "module-index");
            
            if (!new File(DataCrow.moduleDir + jarfile).exists()) continue;
            
            ModuleJar jar = new ModuleJar(DataCrow.moduleDir + jarfile);
            jar.load();
            
            // get the fields to add
            Collection upgradeFlds = getFields(module, index);
            XmlModule xmlModule = jar.getModule();
            
            for (Iterator iter = upgradeFlds.iterator(); iter.hasNext(); ) {
                XmlField field = (XmlField) iter.next();
                XmlField fieldOrg = getField(field.getIndex(), xmlModule.getFields());
                
                if (fieldOrg != null) {
                    xmlModule.getFields().remove(fieldOrg);
                    DcLog.add(DcResources.getText("msgUpgradedModuleXRemoved", 
                                 new String[]{xmlModule.getName(), fieldOrg.getName()}),
                                 DcLog._STANDARD);
                }
            }
            save(xmlModule);
        }        
    }
    
    private Collection getFields(Element element, int module) throws Exception {
        Collection fields = new ArrayList(); 
        NodeList nodes = element.getElementsByTagName("field");
        for (int i = 0; i < nodes.getLength(); i++) {
            Element el = (Element) nodes.item(i);
            XmlModule xmlModule = new XmlModule();
            xmlModule.setIndex(module);
            fields.add(new XmlField(xmlModule, el));
        }
        return fields;
    }    
    
    private Document read(File file) throws Exception {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        FileInputStream bis = new FileInputStream(file);
        return db.parse(bis);
    }
}
