/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import net.sf.dc.core.log.DcLog;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;
import net.sf.dc.util.Directory;
import net.sf.dc.util.Utilities;

/**
 * @deprecated 
 * This class will be removed after the 3.0 production release.
 * @author rj.vanderwaals
 *
 */
public class SystemUpgrade {

    public SystemUpgrade() {}

    public void upgrade() {
        cleanReports();
        cleanModules();
        convertSettings();
    }
    
    /**
     * Removes pre-alpha 2 module files
     */
    private void cleanModules() {
        deleteFile(DataCrow.moduleDir + "audiocds.jar");
        deleteFile(DataCrow.moduleDir + "audiocds.properties");
        deleteFile(DataCrow.baseDir + "data/audiocds.properties");
        
        deleteFile(DataCrow.moduleDir + "audiotracks.jar");
        deleteFile(DataCrow.moduleDir + "audiotracks.properties");
        deleteFile(DataCrow.baseDir + "data/audiotracks.properties");
        
        deleteFile(DataCrow.moduleDir + "books.jar");
        deleteFile(DataCrow.moduleDir + "books.properties");
        deleteFile(DataCrow.baseDir + "data/books.properties");
        
        deleteFile(DataCrow.moduleDir + "musicalbums.jar");
        deleteFile(DataCrow.moduleDir + "musicalbums.properties");
        deleteFile(DataCrow.baseDir + "data/musicalbums.properties");
        
        deleteFile(DataCrow.moduleDir + "musictracks.jar");
        deleteFile(DataCrow.moduleDir + "musictracks.properties");
        deleteFile(DataCrow.baseDir + "data/musictracks.properties");

        deleteFile(DataCrow.moduleDir + "images.jar");
        deleteFile(DataCrow.moduleDir + "images.properties");
        deleteFile(DataCrow.baseDir + "data/images.properties");

        deleteFile(DataCrow.moduleDir + "movies.jar");
        deleteFile(DataCrow.moduleDir + "movies.properties");
        deleteFile(DataCrow.baseDir + "data/movies.properties");
        
        deleteFile(DataCrow.moduleDir + "generic.jar");
        deleteFile(DataCrow.moduleDir + "generic.properties");
        deleteFile(DataCrow.baseDir + "data/generic.properties");
        
        deleteFile(DataCrow.moduleDir + "everything.jar");
        deleteFile(DataCrow.moduleDir + "everything.properties");
        deleteFile(DataCrow.baseDir + "data/everything.properties");
        
        deleteFile(DataCrow.moduleDir + "contactpersons.jar");
        deleteFile(DataCrow.moduleDir + "contactpersons.properties");
        deleteFile(DataCrow.baseDir + "data/contactpersons.properties");
        
        deleteFile(DataCrow.moduleDir + "softwareitem.jar");
        deleteFile(DataCrow.moduleDir + "softwareitem.properties");
        deleteFile(DataCrow.baseDir + "data/softwareitem.properties");

        if (!new File(DataCrow.baseDir + "data/").exists())
            return;
        
        Collection c = Directory.read(DataCrow.baseDir + "data/", false, false, false, new String[] {"properties"}, false);
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            String file = (String) iter.next();
            if (file.toLowerCase().endsWith("_report.properties"))
                deleteFile(file);
        }
    }
    
    private void deleteFile(String name) {
        try {
            new File(name).delete();
        } catch (Exception e) {}
    }
    
    /**
     * Convert the report files to the new directory structure (Data Crow 2 > Data Crow 3)
     */
    private void cleanReports() {
        Map mapping = new HashMap();
        
        mapping.put("audiocds/", "audiocd/");
        mapping.put("books/", "book/");
        mapping.put("musicalbums/", "musicalbum/");
        mapping.put("generic/", "media/");
        mapping.put("everything/", "media/");
        mapping.put("contactpersons/", "contactperson/");
        mapping.put("softwareitem/", "software/");
        
        // copy the files to the new directory
        try {
            for (Iterator iter = mapping.keySet().iterator(); iter.hasNext(); ) {
                String oldName = (String) iter.next();
                String newName = (String) mapping.get(oldName);
                String oldDir = DataCrow.reportDir + oldName;
    
                File file = new File(oldDir);
                if (file.exists()) {
                    Collection files = Directory.read(oldDir, false, true, false, null, false);
                    for (Iterator iterFiles = files.iterator(); iterFiles.hasNext(); ) {
                        String filename = (String) iterFiles.next();
                        
                        while (filename.indexOf("\\") > -1)
                            filename = filename.replace('\\', '/');
                        
                        String filenameNew = filename.replaceAll(oldName, newName);
                        File oldFile = new File(filename);
                        
                        byte[] content = Utilities.readFile(oldFile);
                        new File(filenameNew.substring(0, filenameNew.lastIndexOf('/'))).mkdirs();
                        Utilities.writeToFile(content, filenameNew);
                        oldFile.delete();
                    }
                    
                    new File(oldDir).delete();  
                    new File(oldDir + "html").delete();                
                    new File(oldDir + "pdf").delete();
                }
            }
        } catch (Exception e) {
            DcLog.add(e);
        }
    }
    
    private void convertSettings() {
        try {
            Properties p = new Properties();
            p.load(new FileInputStream(DataCrow.baseDir + "data/data_crow.properties"));
    
            String interval = (String) p.get(DcRepository.settings.stGarbageCollectionIntervalMs);
            try {
                if (Integer.valueOf(interval).intValue() < 240000)
                    p.setProperty(DcRepository.settings.stGarbageCollectionIntervalMs, "240000");
            } catch (Exception e) {
                p.setProperty(DcRepository.settings.stGarbageCollectionIntervalMs, "240000");
            }
            
            convertFieldSettings(p.getProperty("Software_Field_Definitions"), "software.properties");
            convertFieldSettings(p.getProperty("AudioCD_Field_Definitions"), "audiocd.properties");
            convertFieldSettings(p.getProperty("AudioTrack_Field_Definitions"), "audiotrack.properties");
            convertFieldSettings(p.getProperty("Book_Field_Definitions"), "book.properties");
            convertFieldSettings(p.getProperty("Image_Field_Definitions"), "image.properties");
            convertFieldSettings(p.getProperty("Movie_Field_Definitions"), "movie.properties");
            convertFieldSettings(p.getProperty("Music_Album_Field_Definitions"), "musicalbum.properties");
            convertFieldSettings(p.getProperty("Music_Track_Field_Definitions"), "musictrack.properties");
            convertFieldSettings(p.getProperty("Person_Field_Definitions"), "contactperson.properties");
            
            FileOutputStream fos = new FileOutputStream(DataCrow.baseDir + "data/data_crow.properties");
            p.store(fos, "");
            
        } catch (Exception e) {}
    }    

    /**
     * Preserve the labels from the old settings
     * 
     * @param old
     * @param filename
     * @throws IOException
     */
    private void convertFieldSettings(String oldValue, String filename) throws IOException {
        
        if (oldValue == null) return;
        
        String old = oldValue;
        
        Properties p = new Properties();
        FileInputStream fis = new FileInputStream(DataCrow.baseDir + "data/" + filename);
        p.load(fis);
        
        String newValue = (String) p.get(DcRepository.modulesettings.stFieldDefinitions);
        
        int group = old.indexOf("}");
        DcFieldDefinitions oldDefs = new DcFieldDefinitions();
        while (group > -1) {
            String value = old.substring(1, group);
            oldDefs.addDefinition(value);
            if (old.length() > group + 1) {
                old = old.substring(group + 1, old.length());
                group = old.indexOf("}");
            } else {
                group = -1;
            }
        }        
        
        group = newValue.indexOf("}");
        DcFieldDefinitions newDefs = new DcFieldDefinitions();
        while (group > -1) {
            String value = newValue.substring(1, group);
            newDefs.addDefinition(value);
            if (newValue.length() > group + 1) {
                newValue = newValue.substring(group + 1, newValue.length());
                group = newValue.indexOf("}");
            } else {
                group = -1;
            }
        }        

        for (Iterator iterOld = oldDefs.iterator(); iterOld.hasNext(); ) {
            DcFieldDefinition oldDef = (DcFieldDefinition) iterOld.next();
            oldDef.setIndex(oldDef.getIndex() < 6 ? oldDef.getIndex() + 149 : oldDef.getIndex() - 5);
        }
        
        String definitions = "";
        for (Iterator iterNew = newDefs.iterator(); iterNew.hasNext(); ) {
            DcFieldDefinition newDef = (DcFieldDefinition) iterNew.next();

            boolean found = false;
            for (Iterator iterOld = oldDefs.iterator(); iterOld.hasNext(); ) {
                DcFieldDefinition oldDef = (DcFieldDefinition) iterOld.next();
                
                if (oldDef.getIndex() == newDef.getIndex()) {
                    oldDef.isDescriptive(newDef.isDescriptive());
                    definitions += "{" + oldDef.toSettingValue() + "}";
                    found = true;
                }
            }
            
            if (!found) 
                definitions += "{" + newDef.toSettingValue() + "}";
        }
        
        fis.close();
        
        FileOutputStream fos = new FileOutputStream(DataCrow.baseDir + "data/" + filename);
        
        p.setProperty(DcRepository.modulesettings.stFieldDefinitions, definitions);
        p.store(fos, "");
        fos.close();
    }
}
