/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import net.sf.dc.core.DcRepository;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;

public class DataFilterEntry {

    public static final String _AND = DcResources.getText("lblAnd");
    public static final String _OR = DcResources.getText("lblOr");
    
    private int module;
    private int field;

    private Operator operator;
    private Object value; 
    
    private String andOr;
    
    public DataFilterEntry(String andOr, int module, int field, Operator operator, Object value) {
        this.module = module;
        this.field = field;
        this.operator = operator;
        this.value = value;
        this.andOr = andOr;
    }
    
    public String getAndOr() {
        return andOr;
    }

    public boolean isOr() {
        return andOr.equals(_OR);
    }

    public boolean isAnd() {
        return andOr.equals(_AND);
    }

    public int getField() {
        return field;
    }

    public int getModule() {
        return module;
    }

    public Operator getOperator() {
        return operator;
    }

    public Object getValue() {
        return value;
    }
    
    private boolean isEmpty(Object o) {
    	boolean isEmpty = (o == null || o.toString().equals("-1") || o.equals(""));
    	isEmpty = (!isEmpty && o instanceof Collection ? ((Collection) o).size() == 0 : isEmpty);
    	return isEmpty;
    }
    
    public boolean applies(DcObject dco) {

        Object o = dco.getValue(field);
        boolean isInputEmpty = isEmpty(o); // o == null || o.toString().equals("-1") || o.equals("");
        boolean isCheckEmpty = isEmpty(value); // == null || value.toString().equals("-1") || value.equals("");
        
        String input = isInputEmpty ? "" : o.toString().toLowerCase();
        String check = isCheckEmpty ? "" : value.toString().toLowerCase();
        
        if (operator.getIndex() == Operators._CONTAINS) {
            if (dco.getField(field).getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION) {
                Collection c1 = (Collection) o;
                Collection c2 = (Collection) value;
                return contains(c1, c2);
            } else { 
                return input.indexOf(check) > -1 || input.equals(check);
            }
        }
        
        if (operator.getIndex() == Operators._DOES_NOT_CONTAIN) {
            if (dco.getField(field).getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION) {
                Collection c1 = (Collection) o;
                Collection c2 = (Collection) value;
                return !contains(c1, c2);
            } else { 
                return input.indexOf(check) == -1;
            }
        }        
         
        if (operator.getIndex() == Operators._ENDS_WITH)
            return input.endsWith(check);

        if (operator.getIndex() == Operators._EQUAL_TO) {
            if (dco.getField(field).getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION) {
                Collection c1 = (Collection) o;
                
                Collection c2;
                if (value instanceof Collection) {
                    c2 = (Collection) value;
                } else {
                    c2 = new ArrayList();
                    c2.add(value);
                }

                return equals(c1, c2);
            } else { 
                return input.equals(check);
            }
        }

        if (operator.getIndex() == Operators._GREATER_THEN) {
            if (dco.getField(field).getValueType() == DcRepository.datatypes._DATE) {
                return isInputEmpty ? false : ((Date) o).after((Date) value);
            } else {
                return isInputEmpty ? false : 
                    Integer.valueOf(input).intValue() > Integer.valueOf(check).intValue();
            }
        }

        if (operator.getIndex() == Operators._IS_EMPTY)
            return isInputEmpty;

        if (operator.getIndex() == Operators._IS_FILLED)
            return !isInputEmpty;
         
        if (operator.getIndex() == Operators._LESS_THEN) {
            if (dco.getField(field).getValueType() == DcRepository.datatypes._DATE) {
                return isInputEmpty ? false : ((Date) o).before((Date) value);
            } else {
                return isInputEmpty ? false : 
                    Integer.valueOf(input).intValue() < Integer.valueOf(check).intValue();
            }
        }

        if (operator.getIndex() == Operators._NOT_EQUAL_TO) {
            if (dco.getField(field).getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION) {
                Collection c1 = (Collection) o;
                Collection c2 = (Collection) value;
                return !equals(c1, c2);
            } else { 
                return !input.equals(check);
            }            
        }

        if (operator.getIndex() == Operators._STARTS_WITH)
            return input.startsWith(check);
        
        return true;
    }

    private boolean contains(Collection c1, Collection c2) {
        boolean contains = false;
        
        if (c2 == null) {
            contains = true;
        } else if (c1 != null) {
            contains = true;
            for (Iterator iter2 = c2.iterator(); iter2.hasNext(); ) {
                boolean b = false;
                DcObject dco2 = (DcObject) iter2.next();
                for (Iterator iter1 = c1.iterator(); iter1.hasNext(); ) {
                    DcObject dco1 = (DcObject) iter1.next();
                    if (dco1.toString().equals(dco2.toString()))
                        b = true; 
                }
                contains &= b; 
            }
        }

        return contains;
    }
    
    private boolean equals(Collection c1, Collection c2) {
        boolean equals = (c2 == null && c1 == null) || (c2 != null || c1 != null) ? true :
                          c2.size() != c1.size();      

        if (equals) {
            for (Iterator iter2 = c2.iterator(); iter2.hasNext(); ) {
                DcObject dco2 = (DcObject) iter2.next();
                boolean b = false;
                for (Iterator iter1 = c1.iterator(); iter1.hasNext(); ) {
                    DcObject dco1 = (DcObject) iter1.next();
                    if (dco1.toString().equals(dco2.toString()))
                        b = true; 
                }
                equals &= b;
            }
        }

        return equals;
    }     
}
