/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.data;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.util.Utilities;

public class DataFilters {
    
    private static final Map filters = new HashMap(); 
    
    private static final Map activeFilters = new HashMap();
    
    private static final String filename = DataCrow.baseDir + "data" + File.separator + "filters.xml";
    
    public DataFilters() {}
    
    /**
     * Loads the settings file
     */
    public void load() {
        
        if (!new File(filename).exists())
            return;
        
        try {
            byte[] b = Utilities.readFile(new File(filename));
            String xml = new String(b, "UTF-8");
            
            while (xml.indexOf("<FILTER>") > -1) {
                String part = xml.substring(xml.indexOf("<FILTER>"), xml.indexOf("</FILTER>") + 9);
                DataFilter df = new DataFilter(part);
                
                Collection c = (Collection) filters.get(new Integer(df.getModule()));
                c = c == null ? new ArrayList() : c;
                c.add(df);
                filters.put(new Integer(df.getModule()), c);
                
                xml = xml.substring(xml.indexOf("</FILTER>") + 9, xml.length());
            }
        } catch (Exception exp) {
            DcLog.add(exp);
        }
    }
    
    public static void setCurrent(int module, DataFilter df) {
    	activeFilters.put(new Integer(module), df);
    }
    
    public static DataFilter getDefaultDataFilter(int module) {
        DataFilter filter = new DataFilter(module);
        filter.setOrder(DcModules.get(module).getSettings().getStringArray(DcRepository.modulesettings.stSearchOrder));
        return filter;
    }
    
    public static DataFilter getCurrent(int module) {
    	DataFilter df = (DataFilter) activeFilters.get(new Integer(module));
    	if (df == null)
    	    df = getDefaultDataFilter(module);
    		
    	return df;
    }
    
    public void save() {
        String xml = "<FILTERS>\n";
        for (Iterator iter = filters.values().iterator(); iter.hasNext(); ) {
            Collection c = (Collection) iter.next();
            for (Iterator iterFilters = c.iterator(); iterFilters.hasNext(); ) {
                DataFilter df = (DataFilter) iterFilters.next();
                xml += "\n" + df.toStorageString();
            }
        }
        xml += "</FILTERS>";
        
        try {
            FileOutputStream fos = new FileOutputStream(filename);
            fos.write(xml.getBytes("UTF-8"));
            fos.close();
        } catch (Exception exp) {
            DcLog.add(exp);  
        }         
    }
    
    public static void delete(DataFilter df) {
        Collection c = get(df.getModule());
        c.remove(df);
    }
    
    public static Collection get(int module) {
        Collection c = (Collection) filters.get(new Integer(module));
        return c != null ? c : new ArrayList();
    }
    
    public static void add(DataFilter df) {
        Collection c = (Collection) filters.get(new Integer(df.getModule()));
        c = c == null ? new ArrayList() : c;
        c.remove(df);
        c.add(df);
        filters.put(new Integer(df.getModule()), c);
    }
}
