/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import net.sf.dc.core.DcRepository;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.resources.DcResources;

public class Operators {

    public static final int _EQUAL_TO = 1;
    public static final int _NOT_EQUAL_TO = 2;
    public static final int _CONTAINS = 3;
    public static final int _DOES_NOT_CONTAIN = 10;
    
    public static final int _IS_EMPTY = 4;
    public static final int _IS_FILLED = 5;
    
    public static final int _STARTS_WITH = 6;
    public static final int _ENDS_WITH = 7;
    public static final int _LESS_THEN = 8;
    public static final int _GREATER_THEN = 9;

    private static Map operators = new HashMap();
    
    static {
        operators.put(new Integer(_EQUAL_TO), new Operator(_EQUAL_TO, DcResources.getText("lblEqualTo"), true));
        operators.put(new Integer(_NOT_EQUAL_TO), new Operator(_NOT_EQUAL_TO, DcResources.getText("lblNotEqualTo"), true));
        operators.put(new Integer(_CONTAINS), new Operator(_CONTAINS, DcResources.getText("lblContains"), true));
        operators.put(new Integer(_DOES_NOT_CONTAIN), new Operator(_DOES_NOT_CONTAIN, DcResources.getText("lblDoesNotContain"), true));
        operators.put(new Integer(_IS_EMPTY), new Operator(_IS_EMPTY, DcResources.getText("lblIsEmpty"), false));
        operators.put(new Integer(_IS_FILLED), new Operator(_IS_FILLED, DcResources.getText("lblIsFilled"), false));
        operators.put(new Integer(_STARTS_WITH), new Operator(_STARTS_WITH, DcResources.getText("lblStartsWith"), true)); 
        operators.put(new Integer(_ENDS_WITH), new Operator(_ENDS_WITH, DcResources.getText("lblEndsWith"), true));
        operators.put(new Integer(_LESS_THEN), new Operator(_LESS_THEN, DcResources.getText("lblLessThen"), true));
        operators.put(new Integer(_GREATER_THEN), new Operator(_GREATER_THEN, DcResources.getText("lblGreaterThen"), true));
    }
    
    public static Operator getDefault() {
        return getOperator(_EQUAL_TO);
    }
    
    public static Operator getOperator(int index) {
        return (Operator) operators.get(new Integer(index));
    }
    
    public static Collection get(DcField field) {
        Collection operators = new ArrayList();

        operators.add(getOperator(_EQUAL_TO));
        operators.add(getOperator(_NOT_EQUAL_TO));
        
        if (field.getValueType() == DcRepository.datatypes._LONG ||
            field.getValueType() == DcRepository.datatypes._DATE) {
            
            operators.add(getOperator(_LESS_THEN));
            operators.add(getOperator(_GREATER_THEN));
        
        } else if (field.getValueType() != DcRepository.datatypes._BOOLEAN) {
            
            operators.add(getOperator(_IS_EMPTY));
            operators.add(getOperator(_IS_FILLED));
            operators.add(getOperator(_CONTAINS));
            operators.add(getOperator(_DOES_NOT_CONTAIN));
            operators.add(getOperator(_STARTS_WITH));
            operators.add(getOperator(_ENDS_WITH));
        }
        
        return operators;
    }
}
