/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.wf.WorkFlow;
import net.sf.dc.core.wf.requests.IRequest;
import net.sf.dc.core.wf.requests.RequestCollection;
import net.sf.dc.core.wf.requests.SynchronizeWithManagerRequest;
import net.sf.dc.settings.DcSettings;

public class DatabaseManager {

    public static DcDatabase db = null;
    public static boolean isServerClientMode = false;
    
    public static void initialize() {
        db = new DcDatabase();

        Connection connection = null;
        while (connection == null) {
            connection = getConnection();
        }

        try {
            db.initiliaze(connection);
        } catch (Exception exp) {
            exp.printStackTrace();
            DcLog.add("Could not find or connect to the database!", DcLog._ERROR);
            DcLog.add(exp);
            new MessageBox("Could not find or connect to the database!", MessageBox._ERROR);
        }
    }

    public static int getQueueSize() {
    	return db.getQueueSize();
    }
    
    public static void applySettings() {
        db.setDbProperies(getConnection());
    }
    
    public static void closeDatabases(boolean compact) {
        try {
            if (db != null) {
                Connection connection = getConnection();
                Statement stmt = connection.createStatement();
                
                if (!isServerClientMode) {
                    if (compact)
                        stmt.executeUpdate("SHUTDOWN COMPACT");
                    else 
                        stmt.executeUpdate("SHUTDOWN");
                }
                
                connection.close();
            }
        } catch (Exception exp) {
            DcLog.add(exp);
        }
    }

    public static Connection getConnection() {
        try {
            String name = db.getName();
            Class.forName(DcSettings.getString(DcRepository.settings.stDatabaseDriver));

            String address;
            if (name.startsWith("//") || name.startsWith("\\\\")) {
                isServerClientMode = true;
                address = "jdbc:hsqldb:hsql:" + name;
            } else {
                isServerClientMode = false;
                address = "jdbc:hsqldb:" + DataCrow.baseDir +  "data/" + name;
            }
                
            Connection connection = DriverManager.getConnection(address, "SA", "");
            connection.setAutoCommit(true);

            return connection;
        } catch (Exception exp) {
            DcLog.add(exp);
            new MessageBox(exp.getMessage(), MessageBox._ERROR);
        }

        System.exit(1);
        return null;
    }
    
    public static Collection executeQuery(String sql, int type, boolean log) {
        try {
            return executeQuery(DatabaseManager.getConnection().prepareStatement(sql), type, log);
        } catch (SQLException exp) {
            DcLog.add(exp);
        }
        return null;
    }     
    
    public static Collection executeQuery(DcObject dco, boolean log) throws SQLException {
        Query query = new Query(Query._SELECT, dco, null, null);
        DcLog.add(query.getQuery());
        return executeQuery(query.getQuery(), Query._SELECT, log);
    }
    
    public static Collection executeQuery(Query query, boolean logQuery) {
        Collection data = new ArrayList();
        for (Iterator iter = query.getQueries().iterator(); iter.hasNext(); ) {
            PreparedStatement ps = (PreparedStatement) iter.next();
            Collection c = executeQuery(ps, query.getType(), logQuery);
            if (c != null) data.addAll(c);
        }
        
        RequestCollection rc = query.getRequestors();
        if (rc != null) {
            IRequest[] requests = rc.getRequests();
            for (int i = 0; i < requests.length; i++) {
                requests[i].execute(data);
            }
        }
        
        return data;
    }      
    
    public static ResultSet executeSQL(String sql, boolean log) throws Exception {
        if (log)
            DcLog.add(sql, DcLog._QUERY);

        Connection connection = getConnection();
        Statement stmt = connection.createStatement();
        return stmt.executeQuery(sql);
    }
    
    public static Collection executeQuery(PreparedStatement ps, int type, boolean log) {
        Collection data = null;
        
        try {
            if (    type != Query._SELECT && type != Query._SELECTOR && 
                    type != Query._SELECTPRECISE && type != Query._SELECTPRECISEOR &&
                    type != Query._UNDEFINED) {
                
                ps.execute();
            } else {
                ResultSet result = ps.executeQuery();
                data = new WorkFlow().convertToDCObjects(result);
                result.close();
            }
        } catch (SQLException se) {
            if (!se.getMessage().equals("No ResultSet was produced"))
                DcLog.add(se);
        }

        if (log) DcLog.add(ps);

        return data;
    }

    public static void updateValues(DcObject dco) {
        try {
            boolean isChanged = dco.isChanged();
            if (isChanged) {
                Query query = new Query( Query._UPDATE, dco, null, dco.getRequests());
    
                if (dco.isBatch())
                    query.setBatch(dco.isEndOfBatch());
    
                query.setSilence(dco.getSilence());
                db.addQuery(query);
            }
    
            Collection c = dco.getChildren();
            Collection children = new ArrayList();
            children.addAll(c);
            int counter = 1;
            for (Iterator iter = children.iterator(); iter.hasNext(); counter++) {
                DcObject child = (DcObject) iter.next();
                if (child.isChanged()) {
    
                    child.addRequest(new SynchronizeWithManagerRequest(
                            SynchronizeWithManagerRequest._UPDATE, child));
                    
                    boolean exists = false;
                    if (child.getID() != null && child.getID().length() > 0) {
                        DcObject childTest = child.getModule().getDcObject();
                        childTest.setValue(DcObject._ID, child.getID());
                        Collection objects = executeQuery(childTest, true);
                        exists = objects.size() > 0;
                    }
    
                    Query query;
                    if (!exists)
                        query = new Query(Query._INSERT, child, null, child.getRequests());
                    else
                        query = new Query(Query._UPDATE, child, null, child.getRequests());
    
                    if (!isChanged) {
                        query.setBatch(counter == children.size());
                        query.setSilence(counter != children.size());
                    } else {
                        query.setSilence(true);
                    }
                    db.addQuery(query);
                }
            }
        } catch (SQLException se) {
            DcLog.add(se);
        }            
    }

    public static void insertValues(DcObject dco) {
        try {
            if (dco.getID() == null || dco.getID().equals(""))
                dco.setIDs();
    
            Query query = new Query(Query._INSERT, dco, null, dco.getRequests());
            if (dco.isBatch())
                query.setBatch(dco.isEndOfBatch());
    
            query.setSilence(dco.getSilence());
            db.addQuery(query);
    
            Collection children = dco.getChildren();
            for (Iterator iter = children.iterator(); iter.hasNext(); ) {
                DcObject child = (DcObject) iter.next();
                
                child.addRequest(new SynchronizeWithManagerRequest(
                        SynchronizeWithManagerRequest._ADD, child));
                
                child.setValue(child.getParentReferenceFieldIndex(), dco.getID());
                query = new Query(Query._INSERT, child, null, child.getRequests());
                query.setSilence(true);
                db.addQuery(query);
            }
        } catch (SQLException se) {
            DcLog.add(se);
        }            
    }

    public static  void deleteValues(DcObject dco) {
        try {
            Query query = new Query(Query._DELETE, dco, null, dco.getRequests());
            if (dco.isBatch())
                query.setBatch(dco.isEndOfBatch());
        
            query.setSilence(dco.getSilence());
            db.addQuery(query);
        } catch (SQLException se) {
            DcLog.add(se);
        }
    }

    public static boolean isUnique(DcObject o, boolean isUpdate) {
        if (o.hasPrimaryKey() && !o.getModule().isChildModule()) {
            boolean hasRequiredFields = false;
            DcObject dco = o.getModule().getDcObject();

            int[] fields = o.getFieldIndices();
            for (int i = 0 ; i < fields.length; i++) {
                int field = fields[i];
                if (o.isRequired(field)) {
                    hasRequiredFields = true;
                    dco.setValue(field, o.getValue(field));
                }
            }

            if (hasRequiredFields) {
                DataFilter df = new DataFilter(dco);
                DcObject[] objects = DataManager.get(o.getModule().getIndex(), df);
                
                int count = 0;
                for (int i = 0; i < objects.length; i++) {
                	count = !isUpdate || !objects[i].getID().equals(o.getID()) ? count + 1 : count;
                }

                if (count > 0)
                    return false;
            }
        }
        return true;
    }
}

