/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.util.Collection;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.windows.itemforms.DcMinimalisticItemView;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.db.DatabaseManager;
import net.sf.dc.core.db.Query;
import net.sf.dc.core.db.QueryOptions;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.xml.XmlModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcProperty;

public class DcPropertyModule extends DcModule {

    protected DcMinimalisticItemView form;
    
    private boolean isServingMultipleModules = false;
    
    public DcPropertyModule(XmlModule xmlModule) {
        super(xmlModule);
        setServesMultipleModules(xmlModule.isServingMultipleModules());
    }  
    
    public DcPropertyModule(int index, String name, String tableName, String tableShortName, 
                          String objectName, String objectNamePlural) {
        super(index, false, name, "", objectName, objectNamePlural, tableName, tableShortName, "");
    }    

    public DcMinimalisticItemView getForm() {
        initializeUI();
        return form;
    }
    
    public void setServesMultipleModules(boolean b) {
        isServingMultipleModules = b;
    }
    
    public boolean isServingMultipleModules() {
        return isServingMultipleModules;
    }
    
    public Collection loadData() {
        try {
            QueryOptions options = new QueryOptions(new DcField[] {getField(DcProperty._A_NAME)}, true, false);
            Query qry = new Query(Query._SELECT, getDcObject(), options, null);
            return DatabaseManager.executeQuery(qry, true);
        } catch (Exception e) {
            DcLog.add(e);
        }
        return null;
    }    
    
    public int getDefaultSortFieldIdx() {
        return DcProperty._A_NAME;
    }    
    
    public void initializeUI() {
        if (form == null)
            form = new DcMinimalisticItemView(getIndex(), false);
    }
    
    public DcObject getDcObject() {
        return new DcProperty(getIndex());
    }
    
    protected void initializeFields() {
        super.initializeFields();
        addField(new DcField(DcProperty._A_NAME, getIndex(), "Name", 
                             false, true, false, true, false,
                             255, ComponentFactory._SHORTTEXTFIELD, getIndex(), DcRepository.datatypes._STRING,
                             "Name"));
        addField(new DcField(DcProperty._B_ICON, getIndex(), "Icon", 
                             false, true, true, false, false,
                             255, ComponentFactory._PICTUREFIELD, getIndex(), DcRepository.datatypes._ICON,
                             "Icon"));
        
        getField(DcObject._ID).setEnabled(false);
    }   
}
