/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.ModuleUpgradeException;
import net.sf.dc.core.modules.xml.XmlModule;
import net.sf.dc.core.modules.xml.XmlModuleWriter;
import net.sf.dc.util.StringUtils;
import net.sf.dc.util.Utilities;

public class ModuleJar {

    private String filename;
    
    private XmlModule module;
    
    public ModuleJar(XmlModule module) {
        this.module = module;
    }
    
    public ModuleJar(String filename) {
        this.filename = filename;
    }

    public XmlModule getModule() {
        return module;
    }
    
    private void addEntry(ZipOutputStream zout, String name, byte[] bytes) throws IOException {
        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
        
        ZipEntry ze = new ZipEntry(name);
        zout.putNextEntry(ze);
        
        byte b[] = new byte[512];
        int len = 0;
        while ((len = bais.read(b)) != -1) {
            zout.write(b, 0, len);
        }
        
        zout.closeEntry();
    }
    
    public void save() throws ModuleJarException {
        try {
            byte[] icon16 = module.getIcon16();
            byte[] icon32 = module.getIcon32();
            
            icon16 = icon16 == null ? Utilities.getBytes(IconLibrary._icoPicture) : icon16;
            icon32 = icon32 == null ? Utilities.getBytes(IconLibrary._icoPicture) : icon32;

            String moduleName = StringUtils.normalize(module.getName());
            filename = DataCrow.moduleDir + moduleName + ".jar";

            FileOutputStream fos = new FileOutputStream(filename);
            ZipOutputStream zout = new ZipOutputStream(fos);
            
            module.setIcon16Filename("icon16.png");
            module.setIcon32Filename("icon32.png");
            
            XmlModuleWriter writer = new XmlModuleWriter(module);
            byte[] xml  = writer.getXML();
            
            writer.close();
            
            addEntry(zout, module.getFilename(), xml);
        	addEntry(zout, "icon16.png", icon16);
        	addEntry(zout, "icon32.png", icon32);

            zout.close();

        } catch (Exception exp) {
            throw new ModuleJarException(exp);
        }
    }
    
    
    public void load() throws ModuleUpgradeException, ModuleJarException, InvalidModuleXmlException {
        
        try {
            ZipFile zf = new ZipFile(filename);

            Map content = new HashMap();
            Enumeration list = zf.entries();
            while (list.hasMoreElements()) {
                ZipEntry ze = (ZipEntry)list.nextElement();

                BufferedInputStream bis = new BufferedInputStream(zf.getInputStream(ze));
                int size = (int) ze.getSize();
                byte[] bytes = new byte[size];
                bis.read(bytes);
                
                String filename = ze.getName();
                content.put(filename, bytes);
                
                bis.close();
            }        
            
            // first get the xml file
            for (Iterator iter = content.keySet().iterator(); iter.hasNext(); ) {
                String file = (String) iter.next();
                if (file.toLowerCase().endsWith("xml")) {
                    module = new XmlModule((byte[]) content.get(file));
                    module.setFilename(file);
                }
            }
            
            byte[] icon16 = (byte[]) content.get(module.getIcon16Filename());
            byte[] icon32 = (byte[]) content.get(module.getIcon32Filename());
            
            module.setIcon16(icon16);
            module.setIcon32(icon32);
        
        } catch (NullPointerException e) {
            throw new ModuleJarException(e);
        } catch (FileNotFoundException e) {
            throw new ModuleJarException(e);
        } catch (IOException e) {
            throw new ModuleJarException(e);
        }
    }
}
