/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.windows.itemforms.DcMinimalisticItemView;
import net.sf.dc.console.windows.itemforms.TemplateForm;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcTemplate;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;

public class TemplateModule extends DcModule {
    
    private TemplateForm form;
    private DcModule parent;
    
    public TemplateModule(DcModule parent) {
        super(parent.getIndex() + DcModules._TEMPLATE, 
              false, 
              DcResources.getText(DcRepository.systemlabels.sysTemplate), 
              "",
              DcResources.getText("sysTemplate"),
              DcResources.getText("sysTemplatePlural"),
              parent.getTableName() + "_template", 
              parent.getTableShortName() + "temp", 
              parent.getTableName() + "_template");
        
        this.parent = parent;
        
        Collection fields = parent.getFields();
        for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            addField(new DcField(field.getIndex(), getIndex(), field.getLabel(), field.isUiOnly(),
                                 field.isEnabled(), field.isReadOnly(), field.isSearchable(), field.isTechnicalInfo(),
                                 field.getMaximumLength(), field.getFieldType(), field.getSourceModuleIdx(), field.getValueType(),
                                 field.getDatabaseFieldName()));
        }
    }
    
//    public Collection loadData() {
//        try {
//            return DatabaseManager.executeQuery(
//                    "Select * from " + getDcObject().getTableName(), net.sf.dc.core.db.Query._SELECT, true);
//        } catch (Exception e) {
//            DcLog.add(e);
//        }
//        return null;
//    }       
    
    public DcModule getTemplatedModule() {
        return parent;
    }
    
    public ImageIcon getIcon16() {
        return IconLibrary._icoTemplate;
    }

    public ImageIcon getIcon32() {
        return IconLibrary._icoTemplate;
    }        
    
    public DcMinimalisticItemView getForm() {
        initializeUI();
        return form;    
    }  

    public DcObject getDcObject() {
        return new DcTemplate(getIndex(), parent.getIndex());
    }    
    
    public String getObjectName() {
        return DcResources.getText(DcRepository.systemlabels.sysTemplate);
    }

    public String getObjectNamePlural() {
        return DcResources.getText(DcRepository.systemlabels.sysTemplatePlural);
    }
    
    public DcFieldDefinitions getFieldDefinitions() {
        DcFieldDefinitions fds = parent.getFieldDefinitions();
        DcFieldDefinitions definitions = new DcFieldDefinitions();
        
        definitions.addDefinition(new DcFieldDefinition(DcTemplate._SYS_TEMPLATENAME, null, true, true, true, true));
        definitions.addDefinition(new DcFieldDefinition(DcTemplate._SYS_DEFAULT, null, true, true, true, true));
        
        for (Iterator iter = fds.iterator(); iter.hasNext(); ) {
            DcFieldDefinition fd = (DcFieldDefinition) iter.next();
            definitions.addDefinition(new DcFieldDefinition(fd.getIndex(), fd.getLabel(), fd.isEnabled(), false, false ,fd.isVisible()));
        }
        
        return definitions;
    }      

    protected void initializeFields() {
        super.initializeFields();
        addField(new DcField(DcTemplate._SYS_TEMPLATENAME, getIndex(), "Template Name",
                             false, true, false, true, false,
                             255, ComponentFactory._SHORTTEXTFIELD, getIndex(), DcRepository.datatypes._STRING,
                             "TemplateName"));
        addField(new DcField(DcTemplate._SYS_DEFAULT, getIndex(), "Default",
                             false, true, false, true, false,
                             255, ComponentFactory._CHECKBOX, getIndex(), DcRepository.datatypes._BOOLEAN,
                             "DefaultTemplate"));        
    }

    protected void initializeUI() {
        if (form == null)
            form = new TemplateForm(getIndex(), false);
    }
}
