/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects;

import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.xml.XmlField;
import net.sf.dc.enhancers.IValueEnhancer;

public class DcField {

    private String systemName;
    private String label;
    private boolean enabled;
    private boolean required;
    private boolean hidden;
    private boolean searchable;
    private boolean readonly;
    private boolean techinfo;
    private boolean uiOnly;
    private int maximumLength;
    private int fieldType;
    private int valueType;
    private int index;
    private int module;
    private int sourceModuleIdx;
    private String databaseFieldName;
    
    private Collection enhancers = new ArrayList();

    public DcField(XmlField field, int module) {
        this(field.getIndex(), module, field.getName(), field.isUiOnly(), field.isEnabled(),
             field.isReadonly(), field.isSearchable(), field.isTechinfo(), field.getMaximumLength(),
             field.getFieldType(), field.getModuleReference(), field.getValueType(), field.getColumn());
    }
    
    public DcField( int index,
                    int module,
                    String label,
                    boolean uiOnly,
                    boolean enabled,
                    boolean readonly,
                    boolean searchable,
                    boolean techinfo,
                    int maximumLength,
                    int fieldType,
                    int modRef,
                    int valueType,
                    String databaseFieldName) {

        setEnabled(enabled);
        setIndex(index);
        setModule(module);
        setLabel(label);
        setSystemName(label);
        setReadOnly(readonly);
        setSearchable(searchable);
        setTechinfo(techinfo);
        setMaximumLength(maximumLength);
        setFieldType(fieldType);
        setValueType(valueType);
        setDatabaseFieldName(databaseFieldName);
        setUiOnly(uiOnly);
        setSourceModuleIdx(modRef);
    }
    
    public void setSourceModuleIdx(int modRef) {
        this.sourceModuleIdx = modRef;
    }

    public int getSourceModuleIdx() {
        return sourceModuleIdx;
    }

    public int getReferenceIdx() {
        return DcModules.getReferencedModule(this).getIndex();
    }
    
    public void setIndex(int index) {
        this.index = index;
    }

    public int getIndex() {
        return index;
    }

    public void setModule(int module) {
        this.module = module;
    }

    public int getModule() {
        return module;
    }

    public void setTechinfo(boolean techinfo) {
        this.techinfo = techinfo;
    }

    public void setUiOnly(boolean uiOnly) {
        this.uiOnly = uiOnly;
    }

    public boolean isUiOnly() {
        return uiOnly;
    }

    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public void setSearchable(boolean searchable) {
        this.searchable = searchable;
    }

    public String getDatabaseFieldName() {
        return databaseFieldName;
    }

    public int getFieldType() {
        return fieldType;
    }

    public int getValueType() {
        return valueType;
    }

    public String getLabel() {
        return label;
    }

    public String getSystemName() {
        return systemName;
    }

    public boolean isHidden() {
        return hidden;
    }

    public boolean isTechnicalInfo() {
        return techinfo;
    }

    public boolean isRequired() {
        return required;
    }

    public boolean isReadOnly() {
        return readonly;
    }

    public boolean isSearchable() {
        return searchable;
    }

    public int getMaximumLength() {
        return maximumLength;
    }

    public void setLabel(String s) {
        label = s;
    }

    private void setSystemName(String s) {
        systemName = s;
    }

    public void setReadOnly(boolean b) {
        readonly = b;
    }

    public void setRequired(boolean b) {
        required = b;
    }

    public void setHidden(boolean b) {
        hidden = b;
    }

    public void setMaximumLength(int i) {
        maximumLength = i;
    }

    public void setFieldType(int index) {
        fieldType = index;
    }

    public void setValueType(int index) {
        valueType = index;
    }

    public void setDatabaseFieldName(String s) {
        databaseFieldName = s;
    }

    public String toString() {
        return label;
    }
    
    public void removeEnhancers() {
        enhancers.clear();
    }
    
    public void addValueEnhancer(IValueEnhancer enhancer) {
        enhancers.add(enhancer);
    }
    
    public IValueEnhancer[] getValueEnhancers() {
        IValueEnhancer[] e = new IValueEnhancer[enhancers.size()];
        return (IValueEnhancer[]) enhancers.toArray(e);
    }

    public String getDataBaseFieldType() {
        String s = "";
        if (getValueType() == DcRepository.datatypes._STRING) {
            if (getFieldType() == ComponentFactory._LONGTEXTFIELD) {
                s = DcRepository.database._FIELDLONGSTRING +
                    "(" + getMaximumLength() + ")";
            } else {
                s = DcRepository.database._FIELDSTRING +
                    "(" + getMaximumLength() + ")";
            }
        } else if (getValueType() == DcRepository.datatypes._BIGINTEGER ||
                   getValueType() == DcRepository.datatypes._DCOBJECTREFERENCE ||
                   getValueType() == DcRepository.datatypes._DCOBJECTCOLLECTION ||
                   getValueType() == DcRepository.datatypes._DCPARENTREFERENCE ||
                   getValueType() == DcRepository.datatypes._LONG) {
            s = DcRepository.database._FIELDBIGINT;
            
        } else if (getValueType() == DcRepository.datatypes._ICON ||
                   getValueType() == DcRepository.datatypes._PICTURE ||
                   getValueType() == DcRepository.datatypes._BLOB) {
            
            s = DcRepository.database._FIELDOBJECT;
        } else if (getValueType() == DcRepository.datatypes._BOOLEAN) {
            
            s = DcRepository.database._FIELDBOOLEAN;
        } else if (getValueType() == DcRepository.datatypes._DATE) {
            
            s = DcRepository.database._FIELDDATE;
        }

        return s;
    }
}
