/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects;

import java.awt.Toolkit;
import java.io.File;

import javax.swing.ImageIcon;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.util.Utilities;

public class Picture extends DcObject {

    public static final int _A_OBJECTID = 1;
    public static final int _B_FIELD = 2;
    public static final int _C_FILENAME = 3;
    public static final int _D_IMAGE = 4;
    public static final int _E_HEIGHT = 5;
    public static final int _F_WIDTH = 6;
    
    private boolean isNew = false;
    private boolean isUpdated = false;
    private boolean isDeleted = false;
    private boolean isLoaded = false;
    
    private static final Toolkit tk = Toolkit.getDefaultToolkit();
    
    public Picture() {
        super(DcModules._PICTURE);
    }
    
    public void loadImage() {
        String filename = (String) getValue(_C_FILENAME);

        if ((filename != null && !isLoaded()) ||  getValue(Picture._D_IMAGE) == null) {
            try {
                File file = new File(DataCrow.imageDir + filename);

                // if does not exist yet it will be updated later (see QueryQueue)
                if (file.exists()) {
                    byte[] image = Utilities.readFile(file);
                    setValue(Picture._D_IMAGE, image);
                }
                
                isLoaded = true;
            } catch (Exception exp) {
                DcLog.add(exp);
            }
        }
        
        markAsUnchanged();
    }
    
    public void unload() {
        setValueLowLevel(_D_IMAGE, null);
        setChanged(_D_IMAGE, false);
        isLoaded = false;
    }
    
    public ImageIcon getScaledPicture() {
        ImageIcon scaledInstance = null;
        if (getScaledFilename() != null) {
            File file = new File(DataCrow.imageDir + getScaledFilename());
            if (file.exists() && file.canRead())
                scaledInstance = new ImageIcon(tk.createImage(DataCrow.imageDir + getScaledFilename()));
        }
        return scaledInstance;
    }
    
    public String getScaledFilename() {
        return getScaledFilename((String) getValue(Picture._C_FILENAME));
    }

    public String getScaledFilename(String filename) {
        if (filename != null) {
            try {
                int idx = filename.indexOf(".jpg");
                String plain = filename.substring(0, idx);
                return plain + "_small.jpg";
            } catch (Exception exp) {}
        }
        return null;
    }
    
    public void isUpdated(boolean b) {
        isUpdated = b;
        if (b) {
            isNew = false;
            isDeleted = false;
            isLoaded = true;
        }
    }
    
    public void isNew(boolean b) {
        isNew = b;

        if (b) {
            isUpdated = false;
            isDeleted = false;
            isLoaded = true;
        }
    }
    
    public void isDeleted(boolean b) {
        isDeleted = b;
        
        if (b) {
            isUpdated = false;
            isNew = false;
            isLoaded = true;
        }
    }
    
    public boolean isLoaded() {
        return isLoaded;
    }
    
    public boolean isUpdated() {
        return isUpdated;
    }
    
    public boolean isNew() {
        return isNew;
    }
    
    public boolean isDeleted() {
        return isDeleted;
    }
    
    public String toString() {
        return getValue(_C_FILENAME) != null ? (String) getValue(_C_FILENAME) : "";
    }
    
    public int hashCode() {
        return (getValue(Picture._B_FIELD) != null ? getValue(Picture._B_FIELD).hashCode() : 0) +
               (getValue(Picture._A_OBJECTID) != null ? getValue(Picture._A_OBJECTID).hashCode() : 0); 
    }
    
    public boolean equals(Object o) {
        boolean equals = false;
        
        if (o instanceof Picture) {
            Picture picture = (Picture) o;
            
            String fieldname1 = (String) picture.getValue(Picture._B_FIELD);
            String fieldname2 = (String) getValue(Picture._B_FIELD);

            String objectID1 = (String) picture.getValue(Picture._A_OBJECTID);
            String objectID2 = (String) getValue(Picture._A_OBJECTID);
            
            equals = fieldname1.equals(fieldname2) && objectID1.equals(objectID2);
        } else {
            equals = super.equals(o);
        }
        
        return equals;
   }    
}