/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.helpers;

import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.util.isbn.InvalidBarCodeException;
import net.sf.dc.util.isbn.Isbn;

public class Book extends DcMediaObject {

    public static final int _F_PUBLISHER  = 1;
    public static final int _G_AUTHOR = 2;
    public static final int _H_WEBPAGE = 3;
    public static final int _I_CATEGORY = 4;
    public static final int _J_ISBN10 = 5;
    public static final int _K_PICTUREFRONT = 6;
    public static final int _L_STATE = 7;
    public static final int _M_FILENAME = 8;
    public static final int _N_ISBN13 = 9;
    public static final int _O_SERIES = 10;
    public static final int _P_VOLUME_NR = 11;
    public static final int _Q_VOLUME_TITLE = 12;
    public static final int _R_STORAGE_MEDIUM = 13;
    public static final int _S_FILESIZE = 14;
    public static final int _T_NROFPAGES = 15;
    
    public Book() {
       super(DcModules._BOOK);
    }

    protected void beforeSave() {
        String s10 = (String) getValue(_J_ISBN10);
        String s13 = (String) getValue(_N_ISBN13);

        try {
            Isbn isbn = new Isbn();

            boolean isBarcode13 = isbn.isIsbn13(s13);
            boolean isBarcode10 = isbn.isIsbn10(s10);
            if (isBarcode10 && !isBarcode13) {
                String isbn13 = isbn.getIsbn13(s10);  
                setValue(_N_ISBN13, isbn13);
            } else if (isBarcode13 && !isBarcode10) {
                String isbn10 = isbn.getIsbn10(s13);  
                setValue(_J_ISBN10, isbn10);
            }
        } catch (InvalidBarCodeException ibce) {
            DcLog.add(ibce);
        }
    }
}
