/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.helpers;

import net.sf.dc.console.windows.messageboxes.MessageBox;
import net.sf.dc.core.data.DataFilter;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Loan;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.core.wf.WorkFlow;
import net.sf.dc.core.wf.requests.SynchronizeWithManagerRequest;

public class ContactPerson extends DcObject {

    public static final int _A_NAME = 1;
    public static final int _B_DESCRIPTION = 2;
    public static final int _C_PHOTO = 3;
    public static final int _D_CATEGORY = 4;
    public static final int _E_EMAIL = 5;
    public static final int _F_ADDRESS = 6;
    public static final int _G_PHONE_HOME = 7;
    public static final int _H_PHONE_WORK = 8;
    
    public ContactPerson() {
        super(DcModules._CONTACTPERSON);
    }
    
    public void delete() {
        Loan loan = new Loan();
        loan.setValue(Loan._C_CONTACTPERSONID, getID());
        
        DataFilter filter = new DataFilter(loan);
        DcObject[] loans = DataManager.get(DcModules._LOAN, filter);
        
        if (loans == null || loans.length == 0) {
            if (keepInSync)
                addRequest(new SynchronizeWithManagerRequest(SynchronizeWithManagerRequest._DELETE, this));
            WorkFlow.deleteValues(this);  
        } else {
            getRequests().clear();
            new MessageBox(DcResources.getText("msgCantBeDeletedDueToLoans"), MessageBox._WARNING);
        }
    }    
}

