/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.template;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcTemplate;

public class Templates {
    
    private static final Collection templates = new ArrayList();
    
    public static void refresh() {
        templates.clear();
        
        for (Iterator iter = DcModules.getAllModules(); iter.hasNext(); ) {
            DcModule module = (DcModule) iter.next();
            
            if (module.getTemplateModule() != null) {
                DcModule templateModule = module.getTemplateModule();
                DcObject[] objects = DataManager.get(templateModule.getIndex(), null);
                
                for (int i = 0; i < objects.length; i++)
                    templates.add(objects[i]);
            }
        }
    }
    
    public static DcTemplate getDefault(int module) {
        for (Iterator iter = templates.iterator(); iter.hasNext(); ) {
            DcTemplate template = (DcTemplate) iter.next();
            
            if (template.getModule().getIndex() == module && template.isDefault())
                return template;
        }
        
        return null;
    }

    public static Collection getTemplates(int idx) {
        Collection result = new ArrayList();
        for (Iterator iter = templates.iterator(); iter.hasNext(); ) {
            DcTemplate template = (DcTemplate) iter.next();
            
            if (template.getModule().getIndex() == idx)
                result.add(template.clone());
        }
        return result;
    }
}
