/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.settings;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.Properties;

import net.sf.dc.core.log.DcLog;

/**
 * Reads and saves data to the Settings File
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class SettingsFile {

    /**
     * Save the settings file
     */
    public static void save(Settings settings) {
        Properties properties = new Properties();
        
        Collection c = settings.getSettings();
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            Setting setting = (Setting) iter.next();
            if (!setting.isTemporary())
                properties.setProperty(setting.getKey(), setting.getValueAsString());
        }

        try {
            FileOutputStream fos = new FileOutputStream(settings.getSettingsFile());
            properties.store(fos, "");
            fos.close();
        } catch (Exception exp) {
            DcLog.add(exp);  
        } 
    }

    /**
     * Loads the settings file
     */
    public static void load(Settings settings) {
        Properties properties = new Properties();
        
        try {
            FileInputStream fis = new FileInputStream(settings.getSettingsFile());
            properties.load(fis);
            fis.close();
        } catch (Exception exp) {} // use default settings
        
        Collection c = settings.getSettings();
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            Setting setting = (Setting) iter.next();
            if (!setting.isTemporary() && properties.getProperty(setting.getKey()) != null) {
                String sValue = properties.getProperty(setting.getKey()).toString();
                settings.setStringAsValue(setting.getKey(), sValue);
            } 
        }        
    }
}
