/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.settings;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * A settings group contains specific settings. A group can have
 * on child, creating a two level settings hierarchy. Deeper 
 * hierarchies are not supported as yet.
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.9
 */
public class SettingsGroup {
    
    private String helpIndex = "";
    private String sKey = "";
    private LinkedHashMap htSettings = new LinkedHashMap();
    private Hashtable htChildren = new Hashtable();
    
    /**
     * Creates a new settings group
     * 
     * @param sKey a unique identifier of this group
     */
    public SettingsGroup(String sKey, String helpIndex) {
        this.sKey = sKey;
        this.helpIndex = helpIndex;
    }

    /**
     * Adds a child to the settings group
     */
    public void addChild(SettingsGroup child) {
        htChildren.put(child.getKey(), child);
    }
    
    public String getKey() {
        return sKey;   
    }
    
    public String getHelpIndex() {
    	return helpIndex;
    }
    
    public Hashtable getChildren() {
        return htChildren;
    }
    
    public SettingsGroup getChild(String sGroupKey) {
        return (SettingsGroup) htChildren.get(sGroupKey);
    }
    
    public Map getSettings() {
        return htSettings;
    }
    
    protected Setting getSetting(String sKey) {
        Setting setting = (Setting) htSettings.get(sKey);
        
        if (setting != null) {
            return setting; 
        } 
        
        for (Enumeration enumerator = htChildren.elements(); enumerator.hasMoreElements(); ) {
            SettingsGroup settingsGroup = (SettingsGroup) enumerator.nextElement();
            if (settingsGroup != null) {
                setting = settingsGroup.getSetting(sKey);
                if (setting != null) {
                    break;   
                }
            }
        }
        return setting;
    }
    
    protected void add(Setting setting) {
        htSettings.put(setting.getKey(), setting);
    }
    
    public String toString() {
        return sKey; 
    }
}
