/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.wf.requests;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Collection;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.util.Utilities;

public class ImageRequest implements IRequest {
    
    public static final int _SAVE = 1;
    public static final int _DELETE = 2;
    
    private boolean executeOnFail = false;
    
    private String objectID;
    private byte[] image;
    private String filename;
    private int modus;
    
    public ImageRequest(byte[] image, String filename, int modus) {
        this.image = image;
        this.filename = filename;
        this.modus = modus;
    }

    public ImageRequest(String objectID, int modus) {
        this.objectID = objectID;
        this.modus = modus;
    }    
    
    public void execute(Collection objects) {
        if (objectID != null && modus == ImageRequest._DELETE) {
            File file = new File(DataCrow.imageDir);
            String[] files = file.list(new Filter(objectID));
            for (int i = 0; i < files.length; i++) {
                String fullPath = DataCrow.imageDir + files[i];
                File f = new File(fullPath);
                if (f.exists()) {
                    f.deleteOnExit();
                    f.delete();
                }
            }
        } else {
            if (modus == ImageRequest._SAVE) 
                saveImage();
            else if (modus == ImageRequest._DELETE)
                deleteImage();
        }
    }
    
    private class Filter implements FilenameFilter {

        private final String id; 
        
        public Filter(String id) {
            this.id = id;
        }
        
        public boolean accept(File file, String filename) {
            return filename.startsWith(id);
        }
    }
    
    private void deleteImage() {
        String filename1 = DataCrow.imageDir + filename;
        File file1 = new File(filename1);
        if (file1.exists()) {
            file1.deleteOnExit();
            file1.delete();
        }
        
        String filename2 = new Picture().getScaledFilename(DataCrow.imageDir + filename);
        File file2 = new File(filename2);
        if (file2.exists()) {
            file2.deleteOnExit();
            file2.delete();
        }
        
        image = null;
        filename = null;          
        objectID = null;
    }

    private void saveImage() {
        String imageFile = DataCrow.imageDir + filename;

        try {
            File file = new File(imageFile);
            if (file.exists())
                file.delete();
            
            if (image != null && image.length > 10) {
                Utilities.writeToFile(image, imageFile);
                Utilities.writeScaledImageToFile(image, new Picture().getScaledFilename(imageFile));
            }
            
            image = null;
            filename = null;          
            objectID = null;
            
        } catch (Exception exp) {
            DcLog.add("Could not save [" + imageFile + "] : " + exp.getMessage(), DcLog._ERROR);
        }         
    }
    
    public boolean getExecuteOnFail() {
        return executeOnFail;
    }

    public void setExecuteOnFail(boolean b) {
        executeOnFail = b;
    }
}
