/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.wf.requests;

import java.util.Collection;

import net.sf.dc.console.views.MasterView;
import net.sf.dc.console.views.table.TableSearchView;
import net.sf.dc.console.windows.itemforms.DcMinimalisticItemView;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;

public class RefreshChildView implements IUpdateUIRequest {

    private boolean executeOnFail = false;
    private DcMinimalisticItemView form;
    private DcObject child;
    
    private int parentModuleIdx;
    
    private String ID;
    private String parentID;
    private boolean isNew;
    
    public RefreshChildView(DcMinimalisticItemView form, int parentModuleIdx, DcObject child, String ID, String parentID, boolean isNew) {
        this.form = form;
        this.child = child;
        this.ID = ID;
        this.parentID = parentID;
        this.isNew = isNew;
        this.parentModuleIdx = parentModuleIdx;
    }
    
    public void execute(Collection c) {
        if (form != null)
            form.addObjects();
        
        DcModule parentModule = DcModules.get(parentModuleIdx);
        DcModule childModule = parentModule.getChild();

        parentModule.getSearchView().get(MasterView._LIST_VIEW).updateRow(parentID, parentModule.getDcObject(), false, false, false);

        if (!isNew) {
            if (child != null && child.getID() != null) {
                childModule.getSearchView().get(MasterView._TABLE_VIEW).updateRow(child.getID(), child, true, true, false);
            } else if (ID != null) {
                childModule.getSearchView().get(MasterView._TABLE_VIEW).removeRows(new String[] {ID});
            } else {
                TableSearchView tableView = (TableSearchView) parentModule.getSearchView().get(MasterView._TABLE_VIEW);
                tableView.loadChildValues();
            }
        } else {
            TableSearchView tableView = (TableSearchView) parentModule.getSearchView().get(MasterView._TABLE_VIEW);
            tableView.loadChildValues();
        }
    }

    public boolean getExecuteOnFail() {
        return executeOnFail;
    }
    
    public void setExecuteOnFail(boolean b) {
        executeOnFail = b;
    }
}
