/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.wf.requests;

import java.util.Collection;

import net.sf.dc.console.MainFrame;
import net.sf.dc.console.windows.itemforms.DcMinimalisticItemView;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.DcPropertyModule;
import net.sf.dc.core.modules.TemplateModule;
import net.sf.dc.core.resources.DcResources;

public class StatusUpdateRequest implements IRequest {

    private final int module;
    private final int tab;
    private final String msgID;
    private boolean executeOnFail = false;

    public StatusUpdateRequest(int module, int tab, String msgID) {
        this.module = module;
        this.tab = tab;
        this.msgID = msgID;
    }

    public void execute(Collection objects) {
        DcModule mod = DcModules.get(module);

        String msg = DcResources.getText(msgID);
        if (mod instanceof DcPropertyModule) {
            DcMinimalisticItemView view = ((DcPropertyModule) mod).getForm();
            view.setStatus(msg);
        } else if (mod instanceof TemplateModule) {
            DcMinimalisticItemView form = ((TemplateModule) mod).getForm();
            form.setStatus(msg);             
        } else {
            if (tab == MainFrame._INSERTTAB) {
            	mod.getCurrentInsertView().setStatus(msg);
            } else if (tab == MainFrame._SEARCHTAB) {
            	mod.getCurrentSearchView().setStatus(msg);
            }
        }
    }

    public boolean getExecuteOnFail() {
        return executeOnFail;
    }

    public void setExecuteOnFail(boolean b) {
        executeOnFail = b;
    }
}
