/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.fileimporters;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.Movie;
import net.sf.dc.util.Utilities;
import net.sf.ext.mediamanager.FilePropertiesMovie;

public class MovieImporter extends FileImporter {

    public MovieImporter() {
        super(DcModules._MOVIE);
    }

    public String[] getSupportedExtensions() {
        return new String[] {"mpg", "mpeg", "riff", "avi", "vob",
                             "ogm", "mov",  "ifo",  "mkv", "asf"};    
    }
    
    public boolean allowReparsing() {
        return true;
    }    

    public DcObject parse(String filename) throws ParseException {
        Movie movie = new Movie();
        
        try {
            movie.setValue(Movie._M_FILENAME, filename);
            movie.setValue(Movie._4_FILESIZE, new Long(getFileSize(filename)));
            movie.setValue(Movie._A_TITLE, getName(filename));
            movie.setValue(Movie._2_SUBTITLELANGUAGE, Utilities.fillSubtitles(filename, ""));

            FilePropertiesMovie movieFile1 = new FilePropertiesMovie(filename);
            Long playlength = new Long(movieFile1.getDuration());

            if (playlength.intValue() <= 0 && movieFile1.getVideoWidth() <= 0) {
                return movie;
            }
            
            movie.setValue(Movie._L_PLAYLENGTH, playlength);
            movie.setValue(Movie._N_VIDEOCODEC, movieFile1.getVideoCodec());
            movie.setValue(Movie._O_AUDIOCODEC, movieFile1.getAudioCodec());
            movie.setValue(Movie._P_WIDTH, new Long(movieFile1.getVideoWidth()));
            movie.setValue(Movie._Q_HEIGHT, new Long(movieFile1.getVideoHeight()));
            movie.setValue(Movie._R_FPS, new Long(movieFile1.getVideoRate()));
            movie.setValue(Movie._S_FRAMES, new Long(movieFile1.getDuration() * movieFile1.getVideoRate()));
            movie.setValue(Movie._T_AUDIOBITRATE, new Long(movieFile1.getAudioBitRate()));
            movie.setValue(Movie._U_AUDIOSAMPLINGRATE, new Long(movieFile1.getAudioRate()));
            movie.setValue(Movie._V_AUDIOCHANNEL, new Long(movieFile1.getAudioChannels()));
            movie.setValue(Movie._2_SUBTITLELANGUAGE, movieFile1.getSubtitles());
            
            int bitrate = movieFile1.getVideoBitRate();
            if (bitrate <= 0) {
                try {
                    MovieFile movieFile2 = new MovieFile(filename);
                    bitrate = new Double(movieFile2.getVideoBitrate()).intValue();
                } catch (Exception ignore) {}
            }

            movie.setValue(Movie._W_VIDEOBITRATE, new Long(bitrate));

        } catch (Exception exp) {
            throw new ParseException(exp);
        }
        
        return movie;
    }
}
