/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.fileimporters;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import net.sf.dc.core.DcRepository;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.MusicAlbum;
import net.sf.dc.core.objects.helpers.MusicTrack;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchTask;
import net.sf.dc.util.StringUtils;
import entagged.audioformats.AudioFile;
import entagged.audioformats.AudioFileIO;
import entagged.audioformats.Tag;
import entagged.audioformats.exceptions.CannotReadException;
import entagged.audioformats.generic.TagTextField;

public class MusicFile implements IOnlineSearchClient {
    
    private String album;
    private String artist;
    private String genre;
    private String year;
    private String title;
    private String encodingType;
    
    private int track;
    private int bitrate;
    private int length;
    
    private Collection albums = new ArrayList();
    
    public MusicFile() {}
    
    public MusicFile(String filename) throws CannotReadException {
        AudioFile audioFile = AudioFileIO.read(new File(filename));
        
        Tag tag = audioFile.getTag();
        
        if (tag != null) {
            album = getBest(tag.getAlbum());
            artist = getBest(tag.getArtist());
            genre = getGenre(getBest(tag.getGenre()));
            year = getBest(tag.getYear());
            title = getBest(tag.getTitle());
            
            bitrate = audioFile.getBitrate();
            length = audioFile.getLength();
            encodingType = audioFile.getEncodingType();
            
            try {
                String s = getBest(tag.getTrack());
                track = Integer.parseInt(s);
            } catch (Exception exp) {}
            
            try {
                String s = getBest(tag.getTrack());
                track = Integer.parseInt(s);
            } catch (Exception exp) {}

        }
    }
    
    public boolean onlineUpdate(MusicAlbum ma, DcServer server, DcRegion region) {
        
        Collection modes = server.getModes();

        boolean updated = false;
        
        String title = normalize((String) ma.getValue(MusicAlbum._A_TITLE));
        Collection artists = (Collection) ma.getValue(MusicAlbum._F_ARTISTS);
        if ((title == null || title.length() == 0) ||
            (artists == null || artists.size() == 0)) 
            return updated;
        
        DcSearchMode mode = null;
        for (Iterator iterMode = modes.iterator(); iterMode.hasNext(); ) {
            mode = (DcSearchMode) iterMode.next();
        
            OnlineSearchTask process = 
                server.getTask(this, (String) ma.getValue(MusicAlbum._A_TITLE), mode, region);
            process.start();
            
            try {
                process.join();
            } catch (Exception exp) {}
            
            for (Iterator iter = albums.iterator(); iter.hasNext(); ) {
                MusicAlbum musicAlbum = (MusicAlbum) iter.next();

                String titleNew = normalize((String) musicAlbum.getValue(MusicAlbum._A_TITLE));
                Collection artistsNew = (Collection) ma.getValue(MusicAlbum._F_ARTISTS);
                
                if (titleNew != null && artistsNew != null) {
                    boolean sameArtists = false;
                    for (Iterator iterArtists = artists.iterator(); iterArtists.hasNext(); ) {
                        DcObject artistDco = ((DcObject) iterArtists.next());
                        for (Iterator iterArtistsNew = artistsNew.iterator(); iterArtistsNew.hasNext(); ) {
                            sameArtists |= normalize(artistDco.toString()).equals(normalize(iterArtistsNew.next().toString()));
                        }
                    }
                    
                    if ((titleNew.toLowerCase().equals(title.toLowerCase())) && sameArtists) {
                        boolean match = updateTracks(ma.getChildren(), musicAlbum.getChildren());
                        if (match) {
                            updated = true;
                            ma.setValue(MusicAlbum._J_PICTUREFRONT, musicAlbum.getValue(MusicAlbum._J_PICTUREFRONT));
                            ma.setValue(MusicAlbum._K_PICTUREBACK, musicAlbum.getValue(MusicAlbum._K_PICTUREBACK));
                            ma.setValue(MusicAlbum._L_PICTURECD, musicAlbum.getValue(MusicAlbum._L_PICTURECD));
                        }
                    }
                }
            }
            albums.clear();
        }
        return updated;
    }
    
    private String normalize(String s) {
        if (s != null)
            return s.replaceAll("[,.!@#$%^&{}()'~`]", "");
        
        return s;
    }
    
    public void merge(DcObject dco1, DcObject dco2) {
        dco1.setValue(MusicAlbum._J_PICTUREFRONT, dco2.getValue(MusicAlbum._J_PICTUREFRONT));
        dco1.setValue(MusicAlbum._K_PICTUREBACK, dco2.getValue(MusicAlbum._K_PICTUREBACK));
        dco1.setValue(MusicAlbum._L_PICTURECD, dco2.getValue(MusicAlbum._L_PICTURECD));
        dco1.setValue(MusicAlbum._SYS_SERVICEURL, dco2.getValue(MusicAlbum._SYS_SERVICEURL));
        updateTracks(dco1.getChildren(), dco2.getChildren());
    }
    
    private boolean updateTracks(Collection oldTracks, Collection newTracks) {
        boolean match = false;
        for (Iterator iterOld = oldTracks.iterator(); iterOld.hasNext(); ) {
            MusicTrack tr = (MusicTrack) iterOld.next();
            
            String titleOld = (String) tr.getValue(MusicTrack._A_TITLE);
            Long lengthOld = (Long) tr.getValue(MusicTrack._J_PLAYLENGTH);
            Long trackOld = (Long) tr.getValue(MusicTrack._F_TRACKNUMBER);

            for (Iterator iterNew = newTracks.iterator(); iterNew.hasNext(); ) {
                MusicTrack at = (MusicTrack) iterNew.next();
                
                String titleNew = (String) at.getValue(MusicTrack._A_TITLE);
                Long lengthNew = (Long) at.getValue(MusicTrack._J_PLAYLENGTH);
                Long trackNew = (Long) at.getValue(MusicTrack._F_TRACKNUMBER);

                if ((titleOld != null && titleNew != null) && titleNew.toLowerCase().equals(titleOld.toLowerCase())) {
                    
                    match = true;
                    merge(at, tr);
                
                } else if (newTracks.size() == oldTracks.size() && 
                           ((lengthOld != null && lengthNew != null) && lengthNew.equals(lengthOld)) ||  
                           ((trackOld != null && trackNew != null) && trackNew.equals(trackOld))) {
                
                    match = true;
                    merge(at, tr);
                }
            }
        }
        return match;
    }
    
    private void merge(MusicTrack mtOld, MusicTrack mt) {
        setValue(mt, MusicTrack._A_TITLE, mtOld.getValue(MusicTrack._A_TITLE));
        setValue(mt, MusicTrack._B_DESCRIPTION, mtOld.getValue(MusicTrack._B_DESCRIPTION));
        setValue(mt, MusicTrack._C_YEAR, mtOld.getValue(MusicTrack._C_YEAR));
        setValue(mt, MusicTrack._E_RATING, mtOld.getValue(MusicTrack._E_RATING));
        setValue(mt, MusicTrack._F_TRACKNUMBER, mtOld.getValue(MusicTrack._F_TRACKNUMBER));
        setValue(mt, MusicTrack._G_ARTIST, mtOld.getValue(MusicTrack._G_ARTIST));
        setValue(mt, MusicTrack._J_PLAYLENGTH, mtOld.getValue(MusicTrack._J_PLAYLENGTH));
    }
    
    private void setValue(DcObject dco, int field, Object newValue) {
        if (newValue != null && newValue.toString().length() > 0)
            dco.setValue(field, newValue);
    }
    
    private String getGenre(String s) {
        String genre = s != null && s.length() > 0 ? s : null;
        
        if (genre != null) {
            try {
                String genreIdx = StringUtils.getValueBetween("(", ")", genre);
                int index = Integer.parseInt(genreIdx);
                genre = DcRepository.collections.colMusicGenres[index];
            } catch (Exception exp) {}
        }
        
        return genre;
    }    
        
    private String getBest(List list) {
        String s = "";

        Collection all = new ArrayList(list);
        for (Iterator iter = all.iterator(); iter.hasNext(); ) {
            try {
                TagTextField tagField = (TagTextField) iter.next();
                s = tagField.getContent();
            } catch (Exception exp) {
                DcLog.add(exp);
            }
        }

        return s;
    }

    public String getAlbum() {
        return album;
    }

    public String getArtist() {
        return artist;
    }

    public String getGenre() {
        return genre;
    }

    public String getTitle() {
        return title;
    }

    public int getTrack() {
        return track;
    }

    public String getYear() {
        return year;
    }

    public int getBitrate() {
        return bitrate;
    }

    public String getEncodingType() {
        return encodingType;
    }

    public int getLength() {
        return length;
    }

    public void addError(Throwable t) {
        DcLog.add(t);
    }

    public void addError(String message) {}

    public void addMessage(String message) {}

    public void addObject(DcObject dco) {
        albums.add(dco);
    }

    public void addWarning(String warning) {}

    public DcModule getModule() {
        return DcModules.get(DcModules._MUSICALBUM);
    }

    public void processed(int i) {}

    public void processing() {}

    public void processingTotal(int i) {}

    public int resultCount() {
        return albums.size();
    }

    public void stopped() {}

    public DcObject getDcObject() {
        return null;
    }      
}


