/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch;

import java.util.ArrayList;
import java.util.Collection;


public abstract class DcServer {
    
    private final String displayName;
    private final String url;
    private final boolean isKeywordSearchAllowed;
    
    private final int module;
    private final Collection modes = new ArrayList();
    private final Collection regions = new ArrayList();
    
    public DcServer(String displayName, String url, boolean isKeywordSearchAllowed, int module) {
        this.displayName = displayName;
        this.url = url;
        this.module = module;
        this.isKeywordSearchAllowed = isKeywordSearchAllowed;
    }
    
    public int getModule() {
        return module;
    }
    
    public abstract OnlineSearchTask getTask
        (IOnlineSearchClient listener, String query, DcSearchMode mode, DcRegion region);

    public void addMode(DcSearchMode mode) {
        modes.add(mode);
    }
    
    public void addRegion(DcRegion region) {
        regions.add(region);
    }
    
    public String getUrl() {
        return url;
    }

    public String getName() {
        return displayName;
    }
    
    public Collection getModes() {
        return modes;
    }
    
    public Collection getRegions() {
        return regions;
    }
    
    public boolean hasModes() {
        return modes != null && modes.size() > 0;
    }
    
    public boolean isKeywordSearchAllowed() {
        return isKeywordSearchAllowed;
    }
    
    public String toString() {
        return displayName;
    }
}
