/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.DcRepository;
import net.sf.dc.settings.DcSettings;
import net.sf.dc.util.Base64;

public class HttpConnection {
    
    private HttpURLConnection uc;

    public HttpConnection(URL url) throws Exception {
        uc = getConnection(url);
        uc.setRequestMethod("GET");
        uc.setDoOutput(true);
        uc.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible;MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; InfoPath.1; .NET CLR2.0.50727)"); 
    }
    
    public boolean exists() {
        try {
            uc.getInputStream();
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    public String getString(String charset) throws IOException {
        BufferedInputStream in = new BufferedInputStream(uc.getInputStream());
        InputStreamReader reader = new InputStreamReader(in, charset);

        StringBuffer sb = new StringBuffer();
        int c;
        while ((c = reader.read()) != -1) {
            sb.append((char) c);
        }

        reader.close();
        in.close();
        uc.disconnect();
        return sb.toString();
    }
    
    public String getString() throws IOException {
        return getString("UTF-8");
    }

    public int getContentLength() {
    	return uc.getContentLength();
    }

    public void setContentLength(int length) {
        uc.setRequestProperty("Content-Length", String.valueOf(length));
    }

    public void setToSoapRequest(String soapAction) throws Exception {
        uc.setRequestProperty("Content-Type", "text/xml; charset=utf-8");
        uc.setRequestProperty("SOAPAction", soapAction);
        uc.setRequestMethod("POST");
        uc.setDoOutput(true);
        uc.setDoInput(true);
    }

    public OutputStream getOutputStream() throws IOException {
        uc.setDoOutput(true);
        return uc.getOutputStream();
    }

    
    public final byte[] getBytes() throws IOException {
      InputStream raw = uc.getInputStream();
      BufferedInputStream in  = new BufferedInputStream(raw);
      Collection c = new ArrayList();
      while (true) {
          int i = in.read();
          if (i == -1) break;
          c.add(new Byte((byte) i));
      }
      in.close();
      uc.disconnect();
      
      byte[] bytes = new byte[c.size()];
      int counter = 0;
      for (Iterator iter = c.iterator(); iter.hasNext(); counter++) 
          bytes[counter] = ((Byte) iter.next()).byteValue();
      
      c.clear();
      c = null;
      
      return bytes;
    }

    public void close() {
    	uc.disconnect();
        uc = null;
    }

    private final HttpURLConnection getConnection(URL url) throws IOException {
        String proxy = DcSettings.getString(DcRepository.settings.stProxyServerName);

        String username = DcSettings.getString(DcRepository.settings.stProxyUserName);
        String password = DcSettings.getString(DcRepository.settings.stProxyPassword);
        int proxyPort = DcSettings.getInt(DcRepository.settings.stProxyServerPort);

        boolean useProxy = proxy.length() > 0 && proxyPort > 0;
        if (useProxy) {
        	System.setProperty("proxySet", "true");
        	System.setProperty("http.proxyHost", proxy);
        	System.setProperty("http.proxyPort", "" + proxyPort);
        } else {
            System.setProperty("proxySet", "false");
            System.setProperty("http.proxyHost", "");
            System.setProperty("http.proxyPort", "" + -1);
        }

        HttpURLConnection uc = (HttpURLConnection) url.openConnection();
        if (useProxy && username.trim().length() > 0) {
    		String proxyUPB64 = Base64.encode(username + ":" + password);
    		uc.setRequestProperty("Proxy-Authorization", "Basic " + proxyUPB64);
        }

        return uc;
    }
}
