/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch;

import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.onlinesearch.amazon.AmazonImdbServer;
import net.sf.dc.onlinesearch.amazon.AmazonServer;
import net.sf.dc.onlinesearch.amazon.ItemLookupSearchMode;
import net.sf.dc.onlinesearch.amazon.KeywordSearchMode;
import net.sf.dc.onlinesearch.imdb.ImdbServer;
import net.sf.dc.onlinesearch.wikipedia.WikipediaServer;

public class OnlineMovieSearch extends OnlineSearch {

    private Collection servers = new ArrayList();
    
    public OnlineMovieSearch() {
        super(DcModules._MOVIE);
    }
    
    public Collection getServers() {
        if (servers.size() == 0) {
            DcServer amazonServer = new AmazonServer(getModule());
            amazonServer.addMode(new KeywordSearchMode("DVD", "DVD"));
            amazonServer.addMode(new KeywordSearchMode("VHS", "VHS"));
            
            amazonServer.addMode(new ItemLookupSearchMode("Video", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._EAN), ItemLookupSearchMode._EAN));
            amazonServer.addMode(new ItemLookupSearchMode("Video", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._UPC), ItemLookupSearchMode._UPC));
            amazonServer.addMode(new ItemLookupSearchMode("", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._ASIN), ItemLookupSearchMode._ASIN));
            
            AmazonImdbServer combinedServer = new AmazonImdbServer(getModule());
            combinedServer.addMode(new KeywordSearchMode("DVD", "DVD"));
            combinedServer.addMode(new KeywordSearchMode("VHS", "VHS"));
            combinedServer.addMode(new ItemLookupSearchMode("Video", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._EAN), ItemLookupSearchMode._EAN));
            combinedServer.addMode(new ItemLookupSearchMode("Video", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._UPC), ItemLookupSearchMode._UPC));
            combinedServer.addMode(new ItemLookupSearchMode("", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._ASIN), ItemLookupSearchMode._ASIN));
            
            DcServer imdbServer = new ImdbServer(getModule());
            DcServer wikiServer = new WikipediaServer(getModule());
            
            
            servers.add(combinedServer);
            servers.add(amazonServer);
            servers.add(imdbServer);
            servers.add(wikiServer);
        }
        
        return servers; 
    }
}
