/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch;

import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.onlinesearch.amazon.AmazonServer;
import net.sf.dc.onlinesearch.amazon.ItemLookupSearchMode;
import net.sf.dc.onlinesearch.amazon.KeywordSearchMode;
import net.sf.dc.onlinesearch.discogs.DiscogsServer;
import net.sf.dc.onlinesearch.musicbrainz.AlbumTitleSearch;
import net.sf.dc.onlinesearch.musicbrainz.AlbumByArtistSearch;
import net.sf.dc.onlinesearch.musicbrainz.MusicBrainzServer;

public class OnlineMusicAlbumSearch extends OnlineSearch {

    private Collection servers = new ArrayList();

    public OnlineMusicAlbumSearch() {
        super(DcModules._MUSICALBUM);
    }
    
    public Collection getServers() {
        if (servers.size() == 0) {
            MusicBrainzServer musicBrainzServer = new MusicBrainzServer(getModule());
            musicBrainzServer.addMode(new AlbumTitleSearch());
            musicBrainzServer.addMode(new AlbumByArtistSearch());
            
            DcServer amazonServer = new AmazonServer(getModule());
            amazonServer.addMode(new KeywordSearchMode("Music", "Keyword"));
            amazonServer.addMode(new ItemLookupSearchMode("Music", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._EAN), ItemLookupSearchMode._EAN));
            amazonServer.addMode(new ItemLookupSearchMode("Music", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._UPC), ItemLookupSearchMode._UPC));
            amazonServer.addMode(new ItemLookupSearchMode("Music", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._ASIN), ItemLookupSearchMode._ASIN));
            
            servers.add(musicBrainzServer);
            servers.add(amazonServer);
            servers.add(new DiscogsServer(getModule()));
        }

        return servers;    
    }
}
