/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch;

import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

import net.sf.dc.console.windows.itemforms.ItemForm;
import net.sf.dc.console.windows.onlinesearch.OnlineSearchForm;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;

public abstract class OnlineSearch {
    
    private final int module;
    
    private static final int _SERVICE_NAME = 0;
    private static final int _REGION_CODE = 1;
    private static final int _MODE_NAME = 2;
    private static final int _QUERY = 3;    
    
    public OnlineSearch(int module) {
        this.module = module;
    }
    
    public abstract Collection getServers();

    public DcObject query(DcObject dco) throws Exception {
        DcServer server = getServer(dco);
        DcRegion region = getRegion(dco);
        
        if (server != null) {
            OnlineSearchTask task = server.getTask(null, null, null, region);
            return task.query(dco);
        }
        
        return null;
    }
    
    public int getModule() {
        return module;
    }

    public DcServer getServer(DcObject dco) {
        return getServer(getService(dco, _SERVICE_NAME));
    }
    
    public DcRegion getRegion(DcObject dco) {
        return getRegion(getService(dco, _REGION_CODE));
    }

    public DcSearchMode getMode(DcObject dco) {
        return getMode(getService(dco, _MODE_NAME));
    }
    
    public String getQuery(DcObject dco) {
        return getService(dco, _QUERY);
    }    
    
    public DcRegion getDefaultRegion() {
        String code = DcModules.get(module).getSettings().getString(DcRepository.modulesettings.stOnlineSearchDefaultRegion);
        return getRegion(code);
    }
    
    public DcServer getDefaultServer() {
        String name = DcModules.get(module).getSettings().getString(DcRepository.modulesettings.stOnlineSearchDefaultServer);
        return getServer(name);
    }
    
    public DcSearchMode getDefaultSearchMode() {
        String name = DcModules.get(module).getSettings().getString(DcRepository.modulesettings.stOnlineSearchDefaultMode);
        return getMode(name);       
    }
    
    public DcServer getServer(String name) {
        Collection servers = getServers();
        for (Iterator iter = servers.iterator(); iter.hasNext(); ) {
            DcServer server = (DcServer) iter.next();
            if (server.getName().equals(name))
                return server;
        }
        return null;
    }
    
    private DcRegion getRegion(String code) {
        DcServer server = getDefaultServer();
        
        if (server != null && server.getRegions() != null) {
            for (Iterator iter = server.getRegions().iterator(); iter.hasNext(); ) {
                DcRegion region = (DcRegion) iter.next();
                if (region.getCode().equals(code))
                    return region;
            }
        }
        return null;
    }    

    private DcSearchMode getMode(String diplayName) {
        DcServer server = getDefaultServer();
        if (server != null && server.getModes() != null) {
            for (Iterator iter = server.getModes().iterator(); iter.hasNext(); ) {
                DcSearchMode mode = (DcSearchMode) iter.next();
                if (mode.getDisplayName().equals(diplayName))
                    return mode;
            }
        }
        return null;        
    }
    
    private String getService(DcObject dco, int level) {
        String s = (String) dco.getValue(DcMediaObject._SYS_SERVICE);
        
        if (s == null) return null;
        
        StringTokenizer st = new StringTokenizer(s, "/");
        String[] values = new String[st.countTokens()];
        int i = 0;
        while (st.hasMoreElements())
            values[i++] = ((String) st.nextElement()).trim();
        
        String value = null;
        try {
            value = values[level];
            if (level == _QUERY && value.length() > 0) 
                value = value.substring(value.indexOf("[") + 1, value.lastIndexOf("]"));
        } catch (Exception e) {
            DcLog.add(e);
        }
        
        return value;
    }    
    
    public OnlineSearchForm getUI(DcObject dco, ItemForm itemForm, boolean advanced) {
        return new OnlineSearchForm(this, dco, itemForm, advanced);
    }
}
