/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch;

import net.sf.dc.core.objects.DcObject;

public abstract class OnlineSearchTask extends Thread {

    private boolean isCancelled = false;
    
    protected IOnlineSearchClient listener;
    
    private int maximum = 20;
    
    private String searchString;
    
    private String address;
    private DcServer server;
    private DcSearchMode mode;
    private DcRegion region;
    
    public OnlineSearchTask(IOnlineSearchClient listener, 
                            String searchString, 
                            DcServer server,
                            DcRegion region,
                            DcSearchMode mode) {
        
        this.listener = listener;
        this.region = region;
        this.mode = mode;
        this.server = server;
        this.address = region != null ? region.getUrl() : server.getUrl();
        this.searchString = searchString;        
    }

    public OnlineSearchTask(IOnlineSearchClient listener, String searchString, int max) {
        this.listener = listener;
        this.searchString = searchString;
        this.maximum = max;
    }

    public abstract void run();
    public abstract String getWhiteSpaceSubst();
    
    public abstract DcObject query(DcObject dco) throws Exception;
    
    protected void setServiceInfo(DcObject dco) {
        String service =  server.getName() + " / " + 
                         (region != null ? region.getCode() : "none") + " / " +
                         (mode != null ? mode.getDisplayName() : "none") + " / " + 
                          "value=[" + searchString + "]";
        dco.setValue(DcObject._SYS_SERVICE, service);
    }
    
    public void setMaximum(int maximum) {
        this.maximum = maximum;
    }
    
	public void cancelSearch() {
        isCancelled = true;
    }

    protected void isCancelled(boolean b) {
        this.isCancelled = b;
    }

    public boolean isCancelled() {
        return isCancelled;
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public DcSearchMode getMode() {
        return mode;
    }

    public void setMode(DcSearchMode mode) {
        this.mode = mode;
    }

    public DcRegion getRegion() {
        return region;
    }

    public void setRegion(DcRegion region) {
        this.region = region;
    }

    public String getQuery() {
        String s = searchString.replaceAll(" ", getWhiteSpaceSubst());
        
        // replace the & character
        int idx = s.indexOf('&');
        while (idx > -1) {
            s = s.substring(0, s.indexOf('&')) + "%26" + s.substring(s.indexOf('&') + 1, s.length());
            idx = s.indexOf('&');
        }
            
        return s;
    }

    public void setSearchString(String searchString) {
        this.searchString = searchString;
    }

    public DcServer getServer() {
        return server;
    }

    public void setServer(DcServer server) {
        this.server = server;
    }

    public int getMaximum() {
        return maximum;
    }
}
