package net.sf.dc.onlinesearch;

import java.io.IOException;
import java.io.StringReader;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;

public abstract class OnlineSearchTaskImpl extends OnlineSearchTask {

    private JTextPane descriptionPane = ComponentFactory.getTextPane();
    private HTMLEditorKit kit = new HTMLEditorKit();
    private HTMLDocument document = new HTMLDocument();

    
    public OnlineSearchTaskImpl(IOnlineSearchClient listener, String searchString, DcServer server, DcRegion region, DcSearchMode mode) {
        super(listener, searchString, server, region, mode);
        
        descriptionPane.setContentType("text/html");
        descriptionPane.setEditorKit(kit);
        descriptionPane.setDocument(document);
        descriptionPane.setEditable(false);
    }

    public OnlineSearchTaskImpl(IOnlineSearchClient listener, String searchString, int max) {
		super(listener, searchString, max);
	}

	public String getWhiteSpaceSubst() {
        return "+";
    }

    public DcObject query(DcObject dco) throws Exception {
        String link = (String) dco.getValue(DcObject._SYS_SERVICEURL); 
        if (link != null && link.length() > 0)
            return getDcObject(new URL(link));

        return null;
    }

    public void run() {
        Collection IDs = getIDs();
        listener.processingTotal(IDs.size());

        if (IDs.size() == 0) {
            listener.addError(DcResources.getText("msgNoResultsForKeywords", getQuery()));
            listener.stopped();
            return;
        }

        listener.addMessage(DcResources.getText("msgFoundXResults", String.valueOf(IDs.size())));
        listener.addMessage(DcResources.getText("msgStartParsingXResults", String.valueOf(IDs.size())));
        int counter = 1;
        for (Iterator iter = IDs.iterator(); iter.hasNext() && !isCancelled() && counter <= getMaximum(); counter++) {
            String ID = (String) iter.next(); 
            
            try {
                DcObject dco = getDcObject(ID);
                
                if (dco != null) {
                    setServiceInfo(dco);
                    listener.addMessage(DcResources.getText("msgParsingSuccessfull", dco.toString()));
                    listener.addObject(dco);
                }
                
                listener.processed(counter);

            } catch (Exception exp) {
                listener.addMessage(DcResources.getText("msgParsingError", "" + exp));
                DcLog.add(exp);
                listener.processed(counter);
            }
        }
        
        descriptionPane = null;
        kit = null;
        document = null;        
        listener.stopped();        
    }
    
    protected abstract DcObject getDcObject(String id) throws Exception;
    protected abstract DcObject getDcObject(URL url) throws Exception;
    protected abstract Collection getIDs();
    
    protected String getCleanText(String html) throws IOException, BadLocationException {
        String webpage = html;
        
        descriptionPane.read(new StringReader(webpage), "Data Crow");     
        Document doc = descriptionPane.getDocument();
        String text = doc.getText(0, doc.getLength());
        text = text.replaceAll("\r", "\n");
        text = text.replaceAll("\n\n", "\n");
        text = text.replaceAll("" + (char) 160 , " ");
        
        return text;
    }    
}
