/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.amazon;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.AudioCD;
import net.sf.dc.core.objects.helpers.AudioTrack;
import net.sf.dc.core.objects.helpers.MusicAlbum;
import net.sf.dc.core.objects.helpers.MusicTrack;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.util.StringUtils;

public class AmazonAudioCdSearch extends AmazonSearch {

    public AmazonAudioCdSearch(IOnlineSearchClient listener, 
                               String searchString, 
                               DcServer server, 
                               DcRegion region, 
                               DcSearchMode mode) {

        super(listener, searchString, server, region, mode);
    }

    protected Collection toObjects(String xml, String locale) {
        Collection albums = new ArrayList();
        
        try {
            Collection trackSets = getAllTagValues("<Disc Number=", "</Disc>", xml);
            int disc = 1;
            for (Iterator iter = trackSets.iterator(); iter.hasNext(); disc++) {
                String trackSet = (String) iter.next();
        
                DcObject dco = DcModules.get(getServer().getModule()).getDcObject();
                
                setTitle(dco, xml);
                setDescription(dco, xml);
                setYear(dco, xml);
                setRating(dco, xml);
                
                setServiceURL(dco, getAsin(xml));
                setWebpage(dco, xml);

                DcObject artist = setArtist(dco, xml);
                DcObject genre = setGenre(dco, xml);
                
                int fieldIdx = dco.getModule().getIndex() == DcModules._AUDIOCD ? AudioCD._H_PICTUREFRONT : MusicAlbum._J_PICTUREFRONT;
                setImages(dco, xml, fieldIdx, null);
                
                int counter = 1;
                Collection tracks = getAllTagValues("<Track Number=", "</Track>", trackSet);
                for (Iterator iterTracks = tracks.iterator(); iterTracks.hasNext(); counter++) {
                    String s = (String) iterTracks.next();
                    String title = s.substring(s.indexOf(">") + 1, s.length());

                    
                    DcObject track = dco.getModule().getChild().getDcObject();
                    track.setValue(MusicTrack._A_TITLE, title);
                    track.setValue(MusicTrack._F_TRACKNUMBER, new Long(counter));
                    if (genre != null) {
                        int fld = track instanceof AudioTrack ? AudioTrack._J_GENRES : MusicTrack._H_GENRES;
                        DataManager.addMapping(track, genre, fld);
                    }
                    
                    if (artist != null) {
                        int fld = track instanceof AudioTrack ? AudioTrack._K_ARTIST : MusicTrack._G_ARTIST;
                        DataManager.addMapping(track, artist, fld);
                    }

                    dco.addChild(track);
                }

                dco.setIDs();
                albums.add(dco);                
            }
        } catch (Exception exp) {
            listener.addError(exp);
        }     
        
        return albums;
    }
    
    private DcObject setGenre(DcObject dco, String xml) {
        int fieldIdx = dco.getModule().getIndex() == DcModules._AUDIOCD ? AudioCD._G_GENRES : MusicAlbum._G_GENRES;
        return DataManager.createReference(dco, fieldIdx, getStyle(xml));
    }
    
    private void setTitle(DcObject dco, String xml) {
        dco.setValue(DcMediaObject._A_TITLE, getTitle(xml));
    }
    
    private void setDescription(DcObject dco, String xml) {
        dco.setValue(DcMediaObject._B_DESCRIPTION, getDescription(xml));
    }

    private void setYear(DcObject dco, String xml) {
        dco.setValue(DcMediaObject._C_YEAR, getReleaseDate(xml));
    }
    
    private void setRating(DcObject dco, String xml) {
        dco.setValue(DcMediaObject._E_RATING, getRating(xml));
    }
    
    private void setWebpage(DcObject dco, String xml) {
        int fieldIdx = dco.getModule().getIndex() == DcModules._AUDIOCD ? AudioCD._M_WEBPAGE : MusicAlbum._N_WEBPAGE;
        dco.setValue(fieldIdx, getAmazonURL(xml));
    }
    
    private DcObject setArtist(DcObject dco, String xmlString) {
         String name = StringUtils.getValueBetween("<Artist>", "</Artist>", xmlString);
         int fieldIdx = dco.getModule().getIndex() == DcModules._AUDIOCD ? AudioCD._F_ARTIST : MusicAlbum._F_ARTISTS;
         return DataManager.createReference(dco, fieldIdx, name);
    }
}
