/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.amazon;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.objects.helpers.Book;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.util.StringUtils;
import net.sf.dc.util.isbn.InvalidBarCodeException;
import net.sf.dc.util.isbn.Isbn;

public class AmazonBookSearch extends AmazonSearch {

    public AmazonBookSearch(IOnlineSearchClient listener, 
                            String searchString, 
                            DcServer server, 
                            DcRegion region, 
                            DcSearchMode mode) {
        
        super(listener, searchString, server, region, mode);
    }

    protected Collection toObjects(String xml, String locale) {
        Collection objects = new ArrayList();

        try {
            Book book = new Book();
            book.setValue(Book._A_TITLE, getTitle(xml));
            book.setValue(Book._B_DESCRIPTION, getDescription(xml));
    
            setPublisher(book, xml);
            setAuthors(book, xml);
            
            book.setValue(Book._C_YEAR, getPublicationDate(xml));
            book.setValue(Book._H_WEBPAGE, getAmazonURL(xml));
            book.setValue(Book._E_RATING, getRating(xml));
            
            setServiceURL(book, getAsin(xml));
            
            String s = StringUtils.getValueBetween("<ISBN>", "</ISBN>", xml);
            Isbn isbn = new Isbn();
            
            try {
                if (isbn.isIsbn10(s)) {
                    book.setValue(Book._J_ISBN10, s);
                    book.setValue(Book._N_ISBN13, isbn.getIsbn13(s));
                } else if (isbn.isIsbn13(s)) {
                    book.setValue(Book._N_ISBN13, s);
                    book.setValue(Book._J_ISBN10, isbn.getIsbn10(s));
                }
            } catch (InvalidBarCodeException ibe) {
                listener.addError(ibe);
            }
            
            setImages(book, xml, Book._K_PICTUREFRONT, null);
            objects.add(book);
        } catch (Exception e) {
            listener.addError(e);
        }
            
        return objects;
    }
    
    protected void setPublisher(Book book, String rawResult) {
        String name = StringUtils.convertHttpString(StringUtils.getValueBetween("<Publisher>", "</Publisher>", rawResult));
        DataManager.createReference(book, Book._F_PUBLISHER, name);
    }    
    
    private void setAuthors(Book book, String xmlString) {
        Collection c = getAllTagValues("<Author>", "</Author>", xmlString);
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            String name = StringUtils.convertHttpString((String) iter.next());
            DataManager.createReference(book, Book._G_AUTHOR, name);
        }
    }
}
