/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.amazon;

import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.Movie;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchHelper;

public class AmazonImdbMovieSearch extends AmazonMovieSearch  {

    public AmazonImdbMovieSearch(IOnlineSearchClient listener, 
                                 String searchString, 
                                 DcServer server, 
                                 DcRegion region,
                                 DcSearchMode mode) {

        super(listener, searchString, server, region, mode);
    }
    
    protected Collection toObjects(String xml, String locale) {
        Collection objects = super.toObjects(xml, locale);

        try {
            DcServer server = DcModules.get(DcModules._MOVIE).getOnlineSearch().getServer("Imdb");
            OnlineSearchHelper osh = new OnlineSearchHelper(DcModules._MOVIE);
            osh.setServer(server);
            osh.setRegion(getRegion());
            osh.setMaximum(1);
            
            for (Iterator iter = objects.iterator(); iter.hasNext() && !isCancelled(); ) {
                Movie movie = (Movie) iter.next();
                String title = (String) movie.getValue(Movie._A_TITLE);
                if (title != null && title.trim().length() > 0) {
                    DcObject dco = osh.query(movie, getPlainTitle(title), new int[] {Movie._A_TITLE});
                    if (dco != null) movie.merge(dco);
                }
            }
            
            osh.clear();
        } catch (Exception e) {
            listener.addError(e);
        }
        
        return objects;
    }    
    
    private String getPlainTitle(String title) {
        String result = title;
       
        int end = title.indexOf("(") > -1 ? title.indexOf("(") : 
                  title.indexOf("[") > -1 ? title.indexOf("[") : -1;
       
        if (end > -1)
            result = title.substring(0, end);
        
        return result;
    }
}
