/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.amazon;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.objects.helpers.Movie;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.util.StringUtils;

public class AmazonMovieSearch extends AmazonSearch {

    public AmazonMovieSearch(IOnlineSearchClient listener, 
                             String searchString, 
                             DcServer server, 
                             DcRegion region,
                             DcSearchMode mode) {
        
        super(listener, searchString, server, region, mode);
    }

    protected Collection toObjects(String xml, String locale) {
        Collection objects = new ArrayList();

        try {
            Movie movie = new Movie();
            movie.setValue(Movie._A_TITLE, getTitle(xml));
            movie.setValue(Movie._B_DESCRIPTION, getDescription(xml));
            movie.setValue(Movie._0_LANGUAGE, getLanguage(xml));
            movie.setValue(Movie._G_WEBPAGE, getAmazonURL(xml));
            
            setActors(movie, xml);
            setDirectors(movie, xml);
            
            movie.setValue(Movie._C_YEAR, getTheatricalReleaseDate(xml));
            movie.setValue(Movie._E_RATING, getRating(xml));
            
            String asin = getAsin(xml);
            movie.setValue(Movie._5_ASIN, asin);
            setServiceURL(movie, asin);        
            
            setStorageMedium(movie, Movie._7_STORAGEMEDIUM, xml, "<ProductGroup>", "</ProductGroup>");
            
            movie.setValue(Movie._L_PLAYLENGTH, getPlayLength(xml));
            
            String s = getGenre(xml);
            
            if (s != null) {
                StringTokenizer st = new StringTokenizer(s, "&");
                while (st.hasMoreElements())
                    DataManager.createReference(movie, Movie._H_GENRES, ((String) st.nextElement()).trim());
            }
    
            setImages(movie, xml, Movie._X_PICTUREFRONT, null);
            objects.add(movie);
        } catch (Exception e) {
            listener.addError(e);
        }
            
        return objects;
    }
    
    private String getLanguage(String xmlString) {
        Collection languages = getAllTagValues("<Language>", "</Language>", xmlString);
        for (Iterator iter = languages.iterator(); iter.hasNext(); ) {
            String language = (String) iter.next();
            
            if (language.toLowerCase().indexOf("original") > -1 || languages.size() == 1)
                return StringUtils.convertHttpString(StringUtils.getValueBetween("<Name>", "</Name>", xmlString));
        }
        return null;
    }
    
    private Long getPlayLength(String xmlString) {
        String runningTime = StringUtils.getValueBetween("<RunningTime Units=" + '"' + "minutes" + '"' + ">", "</RunningTime>", xmlString);
        try {
            int minutes = Integer.parseInt(runningTime);
            if (minutes > 0)
                return new Long(minutes * 60);
        } catch (Exception ignore) {}
        
        return null;
    }
    
    private void setDirectors(Movie movie, String xmlString) {
        Collection c = getAllTagValues("<Director>", "</Director>", xmlString);
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            String name = StringUtils.convertHttpString((String) iter.next());
            DataManager.createReference(movie, Movie._J_DIRECTOR, name);
        }       
    }    
    
    private void setActors(Movie movie, String xmlString) {
        Collection c = getAllTagValues("<Actor>", "</Actor>", xmlString);
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            String name = StringUtils.convertHttpString((String) iter.next());
            DataManager.createReference(movie, Movie._I_ACTORS, name);
        }       
    }
}
