/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.amazon;

import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchTask;
import net.sf.dc.util.StringUtils;

public abstract class AmazonSearch extends OnlineSearchTask {

    public static final int _KEYWORDSSEARCH = 0;
    public static final int _ASINSEARCH = 1;

    private final static Calendar cal = Calendar.getInstance();

    public AmazonSearch(IOnlineSearchClient listener, 
                        String searchString,
                        DcServer server,
                        DcRegion region,
                        DcSearchMode mode) {
        super(listener, searchString, server, region, mode);
    }

    public void run() {
        try {
            search(getSearchURL(getQuery()));
        } catch (Exception exp) {
            listener.addError(exp);
            listener.stopped();
        }
    }
    
    public String getWhiteSpaceSubst() {
        return "%20";
    }

    private String convertSearchString(String s) {
        String result = "";

		result = s.replaceAll("[]","e");
		result = s.replaceAll("[]","u");
		result = s.replaceAll("[]","i");
		result = s.replaceAll("[]","a");
		result = s.replaceAll("","o");

		result = s.replaceAll("[]","E");
		result = s.replaceAll("[]","U");
		result = s.replaceAll("[]","I");
		result = s.replaceAll("[]","A");
	    result = s.replaceAll("","O");
	    result = result.replaceAll(" ", "%20");
        
        if (getMode() instanceof ItemLookupSearchMode) 
            result = s.replaceAll("-","");

        return result;
    }

    protected URL getSearchURL(String keywords) throws Exception {
        String s = convertSearchString(keywords);
        return new URL(getAddress() + "/onca/xml?Service=AWSECommerceService" +
                       "&AWSAccessKeyId=1N37KF9JXJXVF9JEFB02&" + getMode().getSearchCommand(s) +
                       "&ResponseGroup=Large");
    }
    
    public DcObject query(DcObject dco) throws Exception {
        String url = (String) dco.getValue(DcMediaObject._SYS_SERVICEURL);
        if (url != null && url.length() > 0) {
            HttpConnection connection = new HttpConnection(new URL(url));
            String result = connection.getString();
            connection.close();
            Collection objects = toObjects(result, getRegion().getCode());
            for (Iterator iter = objects.iterator(); iter.hasNext(); )
                return (DcObject) iter.next();
        }
        return null;
    }

    private void search(URL url) {
        listener.processing();
        isCancelled(false);

        try {
            listener.addMessage(DcResources.getText("msgConnectingToServer", getAddress()));
            
            HttpConnection connection = new HttpConnection(url);
            String result = connection.getString();
            connection.close();

            if (!isCancelled())
                createObjects(result);

            if (!isCancelled() && listener.resultCount() == 0)
                listener.addWarning(DcResources.getText("msgNoResultsForKeywords", getQuery()));

        } catch (Exception e) {
            listener.addError(DcResources.getText("msgErrorWhileRetrievingResults", e.getMessage()));
        } finally {
            listener.stopped();
        }
    }

    protected void setStorageMedium(DcObject dco, int fieldIdx, String xmlString, String tagStart, String tagEnd) {
        String medium = StringUtils.getValueBetween(tagStart, tagEnd, xmlString);
        DataManager.createReference(dco, fieldIdx, medium);
    }

    protected String getValues(String rawResult, String startTag, String endTag) {
        int indexBegin = 0;

        String values = "";
        indexBegin = rawResult.indexOf(startTag);
        String tmp = new String(rawResult);
        boolean first = true;
        while(indexBegin > -1) {
            indexBegin = indexBegin + (startTag.length());
            if (!first)
                values += ", ";

            String value = tmp.substring(indexBegin, tmp.length()).trim();
            value = value.substring(0, value.indexOf(endTag));

            if (value.indexOf(",") > -1) {
                String name = value.substring(0, value.indexOf(",")).trim();
                name = value.substring(value.indexOf(",") + 1, value.length()).trim() + " " + name;
                value = name;
            }

            values += value;

            tmp = tmp.substring(indexBegin + value.length() + endTag.length());
            indexBegin = tmp.indexOf(startTag);
            first = false;
        }

        return StringUtils.convertHttpString(values);
    }

    protected void createObjects(String xml) {
        String s = xml;
        Collection parts = new ArrayList();
        int index = s.indexOf("<Item>");

        ArrayList asins = new ArrayList();
        int counter = 0;
        while (index > -1 && !isCancelled() && counter <  getMaximum()) {
            int end = s.indexOf("</Item>") + 7;
            if (end != -1) {
                String part = s.substring(index, end);

                String asin = getAsin(part);
                if (!asins.contains(asin)) {
                    parts.add(part);
                    asins.add(asin);
                }

                s = s.substring(end, s.length());
            }
            counter++;
            index = s.indexOf("<Item>");
        }

        if (parts.size() == 0)
            return;

        listener.addMessage(DcResources.getText("msgFoundXResults", String.valueOf(parts.size())));
        listener.addMessage(DcResources.getText("msgStartParsingXResults", String.valueOf(parts.size())));
        listener.processingTotal(parts.size());
        counter = 1;
        for (Iterator iter = parts.iterator(); iter.hasNext() && !isCancelled(); ) {
            try {
                String asin = (String) asins.get(counter - 1);
                Collection objects = toObjects(iter.next().toString(), getRegion().getCode());
                for (Iterator iter2 = objects.iterator(); iter2.hasNext() && !isCancelled(); ) {
                    DcObject dco = (DcObject) iter2.next();
                    
                    setServiceInfo(dco);
                    setServiceURL(dco, asin);
                    
                    listener.addMessage(DcResources.getText("msgParsingSuccessfull", dco.getValue(DcMediaObject._A_TITLE).toString()));
                    listener.addObject(dco);
                }
                listener.processed(counter++);
            } catch (Exception exp) {
                listener.addMessage(DcResources.getText("msgParsingError", exp.getMessage()));
                DcLog.add(exp);
                listener.processed(counter);
            }
        }
        parts.clear();
        s = null;
    }

    protected void setServiceURL(DcObject dco, String asin) {
        String url = getAddress() + "/onca/xml?Service=AWSECommerceService" +
        "&AWSAccessKeyId=1N37KF9JXJXVF9JEFB02" +
        "&" + new ItemLookupSearchMode("", ItemLookupSearchMode.getDescription(ItemLookupSearchMode._ASIN), ItemLookupSearchMode._ASIN).getSearchCommand(asin) +
        "&ResponseGroup=Large";                    

        dco.setValue(DcMediaObject._SYS_SERVICEURL, url);
    }
    
    protected abstract Collection toObjects(String s, String locale);

    protected String getGenre(String xmlString) {
        return getValueUpInTree("<Name>Genres</Name>", "<Name>", "</Name>", xmlString);
    }

    protected String getStyle(String xmlString) {
        return getValueUpInTree("<Name>Styles</Name>", "<Name>", "</Name>", xmlString);
    }

    private String getValueUpInTree(String base, String startTag, String endTag, String xmlString) {
        int end =  xmlString.indexOf(base);
        if (end > -1) {
            String part = xmlString.substring(0, end);
            int begin = part.lastIndexOf(startTag);
            end = part.lastIndexOf(endTag);

            if (begin > -1 && begin < end) {
                part = part.substring(begin, end + 10);
                return StringUtils.convertHttpString(StringUtils.getValueBetween(startTag, endTag, part));
            }
        }
        return null;
    }

    protected Long getRating(String rawResult) {
        String rating = StringUtils.getValueBetween("<AverageRating>", "</AverageRating>", rawResult);
        Long iRating = new Long(0);
        try {
            if (rating.trim().length() > 0) {
                iRating = new Long(Math.round(Float.valueOf(rating).floatValue()));
            }
        } catch (Exception ignore) {};

        return iRating;
    }

    protected String getAsin(String rawResult) {
        return StringUtils.convertHttpString(StringUtils.getValueBetween("<ASIN>", "</ASIN>", rawResult));
    }

    protected String getTitle(String rawResult) {
        return StringUtils.convertHttpString(StringUtils.getValueBetween("<Title>", "</Title>", rawResult));
    }

    protected String getAmazonURL(String rawResult) {
        return StringUtils.getValueBetween("<DetailPageURL>", "</DetailPageURL>", rawResult);
    }

    protected String getUPC(String rawResult) {
        return StringUtils.getValueBetween("<UPC>", "</UPC>", rawResult);
    }

    protected Long getTheatricalReleaseDate(String rawResult) {
        return getYear(rawResult, "<TheatricalReleaseDate>", "</TheatricalReleaseDate>");
    }

    protected Long getReleaseDate(String rawResult) {
        return getYear(rawResult, "<ReleaseDate>", "</ReleaseDate>");
    }

    protected Long getPublicationDate(String rawResult) {
        return getYear(rawResult, "<PublicationDate>", "</PublicationDate>");
    }

    protected Long getYear(String rawResult, String startTag, String endTag) {
        Long releaseYear = null;
        String releaseDate = StringUtils.getValueBetween(startTag, endTag, rawResult);

        try {
            Date date = new SimpleDateFormat("yyyy-MM-dd").parse(releaseDate);
            cal.setTime(date);
            releaseYear = new Long(cal.get(Calendar.YEAR));
        } catch (Exception ignore) {
            try {
                releaseYear = new Long(releaseDate.substring(0, 4));
            } catch (Exception ignore2) {}
        }

        return releaseYear;
    }

    protected void setImages(DcObject dco, String xmlString, int frontFieldIdx, int[] screenFieldIdx) {
        String mainImage = StringUtils.getValueBetween("<LargeImage>", "</LargeImage>", xmlString);
        String mainImageURL = StringUtils.getValueBetween("<URL>", "</URL>", mainImage);
        setImage(dco, frontFieldIdx, mainImageURL);

        if (screenFieldIdx != null) {
            Collection imageSets = getAllTagValues("<ImageSet Category=" + '"' + "variant", "</ImageSet>", xmlString);
            int counter = 0;
            for (Iterator iter = imageSets.iterator(); iter.hasNext() && counter < screenFieldIdx.length; counter++) {
                Collection screenshots = getAllTagValues("<LargeImage>", "</LargeImage>", (String) iter.next());
                for (Iterator iterScreens = screenshots.iterator(); iterScreens.hasNext();) {
                    String imageURL = StringUtils.getValueBetween("<URL>", "</URL>", (String) iterScreens.next());
                    setImage(dco, screenFieldIdx[counter], imageURL);
                }
            }
        }
    }

    protected Collection getAllTagValues(String startTag, String endTag, String rawResult) {
        Collection result = new ArrayList();

        if (rawResult.indexOf(startTag) > -1) {
            String values = rawResult.substring(rawResult.indexOf(startTag),
                                                   rawResult.lastIndexOf(endTag) + endTag.length());

            int indexBegin = values.indexOf(startTag);
            int indexEnd = 0;
            while(indexBegin > -1) {
                indexBegin = indexBegin + startTag.length();
                indexEnd = values.indexOf(endTag);

                String platform = values.substring(indexBegin, indexEnd) + " ";
                result.add(StringUtils.convertHttpString(platform));

                indexEnd = indexEnd + endTag.length();
                values = values.substring(indexEnd, values.length());
                indexBegin = values.indexOf(startTag);
            }
        }
        return result;
    }

    private void setImage(DcObject dco, int fieldIndex, String URL) {
        try {
            URL url = new URL(URL);
            HttpConnection connection = new HttpConnection(url);
            dco.setValue(fieldIndex, connection.getBytes());
            connection.close();
        } catch (Exception ignore) {}
    }


    protected String getDescription(String xmlString) {
        String description = getEditoralReview(xmlString);
        String userReviews = getUserReview(xmlString);

        if (userReviews.trim().length() > 0) {
            if (description.length() > 0)
                description += "\n\n";

            description += userReviews;
        }

        return StringUtils.convertHttpString(description);
    }

    protected String getEditoralReview(String xmlString) {
        String xmlReviews = StringUtils.getValueBetween("<EditorialReviews>", "</EditorialReviews>", xmlString);
        int counter = 1;
        String reviews = "";
        while (xmlReviews.indexOf("<Content>") > -1) {
            if (counter > 1)
                reviews += "\n\n";

            int begin = xmlReviews.indexOf("<Content>");
            int end = xmlReviews.indexOf("</Content>");
            String review = xmlReviews.substring(begin + 9, end);

            if (review.indexOf("&lt;p clear=" + '"' + "all" + '"') > -1)
                review = review.substring(0, review.indexOf("&lt;p clear=" + '"' + "all" + '"'));

            reviews += review;

            xmlReviews = xmlReviews.substring(end + 10, xmlReviews.length());
            counter++;
        }
        return reviews;
    }


    protected String getUserReview(String xmlString) {
        String xmlReviews = StringUtils.getValueBetween("<CustomerReviews>", "</CustomerReviews>", xmlString);
        int counter = 1;
        String reviews = "";
        while (xmlReviews.indexOf("<Content>") > -1) {
            if (counter > 1) {
                reviews += "\n\n";
            }
            int begin = xmlReviews.indexOf("<Content>");
            int end = xmlReviews.indexOf("</Content>");
            reviews += "-- review " + (counter++) + " --\n" +
                       xmlReviews.substring(begin + 9, end);
            xmlReviews = xmlReviews.substring(end + 10, xmlReviews.length());
        }
        return reviews;
    }
}
