/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.amazon;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchTask;

public class AmazonServer extends DcServer {

    public AmazonServer(int module) {
        this("Amazon", module);
    }
    
    public AmazonServer(String name, int module) {
        super(name, "http://webservices.amazon.com", true, module);
        addRegion(new DcRegion("us", "United States (english)", "http://webservices.amazon.com"));
        addRegion(new DcRegion("uk", "United Kingdom (english)", "http://webservices.amazon.co.uk"));
        addRegion(new DcRegion("de", "Germany (german)", "http://webservices.amazon.de"));
        addRegion(new DcRegion("fr", "France (french)", "http://webservices.amazon.fr"));
        addRegion(new DcRegion("ca", "Canada (english)", "http://webservices.amazon.ca"));
    }
    
    public OnlineSearchTask getTask(IOnlineSearchClient listener, 
                                    String searchString, 
                                    DcSearchMode mode, 
                                    DcRegion region) {
        
        if (getModule() == DcModules._SOFTWARE) {
            return new AmazonSoftwareSearch(listener, searchString, this, region, mode);
        } else if (getModule() == DcModules._MOVIE) {
            return new AmazonMovieSearch(listener, searchString, this, region, mode);
        } else if (getModule() == DcModules._BOOK) {
            return new AmazonBookSearch(listener, searchString, this, region, mode);
        } else if (getModule() == DcModules._AUDIOCD || getModule() == DcModules._MUSICALBUM) {
            return new AmazonAudioCdSearch(listener, searchString, this, region, mode);
        }

        return null;
    }
}
