/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.amazon;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcProperty;
import net.sf.dc.core.objects.helpers.Software;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.util.StringUtils;

public class AmazonSoftwareSearch extends AmazonSearch {

    public AmazonSoftwareSearch(IOnlineSearchClient listener, 
            String searchString, 
            DcServer server, 
            DcRegion region,
            DcSearchMode mode) {

        super(listener, searchString, server, region, mode);
    }

    protected Collection toObjects(String xmlString, String locale) {
        Collection objects = new ArrayList();
        try {
            Software software = new Software();
    
            setStorageMedium(software, Software._W_STORAGEMEDIUM, xmlString, "<Format>", "</Format>");
    
            software.setValue(Software._A_TITLE, getTitle(xmlString));
            software.setValue(Software._B_DESCRIPTION, getDescription(xmlString));
            
            setPublisher(software, xmlString);
            setDeveloper(software, xmlString);
            
            software.setValue(Software._C_YEAR, getReleaseDate(xmlString));
            software.setValue(Software._I_WEBPAGE, getAmazonURL(xmlString));
            software.setValue(Software._E_RATING, getRating(xmlString));
            
            String asin = getAsin(xmlString);
            software.setValue(Software._U_ASIN, asin);
            setServiceURL(software, asin);
            
            setPlatform(software, xmlString);
    
            setImages(software, xmlString,
                      Software._M_PICTUREFRONT, 
                      new int[] {Software._P_SCREENSHOTONE, Software._Q_SCREENSHOTTWO, Software._R_SCREENSHOTTHREE});
            
            objects.add(software);
        } catch (Exception e) {
            listener.addError(e);
        }
            
        return objects;
    }
    
    protected void setDeveloper(Software software, String rawResult) {
        String name = StringUtils.convertHttpString(StringUtils.getValueBetween("<Manufacturer>", "</Manufacturer>", rawResult));
        DataManager.createReference(software, Software._F_DEVELOPER, name);
    }

    protected void setPublisher(Software software, String rawResult) {
        String name = StringUtils.convertHttpString(StringUtils.getValueBetween("<Publisher>", "</Publisher>", rawResult));
        DataManager.createReference(software, Software._G_PUBLISHER, name);
    }
    
    private void setPlatform(Software software, String xmlString) {
        DcModule module = DcModules.get(DcModules._SOFTWARE);
        
        DcObject[] dcPlatforms = DataManager.get(module.getPropertyModule(DcModules._PLATFORM).getIndex(), null); 
        Collection platforms = getAllTagValues("<Platform>", "</Platform>", xmlString);
        
        for (Iterator iter = platforms.iterator(); iter.hasNext(); ) {
            String platform = (String) iter.next();
            for (int i = 0; i < dcPlatforms.length; i++) {
                String dcPlatformName = dcPlatforms[i].getValue(DcProperty._A_NAME).toString().toLowerCase();
                if (platform.toLowerCase().equals(dcPlatformName)) 
                    DataManager.createReference(software, Software._H_PLATFORM, dcPlatforms[i]);
            }        
        }
        
        for (Iterator iter = platforms.iterator(); iter.hasNext(); ) {
            String platform = (String) iter.next();
            for (int i = 0; i < dcPlatforms.length; i++) {
                String dcPlatformName = dcPlatforms[i].getValue(DcProperty._A_NAME).toString().toLowerCase();
                if (!platform.toLowerCase().equals(dcPlatformName) &&
                    platform.toLowerCase().indexOf(dcPlatformName) > -1 || dcPlatformName.indexOf(platform) > -1)
                    DataManager.createReference(software, Software._H_PLATFORM, dcPlatforms[i]);
            }        
        }  
    }
}
