/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.amazon;

import java.util.HashMap;
import java.util.Map;

import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.util.isbn.Isbn;


public class ItemLookupSearchMode extends DcSearchMode {

    private final String label;
    private final String type;
    private String index;
    
    public static final String _ASIN = "ASIN";
    public static final String _UPC = "UPC";
    public static final String _EAN = "EAN";
    public static final String _ISBN = "ISBN";
    
    private static Map descriptions = new HashMap();
    
    static {
    	descriptions.put(_ASIN, "Asin");
    	descriptions.put(_UPC, "UPC Barcode");
    	descriptions.put(_EAN, "EAN Barcode");
    	descriptions.put(_ISBN, "ISBN");
    }
    
    public static String getDescription(String type) {
    	return (String) descriptions.get(type);
    }
    
    public ItemLookupSearchMode(String index, String label, String type) {
        this.label = label;
        this.type = type;
        this.index = index;
    }

    public String getDisplayName() {
        return label;
    }

    public String getSearchCommand(String s) {
    	if (type.equals(_ISBN)) {
    		String isbn13 = s;
    		try {
    			Isbn isbn = new Isbn();
    			if (isbn.isIsbn10(s))
    				isbn13 = isbn.getIsbn13(s);
    		} catch (Exception e) {}
    		
    		return "Operation=ItemLookup&ItemId=" + isbn13 + "&IdType=" + type + "&SearchIndex=" + index;
    	} else if (type.equals(_ASIN)) {
    		return "Operation=ItemLookup&ItemId=" + s + "&IdType=" + type;
    	} else {
    		return "Operation=ItemLookup&ItemId=" + s + "&IdType=" + type + "&SearchIndex=" + index;
    	}
    }
    
    public boolean singleIsPerfect() {
        return true;
    }
    
    public boolean keywordSearch() {
        return false;
    }     
}
