/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.bol;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcAssociate;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.Book;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchTaskImpl;
import net.sf.dc.util.StringUtils;
import net.sf.dc.util.isbn.Isbn;

public class BolBookSearch extends OnlineSearchTaskImpl {

    public BolBookSearch(IOnlineSearchClient listener, String searchString, DcServer server, DcRegion region, DcSearchMode mode) {
        super(listener, searchString, server, region, mode);
    }

    public String getWhiteSpaceSubst() {
        return "+";
    }

    protected Collection getIDs() {
        Collection ids = new ArrayList();
        String webpage = "";
        try {
            listener.addMessage(DcResources.getText("msgConnectingToServer", getAddress()));
            URL url = new URL(getAddress() + getMode().getSearchCommand(getQuery()));
            HttpConnection connection = new HttpConnection(url);
            webpage = connection.getString();
            connection.close();
        } catch (Exception e) {
            listener.addError(DcResources.getText("msgCouldNotConnectTo", getServer().getName()));
        }
        
        int idx = webpage.toUpperCase().indexOf("BOL_OWNER_ID");
        while (idx > -1) {
            String id = StringUtils.getValueBetween("BOL_OWNER_ID=", "&", webpage);
            
            if (!ids.contains(id)) 
                ids.add(id);
            
            webpage = webpage.substring(idx + 30);
            idx = webpage.toUpperCase().indexOf("BOL_OWNER_ID");
        }
        
        return ids;
    }    

    protected DcObject getDcObject(String id) throws Exception { 
        String link = getAddress() +  "/is-bin/INTERSHOP.enfinity/eCS/Store/nl/-/EUR/BOL_DisplayProductInformation-Secondary?" +
                                      "BOL_OWNER_ID=" + id + "&Secondary=description&Section=BOOK";
        return getDcObject(new URL(link));
    }
        
    protected DcObject getDcObject(URL url) throws Exception {
        HttpConnection connection = new HttpConnection(url);
        String webpage = connection.getString("ISO-8859-1");
        
        String title = StringUtils.convertHttpString(StringUtils.getValueBetween("<h1>", "</h1>", webpage));
        String description = StringUtils.getValueBetween("<tr id=\"beschrijving\">", "</tr>", webpage);
        description = description.length() == 0 ? StringUtils.getValueBetween("<tr id=\"bechrijving\">", "</tr>", webpage) : description;
        description = description.length() == 0 ? StringUtils.getValueBetween("<h2>Beschrijving:</h2>", "</td>", webpage) : description;
        if (description.indexOf("</h2>") > -1)
            description = description.substring(description.indexOf("</h2>") + 5);
        
        description = StringUtils.convertHttpString(description);
        
        Book book = new Book();
        book.setValue(Book._A_TITLE, title);
        book.setValue(Book._B_DESCRIPTION, description);
        book.setValue(Book._H_WEBPAGE, url.toString());
        book.setValue(Book._SYS_SERVICEURL, url.toString());
        
        setIsbn(book, webpage);
        setCover(book, webpage);
        setPages(book, webpage);
        setAuthor(book, webpage);
        
        return book;
    }
    
    /**
     * Non-reusable author (Dutch)
     */
    private void setAuthor(Book book, String webpage) {
        String s = webpage;
        
        Collection ids = new ArrayList();
        int idx = s.toUpperCase().indexOf("CREATORPAGEINFORMATION");
        while (idx > -1) {
            String url = s.substring(idx - 120);
            
            url = url.substring(url.indexOf("<a"));
            
            String id = StringUtils.getValueBetween("AUTHOR_ID=", "\"", url);
            
            if (!ids.contains(id)) {
                ids.add(id);
                String name = StringUtils.convertHttpString(StringUtils.getValueBetween(">", "</a>", url));
                DcObject author = null;
                if (name.trim().length() > 0)
                    author = DataManager.getObjectForDisplayValue(DcModules._AUTHOR, name);
                
                if (author == null && name.trim().length() > 0) {
                    
                    url = StringUtils.getValueBetween("<a href=\"", "\"", url);
                    author = DcModules.get(DcModules._AUTHOR).getDcObject();
                    author.setValue(DcAssociate._A_NAME, name);
                    
                    DcModule module = DcModules.get(getServer().getModule());
                    if (module.getSettings().getBoolean(DcRepository.modulesettings.stOnlineSearchSubItems)) {
	                    try {
	                        HttpConnection connection = new HttpConnection(new URL(url));
	                        String page = connection.getString();
	                        connection.close();
	                        idx = page.indexOf(id + ".gif"); 
	                        if (idx > -1) {
	                            String imgUrl = page.substring(0, idx + 4 + id.length());
	                            imgUrl = imgUrl.substring(imgUrl.lastIndexOf("\"") + 1);
	                            
	                            if (imgUrl.startsWith("http")) {
	                                connection = new HttpConnection(new URL(imgUrl));
	                                byte[] image = connection.getBytes();
	                                connection.close();
	                                
	                                if (image != null && image.length > 0)
	                                    author.setValue(DcAssociate._D_PHOTO, image);
	                            }
	                        }
	                    } catch (Exception e) {
	                        DcLog.add(e);
	                    }
                    }
                     
                    try {
                        url.replaceAll("DisplayCreatorPageInformation-Author", "DisplayCreatorPageInformation-AuthorSecondary");
                        url += "&sec=bio";
                        HttpConnection connection = new HttpConnection(new URL(url));
                        String page = connection.getString("ISO-8859-1");
                        connection.close();
                        
                        String description = getCleanText(page);
                        description = StringUtils.getValueBetween("- Biografie", "Terug naar", description);
                        while (description.startsWith("\n") || description.startsWith("\r") || description.startsWith(" "))
                            description = description.substring(1);
                        
                        author.setValue(DcAssociate._B_DESCRIPTION, description);
                    } catch (Exception e) {
                        DcLog.add(e);
                    }        
                    
                    author.setIDs();
                }
                
                DataManager.addMapping(book, author, Book._G_AUTHOR);
            }

            s = s.substring(s.toUpperCase().indexOf("CREATORPAGEINFORMATION") + 22);
            idx = s.toUpperCase().indexOf("CREATORPAGEINFORMATION");
        }
    }
    
    private void setPages(Book book, String webpage) {
        int idx = webpage.toUpperCase().indexOf("PAGINA'S");
        if (idx > -1) {
            String part = webpage.substring(idx - 20, idx);
            
            char[] chars = part.toCharArray();
            String pages = "";
            for (int i = 0; i < chars.length; i++) {
                if (Character.isDigit(chars[i]))
                    pages += chars[i];    
                else if (pages.length() > 0)
                    break;
            }
            
            if (pages.length() > 0)
                book.setValue(Book._T_NROFPAGES, new Integer(pages));
        }
    }
    
    private void setCover(Book book, String webpage) {
        int idx = webpage.toUpperCase().indexOf("BOOKCOVER");
        for (; idx > -1; idx--) {
            if (webpage.charAt(idx) == '"') {
                ++idx;
                break;
            }
        }
        
        String url = idx > -1 ? webpage.substring(idx) : "";
        
        idx = url.toUpperCase().indexOf("?IMAGEURL=");
        if (idx > -1) {
            url = url.replaceAll("%20", " ");
            url = url.replaceAll("%2f", "/");
            url = url.replaceAll("%2e", ".");
            
            url = url.substring(idx + 10);
            url = url.substring(0, url.indexOf(".gif") > -1 ? url.indexOf(".gif") + 4 : url.indexOf("&"));
            url = url.startsWith("http://") ? url : "http://img.bol.com/intershoproot" + (url.startsWith("/") ? url : "/" + url);
        } else {
            try {
                url = url.substring(0, url.indexOf("\""));
            } catch (Exception e) {}
        }
            
        if (url.length() > 0) {
            try {
                HttpConnection connection = new HttpConnection(new URL(url));
                byte[] image = connection.getBytes();
                connection.close();
            
                if (image != null && image.length > 0)
                    book.setValue(Book._K_PICTUREFRONT, image);
            } catch (Exception exp) {
                DcLog.add(exp);
            }
        }
    }
    
    private void setIsbn(Book book, String webpage) {

        String part = webpage;
        try {
            int idx = part.toUpperCase().indexOf("ISBN10:");
            idx = idx == -1 ? part.toUpperCase().indexOf("ISBN13:") : idx;
            if (idx > -1) {
                part = part.substring(idx);
                part = part.substring(part.indexOf("</b>"));
                part = part.substring(0, part.indexOf("</span>"));
            
                char[] chars = part.toCharArray();
                String isbn = "";
                for (int i = 0; i < chars.length; i++) {
                    if (Character.isDigit(chars[i]))
                        isbn += chars[i];    
                    else if (isbn.length() > 0)
                        break;
                }

                Isbn isbnUtil = new Isbn(); 
                if (isbnUtil.isIsbn10(isbn))
                    book.setValue(Book._J_ISBN10, isbn);
                else if (isbnUtil.isIsbn13(isbn))
                    book.setValue(Book._N_ISBN13, isbn);
            }
        } catch (Exception exp) {
            DcLog.add(exp);
        }
    }
}
