/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.discogs;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.JFrame;

import net.sf.dc.console.windows.PictureSelectDialog;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.AudioCD;
import net.sf.dc.core.objects.helpers.AudioTrack;
import net.sf.dc.core.objects.helpers.MusicAlbum;
import net.sf.dc.core.objects.helpers.MusicTrack;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchTaskImpl;
import net.sf.dc.util.StringUtils;

public class DiscogsAudioCdSearch extends OnlineSearchTaskImpl {

    public DiscogsAudioCdSearch(IOnlineSearchClient listener, String searchString, DcServer server, DcRegion region, DcSearchMode mode) {
        super(listener, searchString, server, region, mode);
    }

    public String getWhiteSpaceSubst() {
        return "+";
    }
    
    protected Collection getIDs() {
        Collection ids = new ArrayList();
        
        try {
            String link = getAddress() + getMode().getSearchCommand(getQuery());
            HttpConnection connection = new HttpConnection(new URL(link));
            String page = connection.getString();
            
            int idx = page.indexOf("/release/");
            while (idx > -1) {
                String id = StringUtils.getValueBetween("/release/", "\"", page);
                
                Number number = StringUtils.getContainedNumber(id);
                if (number != null && !ids.contains("" + number))
                    ids.add("" + number);
                
                page = page.substring(idx + 10);
                idx = page.indexOf("/release/");
            }
        } catch (Exception e) {
            listener.addError(e);
        }
        
        return ids;
    }
    
    protected DcObject getDcObject(String id) throws Exception {
        String link = getAddress() +  "release/" + id;
        return getDcObject(new URL(link));
    }
    
    protected DcObject getDcObject(URL url) throws Exception {
    	DcModule module = DcModules.get(getServer().getModule());
        DcObject album = module.getDcObject();
        HttpConnection connection = new HttpConnection(url);
        String page = connection.getString();     
        
        album.setValue(DcObject._SYS_SERVICEURL, url.toString());
        
        setWebpage(album, url);

        setTitle(album, page);
        
        Collection artists = setArtist(album, page);
        
        setYear(album, page);
        setGenres(album, page);
        setRating(album, page);
        
        setImages(album, page);
        
        addTracks(album, artists, page);
        
        return album;
    }
    
    private void setImages(DcObject album, String page) {
    	int idx = page.indexOf("/viewimages?what=");
    	if (idx > -1) {
    		String link = page.substring(idx);
    		link = getAddress() + link.substring(1, link.indexOf("\""));
    		
    	    try {
	            HttpConnection connection = new HttpConnection(new URL(link));
	            String webpage = connection.getString();
	            connection.close();
	            
	            idx = webpage.indexOf("http://www.discogs.com/image");
	            while (idx > -1 && !isCancelled()) {
	            	link = webpage.substring(idx);
	            	link = link.substring(0, link.indexOf("\""));

	            	connection = new HttpConnection(new URL(link));
                    byte[] image = connection.getBytes();
	            	connection.close();
	            	
                    if (!isCancelled()) {
                        PictureSelectDialog dlg = 
                            new PictureSelectDialog(listener instanceof JFrame ? (JFrame) listener : null, 
                                                    album, image);
                        dlg.setVisible(true);
                    }
	            	
	            	webpage = webpage.substring(idx + 30);
	            	idx = webpage.indexOf("http://www.discogs.com/image");
	            }
    	    } catch (Exception e) {
    	    	listener.addError(e);
    	    }
    	}
    }
    
    private void setTitle(DcObject album, String page) {
        String title = StringUtils.getValueBetween("<title>", "</title>", page);
        title = title.indexOf("-") > -1 ? title.substring(title.indexOf("-") + 1) : title;
        album.setValue(DcMediaObject._A_TITLE, title.trim());
    }

    private Collection setArtist(DcObject album, String page) {
        String artist = StringUtils.getValueBetween("<title>", "</title>", page);
        artist = artist.indexOf("-") > -1 ? artist.substring(0, artist.indexOf("-")) : artist;
        artist = artist.trim();
        
        Collection artists = new ArrayList();
        if (artist.length() > 0) {
            Collection names = new ArrayList();
            if (artist.indexOf("/") > -1) {
                StringTokenizer st = new StringTokenizer(artist, "/"); 
                while (st.hasMoreElements()) {
                    names.add(StringUtils.convertHttpString((String) st.nextElement()).trim());
                }
            } else {
                names.add(StringUtils.convertHttpString(artist.trim()));
            }

            DcModule module = DcModules.get(getServer().getModule());
            int fieldIdx = module.getIndex() == DcModules._AUDIOCD ? AudioCD._F_ARTIST : MusicAlbum._F_ARTISTS;
            
            for (Iterator iter = names.iterator(); iter.hasNext(); ) 
                artists.add(DataManager.createReference(album, fieldIdx, iter.next()));
        }
        
        return artists;
    }
    
    private void addTracks(DcObject album, Collection artists, String page) {
        boolean isAudioTrack = album.getModule().getChild().getIndex() == DcModules._AUDIOTRACK;
        
        String tracks = StringUtils.getValueBetween("<b>Tracklisting:</b><br>", "</table>", page);
        int idx = tracks.indexOf("<tr>");
        while (idx > -1) {
            DcObject track = album.getModule().getChild().getDcObject();
            String part = StringUtils.getValueBetween("<tr>", "</tr>", tracks);
            
            int partIdx = part.indexOf("<td");
            ArrayList parts = new ArrayList();
            while (partIdx > -1) {
                part = part.substring(partIdx + 3);
                parts.add(StringUtils.getValueBetween(">", "</td>", part));
                partIdx = part.indexOf("<td");
            }

            if (parts.size() > 0) {
                String s = (String) parts.get(0);
                Long trackNr = StringUtils.getContainedNumber(s);
                track.setValue(isAudioTrack ? AudioTrack._F_TRACKNUMBER : MusicTrack._F_TRACKNUMBER, trackNr);
            }
            
            if (parts.size() > 2) {
                String s = (String) parts.get(2);
                String length = StringUtils.getValueBetween("(", ")", s).trim();
                if (length.indexOf(":") > -1) {
                    String minutes = length.substring(0, length.indexOf(":"));
                    String seconds = length.substring(length.indexOf(":") + 1, length.length());
                    
                    try {
                        Integer min = Integer.valueOf(minutes);
                        Integer sec = Integer.valueOf(seconds);
                        track.setValue(isAudioTrack ? AudioTrack._H_PLAYLENGTH : MusicTrack._J_PLAYLENGTH, 
                                       new Long(sec.intValue() + (min.intValue() * 60)));
                    } catch (NumberFormatException nfe) {}
                    
                    s = s.substring(0, s.indexOf("(")).trim();
                }
                
                track.setValue(DcMediaObject._A_TITLE, s);

                for (Iterator iter = artists.iterator(); iter.hasNext(); ) 
                    DataManager.addMapping(track, (DcObject) iter.next(), 
                                           isAudioTrack ? AudioTrack._K_ARTIST : MusicTrack._G_ARTIST);
            } 
            
            tracks = tracks.substring(idx + 4);
            idx = tracks.indexOf("<tr>");
            album.addChild(track);
        }
    }
    
    private void setWebpage(DcObject dco, URL link) {
    	DcModule module = DcModules.get(getServer().getModule());
        int fieldIdx = module.getIndex() == DcModules._AUDIOCD ? AudioCD._M_WEBPAGE : MusicAlbum._N_WEBPAGE;   
        dco.setValue(fieldIdx, link.toString());
    }
    
    private void setRating(DcObject dco, String page) {
        String rating = StringUtils.getValueBetween("<tr><td align=right>Rating:</td>", "</td>", page);
        rating = StringUtils.getValueBetween("<b>", "</b>", rating);
        
        int idx = rating.indexOf(".");
        if (idx > 0) {
            rating = rating.substring(0, idx);
            dco.setValue(DcMediaObject._E_RATING, new Long(rating));
        }
    }
    
    private void setYear(DcObject dco, String page) {
        String year = StringUtils.getValueBetween("Released:</td><td>", "</td>", page);
        
        DcModule module = DcModules.get(getServer().getModule());
        if (year.length() > 0) {
            year = year.indexOf(" ") > -1 ? year.substring(year.lastIndexOf(" ") + 1) : year;
            int fieldIdx = module.getIndex() == DcModules._AUDIOCD ? AudioCD._C_YEAR : MusicAlbum._C_YEAR;
            dco.setValue(fieldIdx, new Long(year));
        }
    }
    
    private void setGenres(DcObject dco, String page) {
    	DcModule module = DcModules.get(getServer().getModule());
        int fieldIdx = module.getIndex() == DcModules._AUDIOCD ? AudioCD._G_GENRES : MusicAlbum._G_GENRES;
        
        String s = StringUtils.getValueBetween("<td align=right>Genre:</td><td>", "</td>", page);
        StringTokenizer st = new StringTokenizer("\r", s);
        while (st.hasMoreTokens()) {
            String genre = st.nextToken();
            if (genre.trim().length() > 0)
                DataManager.createReference(dco, fieldIdx, genre);
        }
    }
}
