/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.imdb;

import java.io.IOException;
import java.io.StringReader;
import java.net.URL;

import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcAssociate;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.util.StringUtils;

public class ImdbPerson {

    private final int module;
    
    private JTextPane descriptionPane;
    private HTMLEditorKit kit = new HTMLEditorKit();
    private HTMLDocument document = new HTMLDocument();
    
    private String address = "http://www.imdb.com";
    
    public ImdbPerson(int module) {
        this.module = module;
        
        descriptionPane = ComponentFactory.getTextPane();
        descriptionPane.setContentType("text/html");
        descriptionPane.setEditorKit(kit);
        descriptionPane.setDocument(document);
        descriptionPane.setEditable(false);        
    }
    
    public void clear() {
        descriptionPane = null;
        kit = null;
        document = null;
        address = null;
    }
    
    public DcObject get(String id) throws Exception {
        URL url = new URL(address + "/name/nm" + id);
        return get(url);
    }    
    
    public DcObject get(URL url) throws Exception {
        DcObject person = DcModules.get(module).getDcObject();
        HttpConnection connection = new HttpConnection(url);
        String page = connection.getString();
        connection.close();
        
        String s = url.toString();
        String id = s.substring(s.indexOf("/name/nm") + 8, s.length());
        String name = StringUtils.convertHttpString(StringUtils.getValueBetween("<title>", "</title>", page));

        person.setValue(DcAssociate._A_NAME, name);
        person.setValue(DcAssociate._C_WEBPAGE, address + "/name/nm" + id);
        person.setValue(DcObject._SYS_SERVICEURL, s);
        
        setBiography(person, id);
        setPhoto(person, page);

        return person;
    }
    
    private void setBiography(DcObject person, String id) throws Exception {
        URL url = new URL("http://www.imdb.com/name/nm" + id + "/bio");
        HttpConnection connection = new HttpConnection(url);
        String page = connection.getString();
        connection.close();

        if (page.indexOf("<h5>") > -1) {
            page = page.substring(page.indexOf("<h5>"), page.length());
            page = page.substring(0, page.indexOf("<div align=\"center\">"));
            page = "<html><body>" + page + "</body></html>";
            person.setValue(DcAssociate._B_DESCRIPTION, getCleanText(page));
        }
    }
    
    private void setPhoto(DcObject person, String page) throws Exception {
        String s = StringUtils.getValueBetween("<div class=\"photo\">", "</div>", page);
        if (s.length() > 0 && s.indexOf("Photo Not Submitted") == -1) {
            String link = StringUtils.getValueBetween("src=\"", "\"", s);
            URL url = new URL(link);
            HttpConnection connection = new HttpConnection(url);
            byte[] image = connection.getBytes();
            person.setValue(DcAssociate._D_PHOTO, image);
        }
    }
    
    protected synchronized String getCleanText(String html) throws IOException, BadLocationException {
        String webpage = html;
        descriptionPane.read(new StringReader(webpage), "Data Crow");     
        Document doc = descriptionPane.getDocument();
        String text = doc.getText(0, doc.getLength());
        text = text.replaceAll("\r", "\n");
        text = text.replaceAll("\n\n", "\n");
        
        while (text.startsWith("\n"))
            text = text.substring(1, text.length());
        
        return text.trim();
    }
}
