/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.imdb;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;

public class ImdbPersonSearch extends ImdbSearch {

    private int module;

    public ImdbPersonSearch(IOnlineSearchClient listener, int module, String searchString, DcServer server, DcRegion region) {
        super(listener, searchString, server, region, null);
        this.module = module;
    }
    
    public ImdbPersonSearch(IOnlineSearchClient listener, String searchString, int max) {
        super(listener, searchString, max);
    }
    
    protected DcObject getDcObject(URL url) throws Exception {
        ImdbPerson personFinder = new ImdbPerson(module);
        return personFinder.get(url);
    }
    
    protected DcObject getDcObject(String id) throws Exception {
        ImdbPerson personFinder = new ImdbPerson(module);
        DcObject person = personFinder.get(id);
        return person;
    }
    
    protected Collection getIDs() {
        Collection keys = new ArrayList(0);

        try {
            listener.addMessage(DcResources.getText("msgConnectingToServer", getAddress()));
            URL url = new URL(getAddress() + "/find?s=nm&q=" + getQuery());

            HttpConnection connection = new HttpConnection(url);
            String page = connection.getString();
            connection.close();

            String tmp = page;
            if (tmp.indexOf("/name/nm") > -1) {
                int temp = 0;
                int counter = 0;
                while (temp != -1 && counter < getMaximum()) {
                    counter++;
                    temp = tmp.indexOf("/name/nm");
                    if (temp != -1) {
                        tmp = tmp.substring(temp + 8, tmp.length());
                        String key  = tmp.substring(0, 7);
                        if (!keys.contains(key))
                            keys.add(key);
                    }
                }
            }
        } catch (Exception e) {
            listener.addError(DcResources.getText("msgCouldNotConnectToImdb"));
        }
        
        return keys;
    }
}
