/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.imdb;

import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchTaskImpl;
import net.sf.dc.util.StringUtils;

public abstract class ImdbSearch extends OnlineSearchTaskImpl {

    protected final static int _DIRECTEDBY = 0;
    protected final static int _RUNTIME = 1;
    protected final static int _USERRATING = 2;
    protected final static int _COUNTRY = 3;
    protected final static int _CERTIFICATION = 4;
    protected final static int _GENRE = 5;
    protected final static int _TRIVIA = 6;
    
    protected String[][] headers = 
           {{"Director", "Dirigida por", "Ralis par", "Regie", "Regia di"},
            {"Runtime", "Duraci&oacute;n:", "Dure:", "L&auml;nge:", "Durata:"},
            {"User Rating:", "Calificaci&oacute;n de los usuarios:", "Note des utilisateurs:", "Ihre Bewertung:", "Voti degli utenti:"},
            {"Country:", "Pa&iacute;s:", "Pays:", "Produktionsland:", "Nazionalit:"},
            {"Certification", "Clasificaci&oacute;n:", "Classification:", "Altersfreigabe:", "Divieti:"},
            {"Genre", "Genere:"},
            {"Trivia"}};
    
    public ImdbSearch(IOnlineSearchClient listener, 
                      String searchString, 
                      DcServer server, 
                      DcRegion region, 
                      DcSearchMode mode) {
        
        super(listener, searchString, server, region, mode);
    }
    
    public ImdbSearch(IOnlineSearchClient listener, String searchString, int max) {
        super(listener, searchString, max);
    }
    
    public String getWhiteSpaceSubst() {
        return "%20";
    }
    
    protected int indexOf(int headerindex, String source) {
        int index = -1;
        for (int i = 0; i < headers[headerindex].length; i++) {
            index = source.indexOf(headers[headerindex][i]);
            if (index > -1) break;
        }
        return index;
    }

    protected String getPart(int headerindex, String end, String source) {
        String s = "";
        for (int i = 0; i < headers[headerindex].length; i++) {
            s = StringUtils.getValueBetween(headers[headerindex][i], end, source);
            if (s != null && s.length() > 1) break;
        }
        return s;
    }    
}
