/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.imdb;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchTask;

public class ImdbServer extends DcServer {

    public ImdbServer(int module) {
        super("Imdb", "http://www.imdb.com", true, module);
        addRegion(new DcRegion("us", "Default (english)", "http://www.imdb.com"));
        addRegion(new DcRegion("us", "USA (english)", "http://us.imdb.com"));
        addRegion(new DcRegion("it", "Italy (english)", "http://italian.imdb.com"));
        addRegion(new DcRegion("de", "Germany (english)", "http://german.imdb.com"));
        addRegion(new DcRegion("fr", "France (english)", "http://french.imdb.com"));
        addRegion(new DcRegion("sp", "Spain (english)", "http://spanish.imdb.com"));        
    }
    
    public OnlineSearchTask getTask(IOnlineSearchClient listener, 
                                    String searchString, 
                                    DcSearchMode mode, 
                                    DcRegion region) {

        if (getModule() == DcModules._MOVIE)
            return new ImdbMovieSearch(listener, searchString, this, region, null);
        else if (getModule() == DcModules._ACTOR)
            return new ImdbPersonSearch(listener, DcModules._ACTOR, searchString, this, region);
        else 
            return new ImdbPersonSearch(listener, DcModules._DIRECTOR, searchString, this, region);
    }    
}
