/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.mobygames;

import java.net.URL;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcAssociate;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.util.StringUtils;

public class MobyGamesCompany {
    
    private int module;
    
    public MobyGamesCompany(int module) {
        this.module = module;
    }
    
    public DcObject get(URL url) throws Exception {
        DcObject person = DcModules.get(module).getDcObject();
        HttpConnection connection = new HttpConnection(url);
        String page = connection.getString();
        connection.close();
        
        String s = url.toString();
        String name = StringUtils.convertHttpString(StringUtils.getValueBetween("<title>", "</title>", page));
        name = name.toLowerCase().startsWith("mobygames - ") ? name = name.substring(12, name.length()) : name;  

        person.setValue(DcAssociate._A_NAME, name);
        person.setValue(DcAssociate._B_DESCRIPTION, getDescription(s));
        
        person.setValue(DcAssociate._C_WEBPAGE, s);
        person.setValue(DcObject._SYS_SERVICEURL, s);
        setPhoto(person, page);

        return person;
    }    
    
    private void setPhoto(DcObject dco, String page) throws Exception {
        int idx = page.indexOf("<img alt=\"company logo\"");
        if (idx == -1)
            return;
        
        String link = page.substring(idx);
        
        link = StringUtils.getValueBetween("src=\"", "\"", link).trim();
        if (link.length() > 0) {
            link = "http://www.mobygames.com" + link;
            HttpConnection connection = new HttpConnection(new URL(link));
            byte[] image = connection.getBytes();
            dco.setValue(DcAssociate._D_PHOTO, image);
        }
    }

    private String getDescription(String url) throws Exception {
        HttpConnection connection = new HttpConnection(new URL(url + "/overview"));
        String desc = connection.getString();
        
        int idx = desc.toLowerCase().indexOf("<div class=\"rightpanelmain\">");
        if (idx > -1) 
            desc = desc.substring(idx);
        else 
            return null;
        
        idx = desc.toLowerCase().indexOf("overview</h2>");
        if (idx > -1)
            desc = desc.substring(idx + 13);
        
        idx = desc.lastIndexOf("<br>");
        if (idx > -1)
            desc = desc.substring(0, idx);

        idx = desc.indexOf("<div");
        if (idx > -1)
            desc = desc.substring(0, idx);
        
        if (desc.length() > 50)
            return StringUtils.convertHttpString(desc);
        else
            return null;
    }
}
