/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.mobygames;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;

public class MobyGamesCompanySearch extends MobyGamesSearch {
    
    private int module;

    public MobyGamesCompanySearch(IOnlineSearchClient listener, int module, String searchString, DcServer server) {
        super(listener, server, searchString);
        this.module = module;
    }
    
    protected DcObject getDcObject(String id) throws Exception {
        URL url = new URL(new MobyGamesServer(module).getUrl() + "company/" + id);
        return getDcObject(url);
    }
    
    protected DcObject getDcObject(URL url) throws Exception {
        MobyGamesCompany mgc = new MobyGamesCompany(module);
        return mgc.get(url);
    }
    
    protected Collection getIDs() {
        Collection ids = new ArrayList();

        String webpage = "";
        try {
            listener.addMessage(DcResources.getText("msgConnectingToServer", getAddress()));
            URL url = new URL(getAddress() + "search/site?search=" + getQuery() + "&c=1&c2=1");
            HttpConnection connection = new HttpConnection(url);
            webpage = connection.getString();
            connection.close();
        } catch (Exception e) {
            listener.addError(DcResources.getText("msgCouldNotConnectTo", getServer().getName()));
        }
            
        int idx = webpage.indexOf("<a href=\"/company/"); 
        while (idx > -1) {
            webpage = webpage.substring(idx + 18);
            
            int i = webpage.indexOf("/");
            String id = webpage.substring(0, i);
            webpage = webpage.substring(i + 1);
            idx = webpage.indexOf("<a href=\"/company/");
            
            ids.add(id);
        }
            
        return ids;
    }
}
