/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.mobygames;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcAssociate;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.Software;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.util.StringUtils;

public class MobyGamesSoftwareSearch extends MobyGamesSearch {

    public MobyGamesSoftwareSearch(IOnlineSearchClient listener, DcServer server, String query) {
        super(listener, server,  query);
    }
    
    protected Collection getIDs() {
        
        Collection ids = new ArrayList();
        String webpage = "";
        try {
            listener.addMessage(DcResources.getText("msgConnectingToServer", getAddress()));
            URL url = new URL(getAddress() + "search/quick?q=" + getQuery());
            HttpConnection connection = new HttpConnection(url);
            webpage = connection.getString();
            connection.close();
        } catch (Exception e) {
            listener.addError(DcResources.getText("msgCouldNotConnectTo", getServer().getName()));
        }
            
        if (webpage.length() > 0) {    
            String temp = webpage;
            int idx = temp.indexOf("SUMMARY=\"Game Match\"");
            idx = idx == -1 ? temp.indexOf("results:") : idx;

            if (idx > -1) {
                temp = temp.substring(idx, temp.length());
                idx = temp.indexOf("</blockquote>");
                idx = idx == -1 ? temp.indexOf("\n") : idx; 
                
                if (idx > -1) {
                    temp = temp.substring(0, idx);
                    idx = temp.indexOf("<a href=\"");
                    
                    int counter = 0;
                    while (idx > -1 && !isCancelled() && counter < getMaximum()) {
                        temp = temp.substring(idx + 9, temp.length());
                        String part = temp.substring(0, temp.indexOf("\""));
                        
                        if (!ids.contains(part)) {
                            ids.add(part);
                            counter++;
                        }
                        
                        idx = temp.indexOf("<a href=\"");
                    }
                }
            }
        }
        
        return ids;
    }

    protected DcObject getDcObject(String id) throws Exception {
        String ID = id;
        ID = ID.startsWith("/") ? ID.substring(1, ID.length()) : ID;
        String address = ID.toLowerCase().startsWith("http") ? ID : getAddress() + ID;
        return getDcObject(new URL(address));
    }
   
    protected DcObject getDcObject(URL url) throws Exception {
    	DcObject software = new Software();
        
        HttpConnection connection = new HttpConnection(url);
        String page = connection.getString();
        connection.close();
        
        String s = url.toString();
        String ID = s.substring(s.lastIndexOf("/"), s.length());

        String title = StringUtils.getValueBetween("<title>", "</title>", page);
        title = title.indexOf("- MobyGames") > -1 ? title.substring(0, title.indexOf("- MobyGames")) : title;
        software.setValue(Software._A_TITLE, title.trim());
        
        // try to shorten the result string
        int idx = page.toUpperCase().indexOf("<DIV CLASS=\"RIGHTPANEL\">");
        page = idx > -1 ? page.substring(idx, page.length()) : page;
        
        setCompany(software, page, "PUBLISHED BY</DIV>", "PUBLISHED", Software._G_PUBLISHER);
        setCompany(software, page, "DEVELOPED BY</DIV>", "DEVELOPED", Software._F_DEVELOPER);

        software.setValue(Software._B_DESCRIPTION, getDescription(page, "DESCRIPTION</H2>", "RIGHTPANELMAIN", "<DIV"));
        software.setValue(Software._I_WEBPAGE, url.toString());
        software.setValue(DcObject._SYS_SERVICEURL, url.toString());

        DataManager.createReference(software, Software._H_PLATFORM, getInfo(page, "PLATFORM</DIV>", "PLATFORM"));
        DataManager.createReference(software, Software._K_TYPE, getInfo(page, "GENRE</DIV>", "COREGAMEGENRE"));
        
        try {
            String released = getInfo(page, "RELEASED</DIV>", "RELEASED");
            if (released.indexOf(".") > -1)
                released = released.substring(released.lastIndexOf(".") + 1, released.length());
            else if (released.indexOf(",") > -1)
                released = released.substring(released.lastIndexOf(",") + 1, released.length()).trim();
            
            software.setValue(Software._C_YEAR, new Long(released));
        } catch (Exception ignore) {}
        
        setCovers(software, url.toString());
        setScreenshots(software, ID, page);
        
        return software;        
    }
    
    private void setCompany(DcObject software, String page, String tag1, String tag2, int field) {
        String html = page;
        int module = DcModules.getReferencedModule(software.getField(field)).getIndex();
        int idx = html.toUpperCase().indexOf(tag1) > -1 ? html.toUpperCase().indexOf(tag1) :
                  html.toUpperCase().indexOf(tag2);

        String name = null;
        URL url = null;

        if (idx > -1) {
            html = html.substring(idx);
            String link = StringUtils.getValueBetween("a href=\"", "\"", html);
          
            try {
                if (link.trim().length() > 0) {
                    name = StringUtils.getValueBetween("a href=\"", "</a>", html);
                    name = StringUtils.convertHttpString(name.substring(name.indexOf(">") + 1));
                  
                    link = link.startsWith("/") ? link.substring(1) : link;
                    link = getServer().getUrl() + link;
                    url = new URL(link);
                }
            } catch (Exception e) {
                DcLog.add(e);
            }
        }
              
        if (name == null || url == null)
            return;
      
        try {
            DcObject company = DataManager.getObjectForDisplayValue(module, name);
            if (company == null) {
                
                if (DcModules.get(DcModules._SOFTWARE).getSettings().getBoolean(DcRepository.modulesettings.stOnlineSearchSubItems)) {
                    MobyGamesCompany companyFinder = new MobyGamesCompany(module);
                    company = companyFinder.get(url);
                } else {
                    company = DcModules.get(module).getDcObject();
                    company.setValue(DcAssociate._A_NAME, name);
                }
                
                // The name found using this finder might change from the orginal. Check again.
                DcObject existing = DataManager.getObjectForDisplayValue(module, (String) company.getValue(DcAssociate._A_NAME));
                if (existing == null) {
                    company.setIDs();
                } else {
                    company.clearValues();
                    company = existing;
                }
            }
            
            DataManager.createReference(software, field, company);
          
        } catch (Exception e) {
            DcLog.add(e);
        }
    }
    
    private void setCovers(DcObject software, String address) throws Exception {
        String page = "";
        try {
            HttpConnection connection = new HttpConnection(new URL(address + "/cover-art"));
            page = connection.getString();
            connection.close();
        } catch (Exception exp) {
            return; // not available
        }
        
        String frontCoverTag = "FRONT COVER\"";
        String backCoverTag = "BACK COVER\"";
        String mediaCoverTag = "MEDIA\"";
        
        setMediaImage(software, frontCoverTag, Software._M_PICTUREFRONT, page);
        setMediaImage(software, backCoverTag, Software._N_PICTUREBACK, page);
        setMediaImage(software, mediaCoverTag, Software._O_PICTURECD, page);
    }
    
    private void setScreenshots(DcObject software, String ID, String mainPage) throws Exception {
        String link = getImagesLink(ID, mainPage);
        String page = "";
        try {
            HttpConnection connection = new HttpConnection(new URL(getAddress() + link));
            page = connection.getString();
            connection.close();
        } catch (Exception e) {
            DcLog.add(e);
            return;
        }
        
        int[] fields = {Software._P_SCREENSHOTONE, Software._Q_SCREENSHOTTWO, Software._R_SCREENSHOTTHREE};
        
        try {
            String check = (ID + "/SCREENSHOTS/").toUpperCase();
            int idx = page.toUpperCase().indexOf(check);
            int counter = 0;
            while (idx > -1 && counter < fields.length && !isCancelled()) {
                int start = StringUtils.backtrack(page, idx, "<a href=\"");
                start = start > -1 ? start : idx;
                
                String part = page.substring(start);
                part = part.startsWith("/") ? part.substring(1) : part;
                
                String imagePageUrl = part.substring(0,  part.indexOf("\">"));
                
                HttpConnection connection = new HttpConnection(new URL(getAddress() + imagePageUrl));
                part = connection.getString();
                connection.close();
                
                int screenIdx = part.toLowerCase().indexOf("screenshot:");
                if (screenIdx > -1) {
                    try {
                        part = part.substring(screenIdx);
                        part = part.substring(part.toLowerCase().indexOf("src=\"") + 5);
                        String imageUrl = part.substring(0, part.indexOf("\""));
                        
                        imageUrl = !imageUrl.toLowerCase().startsWith("http") ? 
                                   getAddress() + (imageUrl.startsWith("/") ? imageUrl.substring(1) : imageUrl) :
                                   imageUrl;
                        
                        connection = new HttpConnection(new URL(imageUrl));
                        byte[] image = connection.getBytes();
                        software.setValue(fields[counter], image);
                    } catch (Exception exp) {
                        DcLog.add(exp);
                    }
                }
                counter++;
                page = page.substring(idx + check.length());
                idx = page.toUpperCase().indexOf(check);
            }
        } catch (Exception exp) {
            DcLog.add(exp);
        }
    }
    
    private String getImagesLink(String ID, String page) {
        int idx = page.indexOf(ID + "/screenshots");
        idx = idx > -1 ? StringUtils.backtrack(page, idx, "<a href=\"") : idx;
        if (idx > -1) {
            String link = page.substring(idx);
            link = link.substring(0, link.indexOf("\">"));
            return link.startsWith("/") ? link.substring(1) : link;
        } else {
            String link = ID + "/screenshots/";
            try {
                HttpConnection conn = new HttpConnection(new URL(getAddress() + link));
                if (conn.exists()) return link;
                conn.close();
    
                link = "game" + ID + "/screenshots/";
                conn = new HttpConnection(new URL(getAddress() + link));
                if (conn.exists()) return link;
                conn.close();
            } catch (Exception e) {
                DcLog.add(e);
            }
            return link;
        }
    }
    
    private void setMediaImage(DcObject software, String tag, int field, String page) {
        String html = page;
        try {
            int idx = html.toUpperCase().indexOf(tag);
            if (idx > -1) {
                String part = html.substring(0,  idx);
                part = part.substring(part.lastIndexOf("\n"));
                
                idx = part.indexOf("<a href=\"") > -1 ? part.indexOf("<a href=\"") + 9 : part.indexOf("src=\"") + 5;
                part = part.substring(idx);
                part = part.substring(0, part.indexOf("\""));
                part = part.startsWith("/") ? part.substring(1, part.length()) : part;
                
                String imagePageUrl = part.startsWith("http") ? part : getAddress() + part;
                
                HttpConnection connection = new HttpConnection(new URL(imagePageUrl));
                html = connection.getString();
                connection.close();
                
                idx = html.toLowerCase().indexOf("http://www.mobygames.com/images/covers/");
                if (idx > -1) {
                    part = html.substring(idx);
                    part = part.substring(0, part.indexOf("\""));
                    
                    String imageUrl = part.startsWith("http") ? part : getAddress() + part;
                    connection = new HttpConnection(new URL(imageUrl));
                    byte[] image = connection.getBytes();
                    connection.close();
                    software.setValue(field, image);
                }
            }
        } catch (Exception exp) {
            DcLog.add(exp);
        }
    }
    
    private String getDescription(String page, String s1, String s2, String end) {
        String s = page.toUpperCase();
        int idx = s.indexOf(s1) > -1 ? s.indexOf(s1) : s.indexOf(s2);
        String part = null;
        if (idx > -1) {
            part = page.substring(idx, page.length());
            idx = part.toUpperCase().indexOf(end);
            if (idx > -1) {
                idx = part.toUpperCase().indexOf("</H2>");
                part = idx > -1 ? part.substring(idx + 5, part.length()) : part;
                part = part.substring(0, part.toUpperCase().indexOf(end));
                
                int index = part.indexOf("<a");
                while (index > -1) {
                    try {
                        String first = part.substring(0, index);
                        
                        String temp = part.substring(index, part.length());
                        String middle = temp.substring(temp.indexOf(">") + 1, temp.indexOf("</a>"));
                        String last = temp.substring(temp.indexOf("</a>") + 4);
                        part = first + middle + last;
                        
                        index = part.indexOf("<a");
                    } catch (Exception exp) {
                        DcLog.add(exp);
                    }
                }
                part = StringUtils.convertHttpString(part);
            }
        }
        return part;
    }
    
    private String getInfo(String page, String s1, String s2) {
        String orgPage = page;
        String s = page.toUpperCase();
        int idx = s.indexOf(s1) > -1 ? s.indexOf(s1) : s.indexOf(s2);
        String part = null;

        try {
            if (idx > -1) {
                part = orgPage.substring(idx, orgPage.length());
                part = part.substring(part.toUpperCase().indexOf("<A HREF=\""), part.length());
                part = part.substring(part.indexOf(">") + 1, part.toUpperCase().indexOf("</A>"));
                part = StringUtils.convertHttpString(part);
            }
        } catch (StringIndexOutOfBoundsException exp) {}
        
        return part;
    }
}
