/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.musicbrainz;

import java.io.ByteArrayInputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcAssociate;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.util.XMLParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class MusicBrainzArtist {

    private final String address;
    
    public MusicBrainzArtist(String address) {
        this.address = address;
    }
    
    public DcObject get(String id) throws Exception {
        return get(new URL(getLink(id)));
    }    
    
    public DcObject get(URL url) throws Exception {
        HttpConnection connection = new HttpConnection(url);
        String page = connection.getString();
        connection.close();
        return parse(page);
    }
    
    public String getLink(String id) {
        return address + "artist/" + id + "?type=xml&inc=sa-Album";
    }
    
    public DcObject parse(String xml) throws Exception {
        DcAssociate artist = new DcAssociate(DcModules._MUSICARTIST);
        
        Document document = getDocument(xml);
        Element e = document.getDocumentElement();

        Element elArtist = (Element) e.getElementsByTagName("artist").item(0);
        String id = elArtist.getAttribute("id");
        String link = getLink(id);
        artist.setValue(DcObject._SYS_SERVICEURL, link);
        
        String name = XMLParser.getString(e, "name");
        
        StringBuffer sb = new StringBuffer("");
        if (e.getElementsByTagName("life-span") != null) {
            Element ndLife = (Element) e.getElementsByTagName("life-span").item(0);
            if (ndLife != null) {
                NamedNodeMap nnm = ndLife.getAttributes();
                
                Node ndBegin = nnm.getNamedItem("begin");
                Node ndEnd = nnm.getNamedItem("end");
                if (ndBegin != null && ndEnd != null) {
                    String yrBegin = ndBegin.getNodeValue();
                    String yrEnd = ndEnd.getNodeValue();
                    sb.append("Active period: " + yrBegin + " - " + yrEnd + "\r\n");
                }
            }
        }
        
        Collection c = new ArrayList();
        
        NodeList releases = document.getElementsByTagName("release");
        for (int i = 0; i < releases.getLength(); i++) {
            Element release = (Element) releases.item(i);
            String title = XMLParser.getString(release, "title"); 
            if (!c.contains(title))
                c.add(title);
        }
        
        if (c.size() > 0) {
            sb.append(sb.length() > 0 ? "\r\n" : "");
            sb.append("releases:\r\n");
        }
        
        for (Iterator iter = c.iterator(); iter.hasNext(); ) {
            sb.append(iter.next());
            sb.append("\r\n");
        }
        
        artist.setValue(DcAssociate._A_NAME, name);
        artist.setValue(DcAssociate._B_DESCRIPTION, sb.toString());
        
        return artist;                
    }
    
    protected Document getDocument(String xml) {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            ByteArrayInputStream bis = new ByteArrayInputStream(xml.getBytes("UTF-8"));
            return db.parse(bis);
        } catch(Exception e) {
            DcLog.add(e);
        }
        return null;
    }      
}
