/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.musicbrainz;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;

import net.sf.dc.core.objects.DcObject;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;

public class MusicBrainzArtistSearch extends MusicBrainzSearch {

    public MusicBrainzArtistSearch(IOnlineSearchClient listener, 
                                   String searchString, 
                                   DcServer server, 
                                   DcRegion region, 
                                   DcSearchMode mode) {
        super(listener, searchString, server, region, mode);
    }
    
	protected DcObject getDcObject(String id) throws Exception {
		URL url = new URL(new MusicBrainzArtist(getServer().getUrl()).getLink(id));
		return getDcObject(url);
	}

	protected DcObject getDcObject(URL url) throws Exception {
		HttpConnection connection = new HttpConnection(url);
        String xml = connection.getString();
        connection.close();
		return new MusicBrainzArtist(getServer().getUrl()).parse(xml);
	}

	protected Collection getIDs() {
		Collection ids = new ArrayList();
		
		try {
			String httpUrl = getAddress() + "/artist/?type=xml&limit=" + getMaximum() + "&name=" + getQuery();
			URL url = new URL(httpUrl);

			HttpConnection connection = new HttpConnection(url);
			String page = connection.getString();
			connection.close();

			ids.addAll(getIDs(page, "artist"));
		} catch (Exception e) {
			listener.addError(e);
		}
			
		return ids;
	}
}
