/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.musicbrainz;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcAssociate;
import net.sf.dc.core.objects.DcMediaObject;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.AudioCD;
import net.sf.dc.core.objects.helpers.AudioTrack;
import net.sf.dc.core.objects.helpers.MusicAlbum;
import net.sf.dc.core.objects.helpers.MusicTrack;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.amazon.AmazonCoverSource;
import net.sf.dc.util.StringUtils;

public class MusicBrainzAudioCdSearch extends MusicBrainzSearch {

    public MusicBrainzAudioCdSearch(IOnlineSearchClient listener, 
                             String searchString, 
                             DcServer server, 
                             DcRegion region, 
                             DcSearchMode mode) {
        
        super(listener, searchString, server, region, mode);
    }

    private String getLink(String id) {
        return getAddress() + "/release/" + id + "?type=xml&inc=tracks+artist";
    }

    
	protected DcObject getDcObject(String id) throws Exception {
        URL url = new URL(getLink(id));
        return getDcObject(url);
	}

	protected DcObject getDcObject(URL url) throws Exception {
		HttpConnection connection = new HttpConnection(url);
		String xml =  connection.getString();
        connection.close();
        
        DcModule module = DcModules.get(getServer().getModule());
        DcObject dco = module.getDcObject();
        setTitle(dco, xml);
        setYear(dco, xml);
        setWebpage(dco, url);
        Collection artists = setArtists(dco, xml);
        addTracks(dco, artists, xml);
        addCovers(dco);
        
        dco.setValue(DcObject._SYS_SERVICEURL, url.toString());
        
        return dco;
	}

	protected Collection getIDs() {
		Collection ids = new ArrayList();
		
		try {
			String httpUrl = getAddress() + "/release/?type=xml&limit=" + getMaximum();
			httpUrl += getMode().getSearchCommand(getQuery());
			URL url = new URL(httpUrl);
			HttpConnection connection = new HttpConnection(url);
			String page = connection.getString();
			connection.close();

			ids.addAll(getIDs(page, "release"));
		} catch (Exception e) {
			listener.addError(e);
		}
		return ids;		
	}    
    
	private void setTitle(DcObject dco, String xml) {
		String title = StringUtils.convertHttpString(StringUtils.getValueBetween("<title>", "</title>", xml));
		dco.setValue(DcMediaObject._A_TITLE, title);
	}
	
	private void setYear(DcObject dco, String xml) {
        String key = "life-span begin=" + '"';
        int idx = xml.indexOf(key);
        if (idx > -1) {
            String s = xml.substring(idx + key.length(), xml.length());
            
        	String year = s.indexOf("-") > 0 ? s.substring(0, s.indexOf("-")) : 
        				  s.indexOf('"') > 0 ? s.substring(0, s.indexOf('"')) : null;

            if (year != null)
            	dco.setValue(DcMediaObject._C_YEAR, StringUtils.getContainedNumber(year));
        } 
	}	
	
	private void setWebpage(DcObject dco, URL link) {
		dco.setValue(dco.getModule().getIndex() == DcModules._AUDIOCD ? AudioCD._M_WEBPAGE : MusicAlbum._N_WEBPAGE, 
				     link.toString());
	}
	
	private void addCovers(DcObject dco) {
		if (dco instanceof AudioCD) {
			AmazonCoverSource coverSearcher = new AmazonCoverSource(listener, (AudioCD) dco);
			coverSearcher.search();
		} else {
			AudioCD audioCD = new AudioCD();
			audioCD.setValue(AudioCD._A_TITLE, dco.getValue(MusicAlbum._A_TITLE));
			audioCD.setValue(AudioCD._F_ARTIST, dco.getValue(MusicAlbum._F_ARTISTS));
			AmazonCoverSource coverSearcher = new AmazonCoverSource(listener, audioCD);
			coverSearcher.search();
        
			dco.setValue(MusicAlbum._J_PICTUREFRONT, audioCD.getValue(AudioCD._H_PICTUREFRONT));
			dco.setValue(MusicAlbum._K_PICTUREBACK, audioCD.getValue(AudioCD._I_PICTUREBACK));
			dco.setValue(MusicAlbum._L_PICTURECD, audioCD.getValue(AudioCD._J_PICTURECD));
		}
	}
	
    private void addTracks(DcObject dco, Collection artists, String xml) {
    	int childModIdx = dco.getModule().getChild().getIndex();
    	
        Collection c = StringUtils.getValuesBetween("<track id", "</track>", xml);
        int number = 1;
        
        int titleIdx = childModIdx == DcModules._AUDIOTRACK ? AudioTrack._A_TITLE : MusicTrack._A_TITLE; 
        int nrIdx = childModIdx == DcModules._AUDIOTRACK ? AudioTrack._F_TRACKNUMBER : MusicTrack._F_TRACKNUMBER;
        int lengthIdx = childModIdx == DcModules._AUDIOTRACK ? AudioTrack._H_PLAYLENGTH : MusicTrack._J_PLAYLENGTH;
        int artistIdx = childModIdx == DcModules._AUDIOTRACK ? AudioTrack._K_ARTIST : MusicTrack._G_ARTIST;
        
        for (Iterator iter = c.iterator(); iter.hasNext() && !isCancelled(); number++) {
            String s = (String) iter.next();
            DcObject track = DcModules.get(childModIdx).getDcObject();
            
            String title = StringUtils.convertHttpString(StringUtils.getValueBetween("<title>", "</title>", s));
            track.setValue(titleIdx, title);
            track.setValue(nrIdx, new Long(number));

            try {
                String duration = StringUtils.convertHttpString(StringUtils.getValueBetween("<duration>", "</duration>", s));
                int seconds = Integer.parseInt(duration.trim()) / 1000;
                track.setValue(lengthIdx, new Long(seconds));
            } catch (Exception ignore) {}
            
            for (Iterator iterArtist = artists.iterator(); iterArtist.hasNext(); ) 
                DataManager.addMapping(track, (DcObject) iterArtist.next(), artistIdx);
            
            dco.addChild(track);
        }
    }
    
    private Collection setArtists(DcObject dco, String part) {
        Collection artists = StringUtils.getValuesBetween("<artist", "</artist>", part);
        Collection result = new ArrayList();
        
        int fieldIdx = dco.getModule().getIndex() == DcModules._AUDIOCD ? AudioCD._F_ARTIST : MusicAlbum._F_ARTISTS;
        for (Iterator iter = artists.iterator(); iter.hasNext(); ) {
            try {
                String s = (String) iter.next();
                String id = StringUtils.getValueBetween("id=\"", "\"", s);
                String name = StringUtils.getValueBetween("<name>", "</name>", s);
                
                int module = DcModules.getReferencedModule(dco.getField(fieldIdx)).getIndex();
                DcObject person = DataManager.getObjectForDisplayValue(module, name);
                if (person == null) {
                    if (dco.getModule().getSettings().getBoolean(DcRepository.modulesettings.stOnlineSearchSubItems)) {
                        MusicBrainzArtist mbPerson = new MusicBrainzArtist(getServer().getUrl());
                        person = mbPerson.get(id);
                    } else {
                        person = DcModules.get(module).getDcObject();
                        person.setValue(DcAssociate._A_NAME, name);
                    }
                    person.setIDs();
                }
                
                result.add(person);
                DataManager.addMapping(dco, person, dco.getModule().getIndex() == DcModules._AUDIOCD ? AudioCD._F_ARTIST : MusicAlbum._F_ARTISTS);
            } catch (Exception e) {
                DcLog.add(e);
            }
        }
        
        return result;
    }
}
