/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.wikipedia;

import java.util.StringTokenizer;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.objects.helpers.Movie;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;

public class WikipediaDutchMovieSearch extends WikipediaMovieSearch {

    public WikipediaDutchMovieSearch(IOnlineSearchClient listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }

    public String[] getDescriptionTags() {
        return new String[] {"[bewerk]", "categorien"};
    }

    public RemovalTag[] getRemovalTags() {
        return  new RemovalTag[] {new RemovalTag(RemovalTag._REMOVETAG, "[bewerk]"),
                                  new RemovalTag(RemovalTag._REMOVELINE, "verhaal"),
                                  new RemovalTag(RemovalTag._REMOVELINE, "leeswaarschuwing")};
    }

    public void setAdditionInfo(Movie movie, String[] headers, String text, String webpage) {
        setGenres(movie, headers);
    }     
    
    private void setGenres(Movie movie, String[] headers) {
        String genres = getValueForTag(headers, "genre");
        
        if (genres != null) {
            StringTokenizer st = new StringTokenizer(genres, genres.indexOf(">") > -1 ? ">" : ",");
            while (st.hasMoreElements())
                DataManager.createReference(movie, Movie._H_GENRES, ((String) st.nextElement()).trim());
        }
    }
    
    public String[] getTags() {
        return new String[] {"regie", "productie", "uitgebracht", "hoofdrollen", "lengte", "land", "taal"};
    }
}
