/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.wikipedia;

import java.net.URL;

import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.objects.helpers.Movie;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;

public class WikipediaEnglishMovieSearch extends WikipediaMovieSearch {

    public WikipediaEnglishMovieSearch(IOnlineSearchClient listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }

    public String[] getDescriptionTags() {
        return new String[] {"[edit]", " >\n>"};
    }

    public RemovalTag[] getRemovalTags() {
        return new RemovalTag[] {new RemovalTag(RemovalTag._REMOVETAG, "[edit]"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "spoiler"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "plot consummation"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "synopsis"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "plot")};
    }

    public String[] getTags() {
        return new String[] {"directed by", "produced by", "release date", "starring", "running time", "country", "language"};
    }

    protected void setImage(Movie movie, String webpage) {
        int idx = webpage.indexOf("class=\"image\"");
        
        if (idx == -1) return;
        
        // backtrack
        String s = webpage.substring(idx);
        while (s.indexOf("<a href=\"/") > 4) {
            idx -= 4;
            s = webpage.substring(idx);
        }
        
        try {
            s = s.substring(s.indexOf("<a href=\"/") + 10);
            s = getAddress() + s.substring(0, s.indexOf("\""));
        
            HttpConnection connection = new HttpConnection(new URL(s));
            String page = connection.getString();
            connection.close();
            
            page = page.substring(page.indexOf("<a href=\"http://upload.wikimedia.org/"));
            String link = page.substring(page.indexOf("<a href=\"") + 9);
            link = link.substring(0, link.indexOf("\""));
            
            if (link.indexOf("logo") > -1)
                return;
            
            connection = new HttpConnection(new URL(link));
            byte[] image = connection.getBytes();
            connection.close();
            
            movie.setValue(Movie._X_PICTUREFRONT, image);
        } catch (Exception exp) {
            DcLog.add(exp);
        }        
    }
}
