/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.wikipedia;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.objects.helpers.Movie;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;

public class WikipediaGermanMovieSearch extends WikipediaMovieSearch {

    public WikipediaGermanMovieSearch(IOnlineSearchClient listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }

    public String[] getDescriptionTags() {
        return new String[] {"[bearbeiten]", ">\n>"};
    }

    public RemovalTag[] getRemovalTags() {
        return new RemovalTag[] {new RemovalTag(RemovalTag._REMOVETAG, "[Bearbeiten]")};
    }

    public String[] getTags() {
        return new String[] {"regie:", "produktion:", "erscheinungsjahr:", "besetzung", "lnge", "produktionsland:", "originalsprache:"};
    }
    
    
    public void setAdditionInfo(Movie movie, String[] headers, String text, String webpage) {
        setActors(movie, webpage);
        setTitles(movie, headers);
    }    
    
    private void setTitles(Movie movie, String[] headers) {
        movie.setValue(Movie._F_TITLE_LOCAL, getValueForTag(headers, "deutscher titel"));
        
        String title = getValueForTag(headers, "originaltitel");
        if (title != null)
            movie.setValue(Movie._A_TITLE, title);
    }
    
    private void setActors(Movie movie, String webpage) {
        int idx = webpage.toLowerCase().indexOf("besetzung");
        
        try {
            if (idx > -1) {
                String tmp = webpage.substring(idx);
                tmp = tmp.substring(0, webpage.indexOf("</div>"));
                while (tmp.indexOf("<li>") > -1) {
                    String actor = tmp.substring(tmp.indexOf("<li>"), tmp.indexOf("</li>"));
                    actor = actor.substring(actor.indexOf("\">") + 2, actor.indexOf("</a>"));
                    DataManager.createReference(movie, Movie._I_ACTORS, actor);
                    tmp = tmp.substring(tmp.indexOf("</li>") + 6);
                }
            }
        } catch (Exception ignore) {}
    }
    
    protected String getActors(String[] header, String tag) {
        return null;
    }    
}
