/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.wikipedia;

import java.util.StringTokenizer;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.objects.helpers.Movie;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;

public class WikipediaItalionMovieSearch extends WikipediaMovieSearch {

    public WikipediaItalionMovieSearch(IOnlineSearchClient listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }

    public int getTagConstraint() {
        return 3;
    }    
    
    public String[] getDescriptionTags() {
        return new String[] {"[modifica]", ">\n>"};
    }

    public RemovalTag[] getRemovalTags() {
        return new RemovalTag[] {new RemovalTag(RemovalTag._REMOVETAG, "[modifica]"),
                                 new RemovalTag(RemovalTag._REMOVELINE, "attenzione: di seguito viene rivelata")};
    }

    public String[] getTags() {
        return new String[] {"regia:", "soggetto:", "anno:", "attori:", "durata:", "paese:", "dasdadada"};
    }
    
    public void setAdditionInfo(Movie movie, String[] headers, String text, String webpage) {
        setActors(movie, webpage);
        setTitle(movie, headers);
        setGenres(movie, headers);
    }
    
    private void setActors(Movie movie, String webpage) {
        int idx = webpage.toLowerCase().indexOf("attori</a>");
        
        try {
            if (idx > -1) {
                String tmp = webpage.substring(idx);
                tmp = tmp.substring(0, webpage.indexOf("</div>"));
                
                while (tmp.indexOf("<li>") > -1) {
                    String name = tmp.substring(tmp.indexOf("<li>"), tmp.indexOf("</li>"));
                    name = name.substring(name.indexOf("\">") + 2, name.indexOf("</a>"));
                    DataManager.createReference(movie, Movie._I_ACTORS, name);
                    tmp = tmp.substring(tmp.indexOf("</li>") + 6);
                }
            }
        } catch (Exception ignore) {}
    }
    
    private void setTitle(Movie movie, String[] headers) {
        String localTitle = (String) movie.getValue(Movie._A_TITLE);
        String orgTitle =getValueForTag(headers, "titolo originale:"); 
        
        if (orgTitle != null) {
            movie.setValue(Movie._A_TITLE, orgTitle);
            movie.setValue(Movie._F_TITLE_LOCAL, localTitle);
        }
    }
    
    private void setGenres(Movie movie, String[] headers) {
        String genres = getValueForTag(headers, "genere:");
        if (genres != null) {
            StringTokenizer st = new StringTokenizer(genres, ",");
            while (st.hasMoreElements())
                DataManager.createReference(movie, Movie._H_GENRES, ((String) st.nextElement()).trim());
        }
    }
}