/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.wikipedia;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.StringTokenizer;

import javax.swing.text.BadLocationException;

import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.HttpConnection;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchTaskImpl;

/**
 * Allows multiple types of items to be retrieved
 */
public abstract class WikipediaSearch extends OnlineSearchTaskImpl {

    public WikipediaSearch(IOnlineSearchClient listener, DcServer server, DcRegion region, String query) {
        super(listener, query, server, region, null);
    }
    
    public String getWhiteSpaceSubst() {
        return "_";
    }   
    
    protected Collection getIDs() {
        Collection ids = new ArrayList();
        
        String webpage = "";
        try {
            listener.addMessage(DcResources.getText("msgConnectingToServer", getRegion().getUrl()));
    
            URL url = new URL(getAddress() + "wiki/Special:Search?fulltext=Search&search=" + getQuery());
            HttpConnection connection = new HttpConnection(url);
            webpage = connection.getString();

            connection.close();
        } catch (Exception e) {
            listener.addError(DcResources.getText("msgCouldNotConnectTo", getServer().getName()));
        }
        
        String ref = "<li style=\"padding-bottom: 1em;\"><a href=\"/";
        int refIdx = webpage.toLowerCase().indexOf(ref.toLowerCase());
        while (refIdx > -1) {
            webpage = webpage.substring(refIdx + ref.length(), webpage.length());
            String link = webpage.substring(0, webpage.indexOf('"'));
            ids.add(getAddress() + link);
            refIdx = webpage.toLowerCase().indexOf(ref.toLowerCase());
        }
        
        return ids;
    }
  
    protected String getCleanText(String html) throws IOException, BadLocationException {
        String webpage = html;
        
        if (webpage.indexOf("<body") > -1)
            webpage = "<html>" + html.substring(html.indexOf("<body"), html.indexOf("</body>")) + "</html>";
        
        return super.getCleanText(webpage);
    }
    
    protected String cleanup(RemovalTag[] tags, String s) {
        StringTokenizer lines = new StringTokenizer(s, s.indexOf('\n') > -1 ? "\n" : "\r");
        
        String result = "";
        while (lines.hasMoreElements()) {
            String line = ((String) lines.nextElement()).trim();

            for (int i = 0; i < tags.length; i++)
                line = tags[i].apply(line);
            
            if (line.length() > 0 && !line.trim().equals("\n")) {
                result += result.length() > 0 ? "\n" : "";
                result += line;
            }
        }
        return result;
    }    
    

}
