/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.wikipedia;

import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcSearchMode;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;
import net.sf.dc.onlinesearch.OnlineSearchTask;

public class WikipediaServer extends DcServer {

    public WikipediaServer(int module) {
        super("Wikipedia", "http://www.google.nl/search?q=site%3Awikipedia.org+", true, module);
        
        addRegion(new DcRegion("en", "Default (english)", "http://en.wikipedia.org/"));
        addRegion(new DcRegion("nl", "Dutch", "http://nl.wikipedia.org/"));
        addRegion(new DcRegion("de", "German", "http://de.wikipedia.org/"));
        addRegion(new DcRegion("it", "Italien", "http://it.wikipedia.org/"));
        addRegion(new DcRegion("es", "Spanish", "http://es.wikipedia.org/"));
    }
    
    public OnlineSearchTask getTask(IOnlineSearchClient listener, 
            String searchString, 
            DcSearchMode mode, 
            DcRegion region) {

        if (region.getCode().equals("en"))
            return new WikipediaEnglishMovieSearch(listener, this, region, searchString);
        else if (region.getCode().equals("nl"))
            return new WikipediaDutchMovieSearch(listener, this, region, searchString);
        else if (region.getCode().equals("de"))
            return new WikipediaGermanMovieSearch(listener, this, region, searchString);
        else if (region.getCode().equals("it"))
            return new WikipediaItalionMovieSearch(listener, this, region, searchString);        
        else if (region.getCode().equals("es"))
            return new WikipediaSpanishMovieSearch(listener, this, region, searchString);        
        
        return null;
    }     
}
