/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.onlinesearch.wikipedia;

import java.util.StringTokenizer;

import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.helpers.Movie;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.IOnlineSearchClient;

public class WikipediaSpanishMovieSearch extends WikipediaMovieSearch {

    public WikipediaSpanishMovieSearch(IOnlineSearchClient listener, DcServer server, DcRegion region, String query) {
        super(listener, server, region, query);
    }
    
    public int getTagConstraint() {
        return 3;
    } 

    public String[] getDescriptionTags() {
        return new String[] {"[editar]", "secuelas"};
    }

    public RemovalTag[] getRemovalTags() {
        return new RemovalTag[] {new RemovalTag(RemovalTag._REMOVETAG, "[editar]"),
                                 new RemovalTag(RemovalTag._REMOVETAG, "advertencia: esta seccin contiene detalles")};
    }

    public String[] getTags() {
        return new String[] {"direccin", "produccin", "ao", "reparto", "duracin", "pas", "adadadadada"};
    }
    
    public void setAdditionInfo(Movie movie, String[] headers, String text, String webpage) {
        setTitle(movie, headers);
        setGenres(movie, headers);
        setDescription(movie, webpage);
    }
    
    private void setGenres(Movie movie, String[] headers) {
        String genres = getValueForTag(headers, "gnero");
        if (genres != null) {
            StringTokenizer st = new StringTokenizer(genres, genres.indexOf(">") > -1 ? ">" : ",");
            while (st.hasMoreElements())
                DataManager.createReference(movie, DcModules._GENRE, ((String) st.nextElement()).trim());
        }
    }
    
    private void setTitle(Movie movie, String[] headers) {
        movie.setValue(Movie._F_TITLE_LOCAL, getValueForTag(headers, "ttulo"));
    }
    
    private void setDescription(Movie movie, String webpage) {
        String description = (String) movie.getValue(Movie._B_DESCRIPTION);
        description = description == null ? "" : description.trim();
        
        if (description.length() == 0 && webpage.indexOf("</table>") > -1) {
            int idx = webpage.indexOf("</table>");
            description = webpage.substring(idx + 8);
            
            try {
                String s = getCleanText(description);
                idx = s.toLowerCase().indexOf("obtenido de \"http://es.wikipedia.org/");
                s = s.substring(0, idx > -1 ? idx : s.length());
                s = cleanup(getRemovalTags(), s);
                movie.setValue(Movie._B_DESCRIPTION, s);
            } catch (Exception exp) {
                DcLog.add(exp);
            }
        }
    }
}
