/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.reporting.templates;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import net.sf.dc.core.DataCrow;
import net.sf.dc.core.modules.DcMediaModule;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.modules.TemplateModule;
import net.sf.dc.util.Directory;

public class ReportTemplates {

    private static Map folders = new HashMap();
    
    public ReportTemplates(boolean reload) {
    	
    	if (reload || folders.size() == 0) {
    		folders.clear();
	        for (Iterator iter = DcModules.getModules(); iter.hasNext(); ) {
	            DcModule module = (DcModule) iter.next();
	            
	            if (   (module instanceof DcMediaModule || module.getIndex() == DcModules._CONTACTPERSON) && 
	                    module.isTopModule() && !(module instanceof TemplateModule)) {
	                
	                String path = DataCrow.reportDir + 
	                              module.getName().toLowerCase().replaceAll("[/\\*%., ]", "");
	                File file = new File(path);
	                if (!file.exists())
	                    file.mkdirs();
	                
	                folders.put(new Integer(module.getIndex()), path);
	            }
	        }
    	}
    }
    
    public Collection getFolders() {
        return folders.values();
    }
    
    public Collection getReportFiles(int transformer) {
        return getReportFiles(DcModules.getCurrent().getIndex(), transformer);
    }
    
    public Collection getReportFiles(int module, int transformer) {
        String folder = (String) folders.get(new Integer(module));
        
        Collection reports = new ArrayList();
        if (folder != null) {
            String[] extensions = {"xsl", "xslt"};
            Collection files = Directory.read(folder, false, true, false, extensions, false);
            for (Iterator iter = files.iterator(); iter.hasNext(); ) {
                ReportTemplate rf = new ReportTemplate((String) iter.next());
                if (rf.supports(transformer))
                    reports.add(rf);
            }
        }
        
        return reports;
    }
}
