/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.console.views.MasterView;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcLookAndFeel;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.core.settings.Setting;
import net.sf.dc.core.settings.SettingsGroup;
import net.sf.dc.settings.definitions.ProgramDefinitions;

public class DcApplicationSettings extends net.sf.dc.settings.Settings {
    
    public static final String _General = DcResources.getText("lblGroupGeneral");
    public static final String _Module = DcResources.getText("lblModuleSettings");
    public static final String _FileHandlers = DcResources.getText("lblProgramDefinitions");
    public static final String _TableColors = DcResources.getText("lblTableColorSettings");
    public static final String _OddRowColor = DcResources.getText("lblOddRowColor");
    public static final String _EvenRowColor = DcResources.getText("lblEvenRowColor");
    public static final String _HeaderColor = DcResources.getText("lblTableHeaderColor");
    public static final String _RowSelectionColor = DcResources.getText("lblRowSelectionColor");
    public static final String _HTTP = DcResources.getText("lblHTTPSettings");
    public static final String _Font = DcResources.getText("lblFont");

    public DcApplicationSettings() {
        super();
        createSettings();
        getSettings().setSettingsFile(new File(DataCrow.baseDir + "data/data_crow.properties"));
        load();
    }

    protected void createGroups() {
        SettingsGroup generalGroup = new SettingsGroup(_General, "dc.settings.general");
        SettingsGroup tableColorGroup = new SettingsGroup(_TableColors, "dc.settings.colors");
        SettingsGroup http = new SettingsGroup(_HTTP, "dc.settings.http");
        SettingsGroup fileHandlers = new SettingsGroup(_FileHandlers, "dc.settings.fileassociations");
        SettingsGroup moduleGroup = new SettingsGroup(_Module, "dc.settings.module");
        SettingsGroup fontGroup = new SettingsGroup(_Font, "dc.settings.font");

        tableColorGroup.addChild(new SettingsGroup(_OddRowColor, "dc.settings.colors"));
        tableColorGroup.addChild(new SettingsGroup(_EvenRowColor, "dc.settings.colors"));
        tableColorGroup.addChild(new SettingsGroup(_HeaderColor, "dc.settings.colors"));
        tableColorGroup.addChild(new SettingsGroup(_RowSelectionColor, "dc.settings.colors"));
        
        getSettings().addGroup(_Module, moduleGroup);
        getSettings().addGroup(_General, generalGroup);
        getSettings().addGroup(_Font, fontGroup);
        getSettings().addGroup(_HTTP, http);
        getSettings().addGroup(_TableColors, tableColorGroup);
        getSettings().addGroup(_FileHandlers, fileHandlers);
    }

    protected void createSettings() {
        getSettings().addSetting(_Module,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stModuleSettings,
                            "",
                            ComponentFactory._MODULESELECTOR,
                            "",
                            "",
                            false,
                            true));          
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stVersionMajor,
                            new Integer(DcRepository.system.versionMajor),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stVersionMinor,
                            new Integer(DcRepository.system.versionMinor),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stBackupLocation,
                            null,
                            -1,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stVersionBuild,
                            new Integer(DcRepository.system.versionBuild),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stShowTipsOnStartup,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stShowGroupingPanel,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));   
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stShowQuickFilterBar,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));          
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stHsqlCacheScale,
                            new Integer(18),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stHsqlCacheSizeScale,
                            new Integer(20),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stGarbageCollectionIntervalMs,
                            new Integer(480000),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_RowSelectionColor,
                new Setting(DcRepository.datatypes._COLOR,
                            DcRepository.settings.stRowSelectionColor,
                            new Color(255, 255, 153),
                            ComponentFactory._COLORSELECTOR,
                            "",
                            DcResources.getText("lblRowSelectionColor"),
                            false,
                            true));

        getSettings().addSetting(_OddRowColor,
                new Setting(DcRepository.datatypes._COLOR,
                            DcRepository.settings.stOddRowColor,
                            new Color(236, 251, 238),
                            ComponentFactory._COLORSELECTOR,
                            "",
                            DcResources.getText("lblOddRowColor"),
                            false,
                            true));
        getSettings().addSetting(_EvenRowColor,
                new Setting(DcRepository.datatypes._COLOR,
                            DcRepository.settings.stEvenRowColor,
                            new Color(230, 250, 222),
                            ComponentFactory._COLORSELECTOR,
                            "",
                            DcResources.getText("lblEvenRowColor"),
                            false,
                            true));
        getSettings().addSetting(_HeaderColor,
                new Setting(DcRepository.datatypes._COLOR,
                            DcRepository.settings.stTableHeaderColor,
                            new Color(244, 242, 231),
                            ComponentFactory._COLORSELECTOR,
                            "",
                            DcResources.getText("lblTableHeaderColor"),
                            false,
                            true));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stCheckUniqueness,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpUniqueness"),
                            DcResources.getText("lblUniqueness"),
                            false,
                            true));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stCheckRequiredFields,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpRequiredFields"),
                            DcResources.getText("lblCheckRequiredFields"),
                            false,
                            true));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stConnectionString,
                            "dc",
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stDatabaseDriver,
                            "org.hsqldb.jdbcDriver",
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stShowTableTooltip,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpShowTableTooltips"),
                            DcResources.getText("lblShowTableTooltips"),
                            false,
                            true));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stModule,
                            new Integer(DcModules._SOFTWARE),
                            -1,
                            "",
                            "",
                            true,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stShowHiddenFieldsInForm,
                            Boolean.TRUE,
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpShowHiddenFieldsInForms"),
                            DcResources.getText("lblShowHiddenFieldsInForms"),
                            false,
                            true));
        getSettings().addSetting(_Font,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stFontAntiAliasing,
                            Boolean.FALSE,
                            ComponentFactory._CHECKBOX,
                            "",
                            "Font Anti-aliasing",
                            false,
                            true));        
        getSettings().addSetting(_Font,
                new Setting(DcRepository.datatypes._FONT,
                            DcRepository.settings.stSystemFontNormal,
                            new Font("Arial", Font.PLAIN, 11),
                            ComponentFactory._FONTSELECTOR,
                            DcResources.getText("tpFont"),
                            DcResources.getText("lblFontNormal"),
                            false,
                            true));
        getSettings().addSetting(_Font,
                new Setting(DcRepository.datatypes._FONT,
                            DcRepository.settings.stSystemFontBold,
                            new Font("Arial", Font.PLAIN, 11),
                            ComponentFactory._FONTSELECTOR,
                            DcResources.getText("tpFont"),
                            DcResources.getText("lblFontBold"),
                            false,
                            true));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LOOKANDFEEL,
                            DcRepository.settings.stLookAndFeel,
                            new DcLookAndFeel("beigeazulthemepack.zip", null, "beigeazulthemepack.zip", 0),
                            ComponentFactory._LOOKANDFEELSELECTOR,
                            "",
                            DcResources.getText("lblLookAndFeel"),
                            false,
                            false));        
        getSettings().addSetting(_HTTP,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stProxyServerName,
                            "",
                            ComponentFactory._SHORTTEXTFIELD,
                            DcResources.getText("tpProxyServerName"),
                            DcResources.getText("lblProxyServerName"),
                            true,
                            true));
        getSettings().addSetting(_HTTP,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stProxyServerPort,
                            new Integer(0),
                            ComponentFactory._NUMBERFIELD,
                            DcResources.getText("tpProxyServerPort"),
                            DcResources.getText("lblProxyServerPort"),
                            true,
                            true));
        getSettings().addSetting(_HTTP,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stProxyUserName,
                            "",
                            ComponentFactory._SHORTTEXTFIELD,
                            DcResources.getText("tpProxyUserName"),
                            DcResources.getText("lblProxyUserName"),
                            true,
                            true));
        getSettings().addSetting(_HTTP,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stProxyPassword,
                            "",
                            ComponentFactory._PASSWORDFIELD,
                            DcResources.getText("tpProxyPassword"),
                            DcResources.getText("lblProxyPassword"),
                            true,
                            true));
        getSettings().addSetting(_HTTP,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stSelectedView,
                            new Integer(MasterView._LIST_VIEW),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_HTTP,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stBrowserPath,
                            "",
                            ComponentFactory._FILEFIELD,
                            "",
                            DcResources.getText("lblBrowserPath"),
                            true,
                            true));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stExpertFormSize,
                            new Dimension(700, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stHelpFormSize,
                            new Dimension(700, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stTabbedPaneSize,
                            new Dimension(958, 767),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stReferencesDialogSize,
                            new Dimension(300, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stItemWizardFormSize,
                            new Dimension(800, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stModuleWizardFormSize,
                            new Dimension(800, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stUpdateAllDialogSize,
                            new Dimension(600, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stMainViewSize,
                            new Dimension(1095, 871),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stMainViewState,
                            new Integer(-1),
                            -1,
                            "",
                            "",
                            false,
                            false));

        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stTextViewerSize,
                            new Dimension(500, 700),
                            -1,
                            "",
                            "",
                            false,
                            false));

        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                         DcRepository.settings.stReportingDialogSize,
                         new Dimension(700, 490),
                         -1,
                         "",
                         "",
                         false,
                         false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                         DcRepository.settings.stModuleSelectDialogSize,
                         new Dimension(300, 500),
                         -1,
                         "",
                         "",
                         false,
                         false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stShowModuleList,
                            new Boolean(true),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stBroadband,
                            new Boolean(true),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stLastDirectoryUsed,
                            "",
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stShowMenuBarLabels,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stUpdateAllSelectedItemsOnly,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stShowQuickView,
                            Boolean.TRUE,
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.settings.stDeleteImageFileAfterImport,
                            Boolean.FALSE,
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpDeleteImageFile"),
                            DcResources.getText("lblDeleteImageFile"),
                            false,
                            true));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stQuickViewDividerLocation,
                            new Integer(595),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.settings.stTreeDividerLocation,
                            new Integer(200),
                            -1,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_FileHandlers,
                new Setting(DcRepository.datatypes._DEFINITIONGROUP,
                            DcRepository.settings.stProgramDefinitions,
                            new ProgramDefinitions(),
                            ComponentFactory._PROGRAMDEFINITIONFIELD,
                            DcResources.getText("tpProgramDefinitions"),
                            "",
                            false,
                            true));

        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stResourcesEditorViewSize,
                            new Dimension(542, 680),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stBackupDialogSize,
                            new Dimension(500, 500),
                            -1,
                            "",
                            "",
                            false,
                            false));

        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.settings.stSortDialogSize,
                            new Dimension(300, 200),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.settings.stReportFile,
                            "",
                            -1,
                            "",
                            "",
                            false,
                            false));     
    }
}
