/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.settings;

import java.awt.Dimension;
import java.io.File;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.ComponentFactory;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.modules.DcModule;
import net.sf.dc.core.objects.DcField;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.core.settings.Setting;
import net.sf.dc.core.settings.SettingsGroup;
import net.sf.dc.settings.definitions.DcFieldDefinition;
import net.sf.dc.settings.definitions.DcFieldDefinitions;
import net.sf.dc.settings.definitions.QuickViewFieldDefinition;
import net.sf.dc.settings.definitions.QuickViewFieldDefinitions;

public class DcModuleSettings extends net.sf.dc.settings.Settings {
    
    private String _General = DcResources.getText("lblGroupGeneral");
    
    public DcModuleSettings(DcModule module) {
        super();
        createSettings(module);
        createDefinitions(module);
        
        net.sf.dc.core.settings.Settings settings = getSettings();

        // load the default settings (if available)
        String filename = module.getName().toLowerCase() + ".properties";
        File file = new File(DataCrow.moduleDir + filename);
        if (file.exists()) {
            settings.setSettingsFile(file);
            load();
        }
        
        settings.setSettingsFile(new File(DataCrow.baseDir + "/data/" + filename));
        load();
    }
    
    private void createDefinitions(DcModule module) {
        Collection fields = module.getFields();
        DcFieldDefinitions fldDefinitions = new DcFieldDefinitions();
        QuickViewFieldDefinitions qvDefinitions = new QuickViewFieldDefinitions();
        for (Iterator iter = fields.iterator(); iter.hasNext(); ) {
            DcField field = (DcField) iter.next();
            fldDefinitions.addDefinition(new DcFieldDefinition(field.getIndex(), null, true, false, false, true));
            boolean enabled = field.getValueType() == DcRepository.datatypes._STRING ||
                              field.getValueType() == DcRepository.datatypes._LONG;
            
            qvDefinitions.addDefinition(new QuickViewFieldDefinition(
                                                field.getIndex(), 
                                                new Boolean(enabled).booleanValue(), 
                                                DcResources.getText("lblHorizontal")));
        }
        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DEFINITIONGROUP,
                            DcRepository.modulesettings.stQuickViewFieldDefinitions,
                            qvDefinitions,
                            -1,
                            "",
                            "",
                            false,
                            false));           
        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DEFINITIONGROUP,
                            DcRepository.modulesettings.stFieldDefinitions,
                            fldDefinitions,
                            -1,
                            "",
                            "",
                            false,
                            false));        
    }
    
    protected void createGroups() {
        _General = DcResources.getText("lblGroupGeneral");
        
        SettingsGroup generalGroup = new SettingsGroup(_General, "dc.Settings.GeneralSettings");
        getSettings().addGroup(_General, generalGroup);
    }
    
    protected void createSettings(DcModule module) {
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stOnlineSearchFormSize,
                            new Dimension(700, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._STRINGARRAY,
                            DcRepository.modulesettings.stSearchOrder,
                            new String[] {},
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stItemFormSize,
                            new Dimension(700, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));        
        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stFilterDialogSize,
                            new Dimension(700, 400),
                            -1,
                            "",
                            "",
                            false,
                            false));   
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stFieldSettingsDialogSize,
                            new Dimension(500, 300),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._INTEGERARRAY,
                            DcRepository.modulesettings.stGroupedBy,
                            new int[] {},
                            -1,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stQuickViewSettingsDialogSize,
                            new Dimension(500, 300),
                            -1,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stLoanFormSize,
                            new Dimension(500, 500),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
               new Setting(DcRepository.datatypes._DIMENSION,
                           DcRepository.modulesettings.stImportCDDialogSize,
                           new Dimension(550, 400),
                           -1,
                           "",
                           "",
                           false,
                           false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stFileImportDialogSize,
                            new Dimension(550, 800),
                            -1,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stSynchronizerDialogSize,
                            new Dimension(550, 400),
                            -1,
                            "",
                            "",
                            false,
                            false));  
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stFieldSettingsDialogSize,
                            new Dimension(500, 300),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stOnlineSearchFormSize,
                            new Dimension(700, 600),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stSimpleItemViewSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stSimpleItemFormSize,
                            new Dimension(450, 550),
                            -1,
                            "",
                            "",
                            false,
                            false));  
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stOnlineSearchFieldSettingsDialogSize,
                            new Dimension(600, 450),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._LONG,
                            DcRepository.modulesettings.stQuickFilterDefaultField,
                            new Long(DcObject._ID),
                            -1,
                            "",
                            "",
                            false,
                            false));           
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.modulesettings.stAutoCreateSubItems,
                            new Boolean(true),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpAutoCreateMissingItems"),
                            DcResources.getText("lblAutoCreateMissingItems"),
                            false,
                            false));     
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.modulesettings.stOnlineSearchSubItems,
                            new Boolean(false),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpOnlineSearchSubItems"),
                            DcResources.getText("lblOnlineSearchSubItems"),
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.modulesettings.stOnlineSearchDefaultServer,
                            null,
                            ComponentFactory._SHORTTEXTFIELD,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.modulesettings.stOnlineSearchDefaultRegion,
                            null,
                            ComponentFactory._SHORTTEXTFIELD,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._STRING,
                            DcRepository.modulesettings.stOnlineSearchDefaultMode,
                            null,
                            ComponentFactory._SHORTTEXTFIELD,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.modulesettings.stAutoAddPerfectMatch,
                            new Boolean(false),
                            ComponentFactory._CHECKBOX,
                            DcResources.getText("tpAutoAddPerfectMatch"),
                            DcResources.getText("lblAutoAddPerfectMatch"),
                            false,
                            false)); 
        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.modulesettings.stEnabled,
                            new Boolean(true),
                            -1,
                            "",
                            "",
                            false,
                            false)); 
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._DIMENSION,
                            DcRepository.modulesettings.stOnlineSearchFieldSettingsDialogSize,
                            new Dimension(600, 450),
                            -1,
                            "",
                            "",
                            false,
                            false));
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._BOOLEAN,
                            DcRepository.modulesettings.stOnlineSearchOverwrite,
                            new Boolean(false),
                            -1,
                            "",
                            "",
                            false,
                            false));        
        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._INTEGERARRAY,
                            DcRepository.modulesettings.stOnlineSearchFieldOverwriteSettings,
                            new int[] {},
                            -1,
                            "",
                            "",
                            false,
                            false));

        getSettings().addSetting(_General,
                new Setting(DcRepository.datatypes._INTEGERARRAY,
                            DcRepository.modulesettings.stOnlineSearchRetrievedFields,
                            module.getFieldIndices(),
                            -1,
                            "",
                            "",
                            false,
                            false));
    }
}
