/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.synchronizers;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcAssociate;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.OnlineSearchHelper;
import net.sf.dc.util.StringUtils;

public class AssociateSynchronizer extends DefaultSynchronizer {

    private DcObject dco;
    
    public AssociateSynchronizer() {
        super(DcResources.getText("lblMassItemUpdate", DcModules.getCurrent().getObjectName()),
              DcModules.getCurrent().getIndex());    
    }
    
    public String getHelpText() {
        return DcResources.getText("msgAssociateMassUpdateHelp",
                                    new String[] {DcModules.getCurrent().getObjectNamePlural().toLowerCase(),
                                                  DcModules.getCurrent().getObjectName().toLowerCase()});
    }

    public DcObject getDcObject() {
        return dco;
    }
    
    public boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region) {
        this.dco = dco;
        boolean updated = exactSearch(dco);
        
        if (!updated) {
            String name = StringUtils.normalize((String) dco.getValue(DcAssociate._A_NAME));

            if (name == null || name.length() == 0) 
                return updated;
            
            OnlineSearchHelper osh = new OnlineSearchHelper(dco.getModule().getIndex());
            DcObject result = osh.query(dco, (String) dco.getValue(DcAssociate._A_NAME), new int[] {DcAssociate._A_NAME});
            if (result != null) {
                updated = true;
                merge(dco, result);
            }
        }

        return updated;
    }
    
    protected void merge(DcObject dco1, DcObject dco2) {
        setValue(dco1, DcAssociate._A_NAME, dco2.getValue(DcAssociate._A_NAME));
        setValue(dco1, DcAssociate._B_DESCRIPTION, dco2.getValue(DcAssociate._B_DESCRIPTION));
        setValue(dco1, DcAssociate._C_WEBPAGE, dco2.getValue(DcAssociate._C_WEBPAGE));
        setValue(dco1, DcAssociate._D_PHOTO, dco2.getValue(DcAssociate._D_PHOTO));
        setValue(dco1, DcAssociate._SYS_SERVICEURL, dco2.getValue(DcAssociate._SYS_SERVICEURL));
    }    
}
