/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.synchronizers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcAssociate;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.AudioCD;
import net.sf.dc.core.objects.helpers.AudioTrack;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.OnlineSearchHelper;
import net.sf.dc.util.StringUtils;

public class AudioCdSynchronizer extends DefaultSynchronizer {

    private DcObject dco;
    
    public AudioCdSynchronizer() {
        super(DcResources.getText("lblMassItemUpdate", DcModules.get(DcModules._AUDIOCD).getObjectName()),
              DcModules._AUDIOCD);
    }
    
    public String getHelpText() {
        return DcResources.getText("msgAudioCdMassUpdateHelp");
    }
    
    public DcObject getDcObject() {
        return dco;
    }
    
    public boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region) {
        boolean updated = exactSearch(dco);
        
        if (!updated) {
            AudioCD acCurrent = (AudioCD) dco;

            String title = StringUtils.normalize((String) dco.getValue(AudioCD._A_TITLE));
            Collection artists = (Collection) dco.getValue(AudioCD._F_ARTIST);
            
            if ((title == null || title.length() == 0) || (artists == null || artists.size() == 0)) 
                return updated;
            
            OnlineSearchHelper osh = new OnlineSearchHelper(dco.getModule().getIndex());
            osh.setServer(server);
            osh.setRegion(region);
            osh.setMaximum(2);
            Collection c = osh.query((String) acCurrent.getValue(AudioCD._A_TITLE));
            
            for (Iterator iter = c.iterator(); iter.hasNext(); ) {
                AudioCD acNew = (AudioCD) iter.next();
                if (match(dco, acNew)) {
                    boolean match = updateTracks(acCurrent.getChildren(), acNew.getChildren());
                    if (match) {
                        updated = true;
                        merge(acCurrent, acNew);
                    }
                }
            }
            c.clear();
        }
        return updated;
    }
    
    private boolean updateTracks(Collection oldTracks, Collection newTracks) {
        boolean match = false;
        for (Iterator iterOld = oldTracks.iterator(); iterOld.hasNext(); ) {
            AudioTrack currentTrack = (AudioTrack) iterOld.next();
            
            String titleOld = (String) currentTrack.getValue(AudioTrack._A_TITLE);
            Long lengthOld = (Long) currentTrack.getValue(AudioTrack._H_PLAYLENGTH);
            Long trackOld = (Long) currentTrack.getValue(AudioTrack._F_TRACKNUMBER);

            for (Iterator iterNew = newTracks.iterator(); iterNew.hasNext(); ) {
                AudioTrack newTrack = (AudioTrack) iterNew.next();
                
                String titleNew = (String) newTrack.getValue(AudioTrack._A_TITLE);
                Long lengthNew = (Long) newTrack.getValue(AudioTrack._H_PLAYLENGTH);
                Long trackNew = (Long) newTrack.getValue(AudioTrack._F_TRACKNUMBER);

                if ((titleOld != null && titleNew != null) && 
                     titleNew.toLowerCase().equals(titleOld.toLowerCase())) {
                    
                    match = true;
                    merge(currentTrack, newTrack);
                
                } else if (newTracks.size() == oldTracks.size() && 
                           ((lengthOld != null && lengthNew != null) && lengthNew.equals(lengthOld)) ||  
                           ((trackOld != null && trackNew != null) && trackNew.equals(trackOld))) {
                
                    match = true;
                    merge(currentTrack, newTrack);
                }
            }
        }
        return match;
    } 
    
    protected boolean match(DcObject dco1, DcObject dco2) {
        String title1 = StringUtils.normalize((String) dco1.getValue(AudioCD._A_TITLE)).trim();
        String title2 = StringUtils.normalize((String) dco2.getValue(AudioCD._A_TITLE)).trim();

        boolean match = title1.equals(title2);
        if (title1.equals(title2)) {
            Collection artists1 = (Collection) dco1.getValue(AudioCD._F_ARTIST);
            Collection artists2 = (Collection) dco2.getValue(AudioCD._F_ARTIST);
            
            artists1 = artists1 == null ? new ArrayList() : artists1;
            artists2 = artists2 == null ? new ArrayList() : artists2;

            match = false;
            for (Iterator iter1 = artists1.iterator(); iter1.hasNext(); ) {
                DcObject person1 = (DcObject) iter1.next();
                for (Iterator iter = artists1.iterator(); iter.hasNext(); ) {
                    DcObject person2 = (DcObject) iter1.next();

                    String name1 = StringUtils.normalize((String) person1.getValue(DcAssociate._A_NAME)).trim();
                    String name2 = StringUtils.normalize((String) person2.getValue(DcAssociate._A_NAME)).trim();
                    
                    if (name1.equals(name2)) match = true;
                }
            }
        }
        return match;        
    }
}
