/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.synchronizers;

import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.helpers.Book;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.OnlineSearchHelper;
import net.sf.dc.util.StringUtils;

public class BookSynchronizer extends DefaultSynchronizer {

    private DcObject dco;
    
    public BookSynchronizer() {
        super(DcResources.getText("lblMassItemUpdate", DcModules.get(DcModules._BOOK).getObjectName()),
              DcModules._BOOK);
    }
    
    public String getHelpText() {
        return DcResources.getText("msgBookMassUpdateHelp");
    }

    public DcObject getDcObject() {
        return dco;
    }
    
    public boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region) {
        this.dco = dco;
        boolean updated = exactSearch(dco);
        
        if (!updated) {
            String title = StringUtils.normalize((String) dco.getValue(Book._A_TITLE));
            String isbn = StringUtils.normalize((String) dco.getValue(Book._N_ISBN13));
            
            if ((title == null || title.length() == 0)) 
                return updated;
            
            OnlineSearchHelper osh = new OnlineSearchHelper(dco.getModule().getIndex());
            osh.setServer(server);
            osh.setRegion(region);
            osh.setMaximum(2);
            Collection c = osh.query((String) dco.getValue(Book._A_TITLE));
            for (Iterator iter = c.iterator(); iter.hasNext() && !updated; ) {
                Book book = (Book) iter.next();
    
                String titleNew = StringUtils.normalize((String) book.getValue(Book._A_TITLE));
                String isbnNew = StringUtils.normalize((String) book.getValue(Book._N_ISBN13));
                
                boolean match = (isbn.length() > 0) && (isbnNew.length() > 0) ? isbn.equals(isbnNew) :
                                titleNew.toLowerCase().equals(title.toLowerCase());
                if (match) {
                    updated = true;
                    merge(dco, book);
                }
            }
        }

        return updated;
    }
}
