/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.synchronizers;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.views.View;
import net.sf.dc.core.db.DatabaseManager;
import net.sf.dc.core.log.DcLog;
import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.ValidationException;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;

public abstract class DefaultSynchronizer extends Synchronizer {

    public DefaultSynchronizer(String title, int module) {
        super(title, module);
    }
    
    public boolean canParseFiles() {
        return false;
    }

    public boolean canUseOnlineServices() {
        return true;
    }
    
    public abstract boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region);

    protected boolean exactSearch(DcObject dco) {
        try {
            DcObject dcoNew = dco.getModule().getOnlineSearch().query(dco);
            if (dcoNew != null) {
                merge(dco, dcoNew);
                return true;
            }
        } catch (Exception exp) {
            DcLog.add(exp);
        }        
        return false;
    }    
    
    public Thread getTask() {
        return new Task();
    }    
    
    private class Task extends Thread {
        public void run() {
            try {
                initialize();
                
                View view = DcModules.getCurrent().getCurrentSearchView();
                
                int items = view.getRows();
                Collection objects = new ArrayList();
                for (int i = 0; i < items && !isCancelled(); i++)
                    objects.add(view.getObject(i));
                
                initProgressBar(items);
                for ( Iterator iter = objects.iterator(); iter.hasNext() && !isCancelled(); ) {
                    DcObject dco = (DcObject) iter.next();
                    boolean updated = false;
                    
                    addMessage(DcResources.getText("msgSearchingOnlineFor", "" + dco));
                    updated = onlineUpdate(dco, getServer(), getRegion());
                    updateProgressBar();
                    
                    try {
                        if (updated) {
                            dco.setSilence(true);
                            dco.saveUpdate(true);
                            
                            while (DatabaseManager.getQueueSize() > 0) {
                                try {
                                    sleep(100);
                                } catch (Exception exp) {}
                            }
                        }
                    } catch (ValidationException ve) {
                        addMessage(ve.getMessage());
                    }
                }
            } finally {
                addMessage(DcResources.getText("msgSynchronizerEnded"));
                enableAction(true);
            }  
        }
    }
}
