/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.synchronizers;

import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.core.modules.DcModules;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.DcProperty;
import net.sf.dc.core.objects.helpers.Software;
import net.sf.dc.core.resources.DcResources;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;
import net.sf.dc.onlinesearch.OnlineSearchHelper;
import net.sf.dc.onlinesearch.mobygames.MobyGamesServer;
import net.sf.dc.util.StringUtils;

public class SoftwareSynchronizer extends DefaultSynchronizer {

    private DcObject dco;
    
    public SoftwareSynchronizer() {
        super(DcResources.getText("lblMassItemUpdate", DcModules.get(DcModules._SOFTWARE).getObjectName()),
              DcModules._SOFTWARE);
    }
    
    public String getHelpText() {
        return DcResources.getText("msgSoftwareMassUpdateHelp");
    }
    
    public DcObject getDcObject() {
        return dco;
    }
    
    public boolean onlineUpdate(DcObject dco, DcServer server, DcRegion region) {
        this.dco = dco;
        boolean updated = exactSearch(dco);
        
        if (!updated) {
            String title = StringUtils.normalize((String) dco.getValue(Software._A_TITLE));
            DcProperty platform = (DcProperty) dco.getValue(Software._H_PLATFORM);
            
            if ((title == null || title.length() == 0) || platform == null) 
                return updated;
            
            String searchString = (String) dco.getValue(Software._A_TITLE);
            searchString += server instanceof MobyGamesServer && dco.getValue(Software._H_PLATFORM) != null ? 
                                " " + dco.getDisplayString(Software._H_PLATFORM) : "";

            OnlineSearchHelper osh = new OnlineSearchHelper(dco.getModule().getIndex());
            osh.setServer(server);
            osh.setRegion(region);
            Collection c = osh.query(searchString);                                
                                
            for (Iterator iter = c.iterator(); iter.hasNext() && !updated; ) {
                Software software = (Software) iter.next();
    
                String titleNew = StringUtils.normalize((String) software.getValue(Software._A_TITLE));
                DcProperty platformNew = (DcProperty) software.getValue(Software._H_PLATFORM);
                
                if (titleNew.indexOf(" for ") > -1)
                    titleNew = titleNew.substring(0, titleNew.indexOf(" for "));
                
                if ((titleNew.toLowerCase().equals(title.toLowerCase()))) {
                    boolean match = true;
                    if (platform != null && platformNew != null)
                        match = platform.toString().equals(platformNew.toString());
                    
                    if (match) {
                        merge(dco, software);
                        updated = true;
                    }
                }
            }
        }
        return updated;
    }
}
