/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                       This file is part of Data Crow.                      *
 *       Data Crow is free software; you can redistribute it and/or           *
 *        modify it under the terms of the GNU General Public                 *
 *       License as published by the Free Software Foundation; either         *
 *              version 3 of the License, or any later version.               *
 *                                                                            *
 *        Data Crow is distributed in the hope that it will be useful,        *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.             *
 *           See the GNU General Public License for more details.             *
 *                                                                            *
 *        You should have received a copy of the GNU General Public           *
 *  License along with this program. If not, see http://www.gnu.org/licenses  *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.synchronizers;

import java.util.Collection;
import java.util.Iterator;

import net.sf.dc.console.windows.ItemSynchronizerDialog;
import net.sf.dc.core.DcRepository;
import net.sf.dc.core.data.DataManager;
import net.sf.dc.core.objects.DcMapping;
import net.sf.dc.core.objects.DcObject;
import net.sf.dc.core.objects.Picture;
import net.sf.dc.onlinesearch.DcRegion;
import net.sf.dc.onlinesearch.DcServer;

public abstract class Synchronizer {
    
    protected ItemSynchronizerDialog dlg;
    private String title;
    protected final int module;
    
    public Synchronizer(String title, int module) {
        this.title = title;
        this.module = module;
    }
    
    public abstract Thread getTask();
    public abstract String getHelpText();
    
    public abstract boolean canParseFiles();
    public abstract boolean canUseOnlineServices();
    
    public  String getTitle() {
        return title;
    }

    public String getHelpIndex() {
        return "dc.tools.massupdate";
    }
    
    public void synchronize(ItemSynchronizerDialog dlg) {
        this.dlg = dlg;
        
        Thread thread = getTask();
        thread.start();
    }
    
    public void setListener(ItemSynchronizerDialog dlg) {
        this.dlg = dlg;
    }
    
    public void addMessage(String message) {
        dlg.addMessage(message);
    }
    
    protected boolean isReparseFiles() {
        return dlg.isReparseFiles();
    }

    protected boolean useOnlineService() {
        return dlg.useOnlineService();
    }
    
    
    protected boolean isCancelled() {
        return dlg.isCancelled();
    }
    
    protected void initProgressBar(int max) {
        dlg.initProgressBar(max);
    }

    protected void updateProgressBar() {
        dlg.updateProgressBar();
    }
    
    protected void enableAction(boolean b) {
        dlg.enableActions(b);
    }

    protected void initialize() {
        dlg.initialize();
    }
    
    protected DcServer getServer() {
        return dlg.getServer();
    }

    protected DcRegion getRegion() {
        return dlg.getRegion();
    }
    
    protected void merge(DcObject dco1, DcObject dco2) {
        int[] fields = dco1.getFieldIndices();
        for (int i = 0; i < fields.length; i++)
            setValue(dco1, fields[i], dco2.getValue(fields[i]));
    }
    
    protected void setValue(DcObject dco, int field, Object value) {
        boolean overwrite = dco.getModule().getSettings().getBoolean(DcRepository.modulesettings.stOnlineSearchOverwrite);
        int[] fields = overwrite ?
                       dco.getModule().getSettings().getIntArray(DcRepository.modulesettings.stOnlineSearchFieldOverwriteSettings) :
                       dco.getModule().getFieldIndices();
            
        boolean allowed = false;
        for (int i = 0;i < fields.length; i++)
            allowed |= fields[i] == field;
        
        if (allowed) {
            if (dco.isFilled(field)) {
                if (overwrite && 
                    (value != null && !value.equals("") && !value.toString().equals("-1"))) {
                    
                    if (value instanceof Collection) {
                        for (Iterator iter = ((Collection) value).iterator(); iter.hasNext(); ) {
                            DcObject o = (DcObject) iter.next();
                            if (o instanceof DcMapping)
                                o.setValue(DcMapping._A_PARENT_ID, dco.getID());
                            else
                                DataManager.createReference(dco, field, o);
                        }
                    } else if (value instanceof Picture) {
                        dco.setValue(field, ((Picture) value).getValue(Picture._D_IMAGE));
                    } else {
                        dco.setValue(field, value);    
                    }
                }
            } else {
                dco.setValue(field, value);
            }
        }
    }
}