/*
    OpenChart2 Java Charting Library and Toolkit
    Copyright (C) 2005-2007 Approximatrix, LLC
    Copyright (C) 2001  Sebastian Mller
    http://www.approximatrix.com

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
    
*/

package net.sf.ext.approximatrix.charting.model;

/** Implements a ChartDataModelConstraint class specifically for the
 * MultiScatterDataModel class.  This implementation is used to interface
 * some of the easy scaling methods available in MultiScatterDataModel
 *
 * @author armstrong
 */
public class MultiScatterDataModelConstraints implements ChartDataModelConstraints {

	/** The internal pointer to the corresponding data model */
	MultiScatterDataModel model = null;
	
	/** Constructor requiring a data model to be specified
	 *
	 * @param model the MultiScatterDataModel to support
	 */
	public MultiScatterDataModelConstraints(MultiScatterDataModel model) {
		this.model = model;
	}
	
	/** Returns the minimum dependent value to display
	 *
	 * @return a Number which represents the minimum dependent value for display
	 */
	public Number getMinimumValue() {
		if(model.isAutoScale())
			return new Double(model.getMinYValue());
		else
			return model.getManualMinimumValue();
	}

	/** Returns the maximum dependent value to display
	 *
	 * @return a Number which represents the maximum dependent value for display
	 */
	public Number getMaximumValue() {
		if(model.isAutoScale())
			return new Double(model.getMaxYValue());
		else
			return model.getManualMaximumValue();
	}

	/** Returns the minimum independent value to display
	 *
	 * @return a double which represents the minimum independent value for display
	 */
	public double getMinimumColumnValue() {
		if(model.isAutoScale())
			return model.getMinXValue();
		else
			return model.getManualMinimumColumnValue();
	}

	/** Returns the maximum independent value to display
	 *
	 * @return a double which represents the maximum independent value for display
	 */
	public double getMaximumColumnValue() {
		if(model.isAutoScale())
			return model.getMaxXValue();
		else
			return model.getManualMaximumColumnValue();
	}

}
