/**Implements the Edit/Undo command.*/

package net.sf.ext.ij;
import net.sf.ext.ij.gui.Roi;
import net.sf.ext.ij.process.FHT;
import net.sf.ext.ij.process.ImageProcessor;

/** This class consists of static methods and
	fields that implement ImageJ's Undo command. */
public class Undo {

	public static final int NOTHING = 0;
	public static final int FILTER = 1;
	public static final int TYPE_CONVERSION = 2;
	public static final int PASTE = 3;
	public static final int COMPOUND_FILTER = 4;
	public static final int COMPOUND_FILTER_DONE = 5;
	public static final int TRANSFORM = 6;
	
	private static int whatToUndo = Undo.NOTHING;
	private static int imageID;
	private static ImageProcessor ipCopy = null;
	private static ImagePlus impCopy;
	
	public static void setup(int what, ImagePlus imp) {
		if (imp==null) {
			Undo.whatToUndo = Undo.NOTHING;
			Undo.reset();
			return;
		}
		//IJ.log(imp.getTitle() + ": set up undo (" + what + ")");
		if (what==Undo.FILTER && Undo.whatToUndo==Undo.COMPOUND_FILTER)
				return;
		if (what==Undo.COMPOUND_FILTER_DONE) {
			if (Undo.whatToUndo==Undo.COMPOUND_FILTER)
				Undo.whatToUndo = what;
			return;
		}
		Undo.whatToUndo = what;
		Undo.imageID = imp.getID();
		if (what==Undo.TYPE_CONVERSION)
			Undo.ipCopy = imp.getProcessor();
		else if (what==Undo.TRANSFORM) {			
			Undo.impCopy = new ImagePlus(imp.getTitle(), imp.getProcessor().duplicate());
			Object fht  = imp.getProperty("FHT");
			if (fht!=null) {
				fht = new FHT((ImageProcessor)fht); // duplicate
				Undo.impCopy.setProperty("FHT", fht);
			}
		} else if (what==Undo.COMPOUND_FILTER) {
			ImageProcessor ip = imp.getProcessor();
			if (ip!=null)
				Undo.ipCopy = ip.duplicate();
			else
				Undo.ipCopy = null;
		} else
			Undo.ipCopy = null;
	}
	
	
	public static void reset() {
		if (Undo.whatToUndo==Undo.COMPOUND_FILTER)
			return;
		Undo.whatToUndo = Undo.NOTHING;
		Undo.imageID = 0;
		Undo.ipCopy = null;
		Undo.impCopy = null;
		//IJ.log("Undo: reset");
	}
	

	public static void undo() {
		ImagePlus imp = WindowManager.getCurrentImage();
		//IJ.log(imp.getTitle() + ": undo (" + whatToUndo + ")  "+(imageID!=imp.getID()));
		if (Undo.imageID!=imp.getID()) {
			Undo.reset();
			return;
		}
		switch (Undo.whatToUndo) {
			case FILTER:
				ImageProcessor ip = imp.getProcessor();
				if (ip!=null) {
					ip.reset();
					imp.updateAndDraw();
				}
	    		break;
			case TYPE_CONVERSION:
			case COMPOUND_FILTER:
			case COMPOUND_FILTER_DONE:
				if (Undo.ipCopy!=null)
					imp.setProcessor(null, Undo.ipCopy);
	    		break;
			case TRANSFORM:
				if (Undo.impCopy!=null) {
					imp.setProcessor(Undo.impCopy.getTitle(), Undo.impCopy.getProcessor());
					Object fht  = Undo.impCopy.getProperty("FHT");
					if (fht!=null)
						imp.setProperty("FHT", fht);
					else if (imp.getProperty("FHT")!=null)
							imp.getProperties().remove("FHT");
				}
	    		break;
			case PASTE:
				Roi roi = imp.getRoi();
				if (roi!=null)
					roi.abortPaste();
	    		break;
    	}
    	Undo.reset();
	}
	
}
