package net.sf.ext.ij.macro;

public interface MacroConstants {

	// In the tokenized macro, the token and optional symbol table address 
	// are packed into a single 32-bit int. The token is stored in the right-most 
	// 12 bits and the address in the remaining 24 bits. 
	static final int TOK_SHIFT=12, TOK_MASK=0xfff;

	static final int PLUS_PLUS=1, MINUS_MINUS=2, EQ=3, NEQ=4, GT=5, GTE=6, LT=7, LTE=8,
		PLUS_EQUAL=9, MINUS_EQUAL=10, MUL_EQUAL=11, DIV_EQUAL=12, LOGICAL_AND=13, LOGICAL_OR=14,
		SHIFT_RIGHT=15, SHIFT_LEFT=16;

	// Token types
	public static final int EOF=128, WORD=129, NUMBER=130, NOP=131, EOL=132, STRING_CONSTANT=133, 
		PREDEFINED_FUNCTION=134, NUMERIC_FUNCTION=135, STRING_FUNCTION=136, ARRAY_FUNCTION=137, 
		USER_FUNCTION=138, ARRAY=139;

	// Keywords
	static final String[] keywords = {"macro", "var", "if", "else", "while", "do", "for", "function",
		"return", "true", "false", "PI", "NaN"};
	public static final int MACRO=200, VAR=201, IF=202, ELSE=203, WHILE=204, DO=205, FOR=206, FUNCTION=207,
		RETURN=208, TRUE=209, FALSE=210, PI=211, NaN=212;;
	static final int[] keywordIDs = {MacroConstants.MACRO, MacroConstants.VAR, MacroConstants.IF, MacroConstants.ELSE, MacroConstants.WHILE, MacroConstants.DO, MacroConstants.FOR, MacroConstants.FUNCTION,
		MacroConstants.RETURN, MacroConstants.TRUE, MacroConstants.FALSE, MacroConstants.PI, MacroConstants.NaN};

	// Functions that don't return a value
	static final int RUN=300, INVERT=301, SELECT=302, WAIT=303, BEEP=304, RESET_MIN_MAX=305, RESET_THRESHOLD=306,
		PRINT=307, WRITE=308, DO_WAND=309, SET_MIN_MAX=310, SET_THRESHOLD=311, SET_TOOL=312,
		SET_FOREGROUND=313, SET_BACKGROUND=314, MAKE_LINE=315, MAKE_OVAL=316, MAKE_RECTANGLE=317,
		DUMP=318, MOVE_TO=319, LINE_TO=320, DRAW_LINE=321, REQUIRES=322, AUTO_UPDATE=323, UPDATE_DISPLAY=324, DRAW_STRING=325,
		SET_PASTE_MODE=326, DO_COMMAND=327, SHOW_STATUS=328, SHOW_PROGRESS=329, SHOW_MESSAGE=330, PUT_PIXEL=331, SET_PIXEL=332,
		SNAPSHOT=333, RESET=334, FILL=335, SET_COLOR=336, SET_LINE_WIDTH=337, CHANGE_VALUES=338, SELECT_IMAGE=339, EXIT=340,
		SET_LOCATION=341, GET_CURSOR_LOC=342, GET_LINE=343, GET_VOXEL_SIZE=344, GET_HISTOGRAM=345, GET_STATISTICS=346,
		GET_BOUNDING_RECT=347, GET_LUT=348, SET_LUT=349, GET_COORDINATES=350, SHOW_MESSAGE_WITH_CANCEL=351,
		MAKE_SELECTION=352, SET_RESULT=353, UPDATE_RESULTS=354, SET_BATCH_MODE=355, PLOT=356, SET_JUSTIFICATION=357,
		SET_Z_COORDINATE=358, GET_THRESHOLD=359, GET_PIXEL_SIZE=360, SETUP_UNDO=361, SAVE_SETTINGS=362, RESTORE_SETTINGS=363,
		SET_KEY_DOWN=364, OPEN=365, SET_FONT=366, GET_MIN_AND_MAX=367, CLOSE=368, SET_SLICE=369,
		NEW_IMAGE=370, SAVE_AS=371, SAVE=372, SET_AUTO_THRESHOLD=373, RENAME=374, GET_BOUNDS=375, FILL_RECT=376,
		GET_RAW_STATISTICS=377, FLOOD_FILL=378, RESTORE_PREVIOUS_TOOL=379, SET_VOXEL_SIZE=380, GET_LOCATION_AND_SIZE=381,
		GET_DATE_AND_TIME=382, SET_METADATA=383, CALCULATOR=384, SET_RGB_WEIGHTS=385, MAKE_POLYGON=386, SET_SELECTION_NAME=387,
		DRAW_RECT=388, DRAW_OVAL=389, FILL_OVAL=390, SET_OPTION=391, SHOW_TEXT=392, SET_SELECTION_LOC=393, GET_DIMENSIONS=394;
	static final String[] functions = {"run","invert","selectWindow","wait", "beep", "resetMinAndMax", "resetThreshold",
		"print", "write", "doWand", "setMinAndMax", "setThreshold", "setTool",
		"setForegroundColor", "setBackgroundColor", "makeLine", "makeOval", "makeRectangle",
		"dump", "moveTo", "lineTo", "drawLine", "requires", "autoUpdate", "updateDisplay", "drawString",
		"setPasteMode", "doCommand", "showStatus", "showProgress", "showMessage", "putPixel", "setPixel",
		"snapshot", "reset", "fill", "setColor", "setLineWidth", "changeValues", "selectImage", "exit",
		"setLocation", "getCursorLoc", "getLine", "getVoxelSize", "getHistogram", "getStatistics",
		"getBoundingRect", "getLut", "setLut", "getSelectionCoordinates", "showMessageWithCancel",
		"makeSelection", "setResult", "updateResults", "setBatchMode", "Plot", "setJustification",
		"setZCoordinate", "getThreshold", "getPixelSize", "setupUndo", "saveSettings", "restoreSettings",
		"setKeyDown", "open", "setFont", "getMinAndMax", "close", "setSlice",
		"newImage", "saveAs", "save", "setAutoThreshold", "rename", "getSelectionBounds", "fillRect",
		"getRawStatistics", "floodFill", "restorePreviousTool", "setVoxelSize", "getLocationAndSize",
		"getDateAndTime", "setMetadata", "imageCalculator", "setRGBWeights", "makePolygon", "setSelectionName",
		"drawRect", "drawOval", "fillOval", "setOption", "showText", "setSelectionLocation", "getDimensions"};
	static final int[] functionIDs = {MacroConstants.RUN, MacroConstants.INVERT, MacroConstants.SELECT, MacroConstants.WAIT, MacroConstants.BEEP, MacroConstants.RESET_MIN_MAX, MacroConstants.RESET_THRESHOLD,
		MacroConstants.PRINT, MacroConstants.WRITE,	 MacroConstants.DO_WAND, MacroConstants.SET_MIN_MAX, MacroConstants.SET_THRESHOLD, MacroConstants.SET_TOOL,
		MacroConstants.SET_FOREGROUND, MacroConstants.SET_BACKGROUND, MacroConstants.MAKE_LINE, MacroConstants.MAKE_OVAL, MacroConstants.MAKE_RECTANGLE,
		MacroConstants.DUMP, MacroConstants.MOVE_TO, MacroConstants.LINE_TO, MacroConstants.DRAW_LINE, MacroConstants.REQUIRES, MacroConstants.AUTO_UPDATE, MacroConstants.UPDATE_DISPLAY, MacroConstants.DRAW_STRING,
		MacroConstants.SET_PASTE_MODE, MacroConstants.DO_COMMAND, MacroConstants.SHOW_STATUS, MacroConstants.SHOW_PROGRESS, MacroConstants.SHOW_MESSAGE, MacroConstants.PUT_PIXEL, MacroConstants.SET_PIXEL,
		MacroConstants.SNAPSHOT, MacroConstants.RESET, MacroConstants.FILL, MacroConstants.SET_COLOR, MacroConstants.SET_LINE_WIDTH, MacroConstants.CHANGE_VALUES, MacroConstants.SELECT_IMAGE, MacroConstants.EXIT,
		MacroConstants.SET_LOCATION, MacroConstants.GET_CURSOR_LOC, MacroConstants.GET_LINE, MacroConstants.GET_VOXEL_SIZE, MacroConstants.GET_HISTOGRAM, MacroConstants.GET_STATISTICS,
		MacroConstants.GET_BOUNDING_RECT, MacroConstants.GET_LUT, MacroConstants.SET_LUT, MacroConstants.GET_COORDINATES, MacroConstants.SHOW_MESSAGE_WITH_CANCEL,
		MacroConstants.MAKE_SELECTION, MacroConstants.SET_RESULT, MacroConstants.UPDATE_RESULTS, MacroConstants.SET_BATCH_MODE, MacroConstants.PLOT, MacroConstants.SET_JUSTIFICATION,
		MacroConstants.SET_Z_COORDINATE, MacroConstants.GET_THRESHOLD, MacroConstants.GET_PIXEL_SIZE, MacroConstants.SETUP_UNDO, MacroConstants.SAVE_SETTINGS, MacroConstants.RESTORE_SETTINGS,
		MacroConstants.SET_KEY_DOWN, MacroConstants.OPEN, MacroConstants.SET_FONT, MacroConstants.GET_MIN_AND_MAX, MacroConstants.CLOSE, MacroConstants.SET_SLICE,
		MacroConstants.NEW_IMAGE, MacroConstants.SAVE_AS, MacroConstants.SAVE, MacroConstants.SET_AUTO_THRESHOLD, MacroConstants.RENAME, MacroConstants.GET_BOUNDS, MacroConstants.FILL_RECT,
		MacroConstants.GET_RAW_STATISTICS, MacroConstants.FLOOD_FILL, MacroConstants.RESTORE_PREVIOUS_TOOL, MacroConstants.SET_VOXEL_SIZE, MacroConstants.GET_LOCATION_AND_SIZE,
		MacroConstants.GET_DATE_AND_TIME, MacroConstants.SET_METADATA, MacroConstants.CALCULATOR, MacroConstants.SET_RGB_WEIGHTS, MacroConstants.MAKE_POLYGON, MacroConstants.SET_SELECTION_NAME,
		MacroConstants.DRAW_RECT, MacroConstants.DRAW_OVAL, MacroConstants.FILL_OVAL, MacroConstants.SET_OPTION, MacroConstants.SHOW_TEXT, MacroConstants.SET_SELECTION_LOC, MacroConstants.GET_DIMENSIONS};

	// Numeric functions
	static final int GET_PIXEL=1000, ABS=1001, COS=1002, EXP=1003, FLOOR=1004, LOG=1005, MAX_OF=1006, MIN_OF=1007, POW=1008,
		ROUND=1009, SIN=1010, SQRT=1011, TAN=1012, GET_TIME=1013, GET_WIDTH=1014, GET_HEIGHT=1015, RANDOM=1016,
		GET_RESULT=1017, GET_COUNT=1018, GET_NUMBER=1019, NIMAGES=1020, NSLICES=1021,
		LENGTH_OF=1022, NRESULTS=1023, GET_ID=1024, BIT_DEPTH=1025, SELECTION_TYPE=1026, ATAN=1027, IS_OPEN=1028, 
		IS_ACTIVE=1029, INDEX_OF=1030, LAST_INDEX_OF=1031, CHAR_CODE_AT=1032, GET_BOOLEAN=1033,
		STARTS_WITH=1034, ENDS_WITH=1035, ATAN2=1036, IS_NAN=1037, GET_ZOOM=1038, PARSE_INT=1039, PARSE_FLOAT=1040,
		IS_KEY_DOWN=1041, GET_SLICE_NUMBER=1042, SCREEN_WIDTH=1043, SCREEN_HEIGHT=1044, CALIBRATE=1045,
		ASIN=1046, ACOS=1047, ROI_MANAGER=1048, TOOL_ID=1049, IS=1050;
	static final String[] numericFunctions = { "getPixel", "abs", "cos", "exp", "floor", "log", "maxOf", "minOf", "pow",
		"round", "sin", "sqrt", "tan", "getTime", "getWidth", "getHeight", "random",
		"getResult", "getResultsCount", "getNumber", "nImages", "nSlices", 
		"lengthOf", "nResults", "getImageID", "bitDepth", "selectionType", "atan", "isOpen",
		"isActive", "indexOf", "lastIndexOf", "charCodeAt", "getBoolean",
		"startsWith", "endsWith", "atan2", "isNaN", "getZoom", "parseInt", "parseFloat",
		"isKeyDown", "getSliceNumber", "screenWidth", "screenHeight", "calibrate",
		"asin", "acos", "roiManager", "toolID", "is"};
	static final int[] numericFunctionIDs = {MacroConstants.GET_PIXEL, MacroConstants.ABS, MacroConstants.COS, MacroConstants.EXP, MacroConstants.FLOOR, MacroConstants.LOG, MacroConstants.MAX_OF, MacroConstants.MIN_OF, MacroConstants.POW,
		MacroConstants.ROUND, MacroConstants.SIN, MacroConstants.SQRT, MacroConstants.TAN, MacroConstants.GET_TIME, MacroConstants.GET_WIDTH, MacroConstants.GET_HEIGHT, MacroConstants.RANDOM,
		MacroConstants.GET_RESULT, MacroConstants.GET_COUNT, MacroConstants.GET_NUMBER, MacroConstants.NIMAGES, MacroConstants.NSLICES,
		MacroConstants.LENGTH_OF, MacroConstants.NRESULTS, MacroConstants.GET_ID, MacroConstants.BIT_DEPTH, MacroConstants.SELECTION_TYPE, MacroConstants.ATAN, MacroConstants.IS_OPEN, 
		MacroConstants.IS_ACTIVE, MacroConstants.INDEX_OF, MacroConstants.LAST_INDEX_OF, MacroConstants.CHAR_CODE_AT, MacroConstants.GET_BOOLEAN,
		MacroConstants.STARTS_WITH, MacroConstants.ENDS_WITH, MacroConstants.ATAN2, MacroConstants.IS_NAN, MacroConstants.GET_ZOOM, MacroConstants.PARSE_INT, MacroConstants.PARSE_FLOAT,
		MacroConstants.IS_KEY_DOWN, MacroConstants.GET_SLICE_NUMBER, MacroConstants.SCREEN_WIDTH, MacroConstants.SCREEN_HEIGHT, MacroConstants.CALIBRATE,
		MacroConstants.ASIN, MacroConstants.ACOS, MacroConstants.ROI_MANAGER, MacroConstants.TOOL_ID, MacroConstants.IS};

	// String functions
	static final int D2S=2000, TO_HEX=2001, TO_BINARY=2002, GET_TITLE=2003, GET_STRING=2004, SUBSTRING=2005,
		FROM_CHAR_CODE=2006, GET_INFO=2007, GET_DIRECTORY=2008, GET_ARGUMENT=2009, GET_IMAGE_INFO=2010,
		TO_LOWER_CASE=2011, TO_UPPER_CASE=2012, RUN_MACRO=2013, EVAL=2014, TO_STRING=2015, REPLACE=2016,
		DIALOG=2017, GET_METADATA=2018, FILE=2019, SELECTION_NAME=2020, GET_VERSION=2021, GET_RESULT_LABEL=2022,
		CALL=2023, STRING=2024, EXT=2025, EXEC=2026;
	static final String[] stringFunctions = {"d2s", "toHex", "toBinary", "getTitle", "getString", "substring",
		"fromCharCode", "getInfo", "getDirectory", "getArgument", "getImageInfo", 
		"toLowerCase", "toUpperCase", "runMacro", "eval", "toString", "replace",
		"Dialog", "getMetadata", "File", "selectionName", "getVersion", "getResultLabel",
		"call", "String", "Ext", "exec"};
	static final int[] stringFunctionIDs = {MacroConstants.D2S, MacroConstants.TO_HEX, MacroConstants.TO_BINARY, MacroConstants.GET_TITLE, MacroConstants.GET_STRING, MacroConstants.SUBSTRING,
		MacroConstants.FROM_CHAR_CODE, MacroConstants.GET_INFO, MacroConstants.GET_DIRECTORY, MacroConstants.GET_ARGUMENT, MacroConstants.GET_IMAGE_INFO,
		MacroConstants.TO_LOWER_CASE, MacroConstants.TO_UPPER_CASE, MacroConstants.RUN_MACRO, MacroConstants.EVAL, MacroConstants.TO_STRING, MacroConstants.REPLACE,
		MacroConstants.DIALOG, MacroConstants.GET_METADATA, MacroConstants.FILE, MacroConstants.SELECTION_NAME, MacroConstants.GET_VERSION, MacroConstants.GET_RESULT_LABEL,
		MacroConstants.CALL, MacroConstants.STRING, MacroConstants.EXT, MacroConstants.EXEC};

	// Array functions
	static final int GET_PROFILE=3000, NEW_ARRAY=3001, SPLIT=3002, GET_FILE_LIST=3003,
		GET_FONT_LIST=3004, NEW_MENU=3005, GET_LIST=3006;
	static final String[] arrayFunctions = {"getProfile", "newArray", "split", "getFileList",
		"getFontList", "newMenu", "getList"};
	static final int[] arrayFunctionIDs = {MacroConstants.GET_PROFILE, MacroConstants.NEW_ARRAY, MacroConstants.SPLIT, MacroConstants.GET_FILE_LIST,
		MacroConstants.GET_FONT_LIST, MacroConstants.NEW_MENU, MacroConstants.GET_LIST};

}  // interface MacroConstants
