package net.sf.ext.ij.plugin;
import java.awt.Checkbox;
import java.awt.TextField;
import java.awt.event.KeyEvent;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;
import java.util.Vector;

import net.sf.ext.ij.IJ;
import net.sf.ext.ij.ImagePlus;
import net.sf.ext.ij.Macro;
import net.sf.ext.ij.gui.GenericDialog;
import net.sf.ext.ij.gui.HistogramWindow;
import net.sf.ext.ij.gui.ImageWindow;
import net.sf.ext.ij.gui.YesNoCancelDialog;
import net.sf.ext.ij.measure.Calibration;
import net.sf.ext.ij.plugin.filter.PlugInFilter;
import net.sf.ext.ij.plugin.frame.Recorder;
import net.sf.ext.ij.process.ImageProcessor;
import net.sf.ext.ij.process.ImageStatistics;
import net.sf.ext.ij.process.StackStatistics;
import net.sf.ext.ij.util.Tools;


/** This plugin implements the Analyze/Histogram command. */
public class Histogram implements PlugIn, TextListener {

	private static int nBins = 256;
	private static boolean useImageMinAndMax = true;
	private static double xMin, xMax;
	private static String yMax = "Auto";
	private static boolean stackHistogram;
	private static int imageID;	
	private Checkbox checkbox;
	private TextField minField, maxField;
	private String defaultMin, defaultMax;

 	public void run(String arg) {
 		ImagePlus imp = IJ.getImage();
 		int bitDepth = imp.getBitDepth();
 		if (bitDepth==32 || IJ.altKeyDown()) {
			IJ.setKeyUp(KeyEvent.VK_ALT);
 			if (!showDialog(imp))
 				return;
 		} else {
 			int flags = setupDialog(imp, 0);
 			if (flags==PlugInFilter.DONE) return;
			Histogram.stackHistogram = flags==PlugInFilter.DOES_STACKS;
			Calibration cal = imp.getCalibration();
 			Histogram.nBins = 256;
			if (Histogram.stackHistogram && ((bitDepth==8&&!cal.calibrated())||bitDepth==24)) {
				Histogram.xMin = 0.0;
				Histogram.xMax = 256.0;
				Histogram.useImageMinAndMax = false;
			} else
				Histogram.useImageMinAndMax = true;
 			Histogram.yMax = "Auto";
 		}
 		ImageStatistics stats = null;
 		if (Histogram.useImageMinAndMax)
 			{Histogram.xMin=0.0; Histogram.xMax=0.0;}
 		int iyMax = (int)Tools.parseDouble(Histogram.yMax, 0.0);
 		boolean customHistogram = (bitDepth==8||bitDepth==24) && (!(Histogram.xMin==0.0&&Histogram.xMax==0.0)||Histogram.nBins!=256||iyMax>0);
 		ImageWindow.centerNextImage();
 		if (Histogram.stackHistogram || customHistogram) {
 			ImagePlus imp2 = imp;
 			if (customHistogram && !Histogram.stackHistogram && imp.getStackSize()>1)
 				imp2 = new ImagePlus("Temp", imp.getProcessor());
			stats = new StackStatistics(imp2, Histogram.nBins, Histogram.xMin, Histogram.xMax);
			stats.histYMax = iyMax;
			new HistogramWindow("Histogram of "+imp.getShortTitle(), imp, stats);
		} else
			new HistogramWindow("Histogram of "+imp.getShortTitle(), imp, Histogram.nBins, Histogram.xMin, Histogram.xMax, iyMax);
	}
	
	boolean showDialog(ImagePlus imp) {
		ImageProcessor ip = imp.getProcessor();
		double min = ip.getMin();
		double max = ip.getMax();
		if (imp.getID()!=Histogram.imageID || (min==Histogram.xMin&&min==Histogram.xMax))
			Histogram.useImageMinAndMax = true;
		if (imp.getID()!=Histogram.imageID || Histogram.useImageMinAndMax) {
			Histogram.xMin = min;
			Histogram.xMax = max;
			Calibration cal = imp.getCalibration();
			Histogram.xMin = cal.getCValue(Histogram.xMin);
			Histogram.xMax = cal.getCValue(Histogram.xMax);
		}
		defaultMin = IJ.d2s(Histogram.xMin,2);
		defaultMax = IJ.d2s(Histogram.xMax,2);
		Histogram.imageID = imp.getID();
		int stackSize = imp.getStackSize();
		GenericDialog gd = new GenericDialog("Histogram");
		gd.addNumericField("Bins:", HistogramWindow.nBins, 0);
		gd.addCheckbox("Use min/max or:", Histogram.useImageMinAndMax);
		//gd.addMessage("          or");
		gd.addMessage("");
		int fwidth = 6;
		int nwidth = Math.max(IJ.d2s(Histogram.xMin,2).length(), IJ.d2s(Histogram.xMax,2).length());
		if (nwidth>fwidth) fwidth = nwidth;
		gd.addNumericField("X_Min:", Histogram.xMin, 2, fwidth, null);
		gd.addNumericField("X_Max:", Histogram.xMax, 2, fwidth, null);
		gd.addMessage(" ");
		gd.addStringField("Y_Max:", Histogram.yMax, 6);
		if (stackSize>1)
			gd.addCheckbox("Stack Histogram", Histogram.stackHistogram);
		Vector numbers = gd.getNumericFields();
		minField = (TextField)numbers.elementAt(1);
		minField.addTextListener(this);
		maxField = (TextField)numbers.elementAt(2);
		maxField.addTextListener(this);
		checkbox = (Checkbox)(gd.getCheckboxes().elementAt(0));
		gd.showDialog();
		if (gd.wasCanceled())
			return false;			
		Histogram.nBins = (int)gd.getNextNumber();
		if (Histogram.nBins>=2 && Histogram.nBins<=1000)
			HistogramWindow.nBins = Histogram.nBins;
		Histogram.useImageMinAndMax = gd.getNextBoolean();
		Histogram.xMin = gd.getNextNumber();
		Histogram.xMax = gd.getNextNumber();
		Histogram.yMax = gd.getNextString();
		Histogram.stackHistogram = (stackSize>1)?gd.getNextBoolean():false;
		IJ.register(Histogram.class);
		return true;
	}

	public void textValueChanged(TextEvent e) {
		boolean rangeChanged = !defaultMin.equals(minField.getText())
			|| !defaultMax.equals(maxField.getText());
		if (rangeChanged)
			checkbox.setState(false);
	}
	
	int setupDialog(ImagePlus imp, int flags) {
		int stackSize = imp.getStackSize();
		if (stackSize>1) {
			String macroOptions = Macro.getOptions();
			if (macroOptions!=null) {
				if (macroOptions.indexOf("stack ")>=0)
					return flags+PlugInFilter.DOES_STACKS;
				else
					return flags;
			}
			YesNoCancelDialog d = new YesNoCancelDialog(IJ.getInstance(),
				"Histogram", "Include all "+stackSize+" slices?");
			if (d.cancelPressed())
				return PlugInFilter.DONE;
			else if (d.yesPressed()) {
				if (Recorder.record)
					Recorder.recordOption("stack");
				return flags+PlugInFilter.DOES_STACKS;
			}
			if (Recorder.record)
				Recorder.recordOption("slice");
		}
		return flags;
	}

}