package net.sf.ext.ij.plugin;
import java.awt.Color;

import net.sf.ext.ij.IJ;
import net.sf.ext.ij.ImagePlus;
import net.sf.ext.ij.ImageStack;
import net.sf.ext.ij.WindowManager;
import net.sf.ext.ij.gui.GenericDialog;
import net.sf.ext.ij.gui.Toolbar;
import net.sf.ext.ij.measure.Calibration;
import net.sf.ext.ij.process.ByteProcessor;
import net.sf.ext.ij.process.ColorProcessor;
import net.sf.ext.ij.process.ImageProcessor;
import net.sf.ext.ij.process.ImageStatistics;

/** Implements the Image/Stacks/Make Montage command. */
public class MontageMaker implements PlugIn {
			
	private static int columns, rows, first, last, inc, borderWidth;
	private static double scale;
	private static boolean label;
	private static boolean useForegroundColor;
	private static int saveID;

	public void run(String arg) {
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp==null || imp.getStackSize()==1)
			{IJ.error("Stack required"); return;}
		makeMontage(imp);
		MontageMaker.saveID = imp.getID();
    	IJ.register(MontageMaker.class);
	}
	
	public void makeMontage(ImagePlus imp) {
			int nSlices = imp.getStackSize();
			if (MontageMaker.columns==0 || imp.getID()!=MontageMaker.saveID) {
				MontageMaker.columns = (int)Math.sqrt(nSlices);
				MontageMaker.rows = MontageMaker.columns;
				int n = nSlices - MontageMaker.columns*MontageMaker.rows;
				if (n>0) MontageMaker.columns += (int)Math.ceil((double)n/MontageMaker.rows);
				MontageMaker.scale = 1.0;
				if (imp.getWidth()*MontageMaker.columns>800)
					MontageMaker.scale = 0.5;
				if (imp.getWidth()*MontageMaker.columns>1600)
					MontageMaker.scale = 0.25;
				MontageMaker.inc = 1;
				MontageMaker.first = 1;
				MontageMaker.last = nSlices;
			}
			
			GenericDialog gd = new GenericDialog("Make Montage", IJ.getInstance());
			gd.addNumericField("Columns:", MontageMaker.columns, 0);
			gd.addNumericField("Rows:", MontageMaker.rows, 0);
			gd.addNumericField("Scale Factor:", MontageMaker.scale, 2);
			gd.addNumericField("First Slice:", MontageMaker.first, 0);
			gd.addNumericField("Last Slice:", MontageMaker.last, 0);
			gd.addNumericField("Increment:", MontageMaker.inc, 0);
			gd.addNumericField("Border Width:", MontageMaker.borderWidth, 0);
			gd.addCheckbox("Label Slices", MontageMaker.label);
			gd.addCheckbox("Use Foreground Color", MontageMaker.useForegroundColor);
			gd.showDialog();
			if (gd.wasCanceled())
				return;
			MontageMaker.columns = (int)gd.getNextNumber();
			MontageMaker.rows = (int)gd.getNextNumber();
			MontageMaker.scale = gd.getNextNumber();
			MontageMaker.first = (int)gd.getNextNumber();
			MontageMaker.last = (int)gd.getNextNumber();
			MontageMaker.inc = (int)gd.getNextNumber();
			MontageMaker.borderWidth = (int)gd.getNextNumber();
			if (MontageMaker.borderWidth<0) MontageMaker.borderWidth = 0;
			if (MontageMaker.first<1) MontageMaker.first = 1;
			if (MontageMaker.last>nSlices) MontageMaker.last = nSlices;
			if (MontageMaker.first>MontageMaker.last)
				{MontageMaker.first=1; MontageMaker.last=nSlices;}
			if (MontageMaker.inc<1) MontageMaker.inc = 1;
			if (gd.invalidNumber()) {
				IJ.error("Invalid number");
				return;
			}
			MontageMaker.label = gd.getNextBoolean();
			MontageMaker.useForegroundColor = gd.getNextBoolean();
			makeMontage(imp, MontageMaker.columns, MontageMaker.rows, MontageMaker.scale, MontageMaker.first, MontageMaker.last, MontageMaker.inc, MontageMaker.borderWidth, MontageMaker.label);
	}
	
	public void makeMontage(ImagePlus imp, int columns, int rows, double scale, int first, int last, int inc, int borderWidth, boolean labels) {
		int stackWidth = imp.getWidth();
		int stackHeight = imp.getHeight();
		int nSlices = imp.getStackSize();
		int width = (int)(stackWidth*scale);
		int height = (int)(stackHeight*scale);
		int montageWidth = width*columns;
		int montageHeight = height*rows;
		ImageProcessor ip = imp.getProcessor();
		ImageProcessor montage = ip.createProcessor(montageWidth+borderWidth/2, montageHeight+borderWidth/2);
		Color fgColor=Color.white;
		Color bgColor = Color.black;
		if (MontageMaker.useForegroundColor) {
			fgColor = Toolbar.getForegroundColor();
			bgColor = Toolbar.getBackgroundColor();
		} else {
			boolean whiteBackground = false;
			if ((ip instanceof ByteProcessor) || (ip instanceof ColorProcessor)) {
				ImageStatistics is = imp.getStatistics();
				whiteBackground = is.mode>=200;
				if (imp.isInvertedLut())
					whiteBackground = !whiteBackground;
			}
			if (whiteBackground) {
				fgColor=Color.black;
				bgColor = Color.white;
			}
		}
		montage.setColor(bgColor);
		montage.fill();
		montage.setColor(fgColor);
		ImageStack stack = imp.getStack();
		int x = 0;
		int y = 0;
		ImageProcessor aSlice;
	    int slice = first;
		while (slice<=last) {
			aSlice = stack.getProcessor(slice);
			if (scale!=1.0)
				aSlice = aSlice.resize(width, height);
			montage.insert(aSlice, x, y);
			String label = stack.getShortSliceLabel(slice);
			if (borderWidth>0) drawBorder(montage, x, y, width, height, borderWidth);
			if (labels) drawLabel(montage, slice, label, x, y, width, height);
			x += width;
			if (x>=montageWidth) {
				x = 0;
				y += height;
				if (y>=montageHeight)
					break;
			}
			IJ.showProgress((double)(slice-first)/(last-first));
			slice += inc;
		}
		if (borderWidth>0) {
			int w2 = borderWidth/2;
			drawBorder(montage, w2, w2, montageWidth-w2, montageHeight-w2, borderWidth);
		}
		IJ.showProgress(1.0);
		ImagePlus imp2 = new ImagePlus("Montage", montage);
		imp2.setCalibration(imp.getCalibration());
		Calibration cal = imp2.getCalibration();
		if (cal.scaled()) {
			cal.pixelWidth /= scale;
			cal.pixelHeight /= scale;
		}
        imp2.setProperty("Info", "xMontage="+columns+"\nyMontage="+rows+"\n");
		imp2.show();
	}
		
	void drawBorder(ImageProcessor montage, int x, int y, int width, int height, int borderWidth) {
		montage.setLineWidth(borderWidth);
		montage.moveTo(x, y);
		montage.lineTo(x+width, y);
		montage.lineTo(x+width, y+height);
		montage.lineTo(x, y+height);
		montage.lineTo(x, y);
	}
	
	void drawLabel(ImageProcessor montage, int slice, String label, int x, int y, int width, int height) {
		if (label!=null && !label.equals("") && montage.getStringWidth(label)>=width) {
			do {
				label = label.substring(0, label.length()-1);
			} while (label.length()>1 && montage.getStringWidth(label)>=width);
		}
		if (label==null || label.equals(""))
			label = ""+slice;
		int swidth = montage.getStringWidth(label);
		x += width/2 - swidth/2;
		y += height;
		montage.moveTo(x, y); 
		montage.drawString(label);
	}
}


