package net.sf.ext.ij.plugin;
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.TextField;
import java.awt.event.ItemEvent;
import java.awt.event.TextEvent;

import net.sf.ext.ij.IJ;
import net.sf.ext.ij.ImagePlus;
import net.sf.ext.ij.Undo;
import net.sf.ext.ij.WindowManager;
import net.sf.ext.ij.gui.GenericDialog;
import net.sf.ext.ij.gui.ImageWindow;
import net.sf.ext.ij.gui.Roi;
import net.sf.ext.ij.measure.Calibration;
import net.sf.ext.ij.process.ImageProcessor;

/**	This plugin implements the Analyze/Tools/Draw Scale Bar command.
	Divakar Ramachandran added options to draw a background 
	and use a serif font on 23 April 2006.
*/
public class ScaleBar implements PlugIn {

    static final String[] locations = {"Upper Right", "Lower Right", "Lower Left", "Upper Left", "At Selection"};
    static final int UPPER_RIGHT=0, LOWER_RIGHT=1, LOWER_LEFT=2, UPPER_LEFT=3, AT_SELECTION=4;
    static final String[] colors = {"White","Black","Light Gray","Gray","Dark Gray","Red","Green","Blue","Yellow"};
    static final String[] bcolors = {"None","Black","White","Dark Gray","Gray","Light Gray","Yellow","Blue","Green","Red"};
    static final String[] checkboxLabels = {"Bold Text", "Hide Text", "Serif Font"};
    static double barWidth;
    static int defaultBarHeight = 4;
    static int barHeightInPixels = ScaleBar.defaultBarHeight;
    static String location = ScaleBar.locations[ScaleBar.LOWER_RIGHT];
    static String color = ScaleBar.colors[0];
    static String bcolor = ScaleBar.bcolors[0];
    static boolean boldText = true;
    static boolean hideText;
    static int defaultFontSize = 14;
    static int fontSize;
    static boolean labelAll;
    ImagePlus imp;
    double imageWidth;
    double mag;
    int xloc, yloc;
    int barWidthInPixels;
    int roiX=-1, roiY, roiWidth, roiHeight;
    boolean serifFont;
    boolean[] checkboxStates = new boolean[3];


    public void run(String arg) {
        imp = WindowManager.getCurrentImage();
        if (imp!=null) {
            if (showDialog(imp) && imp.getStackSize()>1 && ScaleBar.labelAll)
                labelSlices(imp);
        } else
            IJ.noImage();
     }

    void labelSlices(ImagePlus imp) {
        int slice = imp.getCurrentSlice();
        for (int i=1; i<=imp.getStackSize(); i++) {
            imp.setSlice(i);
            drawScaleBar(imp);
        }
        imp.setSlice(slice);        
    }

    boolean showDialog(ImagePlus imp) {
        Roi roi = imp.getRoi();
        if (roi!=null) {
            Rectangle r = roi.getBounds();
            roiX = r.x;
            roiY = r.y; 
            roiWidth = r.width;
            roiHeight = r.height;
            ScaleBar.location = ScaleBar.locations[ScaleBar.AT_SELECTION];
        } else if (ScaleBar.location.equals(ScaleBar.locations[ScaleBar.AT_SELECTION]))
            ScaleBar.location = ScaleBar.locations[ScaleBar.UPPER_RIGHT];

        Calibration cal = imp.getCalibration();
        ImageWindow win = imp.getWindow();
        mag = (win!=null)?win.getCanvas().getMagnification():1.0;
        if (mag>1.0)
            mag = 1.0;
        if (ScaleBar.fontSize<(ScaleBar.defaultFontSize/mag))
            ScaleBar.fontSize = (int)(ScaleBar.defaultFontSize/mag);
        String units = cal.getUnits();
        // Handle Digital Micrograph unit microns
        if (units.equals("micron"))
            units = IJ.micronSymbol+"m";
        double pixelWidth = cal.pixelWidth;
        if (pixelWidth==0.0)
            pixelWidth = 1.0;
        imageWidth = imp.getWidth()*pixelWidth;
        if (roiX>0 && roiWidth>10)
            ScaleBar.barWidth = roiWidth*pixelWidth;
        else if (ScaleBar.barWidth==0.0 || ScaleBar.barWidth>0.67*imageWidth) {
            ScaleBar.barWidth = (80.0*pixelWidth)/mag;
            if (ScaleBar.barWidth>0.67*imageWidth)
                ScaleBar.barWidth = 0.67*imageWidth;
            if (ScaleBar.barWidth>5.0)
                ScaleBar.barWidth = (int)ScaleBar.barWidth;
        }
        int stackSize = imp.getStackSize();
        int digits = (int)ScaleBar.barWidth==ScaleBar.barWidth?0:1;
        if (ScaleBar.barWidth<1.0)
            digits = 2;
        if (mag<1.0 && ScaleBar.barHeightInPixels<ScaleBar.defaultBarHeight/mag)
            ScaleBar.barHeightInPixels = (int)(ScaleBar.defaultBarHeight/mag);
        imp.getProcessor().snapshot();
        if (!IJ.macroRunning()) updateScalebar();
        GenericDialog gd = new BarDialog("ScaleBar Plus");
        gd.addNumericField("Width in "+units+": ", ScaleBar.barWidth, digits);
        gd.addNumericField("Height in pixels: ", ScaleBar.barHeightInPixels, 0);
        gd.addNumericField("Font Size: ", ScaleBar.fontSize, 0);
        gd.addChoice("Color: ", ScaleBar.colors, ScaleBar.color);
        gd.addChoice("Background: ", ScaleBar.bcolors, ScaleBar.bcolor);
        gd.addChoice("Location: ", ScaleBar.locations, ScaleBar.location);
        checkboxStates[0] = ScaleBar.boldText; checkboxStates[1] = ScaleBar.hideText; checkboxStates[2] = serifFont;
        gd.addCheckboxGroup(2, 2, ScaleBar.checkboxLabels, checkboxStates);
        if (stackSize>1)
            gd.addCheckbox("Label all Slices", ScaleBar.labelAll);
        gd.showDialog();
        if (gd.wasCanceled()) {
            imp.getProcessor().reset();
            imp.updateAndDraw();
            return false;
        }
        ScaleBar.barWidth = gd.getNextNumber();
        ScaleBar.barHeightInPixels = (int)gd.getNextNumber();
        ScaleBar.fontSize = (int)gd.getNextNumber();
        ScaleBar.color = gd.getNextChoice();
        ScaleBar.bcolor = gd.getNextChoice();
        ScaleBar.location = gd.getNextChoice();
        ScaleBar.boldText = gd.getNextBoolean();
        ScaleBar.hideText = gd.getNextBoolean();
        serifFont = gd.getNextBoolean();
        if (stackSize>1)
            ScaleBar.labelAll = gd.getNextBoolean();
        if (IJ.macroRunning()) updateScalebar();
         return true;
    }

    void drawScaleBar(ImagePlus imp) {
          if (!updateLocation())
            return;
        ImageProcessor ip = imp.getProcessor();
        Undo.setup(Undo.FILTER, imp);
        Color color = getColor();
        Color bcolor = getBColor();
        //if (!(color==Color.white || color==Color.black)) {
        //    ip = ip.convertToRGB();
        //    imp.setProcessor(null, ip);
        //}

        int x = xloc;
        int y = yloc;
        int fontType = ScaleBar.boldText?Font.BOLD:Font.PLAIN;
        String font = serifFont?"Serif":"SanSerif";
        ip.setFont(new Font(font, fontType, ScaleBar.fontSize));
        ip.setAntialiasedText(true);
        int digits = (int)ScaleBar.barWidth==ScaleBar.barWidth?0:1;
        if (ScaleBar.barWidth<1.0)
            digits = 1;
        // Handle Digital Micrograph unit microns
        String units = imp.getCalibration().getUnits();
        if (units.equals("microns"))
            units = IJ.micronSymbol+"m";
        String label = IJ.d2s(ScaleBar.barWidth, digits) + " "+ units;
        int swidth = ScaleBar.hideText?0:ip.getStringWidth(label);
        int xoffset = (barWidthInPixels - swidth)/2;
        int yoffset =  ScaleBar.barHeightInPixels + (ScaleBar.hideText?0:ScaleBar.fontSize+ScaleBar.fontSize/(serifFont?8:4));

        // Draw bkgnd box first,  based on bar width and height (and font size if hideText is not checked)
        if (bcolor!=null) {
            int w = barWidthInPixels;
            int h = yoffset;
            if (w<swidth) w = swidth;
            int x2 = x;
            if (x+xoffset<x2) x2 = x + xoffset;
            int margin = w/20;
            if (margin<2) margin = 2;
            x2 -= margin;
            int y2 = y - margin;
            w = w+ margin*2;
            h = h+ margin*2;
            ip.setColor(bcolor);
            ip.setRoi(x2, y2, w, h);
            ip.fill();
        }
        
        ip.resetRoi();
        ip.setColor(color);
        ip.setRoi(x, y, barWidthInPixels, ScaleBar.barHeightInPixels);
        ip.fill();
        ip.resetRoi();
        if (!ScaleBar.hideText)
            ip.drawString(label, x+xoffset, y+yoffset);   
        imp.updateAndDraw();
    }

     boolean updateLocation() {
        Calibration cal = imp.getCalibration();
        barWidthInPixels = (int)(ScaleBar.barWidth/cal.pixelWidth);
        int width = imp.getWidth();
        int height = imp.getHeight();
        int fraction = 20;
        int x = width - width/fraction - barWidthInPixels;
        int y = 0;
        if (ScaleBar.location.equals(ScaleBar.locations[ScaleBar.UPPER_RIGHT]))
             y = height/fraction;
        else if (ScaleBar.location.equals(ScaleBar.locations[ScaleBar.LOWER_RIGHT]))
            y = height - height/fraction - ScaleBar.barHeightInPixels - ScaleBar.fontSize;
        else if (ScaleBar.location.equals(ScaleBar.locations[ScaleBar.UPPER_LEFT])) {
            x = width/fraction;
            y = height/fraction;
        } else if (ScaleBar.location.equals(ScaleBar.locations[ScaleBar.LOWER_LEFT])) {
            x = width/fraction;
            y = height - height/fraction - ScaleBar.barHeightInPixels - ScaleBar.fontSize;
        } else {
            if (roiX==-1)
                 return false;
            x = roiX;
            y = roiY;
        }
        xloc = x;
        yloc = y;
        return true;
    }

    Color getColor() {
        Color c = Color.black;
        if (ScaleBar.color.equals(ScaleBar.colors[0])) c = Color.white;
        else if (ScaleBar.color.equals(ScaleBar.colors[2])) c = Color.lightGray;
        else if (ScaleBar.color.equals(ScaleBar.colors[3])) c = Color.gray;
        else if (ScaleBar.color.equals(ScaleBar.colors[4])) c = Color.darkGray;
        else if (ScaleBar.color.equals(ScaleBar.colors[5])) c = Color.red;
        else if (ScaleBar.color.equals(ScaleBar.colors[6])) c = Color.green;
        else if (ScaleBar.color.equals(ScaleBar.colors[7])) c = Color.blue;
        else if (ScaleBar.color.equals(ScaleBar.colors[8])) c = Color.yellow;
       return c;
    }

    // Div., mimic getColor to write getBColor for bkgnd    
    Color getBColor() {
        if (ScaleBar.bcolor==null || ScaleBar.bcolor.equals(ScaleBar.bcolors[0])) return null;
        Color bc = Color.white;
        if (ScaleBar.bcolor.equals(ScaleBar.bcolors[1])) bc = Color.black;
        else if (ScaleBar.bcolor.equals(ScaleBar.bcolors[3])) bc = Color.darkGray;
        else if (ScaleBar.bcolor.equals(ScaleBar.bcolors[4])) bc = Color.gray;
        else if (ScaleBar.bcolor.equals(ScaleBar.bcolors[5])) bc = Color.lightGray;
        else if (ScaleBar.bcolor.equals(ScaleBar.bcolors[6])) bc = Color.yellow;
        else if (ScaleBar.bcolor.equals(ScaleBar.bcolors[7])) bc = Color.blue;
        else if (ScaleBar.bcolor.equals(ScaleBar.bcolors[8])) bc = Color.green;
        else if (ScaleBar.bcolor.equals(ScaleBar.bcolors[9])) bc = Color.red;
        return bc;
    }


    void updateScalebar() {
        updateLocation();
        imp.getProcessor().reset();
        drawScaleBar(imp);
    }

   class BarDialog extends GenericDialog {

        BarDialog(String title) {
            super(title);
        }

        public void textValueChanged(TextEvent e) {
            TextField widthField = ((TextField)numberField.elementAt(0));
            Double d = getValue(widthField.getText());
            if (d==null)
                return;
            ScaleBar.barWidth = d.doubleValue();
            TextField heightField = ((TextField)numberField.elementAt(1));
            d = getValue(heightField.getText());
            if (d==null)
                return;
            ScaleBar.barHeightInPixels = (int)d.doubleValue();
            TextField fontSizeField = ((TextField)numberField.elementAt(2));
            d = getValue(fontSizeField.getText());
            if (d==null)
                return;
            int size = (int)d.doubleValue();
            if (size>5)
                ScaleBar.fontSize = size;
            updateScalebar();
        }

        public void itemStateChanged(ItemEvent e) {
            Choice col = (Choice)(choice.elementAt(0));
            ScaleBar.color = col.getSelectedItem();
            Choice bcol = (Choice)(choice.elementAt(1));
            ScaleBar.bcolor = bcol.getSelectedItem();
            Choice loc = (Choice)(choice.elementAt(2));
            ScaleBar.location = loc.getSelectedItem();
            ScaleBar.boldText = ((Checkbox)(checkbox.elementAt(0))).getState();
            ScaleBar.hideText = ((Checkbox)(checkbox.elementAt(1))).getState();
            serifFont = ((Checkbox)(checkbox.elementAt(2))).getState();
            updateScalebar();
        }

   } //BarDialog inner class

} //ScaleBar class
