package net.sf.ext.ij.plugin;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;

import net.sf.ext.ij.CompositeImage;
import net.sf.ext.ij.IJ;
import net.sf.ext.ij.ImagePlus;
import net.sf.ext.ij.Prefs;
import net.sf.ext.ij.WindowManager;
import net.sf.ext.ij.gui.GenericDialog;
import net.sf.ext.ij.io.FileInfo;
import net.sf.ext.ij.io.Opener;
import net.sf.ext.ij.plugin.frame.Editor;

/** Opens TIFFs, ZIP compressed TIFFs, DICOMs, GIFs and JPEGs using a URL. 
	TIFF file names must end in ".tif", ZIP file names must end 
	in ".zip" and DICOM file names must end in ".dcm". 
	Opens a Web page in the default browser if the URL ends with "/".
*/
public class URLOpener implements PlugIn {

	private static String url = "http://rsb.info.nih.gov/ij/images/clown.gif";

	/** If 'urlOrName' is a URL, opens the image at that URL. If it is
		a file name, opens the image with that name from the 'images.location'
		URL in IJ_Props.txt. If it is blank, prompts for an image
		URL and open the specified image. */
	public void run(String urlOrName) {
		if (!urlOrName.equals("")) {
			if (urlOrName.endsWith("StartupMacros.txt"))
				openTextFile(urlOrName, true);
			else {
				String url = urlOrName.indexOf("://")>0?urlOrName:Prefs.getImagesURL()+urlOrName;
				ImagePlus imp = new ImagePlus(url);
				if (imp.getType()==ImagePlus.COLOR_RGB)
					Opener.convertGrayJpegTo8Bits(imp);
				WindowManager.checkForDuplicateName = true;
				FileInfo fi = imp.getOriginalFileInfo();
				if (fi!=null && fi.fileType==FileInfo.RGB48)
					imp = new CompositeImage(imp, 3);
				imp.show();
			}
			return;
		}
		
		GenericDialog gd = new GenericDialog("Enter a URL");
		gd.addMessage("Enter URL of a TIFF, JPEG, GIF, PNG or DICOM image");
		gd.addStringField("URL:", URLOpener.url, 40);
		gd.showDialog();
		if (gd.wasCanceled())
			return;
		URLOpener.url = gd.getNextString();
		URLOpener.url = URLOpener.url.trim();
		if (URLOpener.url.indexOf("://")==-1)
			URLOpener.url = "http://" + URLOpener.url;
		if (URLOpener.url.endsWith("/"))
			IJ.runPlugIn("net.sf.ext.ij.plugin.BrowserLauncher", URLOpener.url.substring(0, URLOpener.url.length()-1));
		else if (URLOpener.url.endsWith(".html") || URLOpener.url.endsWith(".htm") ||  URLOpener.url.indexOf(".html#")>0)
			IJ.runPlugIn("net.sf.ext.ij.plugin.BrowserLauncher", URLOpener.url);
		else if (URLOpener.url.endsWith(".txt")||URLOpener.url.endsWith(".ijm"))
			openTextFile(URLOpener.url, false);
		else {
			IJ.showStatus("Opening: " + URLOpener.url);
			ImagePlus imp = new ImagePlus(URLOpener.url);
			WindowManager.checkForDuplicateName = true;
			FileInfo fi = imp.getOriginalFileInfo();
			if (fi!=null && fi.fileType==FileInfo.RGB48)
				imp = new CompositeImage(imp, 3);
			imp.show();
			IJ.showStatus("");
		}
		IJ.register(URLOpener.class);  // keeps this class from being GC'd
	}
	
	void openTextFile(String urlString, boolean install) {
		StringBuffer sb = null;
		try {
			URL url = new URL(urlString);
			InputStream in = url.openStream();
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			sb = new StringBuffer() ;
			String line;
			while ((line=br.readLine()) != null)
				sb.append (line + "\n");
			in.close ();
		} catch (IOException e) {
			if  (!(install&&urlString.endsWith("StartupMacros.txt")))
				IJ.error("URL Opener", ""+e);
			sb = null;
		}
		if (sb!=null) {
			(new Editor()).create(urlString, new String(sb));
		}
	}
 
}
