package net.sf.ext.ij.plugin.filter;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import net.sf.ext.ij.IJ;
import net.sf.ext.ij.ImagePlus;
import net.sf.ext.ij.gui.GenericDialog;
import net.sf.ext.ij.gui.Roi;
import net.sf.ext.ij.process.ImageProcessor;

/** This plugin implements the File/Page Setup and File/Print commands. */
public class Printer implements PlugInFilter, Printable {
	private ImagePlus imp;
	private static double scaling = 100.0;
	private static boolean drawBorder;
	private static boolean center = true;
	private static boolean label;
	private static boolean printSelection;
	private static boolean rotate;
	private static int fontSize = 12;

	public int setup(String arg, ImagePlus imp) {
		if (!IJ.isJava2())
			{IJ.error("Print", "Printing requires Java 1.2 or later"); return PlugInFilter.DONE;}
		if (arg.equals("setup"))
			{pageSetup(); return PlugInFilter.DONE;}
		this.imp = imp;
		IJ.register(Printer.class);
		return PlugInFilter.DOES_ALL+PlugInFilter.NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		//pageSetup();
		print(imp);
	}
	
	void pageSetup() {
		GenericDialog gd = new GenericDialog("Page Setup");
		gd.addNumericField("Scale:", Printer.scaling, 0, 3, "%");
		gd.addCheckbox("Draw Border", Printer.drawBorder);
		gd.addCheckbox("Center on Page", Printer.center);
		gd.addCheckbox("Print Title", Printer.label);
		gd.addCheckbox("Selection Only", Printer.printSelection);
		gd.addCheckbox("Rotate 90"+IJ.degreeSymbol, Printer.rotate);
		gd.showDialog();
		if (gd.wasCanceled())
			return;
		Printer.scaling = gd.getNextNumber();
		if (Printer.scaling<5.0) Printer.scaling = 5;
		Printer.drawBorder = gd.getNextBoolean();
		Printer.center = gd.getNextBoolean();
		Printer.label = gd.getNextBoolean();
		Printer.printSelection = gd.getNextBoolean();
		Printer.rotate = gd.getNextBoolean();
	}

	void print(ImagePlus imp) {
		PrinterJob pj = PrinterJob.getPrinterJob();
		pj.setPrintable(this);
		//pj.pageDialog(pj.defaultPage());
		if (IJ.macroRunning() || pj.printDialog()) {
			imp.startTiming();
			try {pj.print(); }
			catch (PrinterException e) {
				IJ.log(""+e);
			}
		}
	}
	
	public int print(Graphics g, PageFormat pf, int pageIndex) {
		if (pageIndex != 0) return Printable.NO_SUCH_PAGE;
		ImageProcessor ip = imp.getProcessor();
		Roi roi = imp.getRoi();
		if (Printer.printSelection && roi!=null && roi.isArea() )
			ip = ip.crop();
		if (Printer.rotate)
			ip = ip.rotateLeft();
		//new ImagePlus("ip", ip.duplicate()).show();
		int width = ip.getWidth();
		int height = ip.getHeight();
		int margin = 0;
		if (Printer.drawBorder) margin = 1;
		double scale = Printer.scaling/100.0;
		int dstWidth = (int)(width*scale);
		int dstHeight = (int)(height*scale);
		int pageX = (int)pf.getImageableX();
		int pageY = (int)pf.getImageableY();
		int dstX = pageX+margin;
		int dstY = pageY+margin;
		Image img = ip.createImage();
		double pageWidth = pf.getImageableWidth()-2*margin;
		double pageHeight = pf.getImageableHeight()-2*margin;		if (Printer.label && pageWidth-dstWidth<Printer.fontSize+5) {
			dstY += Printer.fontSize+5;
			pageHeight -= Printer.fontSize+5;
		}	
		if (dstWidth>pageWidth || dstHeight>pageHeight) {
			// scale to fit page
			double hscale = pageWidth/dstWidth;
			double vscale = pageHeight/dstHeight;
			double scale2 = hscale<=vscale?hscale:vscale;
			dstWidth = (int)(dstWidth*scale2);
			dstHeight = (int)(dstHeight*scale2);
		} else if (Printer.center) {
			dstX += (pageWidth-dstWidth)/2;
			dstY += (pageHeight-dstHeight)/2;
		}
		g.drawImage(img, 
			dstX, dstY, dstX+dstWidth, dstY+dstHeight,
			0, 0, width, height, 
			null);
		if (Printer.drawBorder)
			g.drawRect(dstX-1, dstY-1, dstWidth+1, dstHeight+1);
		if (Printer.label) {
			g.setFont(new Font("SanSerif", Font.PLAIN, Printer.fontSize));
			g.setColor(Color.black);
			g.drawString(imp.getTitle(), pageX+5, pageY+Printer.fontSize);
		}
		return Printable.PAGE_EXISTS;
	}

}
