package net.sf.ext.ij.plugin.filter;
import java.awt.Checkbox;
import java.awt.Rectangle;
import java.awt.TextField;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;
import java.util.Vector;

import net.sf.ext.ij.IJ;
import net.sf.ext.ij.ImagePlus;
import net.sf.ext.ij.ImageStack;
import net.sf.ext.ij.gui.GenericDialog;
import net.sf.ext.ij.gui.Roi;
import net.sf.ext.ij.gui.ShapeRoi;
import net.sf.ext.ij.measure.Calibration;
import net.sf.ext.ij.process.ImageProcessor;
import net.sf.ext.ij.process.StackProcessor;
import net.sf.ext.ij.util.Tools;

/** This plugin implements ImageJ's Resize command. */
public class Resizer implements PlugInFilter, TextListener, ItemListener  {
	ImagePlus imp;
	private boolean crop;
    private static int newWidth;
    private static int newHeight;
    private static boolean constrain = true;
    private static boolean interpolate = true;
    private Vector fields, checkboxes;
	private double origWidth, origHeight;
	private boolean sizeToHeight;
 
	public int setup(String arg, ImagePlus imp) {
		crop = arg.equals("crop");
		this.imp = imp;
		IJ.register(Resizer.class);
		if (crop)
			return PlugInFilter.DOES_ALL+PlugInFilter.ROI_REQUIRED+PlugInFilter.NO_CHANGES;
		else
			return PlugInFilter.DOES_ALL+PlugInFilter.NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		Roi roi = imp.getRoi();
		if (roi!=null && roi.isLine()) {
			IJ.error("The Crop and Adjust->Size commands\ndo not work with line selections.");
			return;
		}
		Rectangle r = ip.getRoi();
		origWidth = r.width;;
		origHeight = r.height;
		sizeToHeight=false;
	    boolean restoreRoi = crop && roi!=null && roi.getType()!=Roi.RECTANGLE;
		if (roi!=null && IJ.isJava2()) {
			Rectangle b = roi.getBounds();
			int w = ip.getWidth();
			int h = ip.getHeight();
			if (b.x<0 || b.y<0 || b.x+b.width>w || b.y+b.height>h) {
				ShapeRoi shape1 = new ShapeRoi(roi);
				ShapeRoi shape2 = new ShapeRoi(new Roi(0, 0, w, h));
				roi = shape2.and(shape1);
				if (restoreRoi) imp.setRoi(roi);
			}
		}
		if (crop) {
			Rectangle bounds = roi.getBounds();
			Resizer.newWidth = bounds.width;
			Resizer.newHeight = bounds.height;
			Resizer.interpolate = false;
		} else {
			if (Resizer.newWidth==0 || Resizer.newHeight==0) {
				Resizer.newWidth = (int)origWidth/2;
				Resizer.newHeight = (int)origHeight/2;
			}
			if (Resizer.constrain) Resizer.newHeight = (int)(Resizer.newWidth*(origHeight/origWidth));
			GenericDialog gd = new GenericDialog("Resize", IJ.getInstance());
			gd.addNumericField("Width (pixels):", Resizer.newWidth, 0);
			gd.addNumericField("Height (pixels):", Resizer.newHeight, 0);
			gd.addCheckbox("Constrain Aspect Ratio", Resizer.constrain);
			gd.addCheckbox("Interpolate", Resizer.interpolate);
			gd.addMessage("NOTE: Undo is not available");
			fields = gd.getNumericFields();
			for (int i=0; i<fields.size(); i++)
				((TextField)fields.elementAt(i)).addTextListener(this);
			checkboxes = gd.getCheckboxes();
			((Checkbox)checkboxes.elementAt(0)).addItemListener(this);
			gd.showDialog();
			if (gd.wasCanceled())
				return;
			Resizer.newWidth = (int)gd.getNextNumber();
			Resizer.newHeight = (int)gd.getNextNumber();
			if (gd.invalidNumber()) {
				IJ.error("Width or height are invalid.");
				return;
			}
			Resizer.constrain = gd.getNextBoolean();
			Resizer.interpolate = gd.getNextBoolean();
			if (Resizer.constrain && Resizer.newWidth==0)
				sizeToHeight = true;
			if (Resizer.newWidth<=0.0 && !Resizer.constrain)  Resizer.newWidth = 50;
			if (Resizer.newHeight<=0.0) Resizer.newHeight = 50;
		}
		
		if (!crop && Resizer.constrain) {
			if (sizeToHeight)
				Resizer.newWidth = (int)(Resizer.newHeight*(origWidth/origHeight));
			else
				Resizer.newHeight = (int)(Resizer.newWidth*(origHeight/origWidth));
		}
		if (ip.getWidth()==1 || ip.getHeight()==1)
			ip.setInterpolate(false);
		else
			ip.setInterpolate(Resizer.interpolate);
    	
		int nSlices = imp.getStackSize();
		try {
	    	StackProcessor sp = new StackProcessor(imp.getStack(), ip);
	    	ImageStack s2 = sp.resize(Resizer.newWidth, Resizer.newHeight);
	    	int newSize = s2.getSize();
	    	if (s2.getWidth()>0 && newSize>0) {
	    		if (restoreRoi)
	    			imp.killRoi();
	    		//imp.hide();
	    		Calibration cal = imp.getCalibration();
	    		if (cal.scaled()) {
    				cal.pixelWidth *= origWidth/Resizer.newWidth;
    				cal.pixelHeight *= origHeight/Resizer.newHeight;
    				imp.setCalibration(cal);
    			}
	    		imp.setStack(null, s2);
	    		//imp.show();
	    		if (restoreRoi) {
	    			imp.restoreRoi();
	    			roi = imp.getRoi();
	    			if (roi!=null) {
	    				roi.setLocation(0, 0);
	    				imp.draw();
	    			}
	    		}
	    	}
	    	if (nSlices>1 && newSize<nSlices)
	    		IJ.error("ImageJ ran out of memory causing \nthe last "+(nSlices-newSize)+" slices to be lost.");
		} catch(OutOfMemoryError o) {
			IJ.outOfMemory("Resize");
		}
	}

    public void textValueChanged(TextEvent e) {
    	TextField widthField = (TextField)fields.elementAt(0);
    	TextField heightField = (TextField)fields.elementAt(1);
        int width = (int)Tools.parseDouble(widthField.getText(),-99);
        int height = (int)Tools.parseDouble(heightField.getText(),-99);
        if (width==-99 || height==-99)
        	return;
        if (Resizer.constrain) {
        	if (width!=Resizer.newWidth) {
         		sizeToHeight = false;
        		Resizer.newWidth = width;
				updateFields();
         	} else if (height!=Resizer.newHeight) {
         		sizeToHeight = true;
        		Resizer.newHeight = height;
				updateFields();
			}
        }
    }
    
    void updateFields() {
		if (sizeToHeight) {
			Resizer.newWidth = (int)(Resizer.newHeight*(origWidth/origHeight));
			TextField widthField = (TextField)fields.elementAt(0);
			widthField.setText(""+Resizer.newWidth);
		} else {
			Resizer.newHeight = (int)(Resizer.newWidth*(origHeight/origWidth));
			TextField heightField = (TextField)fields.elementAt(1);
			heightField.setText(""+Resizer.newHeight);
		}
   }

	public void itemStateChanged(ItemEvent e) {
		Checkbox cb = (Checkbox)checkboxes.elementAt(0);
        boolean newConstrain = cb.getState();
        if (newConstrain && newConstrain!=Resizer.constrain)
        	updateFields();
        Resizer.constrain = newConstrain;
	}

}