package net.sf.ext.ij.plugin.frame;
import java.awt.Button;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Polygon;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.util.Locale;

import net.sf.ext.ij.IJ;
import net.sf.ext.ij.Macro;
import net.sf.ext.ij.WindowManager;
import net.sf.ext.ij.gui.GUI;
import net.sf.ext.ij.gui.Roi;
import net.sf.ext.ij.plugin.PlugIn;
import net.sf.ext.ij.util.Tools;

/** This is ImageJ's macro recorder. */
public class Recorder extends PlugInFrame implements PlugIn, ActionListener {

	/** This variable is true if the recorder is running. */
	public static boolean record;
	
	/** Set this variable true to allow recording within IJ.run() calls. */
	public static boolean recordInMacros;

	private Button makeMacro, help;
	private TextField macroName;
	private static TextArea textArea;
	private static Frame instance;
	private static String commandName;
	private static String commandOptions;
	private static String defaultName = "Macro";
	private static boolean recordPath = true;

	public Recorder() {
		super("Recorder");
		if (Recorder.instance!=null) {
			Recorder.instance.toFront();
			return;
		}
		WindowManager.addWindow(this);
		Recorder.instance = this;
		Recorder.record = true;
		Panel panel = new Panel(new FlowLayout(FlowLayout.CENTER, 2, 0));
		panel.add(new Label("Name:"));
		macroName = new TextField(Recorder.defaultName,15);
		panel.add(macroName);
		panel.add(new Label("     "));
		makeMacro = new Button("Create");
		makeMacro.addActionListener(this);
		panel.add(makeMacro);
		panel.add(new Label("     "));
		help = new Button("?");
		help.addActionListener(this);
		panel.add(help);
		add("North", panel);
		Recorder.textArea = new TextArea("",15,60,TextArea.SCROLLBARS_VERTICAL_ONLY);
		Recorder.textArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
		if (IJ.isLinux()) Recorder.textArea.setBackground(Color.white);
		add("Center", Recorder.textArea);
		pack();
		GUI.center(this);
		show();
		IJ.register(Recorder.class);
	}

	public static void record(String method) {
		if (Recorder.textArea==null)
			return;
		Recorder.textArea.append(method+"();\n");
	}

	/** Starts recording a command. Does nothing if the recorder is
		not open or the command being recorded has called IJ.run(). 
	*/
	public static void setCommand(String command) {
		if (Recorder.textArea==null || (Thread.currentThread().getName().startsWith("Run$_")&&!Recorder.recordInMacros))
			return;
		Recorder.commandName = command;
		Recorder.commandOptions = null;
		Recorder.recordPath = true;
		//IJ.log("setCommand: "+command+" "+Thread.currentThread().getName());
	}

	/** Returns the name of the command currently being recorded, or null. */
	public static String getCommand() {
		return Recorder.commandName;
	}

	static String fixPath (String path) {
		StringBuffer sb = new StringBuffer();
		char c;
		for (int i=0; i<path.length(); i++) {
			sb.append(c=path.charAt(i));
			if (c=='\\')
				sb.append("\\");
		}
		return new String(sb);
	}

	public static void record(String method, String arg) {
		if (Recorder.textArea==null) return;
		Recorder.textArea.append(method+"(\""+arg+"\");\n");
	}

	public static void record(String method, String arg1, String arg2) {
		if (Recorder.textArea==null) return;
		if (arg1.equals("Open")||arg1.equals("Save")||method.equals("saveAs"))
			arg2 = Recorder.fixPath(arg2);
		Recorder.textArea.append(method+"(\""+arg1+"\", \""+arg2+"\");\n");
	}

	public static void record(String method, String arg1, String arg2, String arg3) {
		if (Recorder.textArea==null) return;
		Recorder.textArea.append(method+"(\""+arg1+"\", \""+arg2+"\",\""+arg3+"\");\n");
	}

	public static void record(String method, int a1) {
		if (Recorder.textArea==null) return;
		if (method.equals("setTool"))
			method = "//"+method;
		Recorder.textArea.append(method+"("+a1+");\n");
	}

	public static void record(String method, int a1, int a2) {
		if (Recorder.textArea==null) return;
		Recorder.textArea.append(method+"("+a1+", "+a2+");\n");
	}

	public static void record(String method, double a1, double a2) {
		if (Recorder.textArea==null) return;
		int places = Math.abs(a1)<0.0001||Math.abs(a2)<0.0001?9:4;
		Recorder.textArea.append(method+"("+IJ.d2s(a1,places)+", "+IJ.d2s(a2,places)+");\n");
	}

	public static void record(String method, int a1, int a2, int a3) {
		if (Recorder.textArea==null) return;
		Recorder.textArea.append(method+"("+a1+", "+a2+", "+a3+");\n");
	}

	public static void record(String method, String a1, int a2) {
		Recorder.textArea.append(method+"(\""+a1+"\", "+a2+");\n");
	}

	public static void record(String method, String args, int a1, int a2) {
		if (Recorder.textArea==null) return;
		method = "//"+method;
		Recorder.textArea.append(method+"(\""+args+"\", "+a1+", "+a2+");\n");
	}

	public static void record(String method, int a1, int a2, int a3, int a4) {
		if (Recorder.textArea==null) return;
		Recorder.textArea.append(method+"("+a1+", "+a2+", "+a3+", "+a4+");\n");
	}

	public static void record(String method, String path, String args, int a1, int a2, int a3, int a4, int a5) {
		if (Recorder.textArea==null) return;
		path = Recorder.fixPath(path);
		method = "//"+method;
		Recorder.textArea.append(method+"(\""+path+"\", "+"\""+args+"\", "+a1+", "+a2+", "+a3+", "+a4+", "+a5+");\n");
	}
	
	public static void recordString (String str) {
		if (Recorder.textArea==null) return;
		Recorder.textArea.append(str);
	}

	public static void recordRoi(Polygon p, int type) {
		if (Recorder.textArea==null) return;
		String method = type==Roi.POLYGON?"makePolygon":"makeLine";
		StringBuffer args = new StringBuffer();
		for (int i=0; i<p.npoints; i++) {
			args.append(p.xpoints[i]+",");
			args.append(""+p.ypoints[i]);
			if (i!=p.npoints-1) args.append(",");
		}
		Recorder.textArea.append(method+"("+args.toString()+");\n");
	}

	public static void recordOption(String key, String value) {
		if (key==null) return;
		key = Recorder.trimKey(key);
		value = Recorder.addQuotes(value);
		Recorder.checkForDuplicate(key+"=", value);
		if (Recorder.commandOptions==null)
			Recorder.commandOptions = key+"="+value;
		else
			Recorder.commandOptions += " "+key+"="+value;
		//IJ.write("  "+key+"="+value);
	}

	public static void recordPath(String key, String path) {
		if (key==null || !Recorder.recordPath)
			{Recorder.recordPath=true; return;}
		key = Recorder.trimKey(key);
		path = Recorder.fixPath(path);
		path = Recorder.addQuotes(path);
		Recorder.checkForDuplicate(key+"=", path);
		if (Recorder.commandOptions==null)
			Recorder.commandOptions = key+"="+path;
		else
			Recorder.commandOptions += " "+key+"="+path;
		//IJ.log("recordPath: "+key+"="+path);
	}

	public static void recordOption(String key) {
		if (key==null) return;
		if (Recorder.commandOptions==null && key.equals(" "))
			Recorder.commandOptions = " ";
		else {
			key = Recorder.trimKey(key);
			Recorder.checkForDuplicate(" "+key, "");
			if (Recorder.commandOptions==null)
				Recorder.commandOptions = key;
			else
				Recorder.commandOptions += " "+key;
		}
	}
	
	static void checkForDuplicate(String key, String value) {
		if (Recorder.commandOptions!=null && Recorder.commandName!=null && Recorder.commandOptions.indexOf(key)!=-1 && (value.equals("") || Recorder.commandOptions.indexOf(value)==-1)) {
			if (key.endsWith("=")) key = key.substring(0, key.length()-1);
			IJ.showMessage("Recorder", "Duplicate keyword:\n \n" 
				+ "    Command: " + "\"" + Recorder.commandName +"\"\n"
				+ "    Keyword: " + "\"" + key +"\"\n"
				+ "    Value: " + value+"\n \n"
				+ "Add an underscore to the corresponding label\n"
				+ "in the dialog to make the first word unique.");
		}
	}
	
	static String trimKey(String key) {
		int index = key.indexOf(" ");
		if (index>-1)
			key = key.substring(0,index);
		index = key.indexOf(":");
		if (index>-1)
			key = key.substring(0,index);
		key = key.toLowerCase(Locale.US);
		return key;
	}

	/** Writes the current command and options to the Recorder window. */
	public static void saveCommand() {
		String name = Recorder.commandName;
		if (name!=null) {
			if (Recorder.commandOptions!=null) {
				if (name.equals("Open..."))
					Recorder.textArea.append("open(\""+Recorder.strip(Recorder.commandOptions)+"\");\n");
				else if (Recorder.isSaveAs()) {
							if (name.endsWith("..."))
									name= name.substring(0, name.length()-3);
							String path = Recorder.strip(Recorder.commandOptions);
							Recorder.textArea.append("saveAs(\""+name+"\", \""+path+"\");\n");
				} else if (name.equals("Image..."))
					Recorder.appendNewImage();
				else if (name.equals("Set Slice..."))
					Recorder.textArea.append("setSlice("+Recorder.strip(Recorder.commandOptions)+");\n");
				else if (name.equals("Rename..."))
					Recorder.textArea.append("rename(\""+Recorder.strip(Recorder.commandOptions)+"\");\n");
				else if (name.equals("Image Calculator..."))
					Recorder.textArea.append("//run(\""+name+"\", \""+Recorder.commandOptions+"\");\n");
				else 
					Recorder.textArea.append("run(\""+name+"\", \""+Recorder.commandOptions+"\");\n");
			} else {
				if (name.equals("Threshold..."))
					Recorder.textArea.append("//run(\""+name+"\");\n");
				else if (name.equals("Start Animation [\\]"))
					Recorder.textArea.append("doCommand(\"Start Animation [\\\\]\");\n");
				else {
					if (IJ.altKeyDown() && (name.equals("Open Next")||name.equals("Plot Profile")))
						Recorder.textArea.append("setKeyDown(\"alt\"); ");
					Recorder.textArea.append("run(\""+name+"\");\n");
				}
			}
		}
		Recorder.commandName = null;
		Recorder.commandOptions = null;
	}
	
	static boolean isSaveAs() {
		return Recorder.commandName.equals("Tiff...")
			|| Recorder.commandName.equals("Gif...")
			|| Recorder.commandName.equals("Jpeg...")
			|| Recorder.commandName.equals("Text Image...")
			|| Recorder.commandName.equals("ZIP...")
			|| Recorder.commandName.equals("Raw Data...")
			|| Recorder.commandName.equals("AVI... ")
			|| Recorder.commandName.equals("BMP...")
			|| Recorder.commandName.equals("PNG...")
			|| Recorder.commandName.equals("PGM...")
			|| Recorder.commandName.equals("FITS...")
			|| Recorder.commandName.equals("LUT...")
			|| Recorder.commandName.equals("Selection...")
			|| Recorder.commandName.equals("XY Coordinates...")
			|| Recorder.commandName.equals("Measurements...")
			|| Recorder.commandName.equals("Text... ");
	}

	static void appendNewImage() {
		String options = Recorder.getCommandOptions() + " ";
		String title = Macro.getValue(options, "name", "Untitled");
		String type = Macro.getValue(options, "type", "8-bit");
		String fill = Macro.getValue(options, "fill", "");
		if (!fill.equals("")) type = type +" " + fill;
		int width = (int)Tools.parseDouble(Macro.getValue(options, "width", "512"));
		int height = (int)Tools.parseDouble(Macro.getValue(options, "height", "512"));
		int depth= (int)Tools.parseDouble(Macro.getValue(options, "slices", "1"));
		Recorder.textArea.append("newImage(\""+title+"\", "+"\""+type+"\", "+width+", "+height+", "+depth+");\n");
	}

	static String strip(String value) {
		int index = value.indexOf('=');
		if (index>=0)
			value = value.substring(index+1);
		if (value.startsWith("["))
			value = value.substring(1, value.length()-1);
		return value;
	}

	static String addQuotes(String value) {
		int index = value.indexOf(' ');
		if (index>-1)
			value = "["+value+"]";
		return value;
	}
	
	/** Use by GenericDialog to determine if any options have been recorded. */
	static public String getCommandOptions() {
		return Recorder.commandOptions;
	}

	void createMacro() {
		String text = Recorder.textArea.getText();
		if (text==null || text.equals("")) {
			IJ.showMessage("Recorder", "A macro cannot be created until at least\none command has been recorded.");
			return;
		}
		Editor ed = (Editor)IJ.runPlugIn("net.sf.ext.ij.plugin.frame.Editor", "");
		if (ed==null)
			return;
		String name = macroName.getText();
		int dotIndex = name.lastIndexOf(".");
		if (dotIndex>=0) name = name.substring(0, dotIndex);
		name += ".txt";
		ed.createMacro(name, text);
	}
	
	/** Temporarily disables path recording. */
	public static void disablePathRecording() {
		Recorder.recordPath = false;
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource()==makeMacro)
			createMacro();
		else if (e.getSource()==help)
			showHelp();
	}

    void showHelp() {
    	IJ.showMessage("Recorder",
			"Click \"Create\" to open recorded commands\n"  
			+"as a macro in an editor window.\n" 
			+" \n" 
			+"In the editor:\n" 
			+" \n"
			+"    Type ctrl+R (Macros>Run Macro) to\n" 
			+"    run the macro.\n"     
			+" \n"    
			+"    Use File>Save As to save it and\n" 
			+"    ImageJ's Open command to open it.\n" 
			+" \n"    
			+"    To create a command, use File>Save As,\n"  
			+"    add a '_' to the name, save in the \n" 
			+"    plugins folder, and restart ImageJ.\n" 
			+" \n"     
			+"    Use Edit>Convert to Plugin to convert\n" 
			+"    the macro to a plugin."
		);
    }
    
    public void windowClosing(WindowEvent e) {
    	close();
	}

	public void close() {
		super.close();
		Recorder.record = false;
		Recorder.textArea = null;
		Recorder.commandName = null;
		Recorder.instance = null;	
	}

}